<?php

# Here are imported all system required files.

class UCIDG_Auto_Load
{
    protected $aMenus;
    protected $aMainMenus;
    protected $oHelpers;
    public $sPageURL = "";

    public function __construct()
    {
        $this->IncludeFiles();

        $this->aMenus     = array();
        $this->aMainMenus = array();
        $this->oHelpers   = new UCIDG_Helpers();

        $this->ShowErrors();
        $this->Localization();
        $this->Hooks();
        $this->Actions();
        $this->Shortcodes();
        $this->DirectAjax();
    }

    public static function instance()
    {
        new UCIDG_Auto_Load();
    }

    public function Hooks()
    {
        #   List of all hooks
    }

    public function Actions()
    {
        $oCodeSearch = new UCIDG_Code_Search($this, $this->oHelpers, $this->aMenus);
        $oCodeList   = new UCIDG_Code_List($this, $this->oHelpers, $this->aMenus);
        $oSettings = new UCIDG_Settings($this, $this->oHelpers, $this->aMenus);
        $oShortcodes = new UCIDG_Shortcodes($this, $this->oHelpers, $this->aMenus);
        $oShortGuide = new UCIDG_Setup_Guide($this, $this->oHelpers, $this->aMenus);

        #   List of all actions
        // add_action( "admin_menu", array( $this, "SetMainMenu" ) , 11, 0 );
        add_action("admin_menu", array( $oCodeSearch, "SetMenu" ), 11, 0);
        add_action("admin_menu", array( $oCodeList, "SetMenu" ), 11, 0);
        add_action("admin_menu", array( $oSettings, "SetMenu" ), 11, 0);
        add_action("admin_menu", array( $oShortcodes, "SetMenu" ), 11, 0);
        // add_action( "admin_menu", array( $oShortGuide, "SetMenu" ) );

        #   Load all admin scripts
        add_action('admin_enqueue_scripts', array( $this, "IncludeScripts" ));

        #   Load all public scripts
        add_action('wp_enqueue_scripts', array( $this, "IncludeWPScripts" ));

        #   Redirect Page
        add_action('wp_loaded', array( $this, "RedirectWP" ));

        #   Set Buffer
        add_action('init', array( $this, 'SetOutputBuffer' ));

        #   LearnDash Certificate Setup
        add_filter('sfwd_cpt_options', array( $this, 'LearnDashCertificateSetup' ), 1, 5);

        #   Localization
        // add_action('plugins_loaded', array( $this, 'Localization' ) );



        #   Ajax
        //  Code Search
        add_action('wp_ajax_ucidg_code_search', array( $oCodeSearch, "CodeSearch" ));
        add_action('wp_ajax_nopriv_ucidg_code_search', array( $oCodeSearch, "CodeSearch" ));
        //  Code Suggestion
        add_action('wp_ajax_ucidg_code_suggestion', array( $oCodeSearch, "CodeSuggestion" ));
        add_action('wp_ajax_nopriv_ucidg_code_suggestion', array( $oCodeSearch, "CodeSuggestion" ));

         //QR Code effect on UserProfile
         add_filter( 'learndash-course-row-class',  array( $oCodeSearch, 'Profile_QR_Code' ) );
         //add_action( 'learndash-course-row-expand-before',  array( $oCodeSearch, 'Profile_QR_Code' ), 10, 2 );
         //add_action( 'learndash-course-row-certificate-before',  array( $oCodeSearch, 'Profile_QR_Code' ), 10, 2 );
         //add_action( 'learndash-course-list-shortcode-course-class',  array( $oCodeSearch, 'Profile_QR_Code' ) );

         
        //  Code List
        add_action('wp_ajax_ucidg_code_list', array( $oCodeList, "CodeList" ));
        add_action('wp_ajax_ucidg_code_delete', array( $oCodeList, "CodeDelete" ));
        add_action('wp_ajax_ucidg_course_certificate', array( $oCodeList, "GetCourseCertificate" ));
        add_action('wp_ajax_ucidg_course_students', array( $oCodeList, "GetCourseStudents" ));
        add_action('wp_ajax_ucidg_quiz_certificate', array( $oCodeList, "GetQuizCertificate" ));
        add_action('wp_ajax_ucidg_quiz_students', array( $oCodeList, "GetQuizStudents" ));
    }

    public function Shortcodes()
    {
        $oCodeSearch = new UCIDG_Code_Search($this, $this->oHelpers, $this->aMenus);

        add_shortcode('unique_certificate_code_generator', array( $oCodeSearch, "CodeGenerator" ));
        add_shortcode('unique_certificate_code_generator_url', array( $oCodeSearch, "CodeGeneratorURL" ));

        add_shortcode('unique_certificate_code_verifier', array( $oCodeSearch, "CodeVerifierStyle" ));
        add_shortcode('unique_certificate_code_verifier_2', array( $oCodeSearch, "CodeVerifierStyle2" ));

        //  QR Code Shortcodes
        add_shortcode('qrcode_unique_certificate_code_generator', array( $oCodeSearch, "QRCodeGenerator" ));
    }

    public function DirectAjax()
    {
        if (isset($_REQUEST['action'])) {
            switch ($_REQUEST['action']) {
                case "ucidg_code_search":
                    //  Code Search
                    $oCodeSearch = new UCIDG_Code_Search($this, $this->oHelpers, $this->aMenus);
                    $oCodeSearch->CodeSearch();
                    die();
                    break;
            }
        }
    }

    public function Localization()
    {
        $plugin_rel_path = UCIDG_PLUGIN_BN. 'languages';
        load_plugin_textdomain(UCIDG_TextDomain, false, $plugin_rel_path);
    }

    public function IncludeFiles()
    {
        require_once UCIDG_PLUGIN_DIR . "resourses/system/activate.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/system/deactivate.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/system/helpers.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/system/wp-users.php";

        //  Controllers
        require_once UCIDG_PLUGIN_DIR . "resourses/controller/code-search.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/controller/code-list.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/controller/settings.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/controller/shortcodes.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/controller/setup-guide.php";

        //  Models
        require_once UCIDG_PLUGIN_DIR . "resourses/model/code.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/model/learndash.php";
        require_once UCIDG_PLUGIN_DIR . "resourses/model/setting.php";

        //  ThirdPaty
        require_once UCIDG_PLUGIN_DIR . "assets/third-parties/barcode/barcode.php";
    }

    public function LearnDashCertificateSetup($content, $post){
        if($post == "sfwd-certificates")
        {
            if(isset($_GET['direct']))
            {
                $course_id = (int) ( isset($_GET['course_id']) && !empty($_GET['course_id']) ? $_GET['course_id'] : 0 );
                $quiz = (int) ( isset($_GET['quiz']) && !empty($_GET['quiz']) ? $_GET['quiz'] : 0 );
                $user_id = (int) ( isset($_GET['user']) && !empty($_GET['user']) ? $_GET['user'] : 0 );

                if($quiz > 0)
                    $course_id = $quiz;

                if($course_id > 0 && $user_id > 0)
                {
                    $certificate = isset($_GET['user']) && !empty($_GET['user']) ? $_GET['user'] : get_current_user_id();
                    $user = isset($_GET['user']) && !empty($_GET['user']) ? $_GET['user'] : get_current_user_id();

                    wp_set_current_user( $user );

                    $_GET['cert-nonce'] = wp_create_nonce( $course_id . $user_id . $user_id );
                }
            }
        }

        return $content;
    }

    public function IncludeScripts($sHook)
    {
        if (count($this->aMenus)) {
            foreach ($this->aMenus as $sMenu) {
                if ($this->oHelpers->bFindInString($sMenu, $sHook)) {
                    $this->IncludeCSS();
                    $this->IncludeJS();
                    break;
                }
            }
        }
    }

    public function IncludeCSS()
    {
        if (UCIDG_LoadScripts) {
            wp_enqueue_style('jquery-ui', UCIDG_PLUGIN_URL . "/assets/css/jquery-ui.css");
            wp_enqueue_style('bootstrap', UCIDG_PLUGIN_URL . "/assets/css/bootstrap.css");
            wp_enqueue_style('glyphicon', UCIDG_PLUGIN_URL . "/assets/css/bootstrap-glyphicon.min.css");
            wp_enqueue_style('watable', UCIDG_PLUGIN_URL . "/assets/css/watable.css");
            wp_enqueue_style('datepicker', UCIDG_PLUGIN_URL . "/assets/css/datepicker.css");
            wp_enqueue_style('noty', UCIDG_PLUGIN_URL . "/assets/css/noty.css");
            wp_enqueue_style('dropzone', UCIDG_PLUGIN_URL . "/assets/css/dropzone.css");
            wp_enqueue_style('ucidg', UCIDG_PLUGIN_URL . "/assets/css/ucidg.css");
        }
    }

    public function IncludeJS()
    {
        if (UCIDG_LoadScripts) {
            // wp_enqueue_script('jquery-1.12.4.js', UCIDG_PLUGIN_URL . "/assets/js/jquery-1.12.4.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('popper-js', UCIDG_PLUGIN_URL . "/assets/js/popper.min.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('bootstrap-js', UCIDG_PLUGIN_URL . "/assets/js/bootstrap.minified.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('bootstrap-datepicker-js', UCIDG_PLUGIN_URL . "/assets/js/bootstrap-datepicker.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('watable-js', UCIDG_PLUGIN_URL . "/assets/js/jquery.watable.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('filesaver-js', UCIDG_PLUGIN_URL . "/assets/js/filesaver.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('xlsx.core-js', UCIDG_PLUGIN_URL . "/assets/js/xlsx.core.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('table.exporter-js', UCIDG_PLUGIN_URL . "/assets/js/table.exporter.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('noty-js', UCIDG_PLUGIN_URL . "/assets/js/noty.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('sweetalert-js', UCIDG_PLUGIN_URL . "/assets/js/sweetalert.min.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('dropzone-js', UCIDG_PLUGIN_URL . "/assets/js/dropzone.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('jquery-ui.js', UCIDG_PLUGIN_URL . "/assets/js/jquery-ui.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('ucidg-js', UCIDG_PLUGIN_URL . "/assets/js/ucidg.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('search-code-js', UCIDG_PLUGIN_URL . "/assets/js/search-code.js", array(), UCIDG_VERSION, true);
        }
    }

    public function IncludeWPScripts($sHook)
    {
        $this->IncludeShortcodeCSS();
        $this->IncludeShortcodeJS();
    }

    public function IncludeShortcodeCSS()
    {
    }

    public function IncludeShortcodeJS()
    {
        global $post;

        if (is_a($post, 'WP_Post') && has_shortcode($post->post_content, 'unique_certificate_code_verifier')) {
            wp_enqueue_style('jquery-ui', UCIDG_PLUGIN_URL . "/assets/css/jquery-ui.css");

            wp_enqueue_script('jquery-1.12.4.js', UCIDG_PLUGIN_URL . "/assets/js/jquery-1.12.4.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('ucidg-js', UCIDG_PLUGIN_URL . "/assets/js/ucidg.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('jquery-ui.js', UCIDG_PLUGIN_URL . "/assets/js/jquery-ui.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('search-code-js', UCIDG_PLUGIN_URL . "/assets/js/search-code.js", array(), UCIDG_VERSION, true);

            // wp_enqueue_script('jquery-ui.js', UCIDG_PLUGIN_URL . "/assets/js/search-code.js", array( 'jquery' ), UCIDG_VERSION, true);
            // wp_enqueue_script('search-code-js', UCIDG_PLUGIN_URL . "/assets/js/search-code.js", array( 'jquery' ), UCIDG_VERSION, true);
        }
        if (is_a($post, 'WP_Post') && has_shortcode($post->post_content, 'unique_certificate_code_verifier_2')) {
            wp_enqueue_style('jquery-ui', UCIDG_PLUGIN_URL . "/assets/css/jquery-ui.css");

            wp_enqueue_script('jquery-1.12.4.js', UCIDG_PLUGIN_URL . "/assets/js/jquery-1.12.4.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('ucidg-js', UCIDG_PLUGIN_URL . "/assets/js/ucidg.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('jquery-ui.js', UCIDG_PLUGIN_URL . "/assets/js/jquery-ui.js", array(), UCIDG_VERSION, true);
            wp_enqueue_script('search-code-js', UCIDG_PLUGIN_URL . "/assets/js/search-code.js", array(), UCIDG_VERSION, true);

            // wp_enqueue_script('jquery-ui.js', UCIDG_PLUGIN_URL . "/assets/js/search-code.js", array( 'jquery' ), UCIDG_VERSION, true);
            // wp_enqueue_script('search-code-js', UCIDG_PLUGIN_URL . "/assets/js/search-code.js", array( 'jquery' ), UCIDG_VERSION, true);
        }
    }

    public function SetMainMenu()
    {
        $sParentSlug = "";
        $sPageSlug   = "ucidg-main";
        $sPageTitle  = "Uniq. Cert. ID Gen.";
        $sMenuTitle  = "Uniq. Cert. ID Gen.";
        $sCapability = "ucidg";

        $this->RegisterMenu($sParentSlug, $sPageSlug, $sPageTitle, $sMenuTitle, $sCapability, "", UCIDG_PLUGIN_URL . "/assets/images/logo.png", 54);
    }

    public function RegisterMenu($sParentSlug, $sPageSlug, $sPageTitle, $sMenuTitle, $sCapability, $objCallBack, $sIconURL = "", $iPosition = null)
    {
        if ($sParentSlug) {
            #   Sub Menu
            return add_submenu_page($sParentSlug, __($sPageTitle, 'LDUCIDG'), __($sMenuTitle, 'LDUCIDG'), $sCapability, $sPageSlug, $objCallBack);
        } else {
            #   Main Menu
            return add_menu_page(__($sPageTitle, 'LDUCIDG'), __($sMenuTitle, 'LDUCIDG'), $sCapability, $sPageSlug, $objCallBack, $sIconURL, $iPosition);
        }
    }

    public function GetMenuPageURL($sSlug)
    {
        return menu_page_url($sSlug, false);
    }

    public function RedirectWP($sPageURL = "")
    {
        if (!$sPageURL) {
            $sPageURL = $this->sPageURL;
        }
        if ($sPageURL) {
            wp_redirect($sPageURL);
            exit;
        }
    }

    public function SetOutputBuffer()
    {
        ob_start();
    }

    public static function PluginActivation()
    {
        $objPlugin = new UCIDG_Auto_Load();

        global $wpdb;

        // check if it is a multisite network
        if (is_multisite()) {
            // check if the plugin has been activated on the network or on a single site
            // get ids of all sites
            $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");
            foreach ($blogids as $blog_id) {
                switch_to_blog($blog_id);
                // create tables for each site
                $objPlugin->PluginActivationTables();
                restore_current_blog();
            }
        } else {
            // activated on a single site
            $objPlugin->PluginActivationTables();
        }
    }

    public function PluginActivationTables()
    {
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        global $wpdb;

        $charset_collate = $wpdb->get_charset_collate();

        $table_name = $wpdb->prefix . 'uci_codes';
        $sql        = "CREATE TABLE $table_name (
          `UserId` int(11) NOT NULL DEFAULT '0',
          `UniqueCode` varchar(30) NOT NULL,
          `UniqueCertificateCode` varchar(30) NOT NULL,
          `CourseId` int(11) DEFAULT NULL,
          `CourseName` varchar(255) DEFAULT NULL,
          `LessonId` int(11) DEFAULT NULL,
          `LessonName` varchar(255) DEFAULT NULL,
          `QuizId` int(11) DEFAULT NULL,
          `QuizName` varchar(255) DEFAULT NULL,
          `CertificateId` int(11) DEFAULT NULL,
          `CertificateName` varchar(255) DEFAULT NULL,
          `CreatedOn` datetime NOT NULL,
          UNIQUE KEY `UniqueCertificateCode` (`UniqueCertificateCode`)
      ) $charset_collate;";

        dbDelta($sql);
    }

    public function ShowErrors()
    {
        if (UCIDG_DISPLAY_ERRORS) {
            ini_set('display_errors', 1);
            ini_set('display_startup_errors', 1);
            error_reporting(E_ALL);
        }
    }
}
