<?php

class UCIDG_Code_List
{
    private $sParentSlug  = "ucidg-main";
    private $sPageSlug    = "ucidg-code-list";
    private $sPageTitle   = "Code List";
    private $sMenuTitle   = "Code List";
    private $sCapability  = "manage_options";
    private $oAutoLoad    = null;
    private $oHelpers     = null;
    private $oCode        = null;
    private $oLearnDash   = null;
    private $sPageLink    = null;
    private $sPageURL     = null;
    private $sAjaxPageURL = null;

    public function __construct($oAutoLoad, $oHelpers, &$aMenus)
    {
        $aMenus[] = $this->sPageSlug;

        $this->oAutoLoad = $oAutoLoad;
        $this->oHelpers  = $oHelpers;

        $temp = array();

        $this->oCode       = new UCIDG_Code();
        $this->oCodeSearch = new UCIDG_Code_Search($oAutoLoad, $oHelpers, $temp);
        $this->oLearnDash  = new UCIDG_LearnDash();

        $this->sAjaxPageURL = $this->oHelpers->sMakeParams(admin_url('admin-ajax.php'));
    }

    public function MainPage()
    {
        $this->sPageURL = $this->oHelpers->sMakeParams($this->oAutoLoad->GetMenuPageURL($this->sPageSlug));

        $sUCIDG = $this->oHelpers->sParamRequest("UCIDG");
        switch ($sUCIDG) {
            case "LoadFormCourses":
                $this->LoadFormCourses();
                break;
            case "LoadFormQuiz":
                $this->LoadFormQuiz();
                break;
            case "ShowGrid":
            default:
                $this->ShowGrid();
                break;
        }
    }

    public function ShowGrid()
    {
        require UCIDG_PLUGIN_DIR . 'resourses/view/code-list-view.php';
    }

    public function LoadFormCourses()
    {
        $aResponse = $this->SaveFormCourses();
        if ($aResponse[ 'Page' ] == "ShowGrid") {
            $this->oAutoLoad->RedirectWP($this->sPageURL);
        } else {
            //  Is Admin Logged In
            $aCourses      = $this->oLearnDash->GetCourses(ARRAY_A);
            $aCertificates = $this->oLearnDash->GetCertificates(ARRAY_A);
            if ($aCourses === false) {
                $aCourses      = array();
            }
            if ($aCertificates === false) {
                $aCertificates = array();
            }

            require UCIDG_PLUGIN_DIR . 'resourses/view/code-course-form.php';
        }
    }

    public function SaveFormCourses()
    {
        $aResponse = array();

        // Now Getting Form Values
        $aResponse[ 'Data' ][ 'ID' ]          = $this->oHelpers->sParamRequest("ID", true);
        $aResponse[ 'Data' ][ 'Course' ]      = $this->oHelpers->sParamPost("course", true);
        $aResponse[ 'Data' ][ 'Certificate' ] = $this->oHelpers->sParamPost("certificate", true);
        $aResponse[ 'Data' ][ 'StudentIDs' ]  = $this->oHelpers->sParamPost("student-ids", true);
        $aResponse[ 'Page' ]                  = "";

        if ($aResponse[ 'Data' ][ 'Course' ] !== false) {
            // Form Is Submitted

            $aResponse = $this->aCheckCourseDetails($aResponse);
            if (!isset($aResponse[ 'Error' ])) {
                // Data is valid and able to store in WordPress.

                $aRecord = $this->oCode->GenerateCourseCodes($aResponse[ 'Data' ], $this->oCodeSearch);

                if ($aRecord[ 'Error' ]) {
                    $aResponse[ 'Error' ][ 'AdminError' ] = __("Records could not be saved. Please! contact with you administrator.", UCIDG_TextDomain);
                } else {
                    $aResponse[ 'Success' ] = __("Courses certificate codes saved successfully!", UCIDG_TextDomain);

                    if ($aResponse[ 'Data' ][ 'ID' ] <= 0) {
                        $aResponse[ 'Page' ] = "ShowGrid";
                    }
                }
            }
        }

        return $aResponse;
    }

    public function aCheckCourseDetails($aResponse)
    {
        if ($aResponse[ 'Data' ][ 'Course' ] <= 0) {
            $aResponse[ 'Error' ][ 'Course' ] = __("Invalid Course", UCIDG_TextDomain);
        } elseif ($aResponse[ 'Data' ][ 'Course' ] != intval($aResponse[ 'Data' ][ 'Course' ])) {
            $aResponse[ 'Error' ][ 'Course' ] = __("Invalid Course", UCIDG_TextDomain);
        }
        return $aResponse;
    }

    public function aCheckQuizDetails($aResponse)
    {
        if ($aResponse[ 'Data' ][ 'Quiz' ] <= 0) {
            $aResponse[ 'Error' ][ 'Quiz' ] = __("Invalid Quiz", UCIDG_TextDomain);
        } elseif ($aResponse[ 'Data' ][ 'Quiz' ] != intval($aResponse[ 'Data' ][ 'Quiz' ])) {
            $aResponse[ 'Error' ][ 'Quiz' ] = __("Invalid Quiz", UCIDG_TextDomain);
        }
        return $aResponse;
    }

    public function LoadFormQuiz()
    {
        $aResponse = $this->SaveFormQuiz();
        if ($aResponse[ 'Page' ] == "ShowGrid") {
            $this->oAutoLoad->RedirectWP($this->sPageURL);
        } else {
            //  Is Admin Logged In
            $aQuiz         = $this->oLearnDash->GetQuiz(ARRAY_A);
            $aCertificates = $this->oLearnDash->GetCertificates(ARRAY_A);
            if ($aQuiz === false) {
                $aQuiz         = array();
            }

            if ($aCertificates === false) {
                $aCertificates = array();
            }

            require UCIDG_PLUGIN_DIR . 'resourses/view/code-quiz-form.php';
        }
    }

    public function SaveFormQuiz()
    {
        $aResponse = array();

        // Now Getting Form Values
        $aResponse[ 'Data' ][ 'ID' ]          = $this->oHelpers->sParamRequest("ID", true);
        $aResponse[ 'Data' ][ 'Quiz' ]        = $this->oHelpers->sParamPost("quiz", true);
        $aResponse[ 'Data' ][ 'Certificate' ] = $this->oHelpers->sParamPost("certificate", true);
        $aResponse[ 'Data' ][ 'StudentIDs' ]  = $this->oHelpers->sParamPost("student-ids", true);
        $aResponse[ 'Page' ]                  = "";

        if ($aResponse[ 'Data' ][ 'Quiz' ] !== false) {
            // Form Is Submitted


            $aResponse = $this->aCheckQuizDetails($aResponse);

            if (!isset($aResponse[ 'Error' ])) {
                // Data is valid and able to store in WordPress.

                $aRecord = $this->oCode->GenerateQuizCodes($aResponse[ 'Data' ], $this->oCodeSearch);

                if ($aRecord[ 'Error' ]) {
                    $aResponse[ 'Error' ][ 'AdminError' ] = __("Records could not be saved. Please! contact with you administrator.", UCIDG_TextDomain);
                } else {
                    $aResponse[ 'Success' ] = __("Quiz certificate codes saved successfully!", UCIDG_TextDomain);

                    if ($aResponse[ 'Data' ][ 'ID' ] <= 0) {
                        $aResponse[ 'Page' ] = "ShowGrid";
                    }
                }
            }
        }

        return $aResponse;
    }

    public function SetMenu()
    {
        $this->sPageLink = $this->oAutoLoad->RegisterMenu($this->sParentSlug, $this->sPageSlug, $this->sPageTitle, $this->sMenuTitle, $this->sCapability, array( $this, "MainPage" ));
    }

    public function CodeList()
    {
        $this->oSetting = new UCIDG_Setting();
        //  Get data from WordPress
        $aData = $this->oSetting->GetSettings();
        $sSearchURL = "";
        if ($aData) {
            $sSearchURL = $aData['SearchURL'];
            if (trim($sSearchURL) != "") {
                if (strpos($sSearchURL, "?") === false) {
                    $sSearchURL .= "?";
                }
            }
        }

        $aCodeList = $this->oCode->GetCodes("", ARRAY_A, $sSearchURL);

        $iCounterColumn = 1;
        $aResponse      = array(
            "cols" => array(
                "ID"                    => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "number",
                    "friendly" => __("No#", UCIDG_TextDomain),
                    "filter"   => false,
                ),
                "UniqueCertificateCode" => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Code", UCIDG_TextDomain),
                    "unique"   => true,
                ),
                "UniqueCertificateCodeURL" => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Search URL", UCIDG_TextDomain),
                    "unique"   => true,
                ),
                "FullName"              => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Full Name", UCIDG_TextDomain),
                ),
                "FirstName"             => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("First Name", UCIDG_TextDomain),
                    "hidden"   => true,
                ),
                "LastName"              => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Last Name", UCIDG_TextDomain),
                    "hidden"   => true,
                ),
                "StudentEmail"          => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Student Email", UCIDG_TextDomain),
                ),
                "CourseName"            => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Course Name", UCIDG_TextDomain),
                ),
                "LessonName"            => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Lesson Name", UCIDG_TextDomain),
                ),
                "QuizName"              => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Quiz Name", UCIDG_TextDomain),
                ),
                "CertificateName"       => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Certificate Name", UCIDG_TextDomain),
                ),
                "CertificateDate"       => array(
                    "index"       => $iCounterColumn++,
                    "type"        => "string",
                    "placeHolder" => "Type date",
                    "friendly"    => __("Date Of Certificate", UCIDG_TextDomain),
                    "tooltip"     => __("You can find date by entering year, month, and/or date.", UCIDG_TextDomain),
                    "hidden"      => true
                ),
                "remove"                => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => "<span class='glyphicon glyphicon-trash'></span>",
                    "format"   => "<a href='#' title='Remove'><span class='glyphicon glyphicon-trash'></span></a>",
                    "filter"   => false,
                    "sorting"  => false,
                    "hidden"   => false, //For the sake of this demo, this column gets hidden for example1
                    "title"    => false,
                ),
                "check"                 => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => "<span class='glyphicon glyphicon-check'></span>",
                    "format"   => "<a href='#' title='Toggle checked'><span class='glyphicon glyphicon-check'></span></a>",
                    "filter"   => false,
                    "sorting"  => false,
                    "hidden"   => false, //For the sake of this demo, this column gets hidden for example1
                    "title"    => false
                ),
            ), "rows" => $aCodeList );

        echo json_encode($aResponse);

        wp_die();
    }

    public function CodeDelete()
    {
        $sMessage = "";
        $sMessage = "Success";
        $sCode    = $this->oHelpers->sParamRequest("code", true);

        if ($this->oCode->DeleteCode($sCode) === false) {
            $sMessage = __("Could not be deleted. Please! Contact with your administrator.", UCIDG_TextDomain);
        } else {
            $sMessage = "Success";
        }

        echo json_encode(array( "type" => $sMessage ));

        wp_die();
    }

    public function GetCourseCertificate()
    {
        $iCourseID = $this->oHelpers->sParamRequest("course", true);

        $objCourse      = get_post_meta($iCourseID, "_sfwd-courses", true);
        $iCertificateID = $objCourse[ 'sfwd-courses_certificate' ];

        echo json_encode(array( "certificate" => $iCertificateID ));
        wp_die();
    }

    public function GetCourseStudents()
    {
        $iCourseID      = $this->oHelpers->sParamRequest("course", true);
        $iCertificateID = $this->oHelpers->sParamRequest("certificate", true);
        $oUsers         = new UCIDG_WP_Users();
        $aUsers         = $oUsers->aGetAllUsers(ARRAY_A);
        $iUsers         = 0;

        $aCodeList = $this->oCode->GetCodes(" AND CD.CertificateID = '$iCertificateID' AND CD.CourseID = '$iCourseID' ", ARRAY_A);
        $iCodeList = 0;

        if ($aCodeList !== false) {
            $iCodeList = count($aCodeList);
        }

        if ($aUsers !== false) {
            $iUsers = count($aUsers);
        }

        //  Removing Those users who have already got certificates.
        if ($iCodeList > 0 && $iUsers > 0) {
            foreach ($aCodeList as $aCode) {
                foreach ($aUsers as $iIndex => $aUser) {
                    if ($aCode[ 'UserId' ] == $aUser[ 'ID' ]) {
                        unset($aUsers[ $iIndex ]);
                        break;
                    }
                }
            }
        }

        //  Removing those students who are unable to get certificate
        if (count($aUsers) > 0) {
            foreach ($aUsers as $iIndex => $aUser) {
                $bUserCourseFound = false;

                $user_courses = learndash_user_get_enrolled_courses($aUser[ 'ID' ]);

                if (empty($user_courses)) {
                    $user_courses = array();
                }

                if (!empty($user_courses)) {
                    foreach ($user_courses as $course_id) {
                        if ($course_id == $iCourseID) {
                            $bUserCourseFound = true;
                        }
                    }
                }

                if (!$bUserCourseFound) {
                    unset($aUsers[ $iIndex ]);
                }
            }
        }

        $aUsersTemp = array();
        if (count($aUsers)) {
            foreach ($aUsers as $aUser) {
                $aUsersTemp[] = $aUser;
            }
        }
        $aUsers = $aUsersTemp;

        //  Again Making Array indexes.

        $iCounterColumn = 1;
        $aResponse      = array(
            "cols" => array(
                "ID"          => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "number",
                    "friendly" => __("ID", UCIDG_TextDomain),
                    "unique"   => true,
                ),
                "DisplayName" => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Display Name", UCIDG_TextDomain),
                ),
                "FirstName"   => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("First Name", UCIDG_TextDomain),
                ),
                "LastName"    => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Last Name", UCIDG_TextDomain),
                ),
                "UserEmail"   => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Student Email", UCIDG_TextDomain),
                ),
            ), "rows" => $aUsers );

        echo json_encode($aResponse);

        wp_die();
    }

    public function GetQuizCertificate()
    {
        $iQuizID = $this->oHelpers->sParamRequest("quiz", true);


        $objCourse      = get_post_meta($iQuizID, "_sfwd-quiz", true);
        $iCertificateID = $objCourse[ 'sfwd-quiz_certificate' ];

        echo json_encode(array( "certificate" => $iCertificateID ));
        wp_die();
    }

    public function GetQuizStudents()
    {
        $iQuizID        = $this->oHelpers->sParamRequest("quiz", true);
        $iCertificateID = $this->oHelpers->sParamRequest("certificate", true);
        $oUsers         = new UCIDG_WP_Users();
        $aUsers         = $oUsers->aGetAllUsers(ARRAY_A);
        $iUsers         = 0;

        $aCodeList = $this->oCode->GetCodes(" AND CD.CertificateID = '$iCertificateID' AND CD.QuizID = '$iQuizID' ", ARRAY_A);
        $iCodeList = 0;

        if ($aCodeList !== false) {
            $iCodeList = count($aCodeList);
        }

        if ($aUsers !== false) {
            $iUsers = count($aUsers);
        }

        if ($iCodeList > 0 && $iUsers > 0) {
            foreach ($aCodeList as $aCode) {
                foreach ($aUsers as $iIndex => $aUser) {
                    if ($aCode[ 'UserId' ] == $aUser[ 'ID' ]) {
                        unset($aUsers[ $iIndex ]);
                        break;
                    }
                }
            }
        }

        //  Removing those students who are unable to get certificate
        if (count($aUsers) > 0) {
            $objCourse = get_post_meta($iQuizID, "_sfwd-quiz", true);
            $iCourseID = $objCourse[ 'sfwd-quiz_course' ];
            if ($iCourseID > 0) {
                foreach ($aUsers as $iIndex => $aUser) {
                    $bUserCourseFound = false;

                    $user_courses = learndash_user_get_enrolled_courses($aUser[ 'ID' ]);
                    if (empty($user_courses)) {
                        $user_courses = array();
                    }

                    if (!empty($user_courses)) {
                        foreach ($user_courses as $course_id) {
                            if ($course_id == $iCourseID) {
                                $bUserCourseFound = true;
                            }
                        }
                    }

                    if (!$bUserCourseFound) {
                        unset($aUsers[ $iIndex ]);
                    }
                }
            }
        }


        $aUsersTemp = array();
        if (count($aUsers)) {
            foreach ($aUsers as $aUser) {
                $aUsersTemp[] = $aUser;
            }
        }
        $aUsers = $aUsersTemp;

        //  Again Making Array indexes.

        $iCounterColumn = 1;
        $aResponse      = array(
            "cols" => array(
                "ID"          => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "number",
                    "friendly" => __("ID", UCIDG_TextDomain),
                    "unique"   => true,
                ),
                "DisplayName" => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Display Name", UCIDG_TextDomain),
                ),
                "FirstName"   => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("First Name", UCIDG_TextDomain),
                ),
                "LastName"    => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Last Name", UCIDG_TextDomain),
                ),
                "UserEmail"   => array(
                    "index"    => $iCounterColumn++,
                    "type"     => "string",
                    "friendly" => __("Student Email", UCIDG_TextDomain),
                ),
            ), "rows" => $aUsers );

        echo json_encode($aResponse);

        wp_die();
    }
}
