<?php

class UCIDG_Code_Search
{
    private $sParentSlug  = "";
    // private $sParentSlug  = "ucidg-main";
    // private $sPageSlug    = "ucidg-code-search";
    private $sPageSlug    = "ucidg-main";
    private $sPageTitle   = "Uniq. Cert. ID Gen.";
    private $sMenuTitle   = "Uniq. Cert. ID Gen.";
    private $sCapability  = "manage_options";
    private $oAutoLoad    = null;
    private $oHelpers     = null;
    private $oCode        = null;
    private $sPageLink    = null;
    private $sPageURL     = null;
    private $sAjaxPageURL = null;

    public function __construct($oAutoLoad, $oHelpers, &$aMenus)
    {
        $aMenus[] = $this->sPageSlug;

        $this->oAutoLoad = $oAutoLoad;
        $this->oHelpers  = $oHelpers;

        $this->oCode = new UCIDG_Code();

        $this->sAjaxPageURL = $this->oHelpers->sMakeParams(admin_url('admin-ajax.php'));
    }

    public function MainPage()
    {
        $this->sPageURL = $this->oHelpers->sMakeParams($this->oAutoLoad->GetMenuPageURL($this->sPageSlug));

        $sUCIDG = $this->oHelpers->sParamRequest("UCIDG");
        switch ($sUCIDG) {
            default:
                $this->LoadForm();
                break;
        }
    }

    public function LoadForm()
    {
        $this->oSetting = new UCIDG_Setting();
        $aData = $this->oSetting->GetSettings_SearchOptions();
        $bName = true;
        $bEmail = true;
        if ($aData) {
            if ($aData['NameSearch'] == "inactive") {
                $bName = false;
            }
            if ($aData['EmailSearch'] == "inactive") {
                $bEmail = false;
            }
        }

        $sText = "";
        if ($bName && $bEmail) {
            $sText = __("Search by User ID, Certificate ID or Email", UCIDG_TextDomain);
        } elseif ($bName) {
            $sText = __("Search by User ID or Certificate ID", UCIDG_TextDomain);
        } elseif ($bEmail) {
            $sText = __("Search by Certificate ID or Email", UCIDG_TextDomain);
        } else {
            $sText = __("Search by Certificate ID", UCIDG_TextDomain);
        }

        require UCIDG_PLUGIN_DIR . 'resourses/view/code-search-form.php';
    }

    public function SetMenu()
    {
        $this->sPageLink = $this->oAutoLoad->RegisterMenu($this->sParentSlug, $this->sPageSlug, $this->sPageTitle, $this->sMenuTitle, $this->sCapability, array(
            $this,
            "MainPage"
          ), UCIDG_PLUGIN_URL . "/assets/images/logo.png", 54);
    }

    public function CodeSearch()
    {
        $sUniqueCertificateCode = $this->oHelpers->sParamRequest("UniqueCertificateCode", true);
        echo json_encode($this->CodeSearchResponse($sUniqueCertificateCode));
    }

    public function CodeSearchResponse($sUniqueCertificateCode)
    {
        $aResponse = array();

        $this->oSetting = new UCIDG_Setting();
        $aData = $this->oSetting->GetSettings_SearchOptions();
        $bName = true;
        $bEmail = true;
        if ($aData) {
            if ($aData['NameSearch'] == "inactive") {
                $bName = false;
            }
            if ($aData['EmailSearch'] == "inactive") {
                $bEmail = false;
            }
        }

        $sText = "";
        if ($bName && $bEmail) {
            $sText = __("Please enter User ID, Certificate ID or Email", UCIDG_TextDomain);
        } elseif ($bName) {
            $sText = __("Please enter User ID or Certificate ID", UCIDG_TextDomain);
        } elseif ($bEmail) {
            $sText = __("Please enter Certificate ID or Email", UCIDG_TextDomain);
        } else {
            $sText = __("Please enter Certificate ID", UCIDG_TextDomain);
        }

        if ($sUniqueCertificateCode) {
            $aStudent = $this->oCode->GetCode($sUniqueCertificateCode);
            if ($aStudent) {
                $sContent = "";

                $iUserId = $aStudent->UserId;
                $sCertificateButton = "";

                $sCertificateButtonQuiz = "";
                if ($aStudent->QuizId > 0) {
                    $aDetails = $this->GetQuizCertificateLink($aStudent->QuizId, $aStudent->UserId);
                    $sCertificateButtonQuiz .= is_array($aDetails) && isset($aDetails['certificateLink']) ? $aDetails['certificateLink'] : '';
                }

                $sCertificateButtonCertificate = "";
                if ($aStudent->CourseId > 0) {
                    $sCertificateButtonCertificate = $this->GetCourseCertificateLink($aStudent->CourseId, $aStudent->UserId);
                }

                if(strlen($sCertificateButtonQuiz) > 0)
                    $sCertificateButton = $sCertificateButtonQuiz;
                else if( strlen($sCertificateButtonCertificate) > 0)
                    $sCertificateButton = $sCertificateButtonCertificate;

                if ($sCertificateButton != "") {
                    $sCertificateButton = '<a href="'.$sCertificateButton.'" class="ucidg_print_button" target="_blank">'.__("Print", UCIDG_TextDomain).'</a>';
                }

                //  Get data from WordPress
                $this->oSetting = new UCIDG_Setting();
                $aStyle = $this->oSetting->GetSettings_FontColor();
                $sFontSize = $aStyle['FontSize'];
                $sColor = $aStyle['Color'];


                $aData = $this->oSetting->GetVerficationPageFields();

                $bUserPhoto = false;

                if (is_array($aData) && isset($aData['VerficationPageFields']) && count($aData['VerficationPageFields']) > 0) {
                    foreach ($aData['VerficationPageFields'] as $aDatum) {
                        if (!($aDatum['ask'] == true && $aDatum['status'] == 'hide')) {
                            // $sContentX .= "<br />". json_encode($aDatum)." <br />";
                            if ($aDatum['name'] == "User Photo") {
                                $bUserPhoto = true;
                                continue;
                            } elseif ($aDatum['name'] == __("Certificate Print Button", UCIDG_TextDomain)) {
                                $sContent .= '
                                    <div class="ucidg_user_info_details" style="'.UCIDG_TextStyle.'">
                                        <div class="ucidg_user_details" style="'.UCIDG_TextStyle.' font-size:'.$sFontSize.'px; color: '.$sColor.'">
                                            '.__("Certificate", UCIDG_TextDomain).':
                                        </div>
                                        <div class="ucidg_user_content" style="'.UCIDG_TextStyle.' font-size:'.$sFontSize.'px; color: '.$sColor.'">
                                            '.$sCertificateButton.'
                                        </div>
                                    </div>
                                ';
                            } else {
                                $sContent .= '
                                    <div class="ucidg_user_info_details" style="'.UCIDG_TextStyle.'">
                                        <div class="ucidg_user_details" style="'.UCIDG_TextStyle.' font-size:'.$sFontSize.'px; color: '.$sColor.'">
                                            '.$aDatum['name'].':
                                        </div>
                                        <div class="ucidg_user_content" style="'.UCIDG_TextStyle.' font-size:'.$sFontSize.'px; color: '.$sColor.'">
                                            '.($aStudent->{$aDatum['field']} ? $aStudent->{$aDatum['field']} : "-----").'
                                        </div>
                                    </div>
                                ';
                            }
                        }
                    }
                }


                $sUserProfileURL = get_avatar_url($bUserPhoto ? $aStudent->user_email : "");

                $sContent .= '
                    <style>

                        .ucidg_print_button
                        {
                            border: none;
                            font-weight: 400;
                            -webkit-font-smoothing: subpixel-antialiased;
                            -webkit-transition: border .25s linear,color .25s linear,background-color .25s linear,background .25s linear;
                            transition: border .25s linear,color .25s linear,background-color .25s linear,background .25s linear;
                            color: #fff;
                            background-color: #3498db;
                            background: #3498db;
                            padding: 10px 19px;
                            font-size: 17px;
                            line-height: 1.471;
                            border-radius: 6px;
                            white-space: normal;

                            color: #f0f0ff;
                            background-color: #5dade2;
                            border-color: #5dade2;
                            text-decoration: none !important;
                        }

                        .ucidg_print_button:hover
                        {
                            text-decoration: none !important;
                            color: #fff;
                        }

                    </style>
                ';

                $aResponse = (array( "type" => "Success",
                    "html" => '

                        <div class="ucidg_user_info">
                        <div class="ucidg_user_photo_container">
                            <img src="'.$sUserProfileURL.'" class="ucidg_user_photo" />
                        </div>
                        <div class="ucidg_user_details_container">
                            <table class="ucidg_user_details_table">
                                <tr>
                                    <td>
                                        '.$sContent.'
                                        <div class="clearfix"></div>
                                    </td>
                                </tr>
                            </table>
                        </div>
                        <div class="clearfix"></div>
                    ',
                    'data' => array(
                        "name" => $aStudent->display_name,
                        "first_name" => $aStudent->first_name,
                        "first_name_field" => __("First Name", UCIDG_TextDomain),
                        "last_name" => $aStudent->last_name,
                        "last_name_field" => __("Last Name", UCIDG_TextDomain),
                        "course" => $aStudent->CourseName,
                        "course_field" => __("Course Name", UCIDG_TextDomain),
                        "lesson" => $aStudent->LessonName,
                        "lesson_field" => __("Lesson Name", UCIDG_TextDomain),
                        "quiz" => $aStudent->QuizName,
                        "quiz_field" => __("Quiz Name", UCIDG_TextDomain),
                        'certificate' => $aStudent->CertificateName,
                        'certificate_field' => __("Certificate Name", UCIDG_TextDomain),
                        'certificate_date' => $aStudent->CertificateDate,
                        'certificate_date_field' => __("Date Of Certificate", UCIDG_TextDomain),
                        'text_style' => UCIDG_TextStyle,
                    )
                ));
            } else {
                $aResponse = (array( "type" => "Fail", "message" => __("Invalid details given.", UCIDG_TextDomain), 'text_style' => UCIDG_TextStyle ));
            }
        } else {
            $aResponse = (array( "type" => "Fail", "message" => $sText, 'text_style' => UCIDG_TextStyle ));
        }

        return $aResponse;
    }

    public function GetCourseCertificateLink($course_id, $cert_user_id = null)
    {
        $cert_user_id = ! empty( $cert_user_id ) ? intval( $cert_user_id ) : get_current_user_id();

        //if ( ( empty( $course_id ) ) || ( empty( $cert_user_id ) ) || ( !sfwd_lms_has_access( $course_id, $cert_user_id ) ) ) {
        if ( ( empty( $course_id ) ) || ( empty( $cert_user_id ) ) ) {
            return '';
        }

        $certificate_id = learndash_get_setting( $course_id, 'certificate' );
        if ( empty( $certificate_id ) ) {
            return '';
        }

        if ( ( learndash_get_post_type_slug( 'certificate' ) !== get_post_type( $certificate_id ) ) ) {
            return '';
        }

        if ( ( learndash_get_post_type_slug( 'course' ) !== get_post_type( $course_id ) ) ) {
            return '';
        }

        if ( ( learndash_is_admin_user() ) || ( learndash_is_group_leader_user() ) )
            $view_user_id = get_current_user_id();
        else
            $view_user_id = $cert_user_id;

        $cert_query_args = array(
            "course_id" =>  $course_id,
        );

        // We add the user query string key/value if the viewing user is an admin. This
        // allows the admin to view other user's certificated
        if ( ( $cert_user_id != $view_user_id ) && ( ( learndash_is_admin_user() ) || ( learndash_is_group_leader_user() ) ) ) {
            $cert_query_args['user'] = $cert_user_id;
        }

        // $cert_query_args['cert-nonce'] = wp_create_nonce( $course_id . $cert_user_id . $view_user_id );
        // $cert_query_args['cert-nonce'] = wp_create_nonce( $course_id . $cert_user_id . $view_user_id );

        $cert_query_args['user'] = $cert_user_id;
        $cert_query_args['direct'] = $cert_user_id;
        $cert_query_args['cert-nonce'] = wp_create_nonce( $course_id . $cert_user_id . $cert_user_id);

        $url = add_query_arg( $cert_query_args, get_permalink( $certificate_id ) );

        return $url;
    }

    public function GetQuizCertificateLink( $post_id, $cert_user_id = null ) {
    	$cert_details = array();

    	$cert_user_id = ! empty( $cert_user_id ) ? intval( $cert_user_id ) : get_current_user_id();

    	if ( ( learndash_is_admin_user() ) || ( learndash_is_group_leader_user() ) )
    		$view_user_id = get_current_user_id();
    	else
    		$view_user_id = $cert_user_id;

    	$certificateLink = '';
    	$post = get_post( $post_id );

    	if ( ( $post instanceof WP_Post ) && ( $post->post_type == 'sfwd-quiz' ) ) {

    		$meta = get_post_meta( $post_id, '_sfwd-quiz', true );
    		if ( is_array( $meta ) && ! empty( $meta ) ) {

    			if ( ( isset( $meta['sfwd-quiz_threshold'] ) ) && ( '' !== $meta['sfwd-quiz_threshold'] ) ) {
    				$certificate_threshold = $meta['sfwd-quiz_threshold'];
    			} else {
    				$certificate_threshold = '0.8';
    			}

    			if ( (isset( $meta['sfwd-quiz_certificate'] )) && ( ! empty( $meta['sfwd-quiz_certificate'] ) ) ) {
    				$certificate_post = intval($meta['sfwd-quiz_certificate']);
    				$certificateLink = get_permalink( $certificate_post );

    				if ( ! empty( $certificateLink ) ) {

    					$cert_query_args = array(
    						"quiz"	=>	$post->ID,
    					);

    					//$course_id = learndash_get_course_id();
    					//if ( !empty( $course_id ) ) {
    					//	$cert_query_args['course_id'] = $course_id;
    					//}

    					// We add the user query string key/value if the viewing user is an admin. This
    					// allows the admin to view other user's certificated
    					if ( ( $cert_user_id != $view_user_id ) && ( ( learndash_is_admin_user() ) || ( learndash_is_group_leader_user() ) ) ) {
    						$cert_query_args['user'] = $cert_user_id;
    					}
    					$cert_query_args['cert-nonce'] = wp_create_nonce( $post->ID . $cert_user_id . $view_user_id );

    					//error_log('cert_query_args<pre>'. print_r($cert_query_args, true) .'</pre>');

                        $cert_query_args['user'] = $cert_user_id;
                        $cert_query_args['direct'] = $cert_user_id;


    					$certificateLink = add_query_arg( $cert_query_args, $certificateLink );
    				}
    				// $certificateLink = apply_filters('learndash_certificate_details_link', $certificateLink, $certificate_post, $post->ID, $cert_user_id);

    				$cert_details = array( 'certificateLink' => $certificateLink, 'certificate_threshold' => $certificate_threshold );
    			}
    		}
    	}

    	return $cert_details;
    }


    public function CodeVerifierStyle()
    {
        if (!is_admin() && (is_single() || is_page() || is_singular())) {
            $this->oSetting = new UCIDG_Setting();
            $aData = $this->oSetting->GetSettings_SearchOptions();
            $bName = true;
            $bEmail = true;
            if ($aData) {
                if ($aData['NameSearch'] == "inactive") {
                    $bName = false;
                }
                if ($aData['EmailSearch'] == "inactive") {
                    $bEmail = false;
                }
            }

            $sText = "";
            if ($bName && $bEmail) {
                $sText = __("Search by User ID, Certificate ID or Email", UCIDG_TextDomain);
            } elseif ($bName) {
                $sText = __("Search by User ID or Certificate ID", UCIDG_TextDomain);
            } elseif ($bEmail) {
                $sText = __("Search by Certificate ID or Email", UCIDG_TextDomain);
            } else {
                $sText = __("Search by Certificate ID", UCIDG_TextDomain);
            }

            $sCode   =  $this->oHelpers->sParamRequest("code", true) ;
            $sResponse = "";
            $aResponse = $this->CodeSearchResponse($sCode);
            if ($aResponse['type'] == "Success") {
                $sResponse = $aResponse['html'];
            }

            ob_start();
            require UCIDG_PLUGIN_DIR . 'resourses/view/code-verifier-style.php';
            $html = ob_get_clean();

            return $html;
        }
    }

    public function CodeVerifierStyle2()
    {
        if (!is_admin() && (is_single() || is_page() || is_singular())) {
            require UCIDG_PLUGIN_DIR . 'resourses/view/code-verifier-style-2.php';
        }
    }

    public function CodeSuggestion()
    {
        $sUniqueCertificateCode = $this->oHelpers->sParamRequest("term", true);
        $aResponse = array();
        if ($sUniqueCertificateCode) {
            $aStudent = $this->oCode->GetCode($sUniqueCertificateCode, true);
            if (count($aStudent) > 0) {
                foreach ($aStudent as $aCertificate) {
                    $sCode = $aCertificate->UniqueCertificateCode;
                    $aResponse[] = array(
                        'id' => $sCode,
                        'label' => $sCode,
                        'value' => $sCode,
                    );
                }
            }
        }
        echo json_encode($aResponse);
        wp_die();
    }

    public function CodeGeneratorURL($iUserId = 0, $iCertificateId = 0)
    {
        $this->oSetting = new UCIDG_Setting();
        //  Get data from WordPress
        $aData = $this->oSetting->GetSettings();
        $sSearchURL = "";
        if ($aData) {
            $sSearchURL = $aData['SearchURL'];
            if (trim($sSearchURL) != "") {
                if (strpos($sSearchURL, "?") === false) {
                    $sSearchURL .= "?";
                }

                $sSearchURL .= "&code=". $this->CodeGenerator($iUserId, $iCertificateId);
            }
        }

        return $sSearchURL;
    }

    public function CodeGenerator($iUserId = 0, $iCertificateId = 0)
    {
        //  1-151-222-2878
        global $post;

        $iQuizId   = intval($this->oHelpers->sParamRequest("quiz", true));
        $sQuizName = "";
        if ($iQuizId > 0) {
            $sQuizName = get_the_title($iQuizId);
        }

        if ($iCertificateId <= 0) {
            $iCertificateId   = $post->ID;
            $sCertificateName = $post->post_title;
        } else {
            $sCertificateName = get_the_title($iCertificateId);
        }

        $iLessonId = 0;
        $sLessonName = "";
        if ($iQuizId > 0) {
            $iLessonId = intval(get_post_meta($iQuizId, 'lesson_id', true));
        }

        $iCourseId = 0;
        $sCourseName = "";
        if ($iQuizId > 0) {
            $iCourseId = intval(get_post_meta($iQuizId, 'course_id', true));
        }

        if ($iLessonId > 0) {
            $sLessonName = get_the_title($iLessonId);
        }


        if ($iCourseId == 0 && $iQuizId == 0) {
            $iCourseId = $this->oHelpers->sParamRequest("course_id", true);
        }


        if ($iCourseId > 0) {
            $sCourseName = get_the_title($iCourseId);
        }

        $sTime   = time() . "";
        if (!$iUserId) {
            $iUserId = get_current_user_id();
        }

        $sUniqueCode            = $sTime[ 0 ] . $sTime[ 1 ] . $sTime[ 2 ] . "-" . $sTime[ 3 ] . $sTime[ 4 ] . $sTime[ 5 ] . "-" . $sTime[ 6 ] . $sTime[ 7 ] . $sTime[ 8 ] . $sTime[ 9 ];
        $sUniqueCertificateCode = $iUserId . "-" . $sUniqueCode;

        global $wpdb;

        $sWhere       = " AND UserId = '" . $iUserId . "' AND QuizId = '" . $iQuizId . "' AND CertificateId = '" . $iCertificateId . "' AND LessonId = '$iLessonId' AND CourseId = '$iCourseId' ";
        $aCertificate = $this->oCode->CheckCode($sWhere);

        if ($aCertificate !== false && $aCertificate) {
            $sUniqueCertificateCode = $aCertificate->UniqueCertificateCode;
        } else {
            $args[ 'UserId' ]                = $iUserId;
            $args[ 'UniqueCode' ]            = $sUniqueCode;
            $args[ 'UniqueCertificateCode' ] = $sUniqueCertificateCode;
            $args[ 'CourseId' ]              = $iCourseId;
            $args[ 'CourseName' ]            = $sCourseName;
            $args[ 'LessonId' ]              = $iLessonId;
            $args[ 'LessonName' ]            = $sLessonName;
            $args[ 'QuizId' ]                = $iQuizId;
            $args[ 'QuizName' ]              = $sQuizName;
            $args[ 'CertificateId' ]         = $iCertificateId;
            $args[ 'CertificateName' ]       = $sCertificateName;

            $result = $this->oCode->InsertCode($args);
        }

        return $sUniqueCertificateCode;
    }

    public function QRCodeGenerator($aParams = array()){

        $iUserId = 0;
        $iCertificateId = 0;

        $sGeneratedCodeURL = $this->CodeGeneratorURL($iUserId, $iCertificateId);

        $sTextToPrint = $sGeneratedCodeURL;

        $sWidth = isset($aParams['width']) ? intval($aParams['width']) : 300;
        $sHeight = isset($aParams['height']) ? intval($aParams['height']) : 300;
        
       // $sImageURL = 'https://chart.googleapis.com/chart?chs=' . $sWidth . 'x' . $sHeight . '&cht=qr&chl='.urlencode($sTextToPrint).'&choe=UTF-8';
		$sImageURL = 'https://api.qrserver.com/v1/create-qr-code/?size=' . $sWidth . 'x' . $sHeight . '&data='.urlencode($sTextToPrint);
        $sImageURL = ' <img src="'. $sImageURL.'" /> ';

        return $sImageURL;
    }

    public function Get_QR_Code_Exists( $iUserId ){
        global $wpdb;
        $table_name = $wpdb->prefix . 'uci_codes';

        // Define the SQL query to retrieve 'UniqueCertificateCode' and 'CourseId'
        $query = $wpdb->prepare(
            "SELECT UniqueCertificateCode, CourseId
            FROM $table_name
            WHERE UserId = %d",
            $iUserId
        );

        // Execute the query
        $result = $wpdb->get_row($query);

        return $result;
    }

    public function Profile_QR_Code( $course_row_class ){

        $iUserId   = get_current_user_id();
        $iMe = $this->Get_QR_Code_Exists( $iUserId );

        if ( !is_null( $iMe ) ) {
            $iCourseId = $iMe->CourseId;
        }

        $iUniqueCertificateCode = 0;

        if ( !is_null( $iMe ) ) {
            $iUniqueCertificateCode = $iMe->UniqueCertificateCode;
        }

        $this->oSetting = new UCIDG_Setting();
        //  Get data from WordPress
        $aData = $this->oSetting->GetSettings();

        if ($aData) {
            $sSearchURL = $aData['SearchURL'];
            if (trim($sSearchURL) != "") {
                if (strpos($sSearchURL, "?") === false) {
                    $sSearchURL .= "?";
                }

                $sSearchURL .= "&code=". $iUniqueCertificateCode;
            }
        }

        //$sImageURL = 'https://chart.googleapis.com/chart?chs=' . 100 . 'x' . 100 . '&cht=qr&chl='.urlencode($sSearchURL).'&choe=UTF-8';
        $sImageURL = 'https://api.qrserver.com/v1/create-qr-code/?size=100x100&data='.urlencode($sSearchURL);
        $sImageURL = ' <img src="'. $sImageURL.'" /> ';


        if( $course_row_class == 'ld-item-list-item ld-item-list-item-course ld-expandable learndash-complete' && $iUniqueCertificateCode != 0 ){
        ?>
        <div class="<?php echo esc_attr( $course_row_class ); ?>" id="<?php echo esc_attr( 'ld-course-list-item-' . $iCourseId ); ?>">
          <div class="ld-item-list-item-preview">
                <span class="ld-course-title" style="align "><?php echo esc_html( 'Scan this QRcode to verify ' . get_the_title( $iCourseId ) . ' Certificate' ); ?></span>
                <span class="ld-item-details" > <?php echo $sImageURL; ?></span>
           </div>
        </div>
        <?php  
        }
        return $course_row_class;
    }
}
