<?php

class UCIDG_Settings
{
    private $sParentSlug  = "ucidg-main";
    private $sPageSlug    = "ucidg-settings";
    private $sPageTitle   = "Settings";
    private $sMenuTitle   = "Settings";
    private $sCapability  = "manage_options";
    private $oAutoLoad = null;
    private $oHelpers = null;
    private $oSetting = null;
    private $sPageLink = null;
    private $sPageURL = null;
    private $sAjaxPageURL = null;

    public function __construct($oAutoLoad, $oHelpers, &$aMenus)
    {
        $aMenus[] = $this->sPageSlug;

        $this->oAutoLoad = $oAutoLoad;
        $this->oHelpers = $oHelpers;

        $this->oSetting = new UCIDG_Setting();

        $this->sAjaxPageURL = $this->oHelpers->sMakeParams(admin_url('admin-ajax.php'));
    }

    public function MainPage()
    {
        $this->sPageURL = $this->oHelpers->sMakeParams($this->oAutoLoad->GetMenuPageURL($this->sPageSlug));

        $aResponse = $this->SaveForm();
        $sSampleImg = UCIDG_PLUGIN_URL . "/assets/images/noimage.png";

        require UCIDG_PLUGIN_DIR . 'resourses/view/settings-form.php';
    }

    public function SaveForm()
    {
        $aResponse = array();

        //  Now Getting Form Values
        $aResponse[ 'Data' ][ 'SearchURL' ] = $this->oHelpers->sParamPost("SearchURL", true);
        $aResponse[ 'Page' ] = "";

        if ($aResponse[ 'Data' ][ 'SearchURL' ] !== false) {
            //  Form Is Submitted
            $aResponse = $this->aCheckSettingSearchURLDetails($aResponse);

            if (!isset($aResponse[ 'Error' ])) {
                //  Data is valid and able to store in WordPress.
                $aRecord = $this->oSetting->SaveSettings_SearchURL($aResponse[ 'Data' ]);
                if ($aRecord[ 'Error' ]) {
                    $aResponse[ 'Error' ][ 'SearchURL' ] = __("Records could not be saved. Please! contact with you administrator.", UCIDG_TextDomain);
                } else {
                    $aResponse[ 'Success' ][ 'SearchURL' ] = __("Settings have been saved successfully!", UCIDG_TextDomain);
                    $aResponse[ 'Page' ] = "ShowGrid";
                }
            }
        } else {
            //  Get data from WordPress
            $aData = $this->oSetting->GetSettings_SearchURL();
            if ($aData) {
                $aResponse[ 'Data' ][ 'SearchURL' ] = $aData['SearchURL'];
            }
        }


        $aResponse[ 'Data' ][ 'NameSearch' ] = $this->oHelpers->sParamPost("NameSearch", true);
        $aResponse[ 'Data' ][ 'EmailSearch' ] = $this->oHelpers->sParamPost("EmailSearch", true);
        if ($aResponse[ 'Data' ][ 'NameSearch' ] !== false && $aResponse[ 'Data' ][ 'EmailSearch' ] !== false) {
            //  Form Is Submitted
            // $aResponse = $this->aCheckSettingDetails($aResponse);

            if (!isset($aResponse[ 'Error' ])) {
                //  Data is valid and able to store in WordPress.
                $aRecord = $this->oSetting->SaveSettings_SearchOptions($aResponse[ 'Data' ]);
                if ($aRecord[ 'Error' ]) {
                    $aResponse[ 'Error' ][ 'SearchOptions' ] = __("Records could not be saved. Please! contact with you administrator.", UCIDG_TextDomain);
                } else {
                    $aResponse[ 'Success' ][ 'SearchOptions' ] = __("Settings have been saved successfully!", UCIDG_TextDomain);
                    $aResponse[ 'Page' ] = "ShowGrid";
                }
            }
        } else {
            //  Get data from WordPress
            $aData = $this->oSetting->GetSettings_SearchOptions();
            if ($aData) {
                $aResponse[ 'Data' ][ 'NameSearch' ] = $aData['NameSearch'];
                $aResponse[ 'Data' ][ 'EmailSearch' ] = $aData['EmailSearch'];
            }
        }


        $aResponse[ 'Data' ][ 'VerficationPageFields' ] = $this->oHelpers->sParamPost("VerficationPageFields");
        if ($aResponse[ 'Data' ][ 'VerficationPageFields' ] !== false) {
            //  Form Is Submitted

            if (!isset($aResponse[ 'Error' ])) {

                //  Data is valid and able to store in WordPress.
                $aRecord = $this->oSetting->SaveSettings_VerficationPageFields($aResponse[ 'Data' ]);
                if ($aRecord[ 'Error' ]) {
                    $aResponse[ 'Error' ][ 'VerficationPageFields' ] = __("Records could not be saved. Please! contact with you administrator.", UCIDG_TextDomain);
                } else {
                    $aResponse[ 'Success' ][ 'VerficationPageFields' ] = __("Settings have been saved successfully!", UCIDG_TextDomain);
                    $aResponse[ 'Page' ] = "ShowGrid";
                }
            }
        }

        //  Get data from WordPress
        $aData = $this->oSetting->GetVerficationPageFields();
        if ($aData) {
            $aResponse[ 'Data' ][ 'VerficationPageFields' ] = $aData['VerficationPageFields'];
        }


        $aResponse[ 'Data' ][ 'FontSize' ] = $this->oHelpers->sParamPost("FontSize", true);
        $aResponse[ 'Data' ][ 'Color' ] = $this->oHelpers->sParamPost("Color", true);
        $aResponse[ 'Page' ] = "";

        if ($aResponse[ 'Data' ][ 'FontSize' ] !== false) {
            //  Form Is Submitted
            $aResponse = $this->aCheckSettingFontColorDetails($aResponse);

            if (!isset($aResponse[ 'Error' ])) {
                //  Data is valid and able to store in WordPress.
                $aRecord = $this->oSetting->SaveSettings_FontColor($aResponse[ 'Data' ]);
                if ($aRecord[ 'Error' ]) {
                    $aResponse[ 'Error' ][ 'FontColor' ] = __("Records could not be saved. Please! contact with you administrator.", UCIDG_TextDomain);
                } else {
                    $aResponse[ 'Success' ][ 'FontColor' ] = __("Settings have been saved successfully!", UCIDG_TextDomain);
                    $aResponse[ 'Page' ] = "ShowGrid";
                }
            }
        } else {
            //  Get data from WordPress
            $aData = $this->oSetting->GetSettings_FontColor();
            if ($aData) {
                $aResponse[ 'Data' ][ 'FontSize' ] = $aData['FontSize'];
                $aResponse[ 'Data' ][ 'Color' ] = $aData['Color'];
            }
        }


        return $aResponse;
    }

    public function aCheckSettingSearchURLDetails($aResponse)
    {
        if (trim($aResponse[ 'Data' ][ 'SearchURL' ]) == "") {
            $aResponse[ 'Error' ][ 'SearchURL' ] = __("You can not leave it blank.", UCIDG_TextDomain);
        }

        return $aResponse;
    }

    public function aCheckSettingFontColorDetails($aResponse)
    {
        if (trim($aResponse[ 'Data' ][ 'FontSize' ]) == "") {
            $aResponse[ 'Error' ][ 'FontSize' ] = __("You can not leave it blank.", UCIDG_TextDomain);
        } else {
            $sFont = $aResponse[ 'Data' ][ 'FontSize' ];
            $iFont = (int) $sFont;
            if ($iFont <= 0) {
                $aResponse[ 'Error' ][ 'FontSize' ] = __("Font size must be greater than zero.", UCIDG_TextDomain);
            }
        }

        if (trim($aResponse[ 'Data' ][ 'Color' ]) == "") {
            $aResponse[ 'Error' ][ 'Color' ] = __("You can not leave it blank.", UCIDG_TextDomain);
        }

        return $aResponse;
    }

    public function SetMenu()
    {
        $this->sPageLink = $this->oAutoLoad->RegisterMenu($this->sParentSlug, $this->sPageSlug, $this->sPageTitle, $this->sMenuTitle, $this->sCapability, array( $this, "MainPage" ));
    }
}
