<?php

if ( !class_exists( "clsImageUploader" ) ) {

    class clsImageUploader {

        private $sKey = "URLImageFiles";

        public function __construct($sKey = false) {
            if ( $sKey !== false )
                $this->sKey = $sKey;
        }

        public function iUploadImageByURL($sURL, $bMetaData = false, $bReplaceUpload = false, $iParenPostID = 0, $bSetThumbnail = true) {
            $iAttachmentID = false;

            $aUploadFile = $this->aUploadFileOnServerViaURL( $sURL );

            if ( !$bReplaceUpload )
                $iAttachmentID = $this->iGetAttachmentIDFromURL( $sURL, $aUploadFile );

            //  Uploading Image
            if ( $iAttachmentID === false )
                $iAttachmentID = $this->iUploadImageFromURL( $sURL, $aUploadFile, $bMetaData, $iParenPostID, $bSetThumbnail );
            else {
                unlink( $aUploadFile[ 'FilePath' ] );
            }

            return $iAttachmentID;
        }

        public function iGetFileSize($sURL) {
            $aHead = array_change_key_case( get_headers( $sURL, 1 ) );
            // content-length of download (in bytes), read from Content-Length: field

            $iSize = isset( $aHead[ 'content-length' ] ) ? $aHead[ 'content-length' ] : 0;

            // cannot retrieve file size, return "-1"
            if ( !$iSize ) {
                return false;
            }

            return $iSize;
            // return size in bytes
        }

        public function sFormatSize($iSize) {
            $sSize = $iSize;
            switch ( $iSize ) {
                case $iSize < 1024:
                    $sSize = $iSize . ' B';
                    break;
                case $iSize < 1048576:
                    $sSize = round( $iSize / 1024, 2 ) . ' KB';
                    break;
                case $iSize < 1073741824:
                    $sSize = round( $iSize / 1048576, 2 ) . ' MB';
                    break;
                case $iSize < 1099511627776:
                    $sSize = round( $iSize / 1073741824, 2 ) . ' GB';
                    break;
            }

            return $sSize;
            // return formatted size
        }

        public function iGetAttachmentIDFromURL($sURL, $aUploadFile = array()) {
            $aUploadedFiles = get_option( $this->sKey );

            $bFileFound    = false;
            $iAttachmentID = false;
            $bUpdateRecord = false;

            $iFileSize = false;

            if ( $aUploadFile )
                $iFileSize = $aUploadFile[ 'FileSize' ];
            if ( !$iFileSize )
                $iFileSize = $this->iGetFileSize( $sURL );

            if ( $aUploadedFiles !== false && count( $aUploadedFiles ) ) {
                if ( isset( $aUploadedFiles[ $sURL ] ) && count( $aUploadedFiles[ $sURL ] ) > 0 ) {
                    foreach ( $aUploadedFiles[ $sURL ] as $iIndex => $aFile ) {

                        //  File Found
                        $bFileFound = true;

                        //  Check if attachment id is not deleted or media is not deleted
                        if ( $bFileFound ) {
                            $oPost = get_post( $aUploadedFiles[ $sURL ][ $iIndex ][ 'AttachID' ] );
                            if ( gettype( $oPost ) != "object" ) {
                                $bFileFound    = false;
                                unset( $aUploadedFiles[ $sURL ][ $iIndex ] );
                                $bUpdateRecord = true;
                            }
                        }

                        //  Check if file exists of this attachment
                        if ( $bFileFound ) {

                            $bFileExists = file_exists( $aUploadedFiles[ $sURL ][ $iIndex ][ 'FilePath' ] );
                            if ( !$bFileExists ) {
                                $bFileFound    = false;
                                unset( $aUploadedFiles[ $sURL ][ $iIndex ] );
                                $bUpdateRecord = true;
                            }
                        }

                        //  Check File Size Is Change
                        if ( $bFileFound ) {
                            if ( $iFileSize != $aUploadedFiles[ $sURL ][ $iIndex ][ 'FileSize' ] ) {
                                $bFileFound = false;
                            }
                        }

                        if ( $bFileFound ) {
                            $iAttachmentID = $aUploadedFiles[ $sURL ][ $iIndex ][ 'AttachID' ];
                            break;
                        }
                    }
                }
            }

            if ( $bUpdateRecord ) {
                update_option( $this->sKey, $aUploadedFiles );
            }

            return $iAttachmentID;
        }

        private function bAddImageDetails($sURL, $aDetails) {
            $aUploadedFiles = get_option( $this->sKey );

            if ( $aUploadedFiles === false )
                $aUploadedFiles = array();

            $aUploadedFiles[ $sURL ][] = $aDetails;

            return update_option( $this->sKey, $aUploadedFiles );
        }

        private function iUploadImageFromURL($sURL, $aUploadFile, $bMetaData = false, $iParenPostID = 0, $bSetThumbnail = true) {

            $iFileSize = $aUploadFile[ 'FileSize' ];
            $sFileName = $aUploadFile[ 'FileName' ];
            $sFilePath = $aUploadFile[ 'FilePath' ];
            //  When file could not be uploaded return false.
            if ( $iFileSize === false ) {
                return false;
            }

            $iAttachmentID = $this->iMakeAttchementOfFile( $sFileName, $sFilePath, $iParenPostID, $bMetaData );
            $iMetaDataID   = 0;
            if ( $bMetaData )
                $iMetaDataID   = $this->iSetAttachmentThumbnailMetaData( $iAttachmentID, $sFilePath );

            if ( $bSetThumbnail && $iParenPostID > 0 ) {
                $result = set_post_thumbnail( $iParenPostID, $iAttachmentID );
            }

            $aUploadFile[ 'AttachID' ] = $iAttachmentID;
            $aUploadFile[ 'MetaID' ]   = $iMetaDataID;

            $this->bAddImageDetails( $sURL, $aUploadFile );

            return $iAttachmentID;
        }

        private function iSetAttachmentThumbnailMetaData($iAttachmentID, $sFilePath) {
            require_once(ABSPATH . 'wp-admin/includes/image.php');
            $attach_data = wp_generate_attachment_metadata( $iAttachmentID, $sFilePath );

            //  Returns meta_id if the meta doesn't exist, otherwise returns true on success and false on failure. 
            $iMetaID = wp_update_attachment_metadata( $iAttachmentID, $attach_data );

            return $iMetaID;
        }

        private function iMakeAttchementOfFile($sFileName, $sFilePath, $iParenPostID = 0) {
            $wp_filetype = wp_check_filetype( $sFileName, null );
            $attachment  = array(
                'post_mime_type' => $wp_filetype[ 'type' ],
                'post_title'     => sanitize_file_name( $sFileName ),
                'post_content'   => '',
                'post_status'    => 'inherit'
            );

            //  $iParenPostID
            //  Attachments may be associated with a parent post or page. Specify the parent's post ID, or 0 if unattached. Default: 0

            $iAttachmentID = wp_insert_attachment( $attachment, $sFilePath, $iParenPostID );

            return $iAttachmentID;
        }

        private function aUploadFileOnServerViaURL($sURL) {
            $aUploadDir = wp_upload_dir();
            $dImageData = file_get_contents( $sURL, false, null , 0 , 40000 );
            $sURL       = urldecode( $sURL );
            $iRandom    = time() . mt_rand( 2000, 9999 );
            $sFileName  = $iRandom . "-" . basename( $sURL );

            if ( wp_mkdir_p( $aUploadDir[ 'path' ] ) ) {
                //  Current set folder in uploads folder like 11 or 12 which are stored month wise
                $sFilePath = $aUploadDir[ 'path' ] . '/' . $sFileName;
                $sFileURL  = $aUploadDir[ 'url' ] . '/' . $sFileName;
            }
            else {
                //  Main upload folder
                $sFilePath = $aUploadDir[ 'basedir' ] . '/' . $sFileName;
                $sFileURL  = $aUploadDir[ 'baseurl' ] . '/' . $sFileName;
            }

            $iFileSize = file_put_contents( $sFilePath, $dImageData );

            return array( "FileSize" => $iFileSize, "FileName" => $sFileName, "FilePath" => $sFilePath, "FileURL" => $sFileURL );
        }

    }

}

