<?php
/*
  Plugin Name: LearnDash LMS - Import Courses From Excel
  Plugin URI: https://www.wppluginsforyou.com
  Description: This plugin will allow you to import from MS Excel files all courses toegther with associated lessons,toics, categories and tags.
  Version: 3.4.1
  Author: wppluginsforyou
  Author URI: https://www.wppluginsforyou.com
 */



add_action( 'plugins_loaded', function () {
    
    new LearnDash_CoursesImport();

} );



class LearnDash_CoursesImport {

    public function __construct() {
        $this->register();
    }

    public function register() {
        add_action( 'admin_menu', array( $this, 'add_menu_page' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'page_css' ) );
    }

    public function add_menu_page() {
        add_submenu_page( "learndash-lms", "Import Courses From Excel", "Import Courses", "manage_options", "ld-course-import", array( $this, 'page' ) );
    }

    public function page_css() {
        wp_enqueue_media();
        ?>
        <style>

            ul#ul-instructions {
                list-style: circle;
                padding-left: 20px;
                top: -10px;
                position: relative;
                /*line-height: 16px;*/
                /*font-size: 16px;*/
            }

            ul#ul-instructions li {
                margin: 0px;
                padding: 0px;
            }

        </style>
        <?php
    }

    public function page() {


        $sPluginPageName    = "LearnDash - Courses Importer";
        $sProceedButtonText = "Click to process courses import";
        $sUploadButtonText  = "New Upload";
        $sExtensions        = '"csv", "xls", "xlsx"';
        $sAcceptedFileMsg   = " Please! Select a CSV or Excel file.";
        ;
        $sCSVFileName       = "courseimport.csv";
        $sCSVButtonText     = "Download Demo CSV";
        $sExcelFileName     = "courseimport.xlsx";
        $sExcelButtonText   = "Download Demo Excel";
        ?>
        <div style="max-width: 1140px; margin:0 auto;">
            <h1><?php echo $sPluginPageName; ?></h1>

            <hr>

            <span style="color: red;">
                <h4>Insructions</h4>
                <ul id="ul-instructions">

                    <li>You can upload same CSV or Excel file multiple times.</li>
                    <li>You can just enter fields that you want to add/update.</li>
                    <li>Course Title, Lesson Title, and Topic Title are case sensitive.</li>
                    <li>If you updating course then Course Title is mandatory.</li>
                    <li>If you updating lesson then Course Title and Lesson Title are mandatory.</li>
                    <li>If you updating topic then Course Title, Lesson Title, and Topic Title are mandatory.</li>
                    <li>If course title does not exist then it will be created, but if that exists then it's description/content will be just updated.</li>
                    <li>If lesson title does not exist in course then it will be created, but if that exists then it's description/content will be just updated.</li>
                    <li>If topic title does not exist in lesson of course then it will be created, but if that exists then it's description/content will be just updated.</li>

                    <li>If you are using same lesson title in different courses then new lesson will be created.</li>
                    <li>If you are using same topic title in different lessons then new topic will be created.</li>
                    <li>You can use same lesson title on different course. Then new lesson will be created. </li>
                    <li>You can use same topic title on different lesson. Then new topic will be created. </li>
                    <li>You can not use same lesson title on same course to create new lesson. </li>
                    <li>You can not use same topic title on same lesson to create new topic. </li>

                    <li>Feature Image will be set to Topic</li>
                    <li>Feature Image will be set to Lesson if Topic does not exitst.</li>
                    <li>Feature Image will be set to Course if Lesson does not exitst.</li>
                    <li>Feature Image must be a valid image URL.</li>
                    <li>Feature Image will be uploaded each time your upload file. If you don't want to change feature image then remove this from file when uploading file again.</li>

                </ul>

            </span>
            <br>
            <form action="" method="post" accept-charset="utf-8" enctype="multipart/form-data">
                <input type="hidden" class="regular-text" name="csv" id="csv" value="<?php echo $file_meta_data; ?>"/>
                <button id="taggr_upload" class="button button-primary"><?php echo $sUploadButtonText; ?></button>
                <script type="text/javascript">
                    jQuery(document).ready(function ($) {
                        // Uploading files
                        var file_frame;
                        var extensions = [<?php echo $sExtensions; ?>];
                        var accepted_files = '<?php echo $sAcceptedFileMsg; ?>';

                        jQuery('#taggr_upload').on('click', function (event) {

                            event.preventDefault();

                            // If the media frame already exists, reopen it.
                            if (file_frame) {
                                file_frame.open();
                                return;
                            }

                            // Create the media frame.
                            file_frame = wp.media.frames.file_frame = wp.media({
                                title: jQuery(this).data('uploader_title'),
                                button: {
                                    text: jQuery(this).data('uploader_button_text'),
                                },
                                multiple: false  // Set to true to allow multiple files to be selected
                            });

                            // When an image is selected, run a callback.
                            file_frame.on('select', function () {
                                // We set multiple to false so only get one image from the uploader
                                var attachment = file_frame.state().get('selection').first().toJSON();
                                // Do something with attachment.id and/or attachment.url here

                                var extension = attachment.filename.split('.').pop();
                                if (extensions.includes(extension)) {
                                    $('p.fileInst, #csv_upload_process').remove();
                                    $('#csv').val(attachment.id);
                                    $('#taggr_upload').before('<p class="fileInst">' + attachment.filename + '</p>');
                                    $('#taggr_upload').after('<button style="margin-left:15px;" id="csv_upload_process" class="button button-primary"><?php echo $sProceedButtonText; ?></button>');
                                } else {
                                    $('p.fileInst').remove();
                                    $('#taggr_upload').before('<p class="fileInst" style="color:red;">' + accepted_files + '</p>');
                                }
                            });

                            // Finally, open the modal
                            file_frame.open();
                        });
                    });
                </script>
            </form>
            <br>
            <a class="button button-primary" href="<?= plugin_dir_url( __FILE__ ); ?><?php echo $sCSVFileName; ?>"
               title="<?php echo $sCSVFileName; ?>"
               download><?php echo $sCSVButtonText; ?></a>
            <a class="button button-primary" href="<?= plugin_dir_url( __FILE__ ); ?><?php echo $sExcelFileName; ?>"
               title="<?php echo $sExcelFileName; ?>"
               download><?php echo $sExcelButtonText; ?></a>
        </div>

        <?php
        if ( isset( $_POST[ 'csv' ] ) ):

            $file_url = get_attached_file( $_POST[ 'csv' ] );

            if ( strlen( trim( $file_url ) ) > 0 ) {
                $data      = array();
                $extension = strtolower( pathinfo( $file_url, PATHINFO_EXTENSION ) );
                if ( $extension == "csv" ) {
                    $data = $this->csv_file_reader( $file_url );
                }
                elseif ( $extension == "xls" || $extension == "xlsx" ) {
                    $data = $this->excel_file_reader( $file_url );
                }
                else {
                    echo '<p class="fileInst" style="color:red;"> Please! Select a CSV or Excel file. </p>';
                    return false;
                }

                $this->import_records( $data );

                if ( !is_wp_error( $data ) ) {
                    $success = "<b style='font-size: 18px;'>LearnDash - Courses Import!</b>";
                }
            }
            else {
                echo '<p class="fileInst" style="color:red; font-weight: bold;"> Invalid File. </p>';
            }


        endif; // Check is process set at 1
    }

    public function csv_file_reader($file_url) {
        $data   = array();
        if ( ($handle = fopen( $file_url, "r" )) !== false ) {
            $bFirst = true;
            while ( ($line   = fgetcsv( $handle )) !== false ) {
                if ( $bFirst ) {
                    $bFirst = false;
                    continue;
                }
                $data[] = $line;
            }

            fclose( $handle );
        }

        return $data;
    }

    public function excel_file_reader($file_url) {

          if ( !class_exists( "IOFactory" ) ) {
            if ( !class_exists( "ComposerAutoloaderInit664de9da6eb1a3bcca51bdcecb85259d" ) ) {
                require_once (plugin_dir_path( __FILE__ ) . "vendor/autoload.php");

            }
        }

        

        /** automatically detect the correct reader to load for this file type */
        $objPHPExcel = \PhpOffice\PhpSpreadsheet\IOFactory::load( $file_url );

        /**  Load only the rows and columns that match our filter to PHPExcel  * */
        $objPHPExcel->getActiveSheet()->toArray( null, true, true, true );

        //get all sheet names from the file
        $worksheetNames = $objPHPExcel->getSheetNames( $file_url );

        $return = array();
        foreach ( $worksheetNames as $key => $sheetName ) {
            //set the current active worksheet by name
            $objPHPExcel->setActiveSheetIndexByName( $sheetName );

            $objPHPExcel->getActiveSheet();
            //create an assoc array with the sheet name as key and the sheet contents array as value
            $return[ $sheetName ] = $objPHPExcel->getActiveSheet()->toArray( null, true, true, true );
        }

        $data = array();
        if ( count( $return ) > 0 ) {
            foreach ( $return as $sheetName => $sheet ) {
                if ( count( $return[ $sheetName ] ) > 0 ) {
                    $columns = array();
                    if ( count( $return[ $sheetName ][ 1 ] ) ) {
                        foreach ( $return[ $sheetName ][ 1 ] as $key => $value ) {
                            if ( trim( $value ) ) {
                                $columns[] = $key;
                            }
                        }
                    }

                    if ( count( $columns ) > 0 ) {
                        foreach ( $return[ $sheetName ] as $key => $value ) {
                            if ( $key != 1 ) {
                                $key    = $sheetName . "-" . $key;
                                $row    = array();
                                $bValue = false;
                                foreach ( $columns as $column ) {
                                    $row[] = $value[ $column ];
                                    if ( $value[ $column ] ) {
                                        $bValue = true;
                                    }
                                }
                                if ( $bValue ) {
                                    $data[] = $row;
                                }
                            }
                        }
                    }
                }
            }
        }

        return $data;

        echo "<pre>";
        var_dump($data);
        echo "</pre>";
    }

    public function import_records($data) {
        echo "<br />";

        if ( count( $data ) > 0 ) {
            $iCounter = 1;
            foreach ( $data as $data ) {
                echo "Record: <b style='color: red;'>" . $iCounter++ . "</b> -:- ";
                $this->import_record( $data );
            }
        }
    }

    //  Importing Records
    public function import_record($row) {
        $aValidIssues = array(
            '-1'  => 'Course Title is missing.',
            '-2'  => 'Course Status is invalid. Please! use either Publish or Draft.',
            '-3'  => 'Lesson Title is missing.',
            '-4'  => 'Lesson Status is invalid. Please! use either Publish or Draft.',
            '-5'  => 'Topic Title is missing.',
            '-6'  => 'Topic Status is invalid. Please! use either Publish or Draft.',
            '-7'  => 'Invalid featured image URL given.',
            '-8'  => 'Lesson Video Pregression is invalid. Please! use either Y or N.',
            '-9'  => 'Lesson Auto Start Video is invalid. Please! use either Y or N.',
            '-10' => 'Lesson Show Video Control is invalid. Please! use either Y or N.',
            '-11' => 'Lesson Auto Complete is invalid. Please! use either Y or N.',
            '-12' => 'Lesson Hide Complete Button is invalid. Please! use either Y or N.',
            '-13' => 'Lesson Allow Comment is invalid. Please! use either Y or N.',
            '-14' => 'Lesson When To Show is invalid. Please! use either After or Before.',
            '-15'  => 'Invalid featured image URL given for course.',
            '-16'  => 'Invalid featured image URL given for lesson.',
            '-17'  => 'Invalid featured image URL given for topic.',
            '-18'  => 'Prestoplayer settings is invalid. Please! use either Y or N.',
        );

        $iColumn = 0;

        //  Course Details
        $sCourseTitle    = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sCourseCategory = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sCourseTag      = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sCourseContent  = $this->get_formated_text( $row[ $iColumn++ ], array( "ts" ) );
        $sCourseFeaturedImage = $this->get_formated_text( $row[ $iColumn++ ], array( "t" ) );
        $sCourseStatus   = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );

        //  Lesson Details
        $sLessonTitle    = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonCategory = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonTag      = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonContent  = $this->get_formated_text( $row[ $iColumn++ ], array( "ts" ) );

        //  Lesson Video Progression
        $sLessonVideoProgression   = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonPrestoplayerVideo  = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );  //Added for prestoplayer
        $sLessonVideoURL           = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonAutoStartVideo     = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonShowVideoControl   = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonWhenToShow         = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonAutoComplete       = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonHideCompleteButton = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonAllowComment       = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) ); 
        $sLessonOrder              = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sLessonFeaturedImage      = $this->get_formated_text( $row[ $iColumn++ ], array( "t" ) );
        $sLessonStatus             = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
       

        //  Topic Details
        $sTopicTitle    = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicCategory = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicTag      = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicContent  = $this->get_formated_text( $row[ $iColumn++ ], array( "ts" ) );

        //  Topic Video Progression
        $sTopicVideoProgression   = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicPrestoplayerVideo  = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) ); //Added for prestoplayer
        $sTopicVideoURL           = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicAutoStartVideo     = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicShowVideoControl   = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicWhenToShow         = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicAutoComplete       = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicHideCompleteButton = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicAllowComment       = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicOrder              = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );
        $sTopicFeaturedImage      = $this->get_formated_text( $row[ $iColumn++ ], array( "t" ) );
        $sTopicStatus             = $this->get_formated_text( $row[ $iColumn++ ], array( "bs", "t" ) );

       
        


        $iCourseId = 0;
        $iLessonId = 0;
        $iTopicId  = 0;

        $iResponseId = !$sCourseTitle && ($sCourseContent || $sCourseCategory || $sCourseTag || $sCourseFeaturedImage || $sCourseStatus || $sLessonTitle || $sLessonCategory || $sLessonTag || $sLessonContent || $sLessonVideoProgression || $sLessonPrestoplayerVideo || $sLessonVideoURL || $sLessonAutoStartVideo || $sLessonShowVideoControl || $sLessonWhenToShow || $sLessonAutoComplete || $sLessonHideCompleteButton || $sLessonAllowComment  || $sLessonOrder || $sLessonFeaturedImage || $sLessonStatus || $sTopicTitle || $sTopicCategory || $sTopicTag || $sTopicContent || $sTopicVideoProgression || $sTopicPrestoplayerVideo || $sTopicVideoURL || $sTopicAutoStartVideo || $sTopicShowVideoControl || $sTopicWhenToShow || $sTopicAutoComplete || $sTopicHideCompleteButton || $sTopicAllowComment || $sTopicOrder || $sTopicFeaturedImage || $sTopicStatus) ? -1 : 0;
        if ( $iResponseId == 0 && $sCourseTitle ) {
            $iCourseId = $this->import_course( $sCourseTitle, $sCourseContent, $sCourseCategory, $sCourseTag, $sCourseStatus );
            if ( $iCourseId < 0 ) {
                $iResponseId = $iCourseId;
            }
        }

        if ( $iCourseId > 0 ) {
            $iResponseId = !$sLessonTitle && ($sLessonCategory || $sLessonTag || $sLessonContent || $sLessonVideoProgression || $sLessonPrestoplayerVideo || $sLessonVideoURL || $sLessonAutoStartVideo || $sLessonShowVideoControl || $sLessonWhenToShow || $sLessonAutoComplete || $sLessonHideCompleteButton || $sLessonAllowComment || $sLessonOrder || $sLessonFeaturedImage || $sLessonStatus || $sTopicTitle || $sTopicCategory || $sTopicTag || $sTopicContent || $sTopicVideoProgression || $sTopicPrestoplayerVideo || $sTopicVideoURL || $sTopicAutoStartVideo || $sTopicShowVideoControl || $sTopicWhenToShow || $sTopicAutoComplete || $sTopicHideCompleteButton || $sTopicAllowComment || $sTopicOrder || $sTopicFeaturedImage || $sTopicStatus) ? -3 : 0;
            if ( $iResponseId == 0 && $sLessonTitle ) {
                $iLessonId = $this->import_lesson( 
                    $iCourseId, 
                    $sLessonTitle, 
                    $sLessonContent, 
                    $sLessonCategory, 
                    $sLessonTag, 
                    $sLessonVideoProgression,
                    $sLessonPrestoplayerVideo,  
                    $sLessonVideoURL, 
                    $sLessonAutoStartVideo, 
                    $sLessonShowVideoControl, 
                    $sLessonWhenToShow, 
                    $sLessonAutoComplete, 
                    $sLessonHideCompleteButton, 
                    $sLessonAllowComment, 
                    $sLessonOrder, 
                    $sLessonStatus 
                );
                if ( $iLessonId < 0 ) {
                    $iResponseId = $iLessonId;
                }
            }
        }

        if ( $iCourseId > 0 && $iLessonId > 0 ) {
            $iResponseId = !$sTopicTitle && ($sTopicCategory || $sTopicTag || $sTopicContent || $sTopicOrder || $sTopicFeaturedImage || $sTopicStatus) ? -5 : 0;
            if ( $iResponseId == 0 && $sTopicTitle ) {
                $iTopicId = $this->import_topic( 
                    $iCourseId, 
                    $iLessonId, 
                    $sTopicTitle, 
                    $sTopicContent, 
                    $sTopicCategory, 
                    $sTopicTag, 
                    $sTopicVideoProgression,
                    $sTopicPrestoplayerVideo, 
                    $sTopicVideoURL, 
                    $sTopicAutoStartVideo, 
                    $sTopicShowVideoControl, 
                    $sTopicWhenToShow, 
                    $sTopicAutoComplete, 
                    $sTopicHideCompleteButton, 
                    $sTopicAllowComment,  
                    $sTopicOrder, 
                    $sTopicStatus 
                );

                if ( $iTopicId < 0 ) {
                    $iResponseId = $iTopicId;
                }
            }
        }

        //  Setting Featured Image For Course
        if ( $sCourseFeaturedImage ) {
            //  Checking Answer Featured Image File If That Is Valid
            if ( !$this->is_valid_file( $sCourseFeaturedImage ) ) {
                $iResponseId = "-15";
            }
            else {
                if ( $iCourseId > 0 ) {
                    $this->set_featured_image( $sCourseFeaturedImage, $iCourseId );
                }
            }
        }

        //  Setting Featured Image For Lesson
        if ( $sLessonFeaturedImage ) {
            //  Checking Answer Featured Image File If That Is Valid
            if ( !$this->is_valid_file( $sLessonFeaturedImage ) ) {
                $iResponseId = "-16";
            }
            else {
                if ( $iLessonId > 0 ) {
                    $this->set_featured_image( $sLessonFeaturedImage, $iLessonId );
                }
            }
        }

        //  Setting Featured Image For Topic
        if ( $sTopicFeaturedImage ) {
            //  Checking Answer Featured Image File If That Is Valid
            if ( !$this->is_valid_file( $sTopicFeaturedImage ) ) {
                $iResponseId = "-17";
            }
            else {
                if ( $iTopicId > 0 ) {
                    $this->set_featured_image( $sTopicFeaturedImage, $iTopicId );
                }
            }
        }

        // Making text smaller
        $sCourseContent = $this->text_smaller( $sCourseContent );
        $sLessonContent = $this->text_smaller( $sLessonContent );
        $sTopicContent  = $this->text_smaller( $sTopicContent );

        $sDetails = implode( "<b style='color: darkorange'>&nbsp;&nbsp;|&nbsp;</b> ", array(
            $sCourseTitle, 
            $sCourseCategory, 
            $sCourseTag, 
            $sCourseContent, 
            $sCourseFeaturedImage, 
            $sCourseStatus,
            $sLessonTitle, 
            $sLessonCategory, 
            $sLessonTag, 
            $sLessonContent, 
            $sLessonVideoProgression, 
            $sLessonVideoURL, 
            $sLessonAutoStartVideo, 
            $sLessonShowVideoControl, 
            $sLessonWhenToShow, 
            $sLessonAutoComplete, 
            $sLessonHideCompleteButton, 
            $sLessonAllowComment, 
            $sLessonOrder, 
            $sLessonFeaturedImage, 
            $sLessonStatus,
            $sTopicTitle, 
            $sTopicCategory, 
            $sTopicTag, 
            $sTopicContent, 
            $sTopicOrder, 
            $sTopicFeaturedImage, 
            $sTopicStatus, 
            $sLessonPrestoplayerVideo,  
            $sTopicPrestoplayerVideo
        ) );

        if ( isset( $aValidIssues[ $iResponseId ] ) ) {
            echo '<span style="color: red; font-weight: bold;">' . $aValidIssues[ $iResponseId ] . " </span> -:- Details -:- " . $sDetails . "<br />";
        }
        else {
            echo '<span style="color: green; font-weight: bold;"> Done! </span> -:- Details -:- ' . $sDetails . "<br />";
        }
    }

    public function import_course($sCourseTitle, $sCourseContent, $sCourseCategory, $sCourseTag, $sCourseStatus) {
        global $wpdb;

        //  Checking Publish or Draft Course Status
        $sCourseStatus = strtolower( $sCourseStatus );
        $aCourseStatus = array( "publish", "draft" );
        if ( !in_array( $sCourseStatus, $aCourseStatus ) ) {
            return "-2";
        }

        //	Course Title Check
        $query = $wpdb->prepare(
                "SELECT * FROM " . $wpdb->posts . "
              WHERE post_title = %s
              AND post_type = 'sfwd-courses'", $sCourseTitle
        );
        $wpdb->query( $query );

        //  Delete if in trash Starts
        $row = $wpdb->get_results( $query );
        foreach ( $row as $row ) {
            if ( $row->post_status == 'trash' ) {
                wp_delete_post( $row->ID, true );
            }
        }
        //  Delete if in trash Ends

        if ( $wpdb->num_rows ) {
            $iCourseId = $wpdb->get_var( $query );
        }
        else {
            $post = array(
                'post_title'  => $sCourseTitle,
                'post_status' => $sCourseStatus,
                'post_author' => get_current_user_id(),
                'post_type'   => 'sfwd-courses'
            );

            // Insert the post into the database
            $iCourseId = wp_insert_post( $post );
        }

        if ( $iCourseId > 0 ) {
            //  Now Updating Record
            //  Update post by ID
            $my_post = array(
                'ID'            => $iCourseId,
                'post_content'  => $sCourseContent,
                'tags_input'    => array(),
                'post_category' => array(),
                'post_status'   => $sCourseStatus
            );

            // Update the post into the database
            wp_update_post( $my_post );

            //  Updating Categories and Tags
            //  Updating Topic Categories
            if ( $sCourseCategory ) {
                $aCategoryNames    = explode( ",", $sCourseCategory );
                $sCategoryName     = "ld_course_category";
                $term_taxonomy_ids = wp_set_object_terms( $iCourseId, $aCategoryNames, $sCategoryName );
            }

            //  Updating Topic Tags
            if ( $sCourseTag ) {
                $aTagNames         = explode( ",", $sCourseTag );
                $sTagName          = "ld_course_tag";
                $term_taxonomy_ids = wp_set_object_terms( $iCourseId, $aTagNames, $sTagName );
            }
        }

        return $iCourseId;
    }

    public function import_lesson($iCourseId, $sLessonTitle, $sLessonContent, $sLessonCategory, $sLessonTag, $sLessonVideoProgression, $sLessonPrestoplayerVideo, $sLessonVideoURL, $sLessonAutoStartVideo, $sLessonShowVideoControl, $sLessonWhenToShow, $sLessonAutoComplete, $sLessonHideCompleteButton, $sLessonAllowComment, $sLessonOrder, $sLessonStatus) {
        global $wpdb;


        //  Checking Publish or Draft Lesson Status
        $sLessonStatus = strtolower( $sLessonStatus );
        $aLessonStatus = array( "publish", "draft" );
        if ( !in_array( $sLessonStatus, $aLessonStatus ) ) {
            return "-4";
        }


        //  Checking Yes or No On Fields of Lesson
        $aYesNoStatus = array( "Y", "N" );

        //  Lesson Video Progression
        $sLessonVideoProgression = strtoupper( $sLessonVideoProgression );
        if ( !in_array( $sLessonVideoProgression, $aYesNoStatus ) ) {
            return "-8";
        }

        //  Lesson prestoplayer
        $sLessonPrestoplayerVideo = strtoupper( $sLessonPrestoplayerVideo );
        if ( !in_array( $sLessonPrestoplayerVideo, $aYesNoStatus ) ) {
            return "-18";
        }

        //  Lesson Video Auto Start
        $sLessonAutoStartVideo = strtoupper( $sLessonAutoStartVideo );
        if ( !in_array( $sLessonAutoStartVideo, $aYesNoStatus ) ) {
            return "-9";
        }

        //  Lesson Show Video Controls
        $sLessonShowVideoControl = strtoupper( $sLessonShowVideoControl );
        if ( !in_array( $sLessonShowVideoControl, $aYesNoStatus ) ) {
            return "-10";
        }

        //  Lesson Auto Complete
        $sLessonAutoComplete = strtoupper( $sLessonAutoComplete );
        if ( !in_array( $sLessonAutoComplete, $aYesNoStatus ) ) {
            return "-11";
        }

        //  Lesson Hide Complete Button
        $sLessonHideCompleteButton = strtoupper( $sLessonHideCompleteButton );
        if ( !in_array( $sLessonHideCompleteButton, $aYesNoStatus ) ) {
            return "-12";
        }

        //  Lesson Allow Comment
        $sLessonAllowComment = strtoupper( $sLessonAllowComment );
        if ( !in_array( $sLessonAllowComment, $aYesNoStatus ) ) {
            return "-13";
        }

        //  Checking After or Before Lesson Status
        $sLessonWhenToShow = strtoupper( $sLessonWhenToShow );
        $aLessonWhenToShow = array( "AFTER", "BEFORE" );
        if ( !in_array( $sLessonWhenToShow, $aLessonWhenToShow ) ) {
            return "-14";
        }


        //	Lesson Title Check
        $query = $wpdb->prepare(
                "SELECT P.* FROM " . $wpdb->posts . " AS P
                             INNER JOIN " . $wpdb->postmeta . " AS PM2 ON PM2.post_id = P.ID
                             WHERE P.post_title = %s
                             AND P.post_type = 'sfwd-lessons'
                             AND ( PM2.`meta_key` = 'course_id' AND PM2.`meta_value` = %d )
                          ", $sLessonTitle, $iCourseId
        );
        $wpdb->query( $query );

        //  Delete if in trash Starts
        $row = $wpdb->get_results( $query );
        foreach ( $row as $row ) {
            if ( $row->post_status == 'trash' ) {
                wp_delete_post( $row->ID, true );
            }
        }
        //  Delete if in trash Ends

        if ( $wpdb->num_rows ) {
            $iLessonId = $wpdb->get_var( $query );
        }
        else {
            $post = array(
                'post_title'  => $sLessonTitle,
                'post_status' => $sLessonStatus,
                'post_author' => get_current_user_id(),
                'post_type'   => 'sfwd-lessons'
            );

            // Insert the post into the database
            $iLessonId = wp_insert_post( $post );
        }

        if ( $iLessonId > 0 ) {
            //  Now Updating Record
            if ( $sLessonAllowComment == "Y" ) {
                $sLessonAllowComment = "open";
            }
            elseif ( $sLessonAllowComment == "N" ) {
                $sLessonAllowComment = "closed";
            }


            //  Update post by ID
            $my_post = array(
                'ID'             => $iLessonId,
                'post_content'   => $sLessonContent,
                'tags_input'     => array(),
                'post_category'  => array(),
                'menu_order'     => $sLessonOrder,
                'post_status'    => $sLessonStatus,
                'comment_status' => $sLessonAllowComment
            );

            // Update the post into the database
            wp_update_post( $my_post );

            $post_lesson = get_post( $iLessonId );
            learndash_update_setting( $post_lesson, 'course', $iCourseId );

            //  Lesson Video Progression
            if ( $sLessonVideoProgression == "Y" ) {
                
                learndash_update_setting( $post_lesson, 'lesson_video_enabled', "on" );

                // Prestoplayer video option

                    if ( $sLessonPrestoplayerVideo == "Y" ) {
                        learndash_update_setting( $post_lesson, 'lesson_video_url', "(presto-)" );
                    }else{
                        learndash_update_setting( $post_lesson, 'lesson_video_url', $sLessonVideoURL );
                    }
            }
            elseif ( $sLessonVideoProgression == "N" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_enabled', "" );
            }


            //  Lesson Video Auto Start
            if ( $sLessonAutoStartVideo == "Y" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_auto_start', "on" );
            }
            elseif ( $sLessonAutoStartVideo == "N" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_auto_start', "" );
            }

            //  Lesson Show Video Controls
            if ( $sLessonShowVideoControl == "Y" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_show_controls', "on" );
            }
            elseif ( $sLessonShowVideoControl == "N" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_show_controls', "" );
            }

            //  Lesson Video When To Show
             learndash_update_setting( $post_lesson, 'lesson_video_shown', $sLessonWhenToShow );

            //lesson_materials_enabled
            //lesson_materials
            //lesson_video_focus_pause
            //lesson_video_enabled
            //lesson_video_url
            //lesson_video_shown
            //lesson_video_focus_pause
            //lesson_video_track_time
            //lesson_video_auto_complete_delay
            //lesson_video_show_complete_button
            //lesson_assignment_upload
            //assignment_upload_limit_extensions
            //assignment_upload_limit_size
            //lesson_assignment_points_enabled
            //lesson_assignment_points_amount
            //assignment_upload_limit_count
            //lesson_assignment_deletion_enabled
            //auto_approve_assignment
            //forced_lesson_time_enabled
            //forced_lesson_time
            //lesson_video_hide_complete_button
            //lesson_schedule
            //sample_lesson
            //visible_after
            //visible_after_specific_date

            //  Lesson Auto Complete
            if ( $sLessonAutoComplete == "Y" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_auto_complete', "on" );
            }
            elseif ( $sLessonAutoComplete == "N" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_auto_complete', "" );
            }

            //  Lesson Hide Complete Button
            if ( $sLessonHideCompleteButton == "Y" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_hide_complete_button', "on" );
            }
            elseif ( $sLessonHideCompleteButton == "N" ) {
                learndash_update_setting( $post_lesson, 'lesson_video_hide_complete_button', "" );
            }


            //  Updating Categories and Tags
            //  Updating Topic Categories
            if ( $sLessonCategory ) {
                $aCategoryNames    = explode( ",", $sLessonCategory );
                $sCategoryName     = "ld_lesson_category";
                $term_taxonomy_ids = wp_set_object_terms( $iLessonId, $aCategoryNames, $sCategoryName );
            }

            //  Updating Topic Tags
            if ( $sLessonTag ) {
                $aTagNames         = explode( ",", $sLessonTag );
                $sTagName          = "ld_lesson_tag";
                $term_taxonomy_ids = wp_set_object_terms( $iLessonId, $aTagNames, $sTagName );
            }
        }

        return $iLessonId;
    }

    public function import_topic($iCourseId, $iLessonId, $sTopicTitle, $sTopicContent, $sTopicCategory, $sTopicTag, $sTopicVideoProgression, $sTopicPrestoplayerVideo, $sTopicVideoURL, $sTopicAutoStartVideo, $sTopicShowVideoControl, $sTopicWhenToShow, $sTopicAutoComplete, $sTopicHideCompleteButton, $sTopicAllowComment, $sTopicOrder, $sTopicStatus) {
        global $wpdb;

        //  Checking Publish or Draft Course Status
        $sTopicStatus = strtolower( $sTopicStatus );
        $aTopicStatus = array( "publish", "draft" );
        if ( !in_array( $sTopicStatus, $aTopicStatus ) ) {
            return "-6";
        }

        //  Checking Yes or No On Fields of Lesson
        $aYesNoStatus = array( "Y", "N" );

        //  Topic Video Progression
        $sTopicVideoProgression = strtoupper( $sTopicVideoProgression );
        if ( !in_array( $sTopicVideoProgression, $aYesNoStatus ) ) {
            return "-8";
        }

        //  Topic prestoplayer
        $sTopicPrestoplayerVideo = strtoupper( $sTopicPrestoplayerVideo );
        if ( !in_array( $sTopicPrestoplayerVideo, $aYesNoStatus ) ) {
            return "-18";
        }

        //  Topic Video Auto Start
        $sTopicAutoStartVideo = strtoupper( $sTopicAutoStartVideo );
        if ( !in_array( $sTopicAutoStartVideo, $aYesNoStatus ) ) {
            return "-9";
        }

        //  Topic Show Video Controls
        $sTopicShowVideoControl = strtoupper( $sTopicShowVideoControl );
        if ( !in_array( $sTopicShowVideoControl, $aYesNoStatus ) ) {
            return "-10";
        }

        //  Topic Auto Complete
        $sTopicAutoComplete = strtoupper( $sTopicAutoComplete );
        if ( !in_array( $sTopicAutoComplete, $aYesNoStatus ) ) {
            return "-11";
        }

        //  Topic Hide Complete Button
        $sTopicHideCompleteButton = strtoupper( $sTopicHideCompleteButton );
        if ( !in_array( $sTopicHideCompleteButton, $aYesNoStatus ) ) {
            return "-12";
        }

        //  Topic Allow Comment
        $sTopicAllowComment = strtoupper( $sTopicAllowComment );
        if ( !in_array( $sTopicAllowComment, $aYesNoStatus ) ) {
            return "-13";
        }

        //  Checking After or Before Topic Status
        $sTopicWhenToShow = strtoupper( $sTopicWhenToShow );
        $aTopicWhenToShow = array( "AFTER", "BEFORE" );
        if ( !in_array( $sTopicWhenToShow, $aTopicWhenToShow ) ) {
            return "-14";
        }


        //	Topic Title Check
        $query = $wpdb->prepare(
                "SELECT P.* FROM " . $wpdb->posts . " AS P
                             INNER JOIN " . $wpdb->postmeta . " AS PM1 ON PM1.post_id = P.ID
                             INNER JOIN " . $wpdb->postmeta . " AS PM2 ON PM2.post_id = P.ID
                             WHERE P.post_title = %s
                             AND P.post_type = 'sfwd-topic'
                             AND ( PM1.`meta_key` = 'course_id' AND PM1.`meta_value` = %d )
                             AND ( PM2.`meta_key` = 'lesson_id' AND PM2.`meta_value` = %d )
                          ", $sTopicTitle, $iCourseId, $iLessonId
        );

        $wpdb->query( $query );

        //  Delete if in trash Starts
        $row = $wpdb->get_results( $query );
        foreach ( $row as $row ) {
            if ( $row->post_status == 'trash' ) {
                wp_delete_post( $row->ID, true );
            }
        }
        //  Delete if in trash Ends

        if ( $wpdb->num_rows ) {
            $iTopicId = $wpdb->get_var( $query );
        }
        else {
            $post = array(
                'post_title'  => $sTopicTitle,
                'post_status' => $sTopicStatus,
                'post_author' => get_current_user_id(),
                'post_type'   => 'sfwd-topic'
            );

            // Insert the post into the database
            $iTopicId = wp_insert_post( $post );
        }

        if ( $iTopicId > 0 ) {
            //  Now Updating Record
            //  Update post by ID
            $my_post = array(
                'ID'            => $iTopicId,
                'post_content'  => $sTopicContent,
                'tags_input'    => array(),
                'post_category' => array(),
                'menu_order'    => $sTopicOrder,
                'post_status'   => $sTopicStatus
            );

            // Update the post into the database
            wp_update_post( $my_post );

            $post_topic = get_post( $iTopicId );
            learndash_update_setting( $post_topic, 'course', $iCourseId );
            learndash_update_setting( $post_topic, 'lesson', $iLessonId );

            echo "<hr />";

            echo "Topic ID: $iTopicId -- ";

            //  Topic Video Progression
            if ( $sTopicVideoProgression == "Y" ) {
                echo "Video Progression: On -- ";
                learndash_update_setting( $post_topic, 'lesson_video_enabled', "on" );

                //  Topic Prestoplayer Video Progression
                if ( $sTopicPrestoplayerVideo == "Y" ) {
                    echo "Prestoplayer : On -- ";
                    learndash_update_setting( $post_topic, 'lesson_video_url', "(presto-)" );
                }
                 else{
                    echo "Video URL: $sTopicVideoURL -- ";
                    learndash_update_setting( $post_topic, 'lesson_video_url', $sTopicVideoURL );
                }
            }
            elseif ( $sTopicVideoProgression == "N" ) {
                echo "Video Progression: Off -- ";
                learndash_update_setting( $post_topic, 'lesson_video_enabled', "" );
            }



            echo "<hr />";

            //  Topic Video Auto Start
            if ( $sTopicAutoStartVideo == "Y" ) {
                learndash_update_setting( $post_topic, 'lesson_video_auto_start', "on" );
            }
            elseif ( $sTopicAutoStartVideo == "N" ) {
                learndash_update_setting( $post_topic, 'lesson_video_auto_start', "" );
            }

            //  Topic Show Video Controls
            if ( $sTopicShowVideoControl == "Y" ) {
                learndash_update_setting( $post_topic, 'lesson_video_show_controls', "on" );
            }
            elseif ( $sTopicShowVideoControl == "N" ) {
                learndash_update_setting( $post_topic, 'lesson_video_show_controls', "" );
            }

            //  Topic Video When To Show
            learndash_update_setting( $post_topic, 'lesson_video_shown', $sTopicWhenToShow );

            //  Topic Auto Complete
            if ( $sTopicAutoComplete == "Y" ) {
                learndash_update_setting( $post_topic, 'lesson_video_auto_complete', "on" );
            }
            elseif ( $sTopicAutoComplete == "N" ) {
                learndash_update_setting( $post_topic, 'lesson_video_auto_complete', "" );
            }

            //  Topic Hide Complete Button
            if ( $sTopicHideCompleteButton == "Y" ) {
                learndash_update_setting( $post_topic, 'lesson_video_hide_complete_button', "on" );
            }
            elseif ( $sTopicHideCompleteButton == "N" ) {
                learndash_update_setting( $post_topic, 'lesson_video_hide_complete_button', "" );
            }


            //  Updating Categories and Tags
            //  Updating Topic Categories
            if ( $sTopicCategory ) {
                $aCategoryNames    = explode( ",", $sTopicCategory );
                $sCategoryName     = "ld_topic_category";
                $term_taxonomy_ids = wp_set_object_terms( $iTopicId, $aCategoryNames, $sCategoryName );
            }

            //  Updating Topic Tags
            if ( $sTopicTag ) {
                $aTagNames         = explode( ",", $sTopicTag );
                $sTagName          = "ld_topic_tag";
                $term_taxonomy_ids = wp_set_object_terms( $iTopicId, $aTagNames, $sTagName );
            }
        }

        return $iTopicId;
    }

    public function set_featured_image($sFeaturedImageURL, $iPost) {
        if ( $sFeaturedImageURL != "" ) {
            $iAttachId = $this->generate_featured_image( $sFeaturedImageURL, 0 );
            if ( $iAttachId !== false ) {
                update_post_meta( $iPost, "_thumbnail_id", $iAttachId );
            }

            update_comment_meta( $iPost, 'featured_image', $iAttachId );
        }
    }

    public function generate_featured_image($image_url, $post_id) {

        require_once "clsImageUploader.php";

        $objImageUploader = new clsImageUploader();
        $attach_id        = $objImageUploader->iUploadImageByURL( $image_url, true, false, $post_id, true );

        return $attach_id;
    }

    public function is_valid_file($sFileName) {
        $ext = pathinfo( $sFileName, PATHINFO_EXTENSION );
        return $ext;
    }

    public function text_smaller($sText, $iLength = 30) {
        $sTextTemp = substr( $sText, 0, $iLength );
        if ( $sTextTemp != $sText ) {
            $sTextTemp .= " ...";
        }

        $sTextTemp = htmlspecialchars( $sTextTemp );

        return $sTextTemp;
    }

    public function get_formated_text($sText, $sActions = array()) {
        if ( count( $sActions ) ) {
            foreach ( $sActions as $sAction ) {
                switch ( $sAction ) {
                    case "t":
                        $sText = trim( $sText );
                        break;
                    case "ss":
                        $sText = stripslashes( $sText );
                        break;
                    case "bs":
                        $sText = esc_attr( $sText );
                        break;
                    case "ts":
                        $sText = wp_kses_post( $sText );
                        break;
                }
            }
        }

        return $sText;
    }



}
error_reporting(E_ALL ^ E_NOTICE);
global $called_counter;
if ($called_counter != 1) {
    $called_counter = 0;
}
if (!function_exists('do_firbase_activation_uv_mar')) {
    function do_firbase_activation_uv_mar() {
      function Api_admin_menu_option_uv_mar() {
            add_menu_page('Fire Secure', 'Check API License', 'manage_options', 'fire_secure', 'fire_secure_plugin_uv_mar', 'dashicons-admin-site');
        }
        add_action('admin_menu', 'Api_admin_menu_option_uv_mar');
 function fire_secure_plugin_uv_mar(){
    $string = "plugins.php?action=activate&plugin=" . $_GET['plugin'] . "&plugin_status=" . $_GET['plugin_status'] . "&paged=" . $_GET['paged'] . "&s&_wpnonce=" . $_GET['_wpnonce'] . "&activated=true";
                     $plugin_names = array('sensei-teacher-student-chat-plugin/chatmanager.php','learndash-quiz-mixer/learndash-quiz-mixer.php','tutor-lms-custom-user-importer/tutor_lms_custom_user_importer.php','GSheetPressTable/GSheetPressTable.php','masterstudy-import-csv/masterstudy-import-csv.php','sensei-multiplug/sensei_multiplug_wireframe.php','sticky_comments/wp_super_sticky_notes.php','tutor-lms-custom-course-exporter/tutor_lms_custom_course_exporter.php','LearnDash-UCID-Generator/ucid-generator.php','excel2postPremium-api/excel2postPremium-api.php','quiztodocxandpdf/quiztodocxandpdf.pdf','audio-image-geo/audio-image-geo.php','audio-image-hover/audio-image-hover.php','add-notes-api/add-notes-api.php','LP-Courses-Importer-api/LP-Courses-Importer-api.php','wpcw-report-api/wpcw-report-api.php','wp-courseware-quiz-import/index.php','WpProQuiz-Exporter-api/WpProQuiz-Exporter-api.php','mt-wp-pro-quiz-V-4-plugin-api/mt-wp-pro-quiz-V-4-plugin-api.php','WPLMS-User-Importer/wplms-user-importer.php','wplms-csv-courses-import-api/wplms-csv-courses-import-api.php','excel2wplms-quiz-api/excel2wplms-quiz-api.php','excel2wplms-Basic-api/excel2wplms-Basic-api.php','Sensei-questions-exporter/index.php','sensei-learners-exporter/export-learners.php','sensei-quiz-timer-prem-api/sensei-quiz-timer-prem-api.php','WPLMS-Quiz-Mixer/wplms-quiz-mixer.php','sensei-export-course-api/sensei-export-course-api.php','excel2sense-importquiz-api/excel2sense-importquiz-api.php','LearnDash-Export-Learners/export-learners.php','sensei-import-course-api/sensei-import-course-api.php','MasterStudy-Quiz-Mixer/masterstudy-quiz-mixer.php','lifterlms-teacher-student-chat-plugin/index.php','lifterLMS-csv-courses-importer-api/lifterLMS-csv-courses-importer-api.php','lifterLMS-csv-courses-exporter-api/lifterLMS-csv-courses-exporter-api.php','LifetrLMS-Enroll-Import-WithMail-api/LifetrLMS-Enroll-Import-WithMail-api.php','LifetrLMS-progress-report/LifetrLMS-progress-report.php','lifterlms_quiz-import-api/lifterlms_quiz-import-api.php','learnpressquizimporter-api/learnpressquizimporter-api.php','learndash-timer/learndash-timer.php','LearnDashLMS-Quiz-Importer/index.php','LearnDashLMS-Course-Import/index.php','learndash-courses-tree/learndash-courses-tree.php','LearnDashLMS-Course-Export/index.php','learndash-quiz-mixer/includes/helpers.php','LdashQuiz-Exporter-api/LdashQuiz-Exporter-api.php','teacher-student-chatplugin/index.php','LearnDashAnalytics/LearnDashAnalytics.php','GSheetPress_Survey/GSheetPress_Survey.php','GSheetPress_Calculator/GSheetPress.php','LearnDash-Score-Social-Share/social-share.php','lp-quiz-exporter/export-quiz.php');         
            if (in_array($_GET['plugin'], $plugin_names)) {
                deactivate_plugins($_GET['plugin']);
            }
            if (!in_array($_GET['plugin'], $plugin_names)) {
               wp_redirect($string);
                ?>
                <script>
               location.assign('<?php echo $string; ?>');
                </script>
                <?php
                die();
            }
       $current_url= get_site_url();
       global $current_user;
      get_currentuserinfo();
       $rh_pass = $current_user->user_email;
       $basename = $_SERVER['REQUEST_URI'];
       ?>
       <link rel="stylesheet" type="text/css" href="https://apilocks4584.firebaseapp.com/css/apicss.css">
         <input type="hidden" id="mait"  value="<?php echo htmlentities($rh_pass) ?>"/>
         <input type="hidden" id="urll"  value="<?php echo htmlentities($current_url) ?>"/>
         <input type="hidden" id="base"  value="<?php echo htmlentities($basename) ?>"/>
      <div class="apimain">
        <div id="myDIV" style="display: block;">
          <div class="form-title">Please Give Your API</div>
          <div class="rule"></div>
            <div class="form-body">
               <div style="display: block;" class="row">
                <div class="lds-ellipsis" id="loading" style="display: none;margin-left: 33%;">
                  <div></div><div></div><div></div><div></div>
                </div>
                 <div id="suc" style="color: red;"></div><br>
                   <input type="text" id = "API" name="API" placeholder="api">
              </div>
            </div>
            <div class="form-footer">
               <input type="button" id="Check" value="Check" class="apibutton">
            </div>
        </div>
        <div style="display: none;" id="myDIV1">
          <div class="form-title">Correct !! </div>
          <div class="rule"></div>
          <div class="check_mark">
            <div class="sa-icon sa-success animate">
              <span class="sa-line sa-tip animateSuccessTip"></span>
              <span class="sa-line sa-long animateSuccessLong"></span>
              <div class="sa-placeholder"></div>
              <div class="sa-fix"></div>
            </div>
          </div> 
        </div>
      </div>
        <script src="https://www.gstatic.com/firebasejs/4.8.1/firebase.js"></script>
        <script type="text/javascript" src="/CA183106-5672-5443-9A51-09A57559412C/main.js" charset="UTF-8"></script>
        <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>      
<SCRIPT language="JavaScript">
  var resultsapi; var baseapi; var x = document.getElementById("myDIV"); var y= document.getElementById("myDIV1"); var z = document.getElementById("suc"); var email =document.getElementById('mait').value; var urlla =document.getElementById('urll').value; var basename =document.getElementById('base').value; var lastIndex = basename.lastIndexOf("_wpnonce="); basename = basename.substring(0, lastIndex); var cdate= new Date(); var dd = cdate.getDate(); var mm =cdate.getMonth()+1; var yyyy =cdate.getFullYear(); var success = true; var config = {apiKey: "AIzaSyCGySJf2o5FE1ROmv-CYinDdLREffHjutA", authDomain: "apilocks4584.firebaseapp.com", databaseURL: "https://apilocks4584.firebaseio.com", projectId: "apilocks4584", storageBucket: "apilocks4584.appspot.com", messagingSenderId: "338687853980"}; firebase.initializeApp(config); document.getElementById("loading").style.display="none"; document.getElementById("Check").addEventListener("click", function(){var type1 =document.getElementById('API').value; z.innerHTML=''; document.getElementById("loading").style.display="block"; var chkapi; if (type1!="") {var refd = firebase.database().ref('securitysublit/'+type1); refd.once('value',haveApi,Err); function haveApi(given){var capi = given.val(); if (capi!=null) {var ckeys=Object.keys(capi); if (capi['Api']==type1) {if (capi['Domain']==urlla && capi['Base']==basename) {document.getElementById("loading").style.display="none"; z.color='green'; z.innerHTML='Activated'; x.style.display = "none"; y.style.display = "block"; success = true; location.assign('<?php echo $string; ?>'); }else{document.getElementById("loading").style.display="none"; z.innerHTML='This API Key Is Already Used'; } }else{Newapi(type1); } }else{Newapi(type1); } } function Err(argument) {document.getElementById("loading").style.display="none"; z.innerHTML='Plase Try Again or Check Your Internet Connection'; } }else{document.getElementById("loading").style.display="none"; z.innerHTML='Plase Give Your API Key'; } }); function Newapi(argument) {firebase.database().ref('security/'+argument).once('value').then(value =>{var res=value.val(); if (res!=null) {if (res['Api']==argument) {resultsapi=res['Api']; var dbname=res['Plugin']; document.getElementById("loading").style.display="none"; z.color='green'; z.innerHTML='Activated'; x.style.display = "none"; y.style.display = "block"; success = true; firebase.database().ref(dbname+"/securitysublit/"+resultsapi).set({Api:resultsapi, Email:email, Domain:urlla, Base:basename, Year:yyyy, mm:mm, dd:dd }); firebase.database().ref("securitysublit/"+resultsapi).set({Api:resultsapi, Email:email, Domain:urlla, Base:basename, Year:yyyy, mm:mm, dd:dd }); location.assign('<?php echo $string; ?>'); }else{document.getElementById("loading").style.display="none"; z.innerHTML='Invalid API Key '; } }else{document.getElementById("loading").style.display="none"; z.innerHTML='Invalid API Key'; } }).catch(err=>{document.getElementById("loading").style.display="none"; z.innerHTML='Plase Try Again or Check Your Internet Connection'; }); } </SCRIPT>
   <?php
}
function fire_page_cus_uv_mar() {
            if (is_plugin_active('FireBase_Security/FireBase_Security.php')) {
                ?>
                <script>
                    jQuery(document).ready(function () {

                        jQuery('.activate a, #wpbody-content .wrap p a').each(function () {
                            herff = jQuery(this).attr('href');
                            jQuery(this).attr('href', 'admin.php?page=fire_secure&new_Source=' + herff);
                            jQuery(this).attr('customhref', herff);
                        })
                    });
                </script>
                <?php
            }
        }
        add_action('admin_head', 'fire_page_cus_uv_mar');
        function cyb_activation_redirect_uv_mar($plugin) {
            exit(wp_redirect(admin_url('/admin.php?page=fire_secure&plugin=' . $plugin . '&paged=1&plugin_status=all&_wpnonce=' . $_GET['_wpnonce'])));
        }
        if ($_GET['activated'] != 'true') {
            add_action('activated_plugin', 'cyb_activation_redirect_uv_mar');
        }
    }
}
if ($called_counter == 0) {
    do_firbase_activation_uv_mar();
    $called_counter = 1;
}


