<?php
/**
 * Plugin Name: Instructor Role
 * Plugin URI: https://learndash.com/instructor-role-by-learndash
 * Description: This extension adds a user role 'Instructor' into your WordPress website and provides capabilities to create courses content and track student progress in your LearnDash LMS.
 * Version: 5.9.4
 * Requires at least: 6.0
 * Tested up to: 6.7
 * Requires PHP: 7.4
 * Author: LearnDash
 * Author URI: https://learndash.com
 * Text Domain: wdm_instructor_role
 * Domain Path: /languages
 *
 * @package LearnDash\Instructor_Role
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

require_once plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';
require_once plugin_dir_path( __FILE__ ) . 'vendor-prefixed/autoload.php';

use LearnDash\Instructor_Role\Activation;
use LearnDash\Instructor_Role\Deactivation;
use LearnDash\Instructor_Role\Dependency_Checker;
use LearnDash\Instructor_Role\Plugin;
use LearnDash\Instructor_Role\Licensing\Migration as License_Migration;

/**
 * Set Plugin Version
 *
 * @since 3.5.0
 */
if ( ! defined( 'INSTRUCTOR_ROLE_PLUGIN_VERSION' ) ) {
	define( 'INSTRUCTOR_ROLE_PLUGIN_VERSION', '5.9.4' );
}

/**
 * Plugin dir path Constant
 *
 * @since 3.1.0
 */
if ( ! defined( 'INSTRUCTOR_ROLE_ABSPATH' ) ) {
	define( 'INSTRUCTOR_ROLE_ABSPATH', plugin_dir_path( __FILE__ ) );
}

/**
 * Plugin BaseName Constant
 *
 * @since 3.1.1
 */
if ( ! defined( 'INSTRUCTOR_ROLE_BASE' ) ) {
	define( 'INSTRUCTOR_ROLE_BASE', plugin_basename( __FILE__ ) );
}

/**
 * Set the plugin slug as default text domain.
 *
 * @since 3.5.0
 */
if ( ! defined( 'INSTRUCTOR_ROLE_TXT_DOMAIN' ) ) {
	define( 'INSTRUCTOR_ROLE_TXT_DOMAIN', 'wdm_instructor_role' );
}

/**
 * Define the main plugin file.
 *
 * @since 5.9.1
 */
if ( ! defined( 'INSTRUCTOR_ROLE_PLUGIN_FILE' ) ) {
	define( 'INSTRUCTOR_ROLE_PLUGIN_FILE', __FILE__ );
}

/**
 * Define LearnDash Licensing URL.
 *
 * @since 5.9.1
 */
if ( ! defined( 'INSTRUCTOR_ROLE_LICENSING_SITE_URL' ) ) {
	define( 'INSTRUCTOR_ROLE_LICENSING_SITE_URL', 'https://checkout.learndash.com/wp-json/learndash/v2/site/auth_token' );
}

/**
 * Define LearnDash Check Licensing URL.
 *
 * @since 5.9.1
 */
if ( ! defined( 'INSTRUCTOR_ROLE_LICENSING_CHECK_LICENSE_URL' ) ) {
	define( 'INSTRUCTOR_ROLE_LICENSING_CHECK_LICENSE_URL', 'https://checkout.learndash.com/wp-json/learndash/v2/site/auth' );
}

register_activation_hook( INSTRUCTOR_ROLE_PLUGIN_FILE, [ Activation::class, 'run' ] );
register_deactivation_hook( INSTRUCTOR_ROLE_PLUGIN_FILE, [ Deactivation::class, 'run' ] );

add_action( 'admin_notices', [ License_Migration::class, 'run' ] );

require INSTRUCTOR_ROLE_ABSPATH . 'includes/class-instructor-role.php';

/**
 * Begins execution of the plugin.
 *
 * @since 3.5.0
 *
 * @return void
 */
function run_instructor_role() { // phpcs:ignore WordPress.NamingConventions.PrefixAllGlobals.NonPrefixedFunctionFound -- Legacy function.
	$plugin = new \InstructorRole\Includes\Instructor_Role();
	$plugin->run();
}

$learndash_instructor_role_dependency_checker = new Dependency_Checker();

$learndash_instructor_role_dependency_checker->set_dependencies(
	[
		'sfwd-lms/sfwd_lms.php' => [
			'label'            => '<a href="https://www.learndash.com" target="_blank">' . __( 'LearnDash LMS', 'wdm_instructor_role' ) . '</a>',
			'class'            => 'SFWD_LMS',
			'version_constant' => 'LEARNDASH_VERSION',
			'min_version'      => '4.7.0',
		],
	]
);

$learndash_instructor_role_dependency_checker->set_message(
	esc_html__( 'Instructor Role requires the following plugin(s) to be active:', 'wdm_instructor_role' )
);

add_action(
	'plugins_loaded', // It would be great to refactor the plugin, so we can use the 'learndash_init' hook.
	function () use ( $learndash_instructor_role_dependency_checker ) {
		// If plugin requirements aren't met, don't run anything else to prevent possible fatal errors.
		if (
			! $learndash_instructor_role_dependency_checker->check_dependency_results()
			|| php_sapi_name() === 'cli'
		) {
			return;
		}

		run_instructor_role(); // The legacy code should be run first.
		learndash_register_provider( Plugin::class );
	},
	50
);
