<?php
/**
 * Provide a admin area view for the plugin.
 *
 * This file is used to markup the admin-facing aspects of the plugin.
 *
 * @link       https://wbcomdesigns.com/plugins
 * @since      1.0.0
 *
 * @package    Ld_Dashboard
 * @subpackage Ld_Dashboard/admin/partials
 */

$function_obj               = Ld_Dashboard_Functions::instance();
$ld_dashboard_settings_data = $function_obj->ld_dashboard_settings_data();
$settings                   = $ld_dashboard_settings_data['general_settings'];
$monetization_settings      = $ld_dashboard_settings_data['monetization_settings'];
?>
<!-- This file should primarily consist of HTML with a little bit of PHP. -->
<div class="wbcom-tab-content-wrapper">
	<div class="wbcom-welcome-main-wrapper">
		<div class="wbcom-admin-option-wrap wbcom-admin-option-wrap-inner">
			<form method="post" action="options.php" enctype="multipart/form-data">
				<?php
				settings_fields( 'ld_dashboard_general_settings' );
				do_settings_sections( 'ld_dashboard_general_settings' );
				?>

				<!-- General Settings -->
				<div class="wbcom-admin-title-section-wrap title-tutorial-section-wrap">
					<div class="wbcom-admin-title-section-inner">
						<h3><?php esc_html_e( 'General Settings', 'ld-dashboard' ); ?></h3>
						<p class="description"><?php esc_html_e( 'You can configure your general dashboard settings and Instructors options from this settings menu.', 'ld-dashboard' ); ?></p>
					</div>
					<a href="https://docs.wbcomdesigns.com/docs/learndash-dashboard/ld-settings-learndash-dashboard/ld-global-settings/" class="ld-tutorial-btn" target="_blank"><?php esc_html_e( 'View Tutorial', 'ld-dashboard' ); ?></a>
				</div>
				<div class="wbcom-tab-content">
						<div class="form-table ld-dashboard-setting-accordian-wrapper general-settings-section">
							<div class="ld-grid-view-wrapper">
							<div class="ld-dashboard-general-setting-accordian-content ld-dashboard-row-section">
							<div class="wbcom-settings-section-wrap allow-instructors-publish-courses">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php printf(/* translators: %s: Course label. */ esc_html__( 'Instructors %s Status', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ); ?></label>
									<p class="description"><?php printf(/* translators: %s: Course label. */ esc_html__( 'The %s status when an instructor creates or updates it.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options ld-radio-fields-course-publish">
									<label for="ld-dashboard-publish-course" class="ld-dashboard-course-publish-toggle <?php echo isset( $settings['enable-instructor-course-publish'] ) && '1' == $settings['enable-instructor-course-publish'] ? 'checked-course-publish' : ''; ?>">
										<span class="dashicons dashicons-yes"></span>
										<input type="radio" id="ld-dashboard-publish-course" name="ld_dashboard_general_settings[enable-instructor-course-publish]" value="1" <?php checked( $settings['enable-instructor-course-publish'], '1' ); ?> /><?php esc_html_e( 'Enable Auto-Approval', 'ld-dashboard' ); ?>
									</label>
									<label for="ld-dashboard-unpublish-course" class="ld-dashboard-course-publish-toggle <?php echo isset( $settings['enable-instructor-course-publish'] ) && '0' == $settings['enable-instructor-course-publish'] ? 'checked-course-publish' : ''; ?>">
										<span class="dashicons dashicons-yes"></span>
										<input type="radio" id="ld-dashboard-unpublish-course" name="ld_dashboard_general_settings[enable-instructor-course-publish]" value="0" <?php checked( $settings['enable-instructor-course-publish'], '0' ); ?> /><?php esc_html_e( 'Disable Auto-Approval', 'ld-dashboard' ); ?>
									</label>
								</div>
							</div>
							<div class="wbcom-settings-section-wrap allow-instructors-caouse-tags">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php printf(/* translators: %s: Course label. */ esc_html__( 'Instructors Can Create %s Tags', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></label>
									<p class="description"><?php printf(/* translators: %s: Course label. */ esc_html__( 'Allow instructors to create new %s tags from Instructors dashboard.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox" name="ld_dashboard_general_settings[enable-instructor-course-tags]" value="1" <?php checked( $settings['enable-instructor-course-tags'], '1' ); ?> />
								</div>
							</div>
							<?php if ( ld_is_envt_ready_for_to_do() ) { ?>
								<div class="wbcom-settings-section-wrap display-course-to-do">
									<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
										<label><?php printf(/* translators: %s: Course label. */ esc_html__( 'Display %s To Do', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></label>
										<p class="description"><?php esc_html_e( 'Enable this option if you want to display To Do widget at dashboard page.', 'ld-dashboard' ); ?></p>
									</div>
									<div class="ld-grid-content wbcom-settings-section-options">
										<input type="checkbox"  name="ld_dashboard_general_settings[display-to-do]" value="1" <?php checked( $settings['display-to-do'], '1' ); ?> />
									</div>
								</div>
							<?php } ?>
							<div class="wbcom-settings-section-wrap become-instructor-button">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Apply Instructor Button', 'ld-dashboard' ); ?></label>
									<p class="description"><?php esc_html_e( 'Enable this option to display the button on the student dashboard.', 'ld-dashboard' ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox" name="ld_dashboard_general_settings[become-instructor-button]" value="1" <?php checked( $settings['become-instructor-button'], '1' ); ?> />
								</div>
							</div>

							<div class="wbcom-settings-section-wrap instructor-role-status">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Instructor Role Status', 'ld-dashboard' ); ?></label>
									<p class="description"><?php esc_html_e( 'Enable this option to set the instructor role is pending.', 'ld-dashboard' ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox" name="ld_dashboard_general_settings[instructor-role-status]" value="1" <?php checked( $settings['instructor-role-status'], '1' ); ?> />
								</div>
							</div>
							
							<?php if ( class_exists( 'BuddyPress' ) ) : ?>
							<div class="wbcom-settings-section-wrap user-avatar">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Set BuddyPress Avatar', 'ld-dashboard' ); ?></label>
									<p class="description"><?php esc_html_e( 'Enabling this option will set Buddypress Profile avatar as Learndash Dashboard avatar', 'ld-dashboard' ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox"  name="ld_dashboard_general_settings[default-avatar]" value="1" <?php ( isset( $settings['default-avatar'] ) ) ? checked( $settings['default-avatar'], '1' ) : ''; ?> data-id="default-avatar"/>
								</div>
							</div>
							<div class="wbcom-settings-section-wrap profile-override">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Profile Override', 'ld-dashboard' ); ?></label>
									<p class="description"><?php esc_html_e( 'Redirect LearnDash Dashboard user profile with the BuddyPress/BuddyBoss member profile.', 'ld-dashboard' ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox"  name="ld_dashboard_general_settings[redirect-profile]" value="1" <?php ( isset( $settings['redirect-profile'] ) ) ? checked( $settings['redirect-profile'], '1' ) : ''; ?> data-id="redirect-profile"/>
								</div>
							</div>
							<?php endif; ?>
						
						<div class="wbcom-settings-section-wrap profile-override">
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php esc_html_e( 'Gravtar', 'ld-dashboard' ); ?></label>
								<p class="description"><?php esc_html_e( 'Set Gravtar as default', 'ld-dashboard' ); ?></p>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<input type="checkbox"  name="ld_dashboard_general_settings[default-gravtar]" value="1" <?php ( isset( $settings['default-gravtar'] ) ) ? checked( $settings['default-gravtar'], '1' ) : ''; ?> data-id="redirect-profile"/>
							</div>
						</div>
			</div>
		</div>
	</div>
</div>

			<!-- Dashboard and Instructor Page Settings -->
				<div class="form-table">
					<div class="ld-grid-view-wrapper ld-page-mapping-tab-content">
						<div class="ld-dashboard-heading-wrapper title-tutorial-section-wrap">
							<h3><?php esc_html_e( 'Dashboard & Instructor Page Settings', 'ld-dashboard' ); ?></h3>
						</div>

						<div class="wbcom-tab-content ld-dashboard-row-section">
						<div class="wbcom-settings-section-wrap my-dashboard-page">
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php esc_html_e( 'Dashboard', 'ld-dashboard' ); ?></label>
								<p class="description"><?php esc_html_e( 'This sets the page used to display the front end Instructor dashboard. This page should contain the following shortcode. [ld_dashboard]', 'ld-dashboard' ); ?></p>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<?php
								$args = array(
									'name'             => 'ld_dashboard_general_settings[my_dashboard_page]',
									'id'               => 'my_dashboard_page',
									'sort_column'      => 'menu_order',
									'sort_order'       => 'ASC',
									'show_option_none' => ' ',
									'class'            => 'my_dashboard_page',
									'echo'             => false,
									'selected'         => absint( ( isset( $settings['my_dashboard_page'] ) ) ? $settings['my_dashboard_page'] : 0 ),
									'post_status'      => 'publish',
								);

								if ( isset( $value['args'] ) ) {
									$args = wp_parse_args( $value['args'], $args );
								}

								echo wp_dropdown_pages( $args ); // WPCS: XSS ok.
								?>

								<?php if ( isset( $settings['my_dashboard_page'] ) && $settings['my_dashboard_page'] != 0 ) : ?>
									<a href="<?php echo esc_url( get_permalink( $settings['my_dashboard_page'] ) ); ?>" class="button-secondary" target="_bp">
										<?php esc_html_e( 'View', 'ld-dashboard' ); ?>
										<span class="dashicons dashicons-external" aria-hidden="true"></span>
										<span class="screen-reader-text"><?php esc_html_e( '(opens in a new tab)', 'ld-dashboard' ); ?></span>
									</a>
								<?php endif; ?>
							</div>
						</div>
						<div class="wbcom-settings-section-wrap instructor-registration-page">
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php esc_html_e( 'Instructor Registration', 'ld-dashboard' ); ?></label>
								<p class="description"><?php esc_html_e( 'This sets the page used to registration a instructor. This page should contain the following shortcode [ld_instructor_registration]', 'ld-dashboard' ); ?></p>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<?php
								$args = array(
									'name'             => 'ld_dashboard_general_settings[instructor_registration_page]',
									'id'               => 'instructor_registration_page',
									'sort_column'      => 'menu_order',
									'sort_order'       => 'ASC',
									'show_option_none' => ' ',
									'class'            => 'instructor_registration_page',
									'echo'             => false,
									'selected'         => absint( ( isset( $settings['instructor_registration_page'] ) ) ? $settings['instructor_registration_page'] : 0 ),
									'post_status'      => 'publish',
								);

								if ( isset( $value['args'] ) ) {
									$args = wp_parse_args( $value['args'], $args );
								}

								echo wp_dropdown_pages( $args ); // WPCS: XSS ok.
								?>
								<?php if ( isset( $settings['instructor_registration_page'] ) && $settings['instructor_registration_page'] != 0 ) : ?>
									<a href="<?php echo esc_url( get_permalink( $settings['instructor_registration_page'] ) ); ?>" class="button-secondary" target="_bp">
										<?php esc_html_e( 'View', 'ld-dashboard' ); ?>
										<span class="dashicons dashicons-external" aria-hidden="true"></span>
										<span class="screen-reader-text"><?php esc_html_e( '(opens in a new tab)', 'ld-dashboard' ); ?></span>
									</a>
								<?php endif; ?>
							</div>
						</div>
						<div class="wbcom-settings-section-wrap instructor-listing-page">
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php esc_html_e( 'Instructors', 'ld-dashboard' ); ?></label>
								<p class="description"><?php esc_html_e( 'This sets the page used to display a Instructor list of all Instructor. This page should contain the following shortcode [ld_dashboard_instructors_list]', 'ld-dashboard' ); ?></p>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<?php
								$args = array(
									'name'             => 'ld_dashboard_general_settings[instructor_listing_page]',
									'id'               => 'instructor_listing_page',
									'sort_column'      => 'menu_order',
									'sort_order'       => 'ASC',
									'show_option_none' => ' ',
									'class'            => 'instructor_listing_page',
									'echo'             => false,
									'selected'         => absint( ( isset( $settings['instructor_listing_page'] ) ) ? $settings['instructor_listing_page'] : 0 ),
									'post_status'      => 'publish',
								);

								if ( isset( $value['args'] ) ) {
									$args = wp_parse_args( $value['args'], $args );
								}

								echo wp_dropdown_pages( $args ); // WPCS: XSS ok.
								?>
								<?php if ( isset( $settings['instructor_listing_page'] ) && $settings['instructor_listing_page'] != 0 ) : ?>
									<a href="<?php echo esc_url( get_permalink( $settings['instructor_listing_page'] ) ); ?>" class="button-secondary" target="_bp">
										<?php esc_html_e( 'View', 'ld-dashboard' ); ?>
										<span class="dashicons dashicons-external" aria-hidden="true"></span>
										<span class="screen-reader-text"><?php esc_html_e( '(opens in a new tab)', 'ld-dashboard' ); ?></span>
									</a>
								<?php endif; ?>
							</div>
						</div>
					</div>
					</div>
				</div>


				<!-- Dashboard Components -->
				<div class="wbcom-admin-title-section-wrap title-tutorial-section-wrap">
					<div class="wbcom-admin-title-section-inner">
						<h4><?php esc_html_e( 'Dashboard Components Menu', 'ld-dashboard' ); ?></h4>
					</div>
					<a href="https://docs.wbcomdesigns.com/docs/learndash-dashboard/ld-settings-learndash-dashboard/ld-global-settings/" class="ld-tutorial-btn" target="_blank"><?php esc_html_e( 'View Tutorial', 'ld-dashboard' ); ?></a>
				</div>

				<div class="form-table ld-dashboard-setting-accordian-wrapper components-section">
					<div class="ld-grid-view-wrapper wbcom-tab-content">
						<div class="ld-dashboard-row-section ld-dashboard-general-setting-accordian-content">
							<div class="wbcom-settings-section-wrap announcements">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Announcements', 'ld-dashboard' ); ?></label>
									<p class="description"><?php esc_html_e( 'Enable this option if you want to enable the announcement feature.', 'ld-dashboard' ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox"  name="ld_dashboard_general_settings[enable-announcements]" value="1" <?php checked( $settings['enable-announcements'], '1' ); ?> data-id="enable-announcements"/>
								</div>
							</div>

							<div class="wbcom-settings-section-wrap email-integration">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Email integration', 'ld-dashboard' ); ?></label>
									<p class="description"><?php esc_html_e( 'Enable this option if you want to enable email integration.', 'ld-dashboard' ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox" name="ld_dashboard_general_settings[enable-email-integration]" value="1" <?php checked( $settings['enable-email-integration'], '1' ); ?> />
								</div>
							</div>
							<?php if ( class_exists( 'BuddyPress' ) && bp_is_active( 'messages' ) ) { ?>
								<div class="wbcom-settings-section-wrap">
									<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
										<label><?php esc_html_e( 'Private Message Support', 'ld-dashboard' ); ?></label>
										<p class="description"><?php esc_html_e( 'Enable this option if you want to integrate private messaging. Private messaging is a feature of the BuddyPress or BuddyBoss Platform. This option requires either BuddyPress or BuddyBoss Platform to be installed and activated.', 'ld-dashboard' ); ?></p>
									</div>
									<div class="ld-grid-content wbcom-settings-section-options">
										<input type="checkbox" name="ld_dashboard_general_settings[enable-messaging-integration]" value="1" <?php checked( $settings['enable-messaging-integration'], '1' ); ?>/>
									</div>
								</div>
							<?php } ?>
							<?php if ( ld_dashboard_is_commission_enabled() ) : ?>
							<div class="wbcom-settings-section-wrap instructor-earning-log">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Earning Logs', 'ld-dashboard' ); ?></label>
									<p class="description"><?php esc_html_e( 'Allow instructor earining logs on instructor dashboard.', 'ld-dashboard' ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox"  name="ld_dashboard_general_settings[enable-instructor-earning-logs]" value="1" <?php ( isset( $monetization_settings['enable-revenue-sharing'] ) ) ? checked( $settings['enable-instructor-earning-logs'], '1' ) : ''; ?> data-id="enable-revenue-sharing"/>
								</div>
							</div>
							<?php endif; ?>
						</div>
					</div>
				</div>

				<!-- Dashboard Widgets -->
				<div class="wbcom-admin-title-section-wrap title-tutorial-section-wrap">
					<div class="wbcom-admin-title-section-inner">
						<h4><?php esc_html_e( 'Dashboard Widgets', 'ld-dashboard' ); ?></h4>
					</div>
					<a href="https://docs.wbcomdesigns.com/docs/learndash-dashboard/ld-settings-learndash-dashboard/ld-global-settings/" class="ld-tutorial-btn" target="_blank"><?php esc_html_e( 'View Tutorial', 'ld-dashboard' ); ?></a>
				</div>
				<div class="form-table ld-dashboard-setting-accordian-wrapper dashboard-widget-section">
					<div class="ld-grid-view-wrapper wbcom-tab-content">
						<div class="ld-dashboard-row-section ld-dashboard-general-setting-accordian-content">
							<div class="wbcom-settings-section-wrap dashboard-page-title">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Dashboard Page Title', 'ld-dashboard' ); ?></label>
									<p class="description"><?php esc_html_e( 'Would you like to display or hide the title of your dashboard page?', 'ld-dashboard' ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox"  name="ld_dashboard_general_settings[ldd_page_title]" value="1" <?php ( isset( $settings['ldd_page_title'] ) ) ? checked( $settings['ldd_page_title'], '1' ) : ''; ?>/>
								</div>
							</div>

							<div class="wbcom-settings-section-wrap ld-statistics-option-role-by">
								<div class="ld-dashboard-widgets-statistics-select-roles">
									<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
										<label><?php esc_html_e( 'Summary of LearnDash Statistics', 'ld-dashboard' ); ?></label>
										<p class="description"><?php esc_html_e( 'Enable this option if you wish to showcase Learndash statistics on the dashboard page.', 'ld-dashboard' ); ?></p>
									</div>
									<div class="ld-grid-content wbcom-settings-section-options">
										<input type="checkbox" name="ld_dashboard_general_settings[statistics-tiles]" value="1" <?php checked( $settings['statistics-tiles'], '1' ); ?> />
									</div>
								</div>
								<div class="ld-grid-content ld-dashboard-select2-user-roles statistics-tiles-user-role">
									<select class="ld-select-user-roles" name="ld_dashboard_general_settings[statistics-tiles-allwoed-roles][]" id="statistics-tiles-allwoed-roles" multiple>
										<?php foreach ( ld_dashboard_get_dashboard_user_roles() as $role => $name ) : ?>
											<option value="<?php echo esc_attr( $role ); ?>" <?php echo in_array( $role, $settings['statistics-tiles-allwoed-roles'] ) ? 'selected' : ''; ?>><?php echo esc_html( $name ); ?></option>
										<?php endforeach; ?>
									</select>
								</div>
							</div>							
							<div class="wbcom-settings-section-wrap ld-statistics-option-role-by">
								<div class="ld-dashboard-widgets-statistics-select-roles">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php printf(/* translators: %s: Course label. */ esc_html__( '%s Completion chart', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></label>
									<p class="description"><?php printf(/* translators: %s: Course label. */ esc_html__( 'To view the  %s completion report, enable this option.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox"  name="ld_dashboard_general_settings[course-completion-report]" value="1" <?php checked( $settings['course-completion-report'], '1' ); ?> data-id="course-completion-report"/>
								</div>
							</div>
								<div class="ld-grid-content ld-dashboard-select2-user-roles course-completion-report-user-role">
									<select class="ld-select-user-roles" name="ld_dashboard_general_settings[course-completion-report-roles][]" id="course-completion-report-roles" multiple>
										<?php foreach ( ld_dashboard_get_dashboard_user_roles( array( 'other' ) ) as $role => $name ) : ?>
											<option value="<?php echo esc_attr( $role ); ?>" <?php echo in_array( $role, $settings['course-completion-report-roles'] ) ? 'selected' : ''; ?> ><?php echo esc_html( $name ); ?></option>
										<?php endforeach; ?>
									</select>
								</div>
							</div>
							<div class="wbcom-settings-section-wrap ld-statistics-option-role-by">
								<div class="ld-dashboard-widgets-statistics-select-roles">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php printf(/* translators: %s: Course label. */ esc_html__( 'Top %s Chart', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ); ?></label>
									<p class="description"><?php printf(/* translators: %s: Course label. */ esc_html__( 'Enable this option to display a report of the most completed  %s during a specific time frame.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox"  name="ld_dashboard_general_settings[top-courses-report]" value="1" <?php checked( $settings['top-courses-report'], '1' ); ?> data-id="top-courses-report"/>
								</div>
							</div>
								<div class="ld-grid-content ld-dashboard-select2-user-roles top-courses-report-user-role">
									<select class="ld-select-user-roles" name="ld_dashboard_general_settings[top-courses-report-roles][]" id="top-courses-report-roles" multiple>
										<?php foreach ( ld_dashboard_get_dashboard_user_roles( 'other' ) as $role => $name ) : ?>
											<option value="<?php echo esc_attr( $role ); ?>" <?php echo in_array( $role, $settings['top-courses-report-roles'] ) ? 'selected' : ''; ?> ><?php echo esc_html( $name ); ?></option>
										<?php endforeach; ?>
									</select>
								</div>
							</div>

							<div class="wbcom-settings-section-wrap ld-statistics-option-role-by">
								<div class="ld-dashboard-widgets-statistics-select-roles">
								<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
									<label><?php esc_html_e( 'Student Details', 'ld-dashboard' ); ?></label>
									<p class="description"><?php printf(/* translators: %s: Course label. */ esc_html__( 'Consider enabling this option to display student\'s  %s  progress.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></p>
								</div>
								<div class="ld-grid-content wbcom-settings-section-options">
									<input type="checkbox"  name="ld_dashboard_general_settings[student-details]" value="1" <?php checked( $settings['student-details'], '1' ); ?> data-id="student-details"/>
								</div>
							</div>
								<div class="ld-grid-content ld-dashboard-select2-user-roles student-details-user-role">
									<select class="ld-select-user-roles" name="ld_dashboard_general_settings[student-details-roles][]" id="student-details-roles" multiple>
										<?php foreach ( ld_dashboard_get_dashboard_user_roles( 'other' ) as $role => $name ) : ?>
											<option value="<?php echo esc_attr( $role ); ?>" <?php echo in_array( $role, $settings['student-details-roles'] ) ? 'selected' : ''; ?> ><?php echo esc_html( $name ); ?></option>
										<?php endforeach; ?>
									</select>
								</div>
							</div>

							<div class="wbcom-settings-section-wrap ld-statistics-option-role-by">
								<div class="ld-dashboard-widgets-statistics-select-roles">
									<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
										<label><?php esc_html_e( 'Student Course Progress', 'ld-dashboard' ); ?></label>
										<p class="description"><?php printf(/* translators: %s: Course label. */ esc_html__( 'Consider enabling this option to display %s  progress to students.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></p>
									</div>
									<div class="ld-grid-content wbcom-settings-section-options">
										<input type="checkbox"  name="ld_dashboard_general_settings[course-progress]" value="1" <?php checked( $settings['course-progress'], '1' ); ?> data-id="course-progress"/>
									</div>
								</div>
							</div>
						</div>
						</div>
						</div>

						<!-- Popular Course Settings	 -->
						<?php
						if ( LearnDash_Settings_Section::get_section_setting( 'LearnDash_Settings_Courses_Taxonomies', 'ld_course_tag' ) == 'yes' ) {
							$terms = get_terms(
								array(
									'taxonomy'   => 'ld_course_tag',
									'hide_empty' => false,
								)
							);
							if ( count( $terms ) > 0 ) {
								?>
						<div class="form-table ld-dashboard-setting-accordian-wrapper">							
							<div class="ld-grid-view-wrapper">
								<div class="wbcom-admin-title-section-wrap title-tutorial-section-wrap">
									<div class="wbcom-admin-title-section-inner">
										<h4><?php printf(/* translators: %s: Course label. */ esc_html__( 'Featured %s', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?>	</h4>
									</div>
								</div>
								<div class="ld-grid-view-wrapper wbcom-tab-content">
								<div class="ld-dashboard-row-section">
									<div class="wbcom-settings-section-wrap most-popular-course">
									<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
										<label><?php printf(/* translators: %s: Course label. */ esc_html__( 'Featured %s', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></label>
										<p class="description"><?php printf(/* translators: %s: Course label. */ esc_html__( 'Activate this option to showcase  %s based on selected tags.', 'ld-dashboard' ), esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) ) ); ?></p>
									</div>
									<div class="ld-grid-content wbcom-settings-section-options">
										<input type="checkbox" class="ld-dashboard-setting ld-dashboard-popular-course-checkbox" name="ld_dashboard_general_settings[popular-course-report]" value="1" <?php checked( $settings['popular-course-report'], '1' ); ?> data-id="popular-course-report"/>
									</div>
									</div>
								<div class="ld-dashboard-popular-course-setting ld-dashboard-popular-course-toggle" style="<?php echo ( isset( $settings['popular-course-report'] ) && 1 == $settings['popular-course-report'] ) ? '' : 'display:none'; ?>">
									<div class="ld-grid-cols-2 ld-grid-cols-gap">
								<div class="wbcom-settings-section-wrap popular-course-tag">
									<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
										<label><?php printf(/* translators: %s: Course label. */ esc_html__( 'Featured %s Tag', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></label>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<select name="ld_dashboard_general_settings[popular_course_tag]">
									<option value=""><?php echo esc_html__( ' Select Tag ', 'ld-dashboard' ); ?></option>
								<?php
								foreach ( $terms as $term ) {
									?>
									<option value="<?php echo esc_attr( $term->term_id ); ?>" <?php ( isset( $settings['popular_course_tag'] ) ) ? selected( $settings['popular_course_tag'], $term->term_id ) : ''; ?> ><?php echo esc_html( $term->name ); ?></option>
									<?php
								}
								?>
								</select>
							</div>
						</div>
						<div class="wbcom-settings-section-wrap most-popular-courses">
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php printf(/* translators: %s: Course label. */ esc_html__( 'Enable Featured %s', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ); ?></label>
								<p class="description"><?php printf(/* translators: %s: Course label. */ esc_html__( 'Control access to the featured %s by enabling this feature..', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ); ?></p>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<div class="ld-dashboard-mpo ld-featured-courses">
									<div class="ld-dashboard-mpo-item">
										<small for="most-popular-courses-students"><?php esc_html_e( 'Students', 'ld-dashboard' ); ?></small>
										<input type="checkbox" id="most-popular-courses-students"  name="ld_dashboard_general_settings[enable-popular-courses-student]" value="student" <?php ( isset( $settings['enable-popular-courses-student'] ) ) ? checked( $settings['enable-popular-courses-student'], 'student' ) : ''; ?> />
									</div>
									<div class="ld-dashboard-mpo-item">
										<small for="most-popular-courses-group-leader"><?php echo esc_html( LearnDash_Custom_Label::get_label( 'group_leader' ) ); ?></small>
										<input type="checkbox" id="most-popular-courses-group-leader"  name="ld_dashboard_general_settings[enable-popular-courses-group-leader]" value="group-leader" <?php ( isset( $settings['enable-popular-courses-group-leader'] ) ) ? checked( $settings['enable-popular-courses-group-leader'], 'group-leader' ) : ''; ?> />
									</div>
								</div>
							</div>
						</div>
						</div>
					</div>
					</div>
				</div>
				</div>
				</div>
								<?php
							}
						}
						?>

				<!-- Dashboard Welcome Screen Settings -->
				<div class="form-table">
					<div class="ld-grid-view-wrapper">
						<div class="wbcom-admin-title-section-wrap title-tutorial-section-wrap">
							<div class="wbcom-admin-title-section-inner">
								<h4><?php esc_html_e( 'Welcome Screen Settings', 'ld-dashboard' ); ?></h4>
							</div>
						</div>
						<div class="ld-grid-view-wrapper wbcom-tab-content">
						<div class="wbcom-settings-section-wrap welcome-screen-cover-image">							
							<div class="ld-grid-content wbcom-settings-section-options">								
								<div class="ld-dashboard-welcomebar-image ld-display-welcomebar_image"
									<?php
									if ( empty( $settings['welcomebar_image'] ) ) :
										?>
										style="display: block;" <?php endif; ?> >
										<img class="welcomebar_image" src="
										<?php
										if ( ! empty( $settings['welcomebar_image'] ) ) :
											echo esc_url( $settings['welcomebar_image'] );
										endif;
										?>
										" height="180" width="130"/>
										<span class="ld-dashboard-image-close" data-slug="welcomebar_image">x</span>
								</div>
							</div>
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php esc_html_e( 'Dashboard Cover Image', 'ld-dashboard' ); ?></label>
								<p class="description"><?php esc_html_e( 'Change image for dashboard top cover message.', 'ld-dashboard' ); ?></p>

								<label class="ld-dashboard-welcomebar-image">
									<input type="button" data-slug="welcomebar_image" class="button-secondary ld_dashboard_upload_image" value="<?php esc_attr_e( 'Upload Cover Image', 'ld-dashboard' ); ?>" />
									<input type="hidden" id="welcomebar_image" name="ld_dashboard_general_settings[welcomebar_image]" value="<?php echo isset( $settings['welcomebar_image'] ) ? esc_url( $settings['welcomebar_image'] ) : ''; ?>" >
								</label>
							</div>
						</div>
					</div>
					</div>
				</div>

				<!-- Single Instructor Page -->
				<div class="form-table ld-dashboard-setting-accordian-wrapper general-settings-section">
					<div class="ld-grid-view-wrapper">
					<div class="ld-dashboard-general-setting-accordian-content">
					<div class="form-table">
					<div class="ld-grid-view-wrapperl">
						<div class="wbcom-admin-title-section-wrap title-tutorial-section-wrap">
							<div class="wbcom-admin-title-section-inner">
								<h4><?php printf(/* translators: %s: Course label. */ esc_html__( 'Single Instructor Page', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></h4>
							</div>
						</div>
						<div class="ld-grid-view-wrapper wbcom-tab-content">
						<div class="ld-dashboard-row-section">
						<div class="wbcom-settings-section-wrap grid-columns">
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php esc_html_e( 'Grid Columns', 'ld-dashboard' ); ?></label>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<input type="number" name="ld_dashboard_general_settings[ld-course-grid-columns]" value="<?php echo isset( $settings['ld-course-grid-columns'] ) ? $settings['ld-course-grid-columns'] : '4'; ?>" />
							</div>
						</div>
						<div class="wbcom-settings-section-wrap progress-bar">
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php esc_html_e( 'Progress Bar', 'ld-dashboard' ); ?></label>
								<p class="description"><?php printf(/* translators: 1%s: Course label; 2%s: Course label. */ esc_html__( 'Enable %1$s progress bar in %2$s Grid.', 'ld-dashboard' ), esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></p>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<input type="checkbox"  name="ld_dashboard_general_settings[ld-course-grid-progress-bar]" value="1" <?php ( isset( $settings['ld-course-grid-progress-bar'] ) ) ? checked( $settings['ld-course-grid-progress-bar'], '1' ) : ''; ?> data-id="ld-course-grid-progress-bar"/>
							</div>
						</div>
						<div class="wbcom-settings-section-wrap course-content">
							<div class="ld-grid-label wbcom-settings-section-options-heading" scope="row">
								<label><?php printf(/* translators: %s: Course label. */ esc_html__( '%s Content', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></label>
								<p class="description"><?php printf(/* translators: 1%s: Course label; 2%s: Course label. */ esc_html__( 'Enable %1$s title, short description and CTA button in %2$s Grid.', 'ld-dashboard' ), esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></p>
							</div>
							<div class="ld-grid-content wbcom-settings-section-options">
								<input type="checkbox"  name="ld_dashboard_general_settings[ld-course-grid-course-content]" value="1" <?php ( isset( $settings['ld-course-grid-course-content'] ) ) ? checked( $settings['ld-course-grid-course-content'], '1' ) : ''; ?> data-id="ld-course-grid-course-content"/>
							</div>
						</div>
					</div>
				</div>
				</div>				
				</div>
				</div>
				</div>
				</div>
				<div class="wbcom-tab-content ld-submit-button-section">
					<?php submit_button(); ?>
					<?php
					submit_button(
						__( 'Reset Settings', 'ld-dashboard' ),
						'primary ld-dashboard-reset-button',
						'ld-dashboard-reset-general-settings',
						true,
						array(
							'data-setting' => 'ld_dashboard_general_settings',
						)
					);
					?>
					<?php wp_nonce_field( 'ld-dashboard-settings-submit', 'ld-dashboard-settings-submit' ); ?>
				</div>
			</form>
		</div>
	</div>
</div>
