<?php

function ld_dashboard_user_statistics_event_scheduler() {

	if ( ! wp_next_scheduled( 'ld_dashboard_course_user_counts' ) ) {
		global $ld_plugin_public;
		wp_schedule_event( time(), 'twicedaily', 'ld_dashboard_course_user_counts' );
	}

	if ( ! wp_next_scheduled( 'ld_dashboard_group_user_counts' ) ) {
		global $ld_plugin_public;
		wp_schedule_event( time(), 'twicedaily', 'ld_dashboard_group_user_counts' );
	}

	if ( ! wp_next_scheduled( 'ld_dashboard_user_statistics' ) ) {
		global $ld_plugin_public;
		wp_schedule_event( time(), 'hourly', 'ld_dashboard_user_statistics' );
	}

	/*
	if ( isset($_GET['test']) && $_GET['test'] == 'test') {
		ld_dashboard_user_statistics_event_callback(29382);
		exit;
	}
	*/

}

add_action( 'ld_dashboard_user_statistics', 'ld_dashboard_user_statistics_callback' );

function ld_dashboard_user_statistics_callback() {

	global $wpdb;

	$args = array(
		'fields'   => array( 'ID', 'user_email' ),
		'role__in' => array( 'administrator', 'group_leader', 'ld_instructor' ),
		'orderby'  => 'ID',
	);
	// Create the WP_User_Query object
	$wp_user_query = new WP_User_Query( $args );
	$users         = $wp_user_query->get_results();
	if ( ! empty( $users ) ) {
		$current_time = time();
		foreach ( $users as $user ) {
			wp_schedule_single_event( $current_time, 'ld_dashboard_user_statistics_event', array( $user->ID ) );
			$current_time += 5;
		}
	}

	/*
	$args  = array(
		'fields' => ['ID', 'user_email' ],
		'role__not_in' => ['administrator', 'group_leader', 'ld_instructor' ],
		'orderby' => 'ID',
	);
	// Create the WP_User_Query object
	$wp_user_query = new WP_User_Query($args);
	$users = $wp_user_query->get_results();
	if ( !empty( $users ) ) {
		$current_time       = time();
		foreach( $users as $user) {
			$user_id 		= $user->ID;
			$current_time += 5;
			wp_schedule_single_event( $current_time, 'ld_dashboard_user_statistics_event', [$user_id] );
		}
	}
	*/
}

add_action( 'ld_dashboard_user_statistics_event', 'ld_dashboard_user_statistics_event_callback' );

function ld_dashboard_user_statistics_event_callback( $user_id ) {
	global $ld_plugin_public, $current_user, $wpdb, $is_student, $total_students;
	$course_ids   = array( 0 );
	$user_ids     = array( 0 );
	$current_user = get_userdata( $user_id );
	$is_student   = ld_dashboard_studnet_user( $user_id );

	/* Get Course Count*/
	$course_count           = 0;
	$lesson_count           = 0;
	$topics_count           = 0;
	$quizzes_count          = 0;
	$active_course_count    = 0;
	$completed_course_count = 0;

	if ( learndash_is_admin_user( $user_id ) || ( learndash_is_group_leader_user( $user_id ) && 'advanced' === learndash_get_group_leader_manage_courses() ) ) {

		$course_ids    = $course_count = $ld_plugin_public->ld_dashboard_count_post_type( 'sfwd-courses' );
		$lesson_count  = $ld_plugin_public->ld_dashboard_count_post_type( 'sfwd-lessons' );
		$topics_count  = $ld_plugin_public->ld_dashboard_count_post_type( 'sfwd-topic' );
		$quizzes_count = $ld_plugin_public->ld_dashboard_count_post_type( 'sfwd-quiz' );

	} elseif ( learndash_is_group_leader_user( $user_id ) ) {

		$course_ids    = $course_count = learndash_get_group_leader_groups_courses();
		$lesson_count  = $ld_plugin_public->get_group_leader_courses_lessons();
		$topics_count  = $ld_plugin_public->get_group_leader_lessons_contents( 'topics' );
		$quizzes_count = $ld_plugin_public->get_group_leader_lessons_contents( 'quizzes' );
	} elseif ( ld_dashboard_instructor_user( $user_id ) ) {

		$course_ids    = $course_count = $ld_plugin_public->get_instructor_courses_list();
		$lesson_count  = $ld_plugin_public->get_instructor_lessons_list();
		$topics_count  = $ld_plugin_public->get_instructor_lessons_contents( 'topics' );
		$quizzes_count = $ld_plugin_public->get_instructor_lessons_contents( 'quizzes' );

	} else {
		$total_quizzes = $ld_plugin_public->ld_dashboard_get_student_data( $user_id );
		$quizzes_count = $total_quizzes['quiz_completed'];
	}

	$course_count  = is_array( $course_count ) ? count( $course_count ) : $course_count;
	$lesson_count  = is_array( $lesson_count ) ? count( $lesson_count ) : $lesson_count;
	$topics_count  = is_array( $topics_count ) ? count( $topics_count ) : $topics_count;
	$quizzes_count = is_array( $quizzes_count ) ? count( $quizzes_count ) : $quizzes_count;

	/* Students Count */
	$students_count = 0;
	$total_students = array();

	if ( learndash_is_admin_user( $user_id ) ) {
		$user_ids = $total_students = ld_dashboard_user_statistics_get_students( $user_id, false );
	} else {
		$user_ids = $total_students = ld_dashboard_user_statistics_get_students( $user_id );
	}
	// Remove Code
	$students_count = count( $total_students );

	/* Group Count*/
	$group_ids   = learndash_get_groups( true, $user_id );
	$group_count = ( ! empty( $group_ids ) ) ? count( $group_ids ) : 0;

	$enrolled_courses      = learndash_user_get_enrolled_courses( $user_id, array(), true );
	$enrolled_course_count = ( ! empty( $enrolled_courses ) ) ? count( $enrolled_courses ) : 0;
	if ( is_array( $enrolled_courses ) && count( $enrolled_courses ) > 0 ) {

		/* Course Finished */
		/*
		$course_completed = "SELECT  count(*) as course_completed FROM {$wpdb->prefix}learndash_user_activity WHERE activity_type= 'course' and user_id={$user_id} and activity_status = 1";
		$course_completed_activities = $wpdb->get_row( $course_completed, 'ARRAY_A');
		if ( !empty($course_completed_activities)) {
			$completed_course_count = $course_completed_activities['course_completed'];
		}
		*/

		/* Course progress */
		/*
		$course_in_progress = "SELECT count(*) as course_in_progress FROM {$wpdb->prefix}learndash_user_activity WHERE activity_type= 'course' and user_id={$user_id} and activity_status = 0";
		$course_in_progress_activities = $wpdb->get_row( $course_in_progress, 'ARRAY_A');
		if ( !empty($course_in_progress_activities)) {
			$active_course_count = $course_in_progress_activities['course_in_progress'];
		}
		*/
		/* Course not in progress */
		/*
		$course_not_started = "SELECT count(*) as course_not_started FROM {$wpdb->prefix}learndash_user_activity WHERE activity_type= 'access' and user_id={$user_id} and activity_status = 0";
		$course_not_started_activities = $wpdb->get_row( $course_not_started, 'ARRAY_A');
		if ( !empty($course_not_started_activities)) {
			$not_started_count 		= ( $course_not_started_activities['course_not_started'] != 0 ) ? $course_not_started_activities['course_not_started'] : $enrolled_course_count;
			//$active_course_count 	= $not_started_count - $completed_course_count - $active_course_count;
			$active_course_count 	= $not_started_count - $completed_course_count;
		}
		*/
		foreach ( $enrolled_courses as $course_id ) {
			$course_data = learndash_user_get_course_progress( $user_id, $course_id, 'summary' );
			if ( $course_data['status'] == 'completed' ) {
				$completed_course_count++;
			}
			if ( $course_data['status'] == 'in_progress' ) {
				$active_course_count++;
			}
		}
	}

	/* */
	$approved_assignment_count     = 0;
	$not_approved_assignment_count = 0;
	$graded_essays_count           = 0;
	$not_graded_essays_count       = 0;
	$assignments_args              = array(
		'post_type'   => 'sfwd-assignment',
		'post_status' => 'publish',
		'fields'      => 'ids',
		'meta_query'  => array(
			array(
				'key'     => 'approval_status',
				'compare' => 1,
			),
		),
	);
	if ( $is_student == 1 ) {
		$assignments_args['author'] = $user_id;
	}
	if ( in_array( 'ld_instructor', (array) $current_user->roles ) ) {
		$get_courses_sql = $wpdb->prepare( "Select ID from {$wpdb->prefix}posts INNER JOIN {$wpdb->prefix}postmeta pm ON ( {$wpdb->prefix}posts.ID = pm.post_id ) where ( post_author=%d OR ( pm.meta_key = '_ld_instructor_ids' AND pm.meta_value REGEXP '.*;s:[0-9]+:\"%d\"*' ) ) AND post_type='sfwd-courses' AND {$wpdb->prefix}posts.post_status = 'publish' Group By {$wpdb->prefix}posts.ID", $user_id, $user_id);
		$cousres         = $wpdb->get_results( $get_courses_sql );

		if ( ! empty( $cousres ) ) {
			$course_ids = array();
			foreach ( $cousres as $course ) {
				$course_ids[] = $course->ID;
			}
		}
		$assignments_args['meta_query'][] = array(
			'key'     => 'course_id',
			'value'   => $course_ids,
			'compare' => 'IN',
		);
	}
	$approved_assignment_count = ld_dashboard_learndash_get_assignments_pending_count( $user_id, $assignments_args, '', $course_ids, $user_ids );

	/* Not Approved Assignments Count */
	$assignments_completed = '';
	$assignments_args      = array(
		'post_type'   => 'sfwd-assignment',
		'post_status' => 'publish',
		'fields'      => 'ids',
		'meta_query'  => array(
			array(
				'key'     => 'approval_status',
				'compare' => 'NOT EXISTS',
			),
		),
	);

	$not_approved_assignment_count = ld_dashboard_learndash_get_assignments_pending_count( $user_id, $assignments_args, '', $course_ids, $user_ids );

	/* graded essays count */
	$graded_essays_count = ld_dashboard_learndash_get_essay_count( 'found_posts', 'graded', $course_ids, $user_ids, $user_id );
	/* not graded essays count */
	$not_graded_essays_count = ld_dashboard_learndash_get_essay_count( 'found_posts', 'not_graded', $course_ids, $user_ids, $user_id );

	/* certifcate Count*/
	$certificate_count                                = learndash_get_certificate_count( $user_id );
	$certificate_count                                = ( ! empty( $certificate_count ) ) ? $certificate_count : 0;
	$user_statistics['user_id']                       = $user_id;
	$user_statistics['course_count']                  = $course_count;
	$user_statistics['lessons_count']                 = $lesson_count;
	$user_statistics['topics_count']                  = $topics_count;
	$user_statistics['quizze_count']                  = $quizzes_count;
	$user_statistics['enrolled_course_count']         = $enrolled_course_count;
	$user_statistics['active_course_count']           = $active_course_count;
	$user_statistics['completed_course_count']        = $completed_course_count;
	$user_statistics['approved_assignment_count']     = $approved_assignment_count;
	$user_statistics['not_approved_assignment_count'] = $not_approved_assignment_count;
	$user_statistics['graded_essays_count']           = $graded_essays_count;
	$user_statistics['not_graded_essays_count']       = $not_graded_essays_count;
	$user_statistics['students_count']                = $students_count;
	$user_statistics['group_count']                   = $group_count;
	$user_statistics['certificate_count']             = $certificate_count;

	$user_statistics['created_date']  = gmdate( 'Y-m-d H:i:s' );
	$user_statistics['modified_date'] = gmdate( 'Y-m-d H:i:s' );
	$query                            = $wpdb->prepare( 'SELECT id FROM ' . $wpdb->prefix . 'ld_dashboard_users_statistics WHERE user_id = %d ', $user_id );
	$user_statistics_id               = $wpdb->get_var( $query );
	if ( $user_statistics_id == 0 || $user_statistics_id == '' ) {
		$result = $wpdb->insert( $wpdb->prefix . 'ld_dashboard_users_statistics', $user_statistics, array( '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%s', '%s' ) );
	} else {
		unset( $user_statistics['created_date'] );
		$wpdb->update(
			$wpdb->prefix . 'ld_dashboard_users_statistics',
			$user_statistics,
			array(
				'id' => $user_statistics_id,
			),
			array( '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%d', '%s' )
		);
	}

}

function ld_dashboard_user_statistics_get_students( $user_id, $open = true ) {

	$cousres        = ld_dashboard_user_statistics_get_user_courses_list( $user_id, false, $open );
	$total_students = array();
	$args           = apply_filters( 'ld_dashbaord_get_users_for_course_args', array( 'fields' => array( 'ID', 'display_name', 'user_email' ) ) );

	if ( ! empty( $cousres ) ) {
		foreach ( $cousres as $course ) {
			$cource_students = get_post_meta( $course, 'ld_dashboard_course_enrolled_users', true );
			if ( ! empty( $cource_students ) ) {
				$total_students = array_unique( array_merge( $total_students, $cource_students ), SORT_REGULAR );
			}
		}
	}
	if ( ! empty( $total_students ) ) {
		$total_students = array_unique( $total_students, SORT_REGULAR );
		$total_students = array_values( $total_students );
	}
	return apply_filters( 'ld_dashboard_user_statistics_get_students', $total_students );
}


function ld_dashboard_user_statistics_get_user_courses_list( $user_id = 0, $id = false, $open = false ) {
	global $wpdb;

	if ( learndash_is_admin_user( $user_id ) ) {
		$sfwd_courses = $wpdb->prepare(
			"SELECT SQL_CALC_FOUND_ROWS DISTINCT ID
								 FROM {$wpdb->posts}
								 WHERE
									post_status = '%s' AND post_type= '%s'",
			'publish',
			'sfwd-courses'
		);
	} else {
		$sfwd_courses = $wpdb->prepare(
			"SELECT SQL_CALC_FOUND_ROWS DISTINCT ID
							 FROM {$wpdb->posts}
							 WHERE
								post_status = %s AND post_type= '%s' AND post_author = %s",
			'publish',
			'sfwd-courses',
			$user_id
		);
	}

	$course_ids = $wpdb->get_col( $sfwd_courses );

	$shared_sfwd_courses = $wpdb->prepare(
		"SELECT SQL_CALC_FOUND_ROWS DISTINCT ID
								 FROM {$wpdb->posts} p
								 INNER JOIN {$wpdb->postmeta} pm ON p.ID=pm.post_id
								 WHERE
									post_status = '%s' AND post_type= '%s' AND pm.meta_key ='_ld_instructor_ids' AND pm.meta_value LIKE '%\"{$user_id}\"%'",
		'publish',
		'sfwd-courses',
	);
	$shared_course_ids   = $wpdb->get_col( $shared_sfwd_courses );
	if ( count( $shared_course_ids ) > 0 ) {
		$course_ids = array_merge( $course_ids, $shared_course_ids );
	}

	$unique_ids     = array();
	$unique_array   = array();
	$not_course_tab = true;

	if ( $not_course_tab && learndash_is_group_leader_user( $user_id ) ) {
		$group_course_ids = learndash_get_group_leader_groups_courses();
		if ( count( $group_course_ids ) > 0 ) {
			$course_ids = array_merge( $course_ids, $group_course_ids );
		}
	}

	if ( count( $course_ids ) > 0 ) {
		$course_ids = array_unique( $course_ids );
		return $course_ids;
	} else {
		return array();
	}
}



add_action( 'ld_dashboard_course_user_counts', 'ld_dashboard_course_user_counts_callback' );
function ld_dashboard_course_user_counts_callback() {
	global $wpdb;
	$args = array(
		'post_type'      => 'sfwd-courses',
		'posts_per_page' => -1,
	);
	// Create the WP_User_Query object
	$course_query = new WP_Query( $args );
	if ( $course_query->have_posts() ) {
		$current_time = time();
		while ( $course_query->have_posts() ) {
			$course_query->the_post();
			wp_schedule_single_event( $current_time, 'ld_dashboard_single_course_user_count_event', array( get_the_ID() ) );
			$current_time += 1;
		}
	}
}

add_action( 'ld_dashboard_single_course_user_count_event', 'ld_dashboard_single_course_user_count_event_callback' );
function ld_dashboard_single_course_user_count_event_callback( $course_id ) {
	$args     = apply_filters( 'ld_dashbaord_get_users_for_course_args', array( 'fields' => array( 'ID' ) ) );
	$students = ld_dashboard_learndash_get_users_for_course( $course_id, $args, true );
	update_post_meta( $course_id, 'ld_dashboard_course_enrolled_users', array_values( $students ) );
}



function ld_dashboard_learndash_get_users_for_course( $course_id = 0, $query_args = array(), $exclude_admin = true ) {
	$course_user_ids = array();

	if ( empty( $course_id ) ) {
		return $course_user_ids;
	}

	$course_price_type = learndash_get_course_meta_setting( $course_id, 'course_price_type' );

	if ( 'open' === $course_price_type ) {

		$users = ld_dashboard_learndash_get_all_users();
		return $users;

	} else {

		$course_access_users = ld_dashboard_learndash_get_course_users_access_from_meta( $course_id );
		$course_user_ids     = array_merge( $course_user_ids, $course_access_users );

		$course_groups_users = ld_dashboard_learndash_get_course_groups_users_access( $course_id, $course_user_ids );
		$course_user_ids     = array_merge( $course_user_ids, $course_groups_users );

		if ( ! empty( $course_user_ids ) ) {
			$course_user_ids = array_unique( $course_user_ids );
		}

		$course_expired_access_users = ld_dashboard_learndash_get_course_expired_access_from_meta( $course_id );
		if ( ! empty( $course_expired_access_users ) ) {
			$course_user_ids = array_diff( $course_user_ids, $course_expired_access_users );
		}
	}

	return $course_user_ids;
}

/*
 * Get all open course users
 */

function ld_dashboard_learndash_get_all_users() {
	global $wpdb;

	if ( false === ( $course_user_ids = get_transient( 'ld_dashboard_all_users_ids' ) ) ) {
		$role_key        = $wpdb->get_blog_prefix( get_current_blog_id() ) . 'capabilities';
		$course_user_ids = $wpdb->get_col(
			$wpdb->prepare(
				"SELECT SQL_CALC_FOUND_ROWS DISTINCT users.ID
					 FROM {$wpdb->users} as users inner join {$wpdb->usermeta} as usermeta on users.ID = usermeta.user_id WHERE meta_key = %s AND meta_value NOT LIKE '%administrator%'",
				$role_key
			)
		);
		set_transient( 'ld_dashboard_all_users_ids', $course_user_ids, DAY_IN_SECONDS );
	}
	return $course_user_ids;
}

function ld_dashboard_learndash_get_course_users_access_from_meta( $course_id = 0 ) {
	global $wpdb;

	$course_user_ids = array();
	$role_key        = $wpdb->get_blog_prefix( get_current_blog_id() ) . 'capabilities';

	if ( ! empty( $course_id ) ) {
		// We have to do it this was because WP_User_Query cannot handle on meta EXISTS and another 'NOT EXISTS' in the same query.
		$course_user_ids = $wpdb->get_col(
			$wpdb->prepare(
				"SELECT SQL_CALC_FOUND_ROWS DISTINCT users.ID
				 FROM {$wpdb->users} as users
				 INNER JOIN {$wpdb->usermeta} as usermeta on users.ID = usermeta.user_id
				 INNER JOIN {$wpdb->usermeta} as usermeta1 on users.ID = usermeta1.user_id AND usermeta.user_id = usermeta1.user_id
				 WHERE
					usermeta.meta_key = %s AND usermeta.meta_value NOT LIKE '%administrator%'
					AND usermeta1.meta_key = %s",
				$role_key,
				'course_' . $course_id . '_access_from'
			)
		);
	}

	return $course_user_ids;
}

function ld_dashboard_learndash_get_course_groups_users_access( $course_id = 0, $course_user_ids = array() ) {

	$course_id = absint( $course_id );
	if ( ! empty( $course_id ) ) {
		global $wpdb;
		$role_key      = $wpdb->get_blog_prefix( get_current_blog_id() ) . 'capabilities';
		$course_groups = learndash_get_course_groups( $course_id );

		$course_user_ids = $wpdb->get_col(
			str_replace(
				"\'",
				"'",
				$wpdb->prepare(
					"SELECT SQL_CALC_FOUND_ROWS DISTINCT users.ID
					 FROM {$wpdb->users} as users
					 INNER JOIN {$wpdb->usermeta} as usermeta on users.ID = usermeta.user_id
					 INNER JOIN {$wpdb->usermeta} as usermeta1 on users.ID = usermeta1.user_id AND usermeta.user_id = usermeta1.user_id
					 WHERE
						usermeta.meta_key = %s AND usermeta.meta_value NOT LIKE '%administrator%'
						AND usermeta1.meta_key IN (%s) AND users.ID NOT IN(%s)",
					$role_key,
					implode(
						"','",
						array_map(
							function( $course_group_id ) {
								return "learndash_group_users_{$course_group_id}";
							},
							$course_groups
						)
					),
					implode( "','", $course_user_ids )
				)
			)
		);
		/*
		if ( ( is_array( $course_groups ) ) && ( ! empty( $course_groups ) ) ) {
			foreach ( $course_groups as $group_id ) {
				$group_users_ids = ld_dashboard_learndash_get_groups_user_ids( $group_id, $course_user_ids );
				if ( ! empty( $group_users_ids ) ) {
					$course_user_ids = array_merge( $course_user_ids, $group_users_ids );
				}
			}
		}
		*/
	}

	if ( ! empty( $course_user_ids ) ) {
		$course_user_ids = array_unique( $course_user_ids );
	}

	return $course_user_ids;
}


function ld_dashboard_learndash_get_groups_user_ids( $group_id, $course_user_ids = array() ) {
	global $wpdb;

	$role_key = $wpdb->get_blog_prefix( get_current_blog_id() ) . 'capabilities';

	// We have to do it this was because WP_User_Query cannot handle on meta EXISTS and another 'NOT EXISTS' in the same query.
	$course_user_ids = $wpdb->get_col(
		$wpdb->prepare(
			"SELECT SQL_CALC_FOUND_ROWS DISTINCT users.ID
			 FROM {$wpdb->users} as users
			 INNER JOIN {$wpdb->usermeta} as usermeta on users.ID = usermeta.user_id
			 INNER JOIN {$wpdb->usermeta} as usermeta1 on users.ID = usermeta1.user_id AND usermeta.user_id = usermeta1.user_id
			 WHERE
				usermeta.meta_key = %s AND usermeta.meta_value NOT LIKE '%administrator%'
				AND usermeta1.meta_key = %s AND users.ID NOT IN(%s)",
			$role_key,
			'learndash_group_users_' . intval( $group_id ),
			implode( ',', $course_user_ids )
		)
	);

	return $course_user_ids;
}


function ld_dashboard_learndash_get_course_expired_access_from_meta( $course_id = 0 ) {
	global $wpdb;
	$role_key         = $wpdb->get_blog_prefix( get_current_blog_id() ) . 'capabilities';
	$expired_user_ids = array();

	if ( ! empty( $course_id ) ) {

		$expired_user_ids = $wpdb->get_col(
			$wpdb->prepare(
				"SELECT SQL_CALC_FOUND_ROWS DISTINCT users.ID
				 FROM {$wpdb->users} as users
				 INNER JOIN {$wpdb->usermeta} as usermeta on users.ID = usermeta.user_id
				 INNER JOIN {$wpdb->usermeta} as usermeta1 on users.ID = usermeta1.user_id AND usermeta.user_id = usermeta1.user_id
				 WHERE
					usermeta.meta_key = %s AND usermeta.meta_value NOT LIKE '%administrator%'
					AND usermeta1.meta_key = %s",
				$role_key,
				'learndash_course_expired_' . $course_id
			)
		);
	}

	return array_map( 'absint', $expired_user_ids );
}


add_action( 'ld_dashboard_group_user_counts', 'ld_dashboard_group_user_counts_callback' );
function ld_dashboard_group_user_counts_callback() {
	global $wpdb;
	$args = array(
		'post_type'      => 'groups',
		'posts_per_page' => -1,
	);
	// Create the WP_User_Query object
	$groups_query = new WP_Query( $args );
	if ( $groups_query->have_posts() ) {
		$current_time = time();
		while ( $groups_query->have_posts() ) {
			$groups_query->the_post();
			wp_schedule_single_event( $current_time, 'ld_dashboard_single_groups_user_count_event', array( get_the_ID() ) );
			$current_time += 1;
		}
	}
}


add_action( 'ld_dashboard_single_groups_user_count_event', 'ld_dashboard_single_groups_user_count_event_callback' );
function ld_dashboard_single_groups_user_count_event_callback( $group_id ) {
	$args = apply_filters( 'ld_dashbaord_get_users_for_course_args', array( 'fields' => array( 'ID' ) ) );

	$students = ld_dashboard_learndash_get_groups_user_ids( $group_id, array() );
	update_post_meta( $group_id, 'ld_dashboard_grroup_enrolled_users', array_values( $students ) );

	$group_course_ids = learndash_group_enrolled_courses( $group_id );
	update_post_meta( $group_id, 'ld_dashboard_group_enrolled_courses', array_values( $group_course_ids ) );
}


function ld_dashboard_learndash_get_assignments_pending_count( $user_id, $query_args = array(), $return_field = 'found_posts', $course_ids = array(), $user_ids = array() ) {
	$return = 0;

	$default_args = array(
		'post_type'   => 'sfwd-assignment',
		'post_status' => 'publish',
		'fields'      => 'ids',
		'meta_query'  => array(
			array(
				'key'     => 'approval_status',
				'compare' => 'NOT EXISTS',
			),
		),
	);

	// added logic for non-admin user like group leaders who will only see a sub-set of assignments.

	if ( learndash_is_group_leader_user( $user_id ) ) {

		if ( ! empty( $course_ids ) && count( $course_ids ) ) {
			$default_args['meta_query'][] = array(
				'key'     => 'course_id',
				'value'   => $course_ids,
				'compare' => 'IN',
			);
		} else {
			return $return;
		}

		if ( ! empty( $user_ids ) && count( $user_ids ) ) {
			$default_args['author__in'] = $user_ids;
		} else {
			return $return;
		}
	}

	$query_args = wp_parse_args( $query_args, $default_args );
	/**
	 * Filters pending assignments count query arguments.
	 *
	 * @param array $query_args An array of pending assignment count query arguments.
	 */
	$query_args = apply_filters( 'learndash_get_assignments_pending_count_query_args', $query_args );

	if ( 'found_posts' == $return_field ) {
		$query_args['posts_per_page'] = 1;
		$query_args['paged']          = 1;
	}

	if ( ( is_array( $query_args ) ) && ( ! empty( $query_args ) ) ) {
		$query = new WP_Query( $query_args );

		if ( ( ! empty( $return_field ) ) && ( property_exists( $query, $return_field ) ) ) {
			$return = $query->$return_field;
		} else {
			$return = $query;
		}
	}

	return $return;
}


function ld_dashboard_learndash_get_essay_count( $return_field = 'found_posts', $type = 'graded', $course_ids = array(), $user_ids = array(), $user_id = 0 ) {
	global $is_student;
	$return = 0;

	$query_args = array(
		'post_type'   => 'sfwd-essays',
		'post_status' => $type,
		'fields'      => 'ids',
	);

	// added logic for non-admin user like group leaders who will only see a sub-set of assignments.
	if ( learndash_is_group_leader_user( $user_id ) ) {

		if ( ! empty( $course_ids ) && count( $course_ids ) ) {
			$query_args['meta_query'][] = array(
				'key'     => 'course_id',
				'value'   => $course_ids,
				'compare' => 'IN',
			);
		} else {
			return $return;
		}

		if ( ! empty( $user_ids ) && count( $user_ids ) ) {
			$query_args['author__in'] = $user_ids;
		} else {
			return $return;
		}
	} else {
		if ( $is_student != 1 && ! learndash_is_admin_user( $user_id ) ) {
			$query_args['author__in'] = ( ! empty( $user_ids ) ) ? $user_ids : array( 0 );
		}
	}

	if ( $is_student == 1 ) {
		$query_args['author'] = $user_id;
	}

	/**
	 * Filters pending essays count query arguments.
	 *
	 * @param array $query_args An array of pending essays count query arguments.
	 */
	$query_args = apply_filters( 'ld_dashboard_get_essay_count_query_args', $query_args );

	if ( 'found_posts' == $return_field ) {
		$query_args['posts_per_page'] = 1;
		$query_args['paged']          = 1;
	}

	if ( ( is_array( $query_args ) ) && ( ! empty( $query_args ) ) ) {
		$query = new WP_Query( $query_args );

		if ( ( ! empty( $return_field ) ) && ( property_exists( $query, $return_field ) ) ) {
			$return = $query->$return_field;
		} else {
			$return = $query;
		}
	}

	return $return;
}