<?php
/**
 * LearnDash Dashboard Activity Template
 */

$activity_steps_completed = ( isset( $activity->activity_meta['steps_completed'] ) ) ? intval( $activity->activity_meta['steps_completed'] ) : '';
$activity_steps_total     = ( isset( $activity->activity_meta['steps_total'] ) ) ? intval( $activity->activity_meta['steps_total'] ) : '';

$user_link = current_user_can( 'edit_user', $activity->user_id ) ? get_edit_user_link( $activity->user_id ) . '#ld_course_info' : '#';

$activity_diff_completed = ( ! empty( $activity->activity_completed ) && ! empty( $activity->activity_started ) ) 
    ? learndash_get_activity_human_time_diff( $activity->activity_started, $activity->activity_completed, 1 ) 
    : 0;

$activity_abbr_label_completed = ! empty( $activity_diff_completed ) 
    ? __( 'Completed Date (Duration)', 'ld-dashboard' ) 
    : __( 'Completed Date', 'ld-dashboard' );

$activity_diff_started = ! empty( $activity->activity_started ) 
    ? learndash_get_activity_human_time_diff( $activity->activity_started, time(), 1 ) 
    : learndash_get_activity_human_time_diff( $activity->activity_updated, time(), 1 );

$activity_abbr_label_started = ! empty( $activity_diff_started ) 
    ? esc_html__( 'Started Date (Duration)', 'ld-dashboard' ) 
    : esc_html__( 'Started Date', 'ld-dashboard' );

// Quiz Activity
if ( 'quiz' === $activity->activity_type ) : ?>
    <div class="activity-item quiz">
        <?php
        $course = $this->ld_dashboard_get_activity_course( $activity );
        if ( $course ) {
            /* translators: %1$s: User Display name; %2$s: Quiz Label; %3$s: Post Title; %4$s: Course Label; %5$s: Course Title. */
            echo '<p>' . sprintf( esc_html__( '%1$s completed a %2$s %3$s of %4$s %5$s', 'ld-dashboard' ), 
            esc_html( $activity->user_display_name ), 
            esc_html( strtolower( LearnDash_Custom_Label::get_label( 'quiz' ) ) ), 
            esc_html( $activity->post_title ), 
            esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) ), 
            esc_html( get_the_title( $course->ID ) ) ) . '</p>';
        } else {
            /* translators: %1$s: User Display name; %2$s: Quiz Label; %3$s: Post Title. */
            echo '<p>' . sprintf( esc_html__( '%1$s completed a %2$s %3$s', 'ld-dashboard' ), 
            esc_html( $activity->user_display_name ), 
            esc_html( strtolower( LearnDash_Custom_Label::get_label( 'quiz' ) ) ), 
            esc_html( $activity->post_title ) ) . '</p>';
        }
        /* translators: %1$s: Activity start time. */
        echo '<i>' . sprintf( esc_html__( '%1$s ago', 'ld-dashboard' ), esc_html( $activity_diff_started ) ) . '</i>';
        ?>
    </div>

<?php endif; ?>

<!-- Course Activity -->
<?php if ( 'course' === $activity->activity_type && $activity_steps_completed == $activity_steps_total ) : ?>
    <div class="activity-item course">
        <?php echo sprintf( /* translators: %1$s: User Display name; %2$s: Course Label; %3$s: Post Title. */ esc_html__( '%1$s completed a %2$s %3$s', 'ld-dashboard' ), 
        esc_html( $activity->user_display_name ), 
        esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) ), 
        esc_html( $activity->post_title ) ); ?>
    </div>
<?php endif; ?>

<!-- Access Activity -->
<?php if ( 'access' === $activity->activity_type ) : ?>
    <div class="activity-item course">
        <?php
         /* translators: %1$s: User Display name; %2$s: Course Label; %3$s: Post Title. */
        echo '<p>' . sprintf( esc_html__( '%1$s enrolled for a %2$s %3$s', 'ld-dashboard' ), 
        esc_html( $activity->user_display_name ), 
        esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) ), 
        esc_html( $activity->post_title ) ) . '</p>';

         /* translators: %1$s: Activity start time. */
        echo '<i>' . sprintf( esc_html__( '%1$s ago', 'ld-dashboard' ), esc_html( $activity_diff_started ) ) . '</i>';
        ?>
    </div>
<?php endif; ?>

<!-- Lesson Activity -->
<?php if ( 'lesson' === $activity->activity_type ) : ?>
    <div class="activity-item lesson">
        <?php
        $course    = $this->ld_dashboard_get_activity_course( $activity );
        $course_id = isset( $course ) ? $course->ID : $activity->activity_course_id;
        /* translators: %1$s: User Display name; %2$s: Post title; %3$s: Lesson Label; %4$s: Course Label; %5$s: Post Title. */
        echo '<p>' . sprintf( __( '%1$s completed a %2$s %3$s of %4$s %5$s', 'ld-dashboard' ), 
        esc_html( $activity->user_display_name ), 
        esc_html( $activity->post_title ), 
        esc_html( strtolower( LearnDash_Custom_Label::get_label( 'lesson' ) ) ), 
        esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) ), 
        esc_html( get_the_title( $course_id ) ) ) . '</p>';
        /* translators: %1$s: Activity start time. */
        echo '<i>' . sprintf( __( '%1$s ago', 'ld-dashboard' ), esc_html( $activity_diff_started ) ) . '</i>';
        ?>
    </div>
<?php endif; ?>

<!-- Topic Activity -->
<?php if ( 'topic' === $activity->activity_type && isset( $course ) && is_object( $course ) ) : ?>
    <div class="activity-item topic">
        <?php
        /* translators: %1$s: User Display name; %2$s: Topic Label; %3$s: Post title; %4$s: Course Label; %5$s: Post Title. */
        echo '<p>' . sprintf( __( '%1$s completed a %2$s %3$s of %4$s %5$s', 'ld-dashboard' ), 
        esc_html( $activity->user_display_name ), 
        esc_html( strtolower( LearnDash_Custom_Label::get_label( 'topic' ) ) ), 
        esc_html( $activity->post_title ), 
        esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) ), 
        esc_html( get_the_title( $course->ID ) ) ) . '</p>';
        /* translators: %1$s: Activity start time. */
        echo '<i>' . sprintf( __( '%1$s ago', 'ld-dashboard' ), esc_html( $activity_diff_started ) ) . '</i>';
        ?>
    </div>
<?php endif; ?>

<!-- See User Progress -->
<?php if ( is_null( $activity->activity_type ) ) : ?>
    <div class="activity-item not-started">
        <div class="header">
            <span class="user"><i class="fa fa-user"></i><a href="<?php echo esc_url( $user_link ); ?>" title="<?php esc_attr_e( 'See User Progress', 'ld-dashboard' ); ?>"><?php echo esc_html( $activity->user_display_name ); ?></a></span>
        </div>

        <div class="content">
            <?php if ( $course = $this->ld_dashboard_get_activity_course( $activity ) ) : ?>
                <i class="fa fa-graduation-cap"></i>
                <strong><?php echo sprintf( /* translators: %s: Course Label. */ _x( '%s:', 'Course:', 'ld-dashboard' ), LearnDash_Custom_Label::get_label( 'course' ) ); ?></strong> 
                <strong><a href="<?php echo esc_url( get_permalink( $course->ID ) ); ?>" class="link"><?php echo esc_html( get_the_title( $course->ID ) ); ?></a></strong>
                <?php
                edit_post_link(
                    sprintf(
                          /* translators: %s: Title. */
                        __( ' (edit<span class="screen-reader-text"> "%s"</span>)', 'ld-dashboard' ),
                        get_the_title( $course->ID )
                    ),
                    '<span class="ld-dashboard-edit-link edit-link">',
                    '</span>',
                    $course->ID
                );
                ?>
                <br/>
            <?php endif; ?>

            <strong><?php esc_html_e( 'Result:', 'ld-dashboard' ); ?> </strong><?php esc_html_e( 'Not Started', 'ld-dashboard' ); ?>
        </div>
    </div>
<?php endif; ?>
