<?php
$user              = wp_get_current_user();
$user_id           = $user->ID;
$course_user_id    = get_the_author_meta( 'ID' );
$enrolled_users    = ld_dashboard_get_course_students( get_the_ID() );
$course_nonce      = wp_create_nonce( 'course-nonce' );
$course_pricing    = learndash_get_course_price( get_the_ID() );
$course_price_html = '';
$dashboard_url     = Ld_Dashboard_Functions::instance()->ld_dashboard_get_url( 'dashboard' );
$elementor_support = get_option( 'elementor_cpt_support' );
$edit_url          = add_query_arg(
	array(
		'action'    => 'edit-course',
		'ld-course' => get_the_ID(),
		'tab'       => 'my-courses',
		'_lddnonce' => $course_nonce,
	),
	$dashboard_url
);


switch ( $course_pricing['type'] ) {
	case ( 'open' ):
		break;
	case ( 'free' ):
		break;
	case ( 'paynow' ):
		$currency           = ( version_compare( LEARNDASH_VERSION, '4.1.0', '<' ) ) ? learndash_30_get_currency_symbol() : learndash_get_currency_symbol();
		$currency           = ! empty( $currency ) ? $currency : __( 'Price', 'ld-dashboard' );
		$course_price_html .= '<span class="ld-dashboard-currency-symbol">' . $currency . ' </span>' . wp_kses_post( $course_pricing['price'] );
		break;
	case ( 'subscribe' ):
		$currency           = ( version_compare( LEARNDASH_VERSION, '4.1.0', '<' ) ) ? learndash_30_get_currency_symbol() : learndash_get_currency_symbol();
		$currency           = ! empty( $currency ) ? $currency : __( 'Price', 'ld-dashboard' );
		$course_price_html .= '<span class="ld-dashboard-currency-symbol">' . $currency . ' </span>' . wp_kses_post( $course_pricing['price'] );
		break;
	case ( 'closed' ):
		$currency           = ( version_compare( LEARNDASH_VERSION, '4.1.0', '<' ) ) ? learndash_30_get_currency_symbol() : learndash_get_currency_symbol();
		$currency           = ! empty( $currency ) ? $currency : __( 'Price', 'ld-dashboard' );
		$course_price_html .= '<span class="ld-dashboard-currency-symbol">' . $currency . ' </span>' . wp_kses_post( $course_pricing['price'] );
		break;
}


if ( has_post_thumbnail() ) {
	$featured_image = get_the_post_thumbnail_url();
} else {
	$featured_image = LD_DASHBOARD_PLUGIN_URL . 'public/img/course-default.png';
}

$post_status = '';
if ( 'publish' == get_post_status() ) {
	$post_status = esc_html__( 'Published', 'ld-dashboard' );
} elseif ( 'draft' == get_post_status() ) {
	$post_status = esc_html__( 'Draft', 'ld-dashboard' );
} elseif ( 'pending' == get_post_status() ) {
	$post_status = esc_html__( 'Pending', 'ld-dashboard' );
} elseif ( 'trash' === get_post_status() ) {
	$post_status = esc_html__( 'Trash', 'ld-dashboard' );
}

?>


<div id="ld-dashboard-course-<?php esc_attr( the_ID() ); ?>" class="ld-mycourse-wrap ld-mycourse-<?php esc_attr( the_ID() ); ?> __web-inspector-hide-shortcut__">
	<a href="<?php the_permalink(); ?>" class="ld-mycourse-thumbnail" style="background-image: url(<?php echo esc_url( $featured_image ); ?>)">
		<?php if ( '' != $course_price_html ) : ?>
		<span class="ld-dashboard-price-amount amount">
			<bdi>
				<?php echo wp_kses_post( $course_price_html ); ?>
			</bdi>
		</span>
		<?php endif; ?>
	</a>
	<div class="ld-mycourse-content">
		<?php do_action( 'ld_add_course_content_before' ); ?>
		<h3><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h3>
		<div class="ld-meta ld-course-metadata">
			<ul>
				<li class="ld-dashboard-course-status">
					<span><?php echo esc_html( $post_status ); ?></span>
				</li>
				<li><span class="ldd-icons ldd-icon-users"></span> 
					<?php esc_html_e( 'Students:', 'ld-dashboard' ); ?><span><?php echo count( $enrolled_users ); ?></span>
				</li>
				<?php if ( ! empty( ld_dashboard_get_course_categories( get_the_ID() ) ) ) : ?>
				<li><span class="ldd-icons ldd-icon-price-tag-3-line"></span>
					<?php echo ld_dashboard_get_course_categories( get_the_ID() ); ?>
				</li>
				<?php endif; ?>
			</ul>
		</div>
		<?php if ( 'trash' !== get_post_status() ) : ?>
		<div class="mycourse-footer">
			<div class="ld-mycourses-stats">									
				<a href="<?php the_permalink(); ?>" class="ld-mycourse-view" target="_blank">
					<span class="ldd-icons ldd-icon-eye-line visibility-icon"></span> <small class="ld-dashboard-tooltips"><?php esc_html_e( 'View', 'ld-dashboard' ); ?></small>
				</a>
				<a href="<?php echo esc_url( $edit_url ); ?>" class="ld-mycourse-edit">
					<span class="ldd-icons ldd-icon-edit edit_square"></span> <small class="ld-dashboard-tooltips"><?php esc_html_e( 'Edit', 'ld-dashboard' ); ?></small>
				</a>
				
				<?php if ( ! empty( $elementor_support ) && in_array( 'sfwd-courses', $elementor_support ) && did_action( 'elementor/loaded' ) && apply_filters( 'ld-dashboard-edit-with-elementor', true ) ) : ?>
					<a href="<?php echo esc_url( admin_url( 'post.php?post=' . esc_attr( get_the_ID() ) . '&action=elementor' ) ); ?>" class="ld-mycourse-edit">
					<span class="ldd-icons ldd-icon-edit edit_square"></span> <small class="ld-dashboard-tooltips"><?php esc_html_e( 'Edit with Elementor', 'ld-dashboard' ); ?></small>
				</a>
				<?php endif; ?>
				
				<a href="<?php echo esc_url( get_permalink() ) . '?action=delete-course&ld-course=' . esc_attr( get_the_ID() ); ?>" class="ld-dashboard-element-delete-btn" data-type="course" data-type_id=<?php echo esc_attr( get_the_ID() ); ?>>
					<span class="ldd-icons ldd-icon-trash-empty"></span> <small class="ld-dashboard-tooltips"><?php esc_html_e( 'Delete', 'ld-dashboard' ); ?></small>
				</a>
			</div>
		</div>
		<?php endif; ?>
		<?php do_action( 'ld_dashboard_course_content_after' ); ?>
	</div>
</div>
