<?php
/**
 * Provide a public-facing view for the plugin
 *
 * This file is used to markup the public-facing aspects of the plugin.
 *
 * @link       https://wbcomdesigns.com/
 * @since      1.0.0
 *
 * @package    Custom_Learndash
 * @subpackage Custom_Learndash/public/partials
 */

$page_num           = max( 1, get_query_var( 'paged', 1 ) );
$params             = ( isset( $_SERVER['QUERY_STRING'] ) && '' !== $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
$user               = wp_get_current_user();
$user_id            = $user->ID;
$dashboard_page_url = Ld_Dashboard_Functions::instance()->ld_dashboard_get_url( 'dashboard' );
$course_ids         = array();

// Reset shared_course_ids on each new page load to prevent duplicates across pagination
$shared_course_ids = array();

$args = array(
    'post_type'      => learndash_get_post_type_slug( 'course' ),
    'post_status'    => array( 'publish', 'pending', 'draft' ),
    'posts_per_page' => Ld_Dashboard_Functions::instance()->get_courses_per_page(),
    'paged'          => $page_num,
    'orderby'        => array(
        'date' => 'DESC', // Primary order by date, newest first
        'ID'   => 'ASC',  // Secondary order to ensure unique results across pagination
    ),
    'suppress_filters' => true,
);

// First, we get courses for instructor users
if ( ld_dashboard_instructor_user( get_current_user_id() ) ) {
    $instructor_courses = Ld_Dashboard_Public::get_instructor_courses_list( 0 );
    if ( is_array( $instructor_courses ) && ! empty( $instructor_courses ) ) {
        foreach ( $instructor_courses as $instructor_course ) {
            $course_ids[] = $instructor_course->ID;
        }
    }

    if ( ! empty( $course_ids ) ) {
        $args['post__in'] = array_unique( $course_ids );
    } else {
        $args['post__in'] = array( 0 );
    }
} elseif ( learndash_is_group_leader_user( get_current_user_id() ) ) { // then we check for group leader
    if ( 'advanced' !== learndash_get_group_leader_manage_courses() ) { // if group leader has basic course permission
        $args['author'] = get_current_user_id();
    }
}

$courses_query = new WP_Query( $args );

?>
<div class="wbcom-front-end-course-dashboard-my-courses-content">
    <div class="custom-learndash-list custom-learndash-my-courses-list">
        <div class="ld-dashboard-course-content instructor-courses-list"> 
            <div class="ld-dashboard-section-head-title ld-dashboard-course-search-section-inner">
                <h3><?php 
                /* translators: %s: Courses label. */
                printf( esc_html__( 'My %s', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ); ?></h3>
                <div class="ld-dashboard-course-search-section ld-flex-wrap">
                    <form class="ld-dashboard-course-search">
                        <span class="ldd-icon-search"></span>
                        <input id="ld-dashboard-post-search" type="search" placeholder="<?php echo esc_html__( 'Search Course', 'ld-dashboard' ); ?>" />
                        <input type="hidden" id="ld-dashboard-post-paged" value="<?php echo esc_attr( $page_num ); ?>" />
                    </form>
                    <div class="ld-dashboard-course-grid-list-toggle">
                        <a href="javascript:void(0);" class="ld-dashboard-listviews ld-dashboard-post-view"><span class="ldd-icons ldd-icon-list-check" data-view="list"></span></a>
                        <a href="javascript:void(0);" class="ld-dashboard-gridviews ld-dashboard-post-view <?php echo isset( $_COOKIE['view'] ) && 'grid' === $_COOKIE['view'] ? 'ld-grid-active' : ''; ?>"><span class="ldd-icons ldd-icon-function-line" data-view="grid"></span></a>
                    </div>
                </div>
            </div>

            <div class="ld-dashboard-tab-buttons-filter ld-flex-wrap">
                <ul class="ld-dashboard-filter-links ld-flex-wrap">
                    <li><a href="javascript:void(0);" class="ld-dashboard-status-filter ld-dashbaord-fillter-active" data-status="all"><?php echo esc_html__( 'All', 'ld-dashboard' ); ?></a></li>
                    <li><a href="javascript:void(0);" class="ld-dashboard-status-filter" data-status="publish"><?php echo esc_html__( 'Published', 'ld-dashboard' ); ?></a></li>
                    <li><a href="javascript:void(0);" class="ld-dashboard-status-filter" data-status="pending"><?php echo esc_html__( 'Pending', 'ld-dashboard' ); ?></a></li>
                    <li><a href="javascript:void(0);" class="ld-dashboard-status-filter" data-status="draft"><?php echo esc_html__( 'Draft', 'ld-dashboard' ); ?></a></li>
                    <li><a href="javascript:void(0);" class="ld-dashboard-status-filter" data-status="trash"><?php echo esc_html__( 'Trash', 'ld-dashboard' ); ?></a></li>
                </ul>
                <div class="ld-dashboard-filter-links-button ld-flex-wrap">
                    <?php if ( function_exists( 'learndash_is_learndash_license_valid' ) && learndash_is_learndash_license_valid() ) : ?>
                        <div class="ld-dashboard-header-button ld-dashboard-add-new-button-container">
                            <?php $course_nonce = wp_create_nonce( 'course-nonce' ); ?>
                            <a class="ld-dashboard-add-course-video-playlist" href="<?php echo esc_url( $dashboard_page_url ); ?>?action=add-course-playlist&tab=my-courses&_lddnonce=<?php echo esc_attr( $course_nonce ); ?>">
                            <span class="ldd-icons ldd-icon-video-add-line add-icon"></span> <?php echo esc_html__( 'Create from video playlist', 'ld-dashboard' ); ?></a>
                        </div>
                    <?php endif; ?>
                </div>

            </div>

            <div class="my-courses ld-dashboard-content-inner ld-dashboard-tab-content-wrapper <?php echo isset( $_COOKIE['view'] ) && 'grid' === $_COOKIE['view'] ? 'ld-dashboard-wbgrid' : ''; ?>">
                <?php do_action( 'ld_dashboard_before_courses_content' ); ?>
                <?php

                if ( $courses_query->have_posts() ) {
                    while ( $courses_query->have_posts() ) {
                        $courses_query->the_post();
                        $current_post_id = get_the_ID();

                        // Check if the post ID is already processed to prevent duplicates
                        if ( in_array( $current_post_id, $shared_course_ids ) ) {
                            continue;
                        }

                        // Track the processed post ID
                        $shared_course_ids[] = $current_post_id;

                        // Load the appropriate template
                        if ( false !== Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-course-listing.php' ) ) {
                            include Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-course-listing.php' );
                        } else {
                            include LD_DASHBOARD_PLUGIN_DIR . 'templates/ld-dashboard-course-listing.php';
                        }
                    }
                    wp_reset_postdata();
                } else {
                    ?>
                        <p class="ld-dashboard-warning"><?php 
                        /* translators: %s: Courses label. */
                        printf( esc_html__( 'No %s available at the moment.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ); ?></p>
                    <?php
                }
                ?>
            <?php do_action( 'ld_dashboard_after_courses_content' ); ?>
            </div>
        </div>
        <?php
        if ( ! empty( $courses_query->posts ) && count( $courses_query->posts ) > 0 && $courses_query->max_num_pages > 1 ) :
            ?>
            <nav class="custom-learndash-pagination-nav">
                <ul class="custom-learndash-pagination course-pagination-wrapper">
                    <?php if ( $courses_query->query_vars['paged'] > 1 ) : ?>
                    <li class="custom-learndash-pagination-first"><a href="<?php echo esc_url( $dashboard_page_url . '/?tab=my-courses' ); ?>" title="<?php esc_html_e( 'First', 'ld-dashboard' ); ?>">&#8606;</a></li> 
                    <?php endif; ?>
                    <li class="custom-learndash-pagination-prev"><?php previous_posts_link( '&larr;', $courses_query->max_num_pages ); ?></li>
                    <li class="custom-learndash-pagination-pagedisplay">
                        <span>
                            <?php esc_html_e( 'Page', 'ld-dashboard' ); ?>
                            <span class="pagedisplay">
                                <span class="current_page"><?php echo esc_html( $courses_query->query_vars['paged'] ); ?></span> / 
                                <span class="total_pages"><?php echo esc_html( $courses_query->max_num_pages ); ?></span>
                                (<span class="total_items"><?php echo esc_html( $courses_query->found_posts ); ?></span>)
                            </span>
                        </span>
                    </li>
                    <li class="custom-learndash-pagination-next"><?php next_posts_link( '&rarr;', $courses_query->max_num_pages ); ?></li>
                    <?php if ( $courses_query->query_vars['paged'] != $courses_query->max_num_pages ) : ?>
                    <li class="custom-learndash-pagination-last"><a href="<?php echo esc_url( $dashboard_page_url . '/page/' . $courses_query->max_num_pages . '/?tab=my-courses' ); ?>" title="<?php esc_html_e( 'Last', 'ld-dashboard' ); ?>">&#8608;</a></li>
                    <?php endif; ?> 
                </ul>
            </nav>
        <?php endif; ?>
    </div>
</div>
