<div class="zoom-meeting-list-wrap">
	<?php
	try {
		$meeting_status = '';
		$param          = '';
		$show_edit_btn  = true;

		if ( isset( $_GET['filter'] ) && 'prev' === $_GET['filter'] ) {
			$meeting_status = 'prev-meeting';
			$param          = 'previous_meetings';
			$show_edit_btn  = false;
		} else {
			$meeting_status = 'upcoming-meeting';
			$param          = 'upcoming_meetings';
		}

		// Initialize the list of meetings
		$all_meetings = array();

		// Query 1: Meetings authored by the user
		$authored_meetings = get_posts(
			array(
				'post_type'   => 'zoom_meet',
				'post_status' => 'publish',
				'fields'      => 'ids',
				'author'      => get_current_user_id(),
			)
		);

		// Query 2: Meetings where the user is a co-host
		$cohost_meetings = get_posts(
			array(
				'post_type'   => 'zoom_meet',
				'post_status' => 'publish',
				'fields'      => 'ids',
				'meta_query'  => array(
					array(
						'key'     => 'co_host_id',
						'value'   => get_current_user_id(),
						'compare' => '=',
					),
				),
			)
		);

		// Combine authored and co-hosted meetings
		$all_meetings = array_unique( array_merge( $authored_meetings, $cohost_meetings ) );

		// Admin logic: Add meetings with 'using_admin_credentials' set to 'yes'
		if ( in_array( 'administrator', $current_user->roles ) ) {
			$admin_meetings = get_posts(
				array(
					'post_type'   => 'zoom_meet',
					'post_status' => 'publish',
					'fields'      => 'ids',
					'meta_query'  => array(
						array(
							'key'     => 'using_admin_credentials',
							'value'   => 'yes',
							'compare' => '=',
						),
					),
				)
			);

			$all_meetings = array_unique( array_merge( $all_meetings, $admin_meetings ) );
		}

		// Fetch meetings from Zoom API
		$response = $zoom_meeting->get_all_meetings( '?page_size=10&page_number=1&type=' . $param );

		$no_meetings_found = false;

		if ( ! empty( $response ) && property_exists( $response, 'meetings' ) ) {
			$meetings = $response->meetings;

			if ( is_array( $meetings ) ) {
				$iterated = array();
				foreach ( $meetings as $meeting ) {
					$args = array(
						'post_type'  => 'zoom_meet',
						'meta_query' => array(
							array(
								'key'     => 'zoom_meeting_id',
								'value'   => $meeting->id,
								'compare' => '=',
							),
						),
					);

					$meeting_post    = get_posts( $args );
					$meeting_post_id = 0;

					if ( ! empty( $meeting_post ) ) {
						$meeting_post_id = $meeting_post[0]->ID;
					}

					// Ensure meeting visibility for current user
					if ( 0 === $meeting_post_id || ! in_array( $meeting_post_id, $all_meetings ) ) {
						continue;
					}

					// Meeting display variables
					$start_url      = get_post_meta( $meeting_post_id, 'zoom_meeting_start_url', true );
					$start_text     = esc_html__( 'Start Meeting', 'ld-dashboard' );
					$iterated[]     = $meeting_post_id;
					$has_recordings = get_post_meta( $meeting_post_id, 'ldd_meeting_has_recordings', true );
					$edit_url       = add_query_arg(
						array(
							'tab'        => 'meetings',
							'action'     => 'edit-meeting',
							'meeting-id' => $meeting_post_id,
						),
						Ld_Dashboard_Functions::instance()->ld_dashboard_get_url( 'dashboard' )
					);

					$delete_url = add_query_arg(
						array(
							'tab'        => 'meetings',
							'action'     => 'delete-meeting',
							'meeting-id' => $meeting_post_id,
							'_ldnonce'   => wp_create_nonce( 'delete-meeting-' . $meeting_post_id ),
						),
						Ld_Dashboard_Functions::instance()->ld_dashboard_get_url( 'dashboard' )
					);
					?>
					<div class="zoom-meeting-inner-list">
						<div class="zoom-meeting-left-inner">
							<h4>
								<?php echo esc_html( $meeting->topic ); ?>
								<span class="meeting-author">
									<?php
									$author_id = get_post_field( 'post_author', $meeting_post_id );
									echo esc_html__( 'by ', 'ld-dashboard' ) . get_the_author_meta( 'display_name', $author_id );
									?>
								</span>								
							</h4>
							<div class="zoom-meeting-meta">
								<span><?php echo esc_html__( 'ID:', 'ld-dashboard' ); ?> <?php echo esc_html( $meeting->id ); ?></span>
								<span>
									<?php echo esc_html__( 'Date:', 'ld-dashboard' ); ?> 
									<?php echo esc_html( ld_dashboard_date_converter( $meeting->start_time, $meeting->timezone, 'M d Y h:i A', true ) ); ?>
								</span>
								<span class="meeting-status <?php echo esc_attr( $meeting_status ); ?>"></span>
								<span class="using-admin-credentials" style="display: none;">
									<?php
									$using_admin_credentials = get_post_meta( $meeting_post_id, 'using_admin_credentials', true ) === 'yes' ? 'Yes' : 'No';
									echo esc_html__( 'Using Admin Credentials:', 'ld-dashboard' ) . ' ' . esc_html( $using_admin_credentials );
									?>
								</span>
							</div>
							<div class="ld-dashboard-zoom-meeting-link-wrapper">
								<?php if ( '' !== $meeting->join_url ) : ?>
									<div class="zoom-meeting-join-link-wrapper">
										<div class="ld-join-link-url" style="display:none;"><?php echo esc_attr( $meeting->join_url ); ?></div>
										<button class="ld-dashboard-copy-join-link"><?php echo esc_html__( 'Meeting Join Link', 'ld-dashboard' ); ?></button>
										<span class="ld-dashboard-copy-join-link-message"><?php echo esc_html__( 'Link has been copied to clipboard', 'ld-dashboard' ); ?></span>
									</div>
								<?php endif; ?>
								<?php if ( ld_dashboard_is_sdk_enabled() ) : ?>
									<?php $meeting_client_url = ld_dashboard_get_meeting_embed_url( $meeting->id ); ?>
									<div class="ld-dashboard-meeting-action browser-join">
										<a href="<?php echo esc_url( $meeting_client_url ); ?>"><?php echo esc_html__( 'Browser Join', 'ld-dashboard' ); ?> <span class="dashicons dashicons-arrow-right-alt"></span></a>
									</div>
								<?php endif; ?>
							</div>
							<?php if ( 'yes' === $has_recordings ) : ?>
								<div class="ld-dashboard-meeting-recordings-btn"><?php echo esc_html__( 'Recordings', 'ld-dashboard' ); ?></div>
							<?php endif; ?>
							<?php do_action( 'ld_dashboard_after_meeting_details', $meeting_post_id, $meeting->id ); ?>
						</div>
						<?php if ( $meeting_post_id > 0 ) : ?>
							<div class="edit-delete-meeting-zoom">
								<?php if ( $show_edit_btn ) : ?>
									<a href="<?php echo esc_url( $edit_url ); ?>" class="ld-create-meeting-action"><span class="ldd-icons ldd-icon-edit"></span></a>
								<?php endif; ?>
								<a href="<?php echo esc_url( $delete_url ); ?>" class="ld-delete-meeting-action"><span class="ldd-icons ldd-icon-delete-bin-line"></span></a>
								<a href="<?php echo esc_url( get_permalink( $meeting_post_id ) ); ?>" class="ld-view-meeting-action"><span class="ldd-icons ldd-icon-eye-line"></span></a>
							</div>
						<?php endif; ?>
						<?php if ( ld_dashboard_can_user_start_meeting( $meeting_post_id ) ) : ?>
							<div class="zoom-meeting-start-wrap">
								<a href="<?php echo esc_url( $start_url ); ?>" target="_blank"><span class="ldd-icons ldd-icon-play-line"></span></a>
							</div>
						<?php endif; ?>
					</div>
					<?php
				}

				if ( empty( $iterated ) ) {
					$no_meetings_found = true;
				}
			} else {
				$no_meetings_found = true;
			}
		} else {
			$no_meetings_found = true;
		}

		if ( $no_meetings_found ) {
			?>
			<div class="ld-dashboard-meeting-error-msg"><?php echo esc_html__( 'No meetings found.', 'ld-dashboard' ); ?></div>
			<?php
		}
	} catch ( Exception $ex ) {
		echo '<pre>';
		echo 'Error: ';
		print_r( $ex->getMessage() );
		echo '</pre>';
	}
	?>
</div>
