<?php

// Get the post ID based on query parameters
if ( isset( $_GET['ld-course'] ) ) {
    $post_id = sanitize_text_field( wp_unslash( $_GET['ld-course'] ) );
} elseif ( isset( $_GET['ld-lesson'] ) ) {
    $post_id = sanitize_text_field( wp_unslash( $_GET['ld-lesson'] ) );
} elseif ( isset( $_GET['ld-quiz'] ) ) {
    $post_id = sanitize_text_field( wp_unslash( $_GET['ld-quiz'] ) );
} elseif ( isset( $_GET['ld-topic'] ) ) {
    $post_id = sanitize_text_field( wp_unslash( $_GET['ld-topic'] ) );
} elseif ( isset( $_GET['ld-question'] ) ) {
    $post_id = sanitize_text_field( wp_unslash( $_GET['ld-question'] ) );
} elseif ( isset( $_GET['ld-certificate'] ) ) {
    $post_id = sanitize_text_field( wp_unslash( $_GET['ld-certificate'] ) );
} elseif ( isset( $_GET['ld-announcement'] ) ) {
    $post_id = sanitize_text_field( wp_unslash( $_GET['ld-announcement'] ) );
}

// Get post type object to retrieve the singular name
$post_type_object = ! empty( $post_id ) ? get_post_type_object( get_post_type( $post_id ) ) : null;
$post_type_singular_name = $post_type_object ? $post_type_object->labels->singular_name : esc_html__( 'Content', 'ld-dashboard' );

// Define status labels
$status_labels = array(
    'publish' => __( 'Published', 'ld-dashboard' ),
    'pending' => __( 'Pending', 'ld-dashboard' ),
    'draft'   => __( 'Draft', 'ld-dashboard' ),
    // Add more status mappings if needed
);

// Get the status of the post and map it to the corresponding label
$post_status = ! empty( $post_id ) ? get_post_status( $post_id ) : '';
$display_status = isset( $status_labels[ $post_status ] ) ? $status_labels[ $post_status ] : __( 'Unknown', 'ld-dashboard' );

?>

<div class="ld-dashboard-post-meta">
    <!-- Share course content for sidebar -->
    <div class="ld-dashboard-status-author-date-content">
        <h2 class="ld-dashboard-share-course-title"><?php echo esc_html( $post_type_singular_name . ' ' . __( 'Info', 'ld-dashboard' ) ); ?></h2>
        <div class="ld-dashboard-share-status">
            <?php if ( ! empty( $post_id ) && $post_status ) : ?>
                <div class="ld-dashboard-share-status-item">
                    <span id="ld-status-label" class="ld-dashboard-item-label"><?php esc_html_e( 'Status', 'ld-dashboard' ); ?></span>
                    <span id="ld-status-value"><?php echo esc_html( $display_status ); ?></span>
                </div>
                <div class="ld-dashboard-share-status-item">
                    <?php 
                        $author_id = get_post_field( 'post_author', $post_id );
                        $author    = get_the_author_meta( 'display_name', $author_id );
                    ?>
                    <span id="ld-author-label" class="ld-dashboard-item-label"><?php esc_html_e( 'Author', 'ld-dashboard' ); ?></span>
                    <span id="ld-author-value"><?php echo esc_html( $author ); ?></span>
                </div>
                <div class="ld-dashboard-share-status-item">
                    <?php $post_date = get_post_field( 'post_date', $post_id ); ?>
                    <span id="ld-publish-label" class="ld-dashboard-item-label"><?php esc_html_e( 'Publish', 'ld-dashboard' ); ?></span>
                    <span id="ld-publish-value"><?php echo date_i18n( get_option( 'date_format' ), strtotime( $post_date ) ); ?></span>
                </div>
            <?php else : ?>
                <div class="ld-dashboard-share-status-item">
                    <span id="ld-status-label" class="ld-dashboard-item-label"><?php esc_html_e( 'Status', 'ld-dashboard' ); ?></span>
                    <span id="ld-status-value"><?php esc_html_e( 'Status Unknown', 'ld-dashboard' ); ?></span>
                </div>
                <div class="ld-dashboard-share-status-item">
                    <span id="ld-author-label" class="ld-dashboard-item-label"><?php esc_html_e( 'Author', 'ld-dashboard' ); ?></span>
                    <span id="ld-author-value"><?php esc_html_e( 'A ghost writer', 'ld-dashboard' ); ?></span>
                </div>
                <div class="ld-dashboard-share-status-item">
                    <span id="ld-publish-label" class="ld-dashboard-item-label"><?php esc_html_e( 'Publish', 'ld-dashboard' ); ?></span>
                    <span id="ld-publish-value"><?php esc_html_e( 'In a parallel universe', 'ld-dashboard' ); ?></span>
                </div>
            <?php endif; ?>
        </div>
    </div>
</div>
