<?php
/**
 * LD Dashboard Reports Content
 *
 * This file is used to render the report section for LearnDash dashboard.
 *
 * @link       https://wbcomdesigns.com/
 * @since      5.9.9
 *
 * @package    Custom_Learndash
 * @subpackage Custom_Learndash/public/partials
 */

// Get the report instance and current user ID.
$essay_report = Ld_Dashboard_Reports::getInstance();
$user_id      = get_current_user_id();
$report_title = '';
$report_tab   = '';

// Determine the active report tab based on URL parameters.
if ( isset( $_GET['tab'] ) && ! empty( $_GET['tab'] ) ) {
    switch ( $_GET['tab'] ) {
        case 'essay-report':
            $report_title = __( 'Essay Report', 'ld-dashboard' );
            $report_tab   = 'essay-report';
            break;
        case 'assignment-report':
            $report_title = __( 'Assignment Report', 'ld-dashboard' );
            $report_tab   = 'assignment-report';
            break;
        case 'quiz-report':
            $report_title = __( 'Quiz Report', 'ld-dashboard' );
            $report_tab   = 'quiz-report';
            break;
    }
}
?>

<div class="wbcom-front-end-course-dashboard-my-courses-content">
    <div class="custom-learndash-list custom-learndash-my-courses-list">
        <div class="ld-dashboard-course-content instructor-courses-list">
            <!-- Report Section Title -->
            <div class="ld-dashboard-section-head-title">
                <h3><?php echo esc_html( $report_title ); ?></h3>
            </div>
            
            <div class="my-courses ld-dashboard-content-inner ld-dashboard-tab-content-wrapper">
                <?php do_action( 'ld_dashboard_before_report' ); ?>
                
                <!-- Check if the user has the correct roles to access the report -->
                <?php if ( array_intersect( wp_get_current_user()->roles, array_keys( ld_dashboard_get_dashboard_user_roles() ) ) ) : ?>
                    <?php $ldd_filter_drop_downs = $essay_report->ld_dasboard_report_dropdown( $user_id ); ?>

                    <div id="ld-dashboard-report" class="ld-dashboard-report ld-dashboard-reports ld-dashboard-report-container">
                        <div class="ld-dashboard-eassay-report-filter ld-dashboard-report-filters">
                            <div class="ld-dashboard-report-filters-section <?php echo esc_attr( $report_tab ); ?>">

                                <!-- Group Filter (Visible to Group Leaders) -->
                                <?php if ( isset( $ldd_filter_drop_downs['groups'] ) && learndash_is_group_leader_user( $user_id ) ) : ?>
                                    <div class="ld-row ld-select-filter">
                                        <div class="ld-select">
                                            <label><?php _e( 'Group', 'ld-dashboard' ); ?></label>
                                            <select class="ld-change-drop-down" id="ld-dashboard-report-group" name="groupId">
                                                <?php echo $ldd_filter_drop_downs['groups']; ?>
                                            </select>
                                        </div>
                                    </div>
                                <?php endif; ?>

                                <!-- Course Filter -->
                                <?php if ( isset( $ldd_filter_drop_downs['courses'] ) ) : ?>
                                    <div class="ld-row ld-select-filter">
                                        <div class="ld-select">
                                            <label><?php echo esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ); ?></label>
                                            <select class="ld-change-drop-down <?php echo isset( $ldd_filter_drop_downs['courses_class'] ) ? esc_attr( $ldd_filter_drop_downs['courses_class'] ) : ''; ?>" id="ld-dashboard-report-course" name="courseId">
                                                <?php echo $ldd_filter_drop_downs['courses']; ?>
                                            </select>
                                            <div id="ld-essay-report-nocourses" class="group-management-rest-message" style="display: none;">
                                                <?php printf( __( 'No %s available at the moment.', 'ld-dashboard' ), LearnDash_Custom_Label::get_label( 'courses' ) ); ?>
                                            </div>
                                        </div>
                                    </div>
                                <?php endif; ?>

                                <!-- Lesson Filter (Visible for Essay and Assignment Reports) -->
                                <?php if ( isset( $ldd_filter_drop_downs['lessons'] ) && in_array( $report_tab, [ 'assignment-report', 'essay-report' ] ) ) : ?>
                                    <div class="ld-row ld-select-filter">
                                        <div class="ld-select">
                                            <label><?php printf( _x( '%1$s / %2$s', 'LearnDash lesson and topic labels', 'ld-dashboard' ), LearnDash_Custom_Label::get_label( 'lesson' ), LearnDash_Custom_Label::get_label( 'topic' ) ); ?></label>
                                            <select class="ld-change-drop-down <?php echo isset( $ldd_filter_drop_downs['lessons_class'] ) ? esc_attr( $ldd_filter_drop_downs['lessons_class'] ) : ''; ?>" id="ld-dashboard-report-lesson" name="lessonId" disabled="disabled">
                                                <?php echo $ldd_filter_drop_downs['lessons']; ?>
                                            </select>
                                            <div id="ld-essay-report-nolessons" class="group-management-rest-message" style="display: none;">
                                                <?php printf( _x( 'No %s available at the moment.', 'No lessons found.', 'ld-dashboard' ), LearnDash_Custom_Label::get_label( 'lessons' ) ); ?>
                                            </div>
                                        </div>
                                    </div>
                                <?php endif; ?>

                                <!-- Quiz Filter (Visible for Quiz and Essay Reports) -->
                                <?php if ( isset( $ldd_filter_drop_downs['quizzes'] ) && in_array( $report_tab, [ 'quiz-report', 'essay-report' ] ) ) : ?>
                                    <div class="ld-row ld-select-filter">
                                        <div class="ld-select">
                                            <label><?php echo esc_html( LearnDash_Custom_Label::get_label( 'quizzes' ) ); ?></label>
                                            <select class="ld-change-drop-down <?php echo isset( $ldd_filter_drop_downs['quizzes_class'] ) ? esc_attr( $ldd_filter_drop_downs['quizzes_class'] ) : ''; ?>" id="ld-dashboard-report-quiz" name="quizId" disabled="disabled">
                                                <?php echo $ldd_filter_drop_downs['quizzes']; ?>
                                            </select>
                                            <div id="ld-essay-report-noquizzes" class="group-management-rest-message" style="display: none;">
                                                <?php printf( _x( 'No %s available at the moment.', 'No quizzes found', 'ld-dashboard' ), LearnDash_Custom_Label::get_label( 'quizzes' ) ); ?>
                                            </div>
                                        </div>
                                    </div>
                                <?php endif; ?>

                                <!-- Status Filter (Not visible for Quiz Report) -->
                                <?php if ( 'quiz-report' !== $report_tab ) : ?>
                                    <div class="ld-row ld-select-filter">
                                        <div class="ld-select">
                                            <label><?php _e( 'Status', 'ld-dashboard' ); ?></label>
                                            <select class="ld-change-drop-down" id="ld-dashboard-report-status" name="status">
                                                <option value="all"><?php _e( 'All', 'ld-dashboard' ); ?></option>
                                                <?php if ( 'assignment-report' === $report_tab ) : ?>
                                                    <option value="approved"><?php _e( 'Approved', 'ld-dashboard' ); ?></option>
                                                    <option value="not-approved"><?php _e( 'Not approved', 'ld-dashboard' ); ?></option>
                                                <?php elseif ( 'essay-report' === $report_tab ) : ?>
                                                    <option value="graded"><?php _e( 'Graded', 'ld-dashboard' ); ?></option>
                                                    <option value="ungraded"><?php _e( 'Ungraded', 'ld-dashboard' ); ?></option>
                                                <?php endif; ?>
                                            </select>
                                        </div>
                                    </div>
                                <?php endif; ?>
                            </div>

                            <!-- Report Table -->
                            <div class="ld-dashboard-essay-report-table-wrapper ld-dashboard-report-table">
                                <table id="ld-dashboard-report-table" class="ld-dashboard-table display ld-dashboard-datatable" cellspacing="0" width="100%" data-table="<?php echo esc_attr( $report_tab ); ?>"></table>
                            </div>
                        </div>
                    </div>

                <?php else : ?>
                    <!-- Display a message if the user does not have access to the report -->
                    <p><?php _e( 'You must be an admin, group leader, or instructor to access this page.', 'ld-dashboard' ); ?></p>
                <?php endif; ?>

                <?php do_action( 'ld_dashboard_after_report' ); ?>
            </div>
        </div>
    </div>
</div>
