<?php

// Get the current user ID and user data.
$user_id      = get_current_user_id();
$current_user = wp_get_current_user();

// Load necessary functions and settings.
$function_obj               = Ld_Dashboard_Functions::instance();
$ld_dashboard_settings_data = $function_obj->ld_dashboard_settings_data();
$settings                   = $ld_dashboard_settings_data['general_settings'];
$zoom_settings              = $ld_dashboard_settings_data['zoom_meeting_settings'];

// Check if the user is an instructor or administrator.
$is_instructor = in_array( 'ld_instructor', $current_user->roles ) || in_array( 'administrator', $current_user->roles );

// Get the URL for the dashboard page.
$dashboard_url = $function_obj->ld_dashboard_get_url( 'dashboard' );

// Determine the active tab based on the URL parameter. Default is 'profile'.
$active_tab = isset( $_GET['action'] ) ? sanitize_text_field( wp_unslash( $_GET['action'] ) ) : 'profile';
$active_tab = apply_filters('ld-dashboard-profile-setting-active-tab', $active_tab, $dashboard_url, $is_instructor);
?>

<div class="ld-dashboard-course-content instructor-courses-list ld-dashboard-profile-setting-view">
	<!-- Section Title -->
	<div class="ld-dashboard-section-head-title">
		<h3 class="ld-dashboard-nav-title"><?php esc_html_e( 'Settings', 'ld-dashboard' ); ?></h3>
	</div>

	<div class="ld-dashboard-content-inner">
		<!-- Tab Navigation Links -->
		<div class="ld-dashboard-inline-links">
			<ul>
				<li class="ld-tab-profile <?php echo ( 'profile' === $active_tab ) ? 'course-nav-active' : ''; ?>">
					<a href="<?php echo esc_url( $dashboard_url ) . '?tab=settings'; ?>">
						<?php esc_html_e( 'Profile', 'ld-dashboard' ); ?>
					</a>
				</li>
				<li class="ld-tab-reset <?php echo ( 'reset' === $active_tab ) ? 'course-nav-active' : ''; ?>">
					<a href="<?php echo esc_url( $dashboard_url ) . '?tab=settings&action=reset'; ?>">
						<?php esc_html_e( 'Reset Password', 'ld-dashboard' ); ?>
					</a>
				</li>
				<!-- Withdraw Tab: Only visible to instructors if commission is enabled -->
				<?php if ( $is_instructor && ld_dashboard_is_commission_enabled() ) : ?>
					<li class="ld-tab-withdraw <?php echo ( 'withdraw' === $active_tab ) ? 'course-nav-active' : ''; ?>">
						<a href="<?php echo esc_url( $dashboard_url ) . '?tab=settings&action=withdraw'; ?>">
							<?php esc_html_e( 'Withdraw', 'ld-dashboard' ); ?>
						</a>
					</li>
				<?php endif; ?>
				<!-- Zoom Tab: Only visible to instructors when Zoom is enabled and admin account is not used -->
				<?php if ( ld_dashboard_is_zoom_enabled() && $is_instructor && ! in_array( 'administrator', $current_user->roles ) && isset( $zoom_settings['use-admin-account'] ) && 0 === $zoom_settings['use-admin-account'] ) : ?>
					<li class="ld-tab-zoom <?php echo ( 'zoom' === $active_tab ) ? 'course-nav-active' : ''; ?>">
						<a href="<?php echo esc_url( $dashboard_url ) . '?tab=settings&action=zoom'; ?>">
							<?php esc_html_e( 'Zoom', 'ld-dashboard' ); ?>
						</a>
					</li>
				<?php endif; ?>
				<?php do_action('ld-dashboard-profile-setting-view', $dashboard_url, $active_tab, $is_instructor); ?>
			</ul>
		</div>

		<!-- Content based on the active tab -->
		<div class="ld-dashboard-settings-content-wrap">
			<?php
			// Include the relevant settings tab based on the active tab.
			switch ( $active_tab ) {
				case 'reset':
					if ( false !== Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-password-settings-tab.php' ) ) {
						include Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-password-settings-tab.php' );
					} else {
						include LD_DASHBOARD_PLUGIN_DIR . 'templates/ld-dashboard-password-settings-tab.php';
					}
					break;

				case 'withdraw':
					if ( $is_instructor && ld_dashboard_is_commission_enabled() ) {

						if ( false !== Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-withdraw-settings-tab.php' ) ) {
							include Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-withdraw-settings-tab.php' );
						} else {
							include LD_DASHBOARD_PLUGIN_DIR . 'templates/ld-dashboard-withdraw-settings-tab.php';
						}
					}
					break;

				case 'zoom':
					if ( $is_instructor ) {
						if ( false !== Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-zoom-settings-tab.php' ) ) {
							include Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-zoom-settings-tab.php' );
						} else {
							include LD_DASHBOARD_PLUGIN_DIR . 'templates/ld-dashboard-zoom-settings-tab.php';
						}
					}
					break;

				case 'profile':
					if ( false !== Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-profile-settings-tab.php' ) ) {
						include Ld_Dashboard_Public::template_override_exists( 'ld-dashboard-profile-settings-tab.php' );
					} else {
						include LD_DASHBOARD_PLUGIN_DIR . 'templates/ld-dashboard-profile-settings-tab.php';
					}
					break;
	
				default:
					do_action( 'ld_dashboard_custom_settings_tab', $active_tab, $is_instructor );
			}
			?>
		</div>
	</div>
</div>
