<?php
if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

// Fetch the instructor username from the query variable
$instructor_username = get_query_var('instructor_username');

// Validate the instructor username
if (empty($instructor_username)) {
    echo '<h1>' . esc_html__('Instructor not found.', 'ld-dashboard') . '</h1>';
    return;
}

// Get the instructor user object
$instructor = get_user_by('login', $instructor_username);

if (!$instructor) {
    echo '<h1>' . esc_html__('Instructor not found.', 'ld-dashboard') . '</h1>';
    return;
}

// Define instructor variables
$instructor_id = $instructor->ID;
$instructor_courses = Ld_Dashboard_Public::get_instructor_courses_list($instructor_id);
$bio                        = get_the_author_meta( 'description', $instructor_id );
$website                    = get_the_author_meta( 'user_url', $instructor_id );
$total_students             = array();
$total_courses = is_array($instructor_courses) ? count($instructor_courses) : 0;
$items_per_load = 20; // Number of courses to display per load
$paged_courses = array_slice($instructor_courses, 0, $items_per_load); // Initial set of courses

// Define the placeholder image path
$placeholder_image = LD_DASHBOARD_PLUGIN_URL . 'public/img/course-default.png';

// Initialize $total_students as an empty array to avoid warnings and errors.
$total_students = array();

// Calculate total students across all courses.
if ( ! empty( $instructor_courses ) && is_array( $instructor_courses ) ) {
    foreach ( $instructor_courses as $course ) {
        $course_students = ld_dashboard_get_course_students( $course->ID );
        if ( is_array( $course_students ) ) {
            $result         = array_diff( $course_students, $total_students );
            $total_students = array_merge( $total_students, $result );
        }
    }
}

// Safely calculate the number of students and courses.
$total_students_count = ! empty( $total_students ) ? count( $total_students ) : 0;
$total_courses_count  = ! empty( $instructor_courses ) ? count( $instructor_courses ) : 0;


?>

<div class="ld-dashboard-full-width-instructor-profile ld-dashboard-instructor-profile-wrap">
    <div class="ld-dashboard-instructor-profile-area">

        <!-- Instructor Profile Details -->
        <div class="ld-dashboard-single-instructor-toparea">
            <div class="ld-dashboard-instructor-details-wrap">
                <!-- Instructor Profile Picture -->
                <div class="ld-dashboard-instructor-profile-pic">
                    <img src="<?php echo esc_url(get_avatar_url($instructor_id)); ?>" alt="<?php echo esc_attr($instructor->display_name); ?>">
                </div>

                <!-- Instructor Details -->
                <div class="ld-dashboard-instructor-profile-details">
                    <h2><?php echo esc_html($instructor->display_name); ?></h2>

                    <div class="ld-dashboard-instructor-details-metrics">
                        <div class="instructor-metric">
                            <span class="metric-label"><?php esc_html_e('Courses', 'ld-dashboard'); ?></span>
                            <strong class="metric-value"><?php echo esc_html($total_courses); ?></strong>
                        </div>
                        <div class="instructor-metric">
                            <span class="metric-label"><?php esc_html_e('Students', 'ld-dashboard'); ?></span>
                            <strong class="metric-value"><?php echo esc_html(count($total_students)); ?></strong>
                        </div>
                    </div>

                    <!-- Instructor Website -->
                    <?php if (!empty($website)) : ?>
                        <a href="<?php echo esc_url($website); ?>" target="_blank" class="instructor-website-link">
                            <?php esc_html_e('Visit Website', 'ld-dashboard'); ?>
                        </a>
                    <?php endif; ?>
                </div>
            </div>
        </div>

        <!-- Instructor Biography -->
        <?php if (!empty($bio)) : ?>
            <div class="ld-dashboard-container-instructor-biography-wrap">
                <h3><?php esc_html_e('About Me', 'ld-dashboard'); ?></h3>
                <p><?php echo wp_kses_post($bio); ?></p>
            </div>
        <?php endif; ?>

        <!-- Instructor Courses -->
        <div class="ld-dashboard-instructor-course-container">
            <h3><?php esc_html_e('Courses', 'ld-dashboard'); ?></h3>

            <div id="instructor-course-grid" class="instructor-course-listing-grid">
                <?php if (!empty($paged_courses) && is_array($paged_courses)) : ?>
                    <?php foreach ($paged_courses as $course) : ?>
                        <?php if ($course instanceof WP_Post) : ?>
                            <div class="instructor-course-item">
                                <div class="instructor-course-thumbnail">
                                    <?php
                                    // Display course featured image or placeholder
                                    $featured_image = get_the_post_thumbnail($course->ID, [60, 60], ['class' => 'instructor-course-featured-image']);
                                    echo $featured_image ? $featured_image : '<img src="' . esc_url($placeholder_image) . '" alt="' . esc_attr__('Placeholder', 'ld-dashboard') . '" class="instructor-course-featured-image">';
                                    ?>
                                </div>
                                <div class="instructor-course-details">
                                    <a href="<?php echo esc_url(get_permalink($course->ID)); ?>" class="instructor-course-title">
                                        <?php echo esc_html($course->post_title); ?>
                                    </a>
                                    <a href="<?php echo esc_url(get_permalink($course->ID)); ?>" class="instructor-course-button">
                                        <?php esc_html_e('View Course', 'ld-dashboard'); ?>
                                    </a>
                                </div>
                            </div>
                        <?php endif; ?>
                    <?php endforeach; ?>
                <?php else : ?>
                    <p><?php esc_html_e('No courses found for this instructor.', 'ld-dashboard'); ?></p>
                <?php endif; ?>
            </div>

            <!-- Load More Button -->
            <?php if ($total_courses > $items_per_load) : ?>
                <div id="instructor-course-load-more-wrapper" class="instructor-course-load-more-wrapper">
                    <button id="instructor-course-load-more" class="instructor-course-load-more-button" data-offset="<?php echo $items_per_load; ?>" data-totalcourses="<?php echo $total_courses; ?>" data-totalload="<?php echo $items_per_load; ?>" data-instructor_id="<?php echo $instructor_id; ?>">
                        <?php esc_html_e('Load More', 'ld-dashboard'); ?>
                    </button>
                </div>
            <?php endif; ?>
        </div>
    </div>
</div>