<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

$user_id       = get_current_user_id();
$total_courses = learndash_user_get_enrolled_courses( $user_id, array(), true );
$loader        = includes_url( 'images/spinner-2x.gif' );
$loader        = apply_filters( 'ld_dashboard_loader_img_url', $loader );

$ld_dashboard_settings_data = Ld_Dashboard_Functions::instance()->ld_dashboard_settings_data();
$settings                   = $ld_dashboard_settings_data['tiles_options'];
$course_not_started_color    = isset( $settings['course_not_started_color'] ) ? $settings['course_not_started_color'] : '#FF7272';
$course_progress_color       = isset( $settings['course_progress_color'] ) ? $settings['course_progress_color'] : '#0EACF5';
$course_completed_color      = isset( $settings['course_completed_color'] ) ? $settings['course_completed_color'] : '#00BB30';

$page        =  1;
$total       = count( $total_courses ); // Total number of courses.
$limit       = apply_filters( 'ld_dashboard_student_course_progress_per_page', 20 );
$total_pages = ceil( $total / $limit ); // Calculate total pages.
$page        = max( $page, 1 ); // Ensure at least page 1 is shown.
$page        = min( $page, $total_pages ); // Prevent exceeding total pages.
$offset      = ( $page - 1 ) * $limit;
if ( $offset < 0 ) {
	$offset = 0;
}
$total_courses_lists = array_slice( $total_courses, $offset, $limit );
?>
<div class="ld-dashboard-course-quizzes-assignment-progress">
	<div class="ld-dashboard-courses">
		<select id="ld-dashboard-student-courses-id" class="ld-dashboard-select">
			<?php
			if ( ! empty( $total_courses ) ) :
				foreach ( $total_courses as $index => $course_id ) {
					?>
				<option value="<?php echo esc_attr( $course_id ); ?>"><?php echo esc_html( html_entity_decode( get_the_title( $course_id ) ) ); ?></option>
					<?php
				}
			else :
				/* translators: %s: Courses Label. */
				echo "<option value='' disabled selected>" . sprintf( esc_html__( 'No enrolled %s', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) ) . '</option>';
			endif;
			?>
		</select>
	</div>

	<div class="ld-dashboard-loader">
		<img src="<?php echo esc_url( $loader ); ?>">
		<p><?php echo apply_filters( 'ld_dashboard_waiting_text', __( 'Please wait while the details load...', 'ld-dashboard' ) );	// phpcs:ignore. ?></p>
	</div>
	<div class="ld-dashboard-student-course-report-container"></div>
</div>

<div class="ld-dashboard-course-progress">
	<h3 class="ld-dashboard-instructor-earning-title">
		<?php 
		/* translators: %s: Courses Label. */
		printf( esc_html__( '%s Progress', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?>			
	</h3>
	<div class="ld-dashboard-student-courses-progress-section">
		<ul class="ld-dashboard-student-courses-progress">
			<?php
			if ( ! empty( $total_courses_lists ) ) :
				foreach ( $total_courses_lists as $course_id ) :
					$course_progress_data = $this->ld_dashboard_check_course_progress_data( $user_id, $course_id );
					$course_progress      = isset( $course_progress_data['percentage'] ) ? $course_progress_data['percentage'] : 0;
					$total_steps          = isset( $course_progress_data['total_steps'] ) ? $course_progress_data['total_steps'] : 0;
					$completed_steps      = isset( $course_progress_data['completed_steps'] ) ? $course_progress_data['completed_steps'] : 0;

					$student_course_progress_color = $course_progress_color;
					if ( $course_progress == 100 ) {
						$student_course_progress_color = $course_completed_color;
					}
					?>
					<li>
						<strong>
							<a href="<?php echo esc_url( get_the_permalink( $course_id ) ); ?>"><?php echo esc_html( html_entity_decode( get_the_title( $course_id ) ) ); ?></a>&nbsp;
							<span class="ld-dashboard-progress-percentage">
							<?php
							echo sprintf(
								/* translators: %1$s: Course Progress. */
								esc_html__( '%1$s%% Complete', 'ld-dashboard' ),
								esc_html( $course_progress )
							);
							?>
							</span>
							<span class="ld-dashboard-progress-steps">
							<?php
							echo sprintf(
								/* translators: %1$s: Completed steps; %2$s: Total steps. */
								esc_html__( '%1$s/%2$s Steps', 'ld-dashboard' ),
								esc_html( $completed_steps ),
								esc_html( $total_steps )
							);
							?>
							</span>
						</strong>
						<div class="ld-dashboard-progress progress_bar_wrap" data-course="<?php echo esc_attr( $course_id ); ?>">
							<div class="ld-dashboard-progressbar ld-dashboard-animate ld-dashboard-stretch-right" data-percentage-value="<?php echo esc_attr( $course_progress ); ?>" style="background-color:<?php echo esc_attr( $student_course_progress_color ); ?>; width:0;"></div>
						</div>
					</li>
					<?php
				endforeach;
			else :
				?>
				<p><?php 
				/* translators: %s: Courses Label. */
				printf( esc_html__( 'You are not enrolled in any %s.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'course' ) ) ); ?></p>
			<?php endif; ?>
		</ul>
		
		<?php
		$page_container = '';
		if ( $total_pages != 0 ) {			
			$loader          = includes_url( 'images/spinner-2x.gif' );
			$page_container  = '<div class="ld-dashboard-student-course-progress-loader" style="display:none;">
				<img src="' . apply_filters( 'ld_dashboard_loader_img_url', $loader ) . '">
				<p>' . apply_filters( 'ld_dashboard_waiting_text', __( 'Please wait while details are loading...', 'ld-dashboard' ) ) . '</p>
			</div>';
			$page_container .= '<div class="ld-student-course-progress ld-dashboard-pagination">';
			if ( $page != 1 ) {
				$page_container .= sprintf( '<a class="ld-pagination" href="#"  data-page="%d" data-student="%d"> ' . esc_html__( '&#171; prev', 'ld-dashboard' ) . '</a>', $page - 1, $user_id );
			}
			$page_container .= ' <span>' . esc_html__( 'Page', 'ld-dashboard' ) . ' <strong>' . $page . '</strong> ' . esc_html__( 'of', 'ld-dashboard' ) . ' ' . $total_pages . '</span> ';
			if ( $page != $total_pages ) {
				$page_container .= sprintf( '<a class="ld-pagination" href="#"  data-page="%d" data-student="%d">' . esc_html__( 'next &#187;', 'ld-dashboard' ) . '</a>', $page + 1, $user_id );
			}
			$page_container .= '</div>';
		}
		echo $page_container; 	
		?>
	</div>
</div>

<?php do_action( 'ld_dashboard_show_course_to_do_list', $total_courses ); ?>
