<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

global $students, $course_ids, $wpdb;
$courseIDs = $course_ids ? $course_ids : array(); // Set course IDs or initialize empty array.

$curr_user_id = get_current_user_id();
$curr_user    = wp_get_current_user();
$group_ids    = array();

if ( learndash_is_group_leader_user() && ! in_array( 'ld_instructor', $curr_user->roles ) ) {
	$group_ids     = learndash_get_administrators_group_ids( $curr_user_id );
	$group_student = array();

	if ( ! empty( $group_ids ) ) {
		foreach ( $group_ids as $group_id ) {
			$group_user_ids = get_post_meta( $group_id, 'ld_dashboard_group_enrolled_users', true );

			$group_user_ids = empty( $group_user_ids ) ? learndash_get_groups_user_ids( $group_id ) : $group_user_ids;
			$group_student  = array_merge( $group_student, $group_user_ids );
		}
	}

	$group_student = ( is_array( $group_student ) && ! empty( $group_student ) ) ? array_unique( $group_student ) : array( 0 );

	$args = array(
		'orderby' => 'user_nicename',
		'order'   => 'ASC',
		'fields'  => array( 'ID', 'display_name', 'user_login' ),
		'include' => $group_student,
	);
} elseif ( learndash_is_admin_user() ) {
	add_action(
		'pre_user_query',
		function ( $query ) {
			global $wpdb;
			$query->query_from   .= " INNER JOIN {$wpdb->usermeta} as usermeta ON usermeta.user_id = {$wpdb->users}.ID";
			$query->query_where  .= " AND ( usermeta.meta_key LIKE 'course_%_access_from' OR usermeta.meta_key LIKE 'group_%_access_from') ";
			$query->query_orderby = " GROUP BY {$wpdb->users}.ID " . $query->query_orderby;
		}
	);

	$args = array(
		'orderby' => 'user_nicename',
		'order'   => 'ASC',
		'fields'  => array( 'ID', 'display_name', 'user_login' ),
	);
} elseif ( in_array( 'ld_instructor', $curr_user->roles ) ) {
	$stdnts  = array(); // Start with an empty array for student IDs.
	$courses = Ld_Dashboard_Public::get_instructor_courses_list();

	if ( is_array( $courses ) && ! empty( $courses ) ) {
		foreach ( $courses as $course ) {
			$course_students = get_post_meta( $course->ID, 'ld_dashboard_course_enrolled_users', true );

			// Only merge if course students are found
			if ( ! empty( $course_students ) ) {
				$stdnts = array_merge( $stdnts, $course_students );
			} else {
				// Fallback if no specific course students are found
				$course_pricing  = learndash_get_course_price( $course->ID );
				$course_user_ids = ( 'open' !== $course_pricing['type'] ) ? learndash_get_course_users_access_from_meta( $course->ID ) : array();

				$course_group_ids = learndash_get_course_groups( $course->ID );
				if ( is_array( $course_group_ids ) && ! empty( $course_group_ids ) ) {
					foreach ( $course_group_ids as $group_id ) {
						$group_users = learndash_get_groups_user_ids( $group_id );
						if ( ! empty( $group_users ) ) {
							$course_user_ids = array_merge( $course_user_ids, $group_users );
						}
					}
				}

				// Only add unique users from $course_user_ids to $stdnts
				if ( ! empty( $course_user_ids ) ) {
					foreach ( $course_user_ids as $std ) {
						if ( ! in_array( $std, $stdnts ) ) {
							$stdnts[] = $std;
						}
					}
				}
			}
		}

		// Remove any duplicate entries in $stdnts
		$stdnts = array_unique( $stdnts );
	}

	if ( learndash_is_group_leader_user() ) {
		$group_users = learndash_get_group_leader_groups_users();
		if ( is_array( $group_users ) && ! empty( $group_users ) ) {
			$stdnts = array_unique( array_merge( $stdnts, $group_users ) );
		}
	}

	// Set up the query arguments for get_users
	$args = array(
		'orderby' => 'user_nicename',
		'order'   => 'ASC',
		'fields'  => array( 'ID', 'display_name', 'user_login' ),
		'include' => ! empty( $stdnts ) ? $stdnts : array( 0 ),  // Only include if $stdnts has values
		'number'  => 100,
	);

	$group_ids = learndash_get_administrators_group_ids( $curr_user_id );
} else {
	$instructor_students = ld_dashboard_get_students( $curr_user_id );
	$course_student_ids  = ! empty( $instructor_students ) ? array_column( $instructor_students, 'ID' ) : array( 0 );

	$args = array(
		'meta_key' => 'course_%_access_from',
		'orderby'  => 'user_nicename',
		'order'    => 'ASC',
		'fields'   => array( 'ID', 'display_name', 'user_login' ),
		'include'  => $course_student_ids,
	);

	$sql_str       = $wpdb->prepare(
		"SELECT post_id, meta_key FROM {$wpdb->postmeta} as postmeta INNER JOIN {$wpdb->posts} as posts ON posts.ID = postmeta.post_id
		WHERE posts.post_type = %s AND posts.post_status = %s AND meta_key LIKE 'learndash_group_enrolled_%'",
		'sfwd-courses',
		'publish'
	);
	$group_courses = $wpdb->get_results( $sql_str );

	if ( ! empty( $group_courses ) ) {
		foreach ( $group_courses as $grp_course ) {
			$group_id = explode( 'learndash_group_enrolled_', $grp_course->meta_key );
			if ( in_array( $grp_course->post_id, $courseIDs ) && ! in_array( $group_id[1], $group_ids ) ) {
				$group_ids[] = $group_id[1];
			}
		}
	}
}

$args['number'] = 100;
$students       = get_users( $args );
$loader         = includes_url( 'images/spinner-2x.gif' );
?>
<div class="ld-dashboard-student-status">
	<div class="ld-dashboard-box-style ld-dashboard-pd20">
		<div class="ld-dashboard-seperator">
			<h3 class="ld-dashboard-instructor-earning-title"><?php esc_html_e( 'Student Details', 'ld-dashboard' ); ?></h3>
		</div>

		<?php do_action( 'ld_dashboard_student_status_before', $curr_user_id ); ?>
		<?php if ( ! empty( $students ) ) { ?>
			<div class="ld-dashboard-student-status-block">
				<div class="ld-dashboard-student-lists">
					<?php if ( ! empty( $group_ids ) ) : ?>
						<select name="ld-dashboard-groups" class="ld-dashboard-groups ld-dashboard-select" data-course-id="<?php echo esc_attr( join( ',', $courseIDs ) ); ?>">
							<option value="all"><?php esc_html_e( 'All', 'ld-dashboard' ); ?></option>
							<?php foreach ( $group_ids as $grp_id ) : ?>
								<option value="<?php echo esc_attr( $grp_id ); ?>" ><?php echo esc_html( get_the_title( $grp_id ) ); ?></option>
							<?php endforeach; ?>
						</select>
					<?php endif; ?>
					<select name="ld-dashboard-student" class="ld-dashboard-student ld-dashboard-student-select">
						<?php foreach ( $students as $student ) { ?>
							<option value="<?php echo esc_attr( $student->ID ); ?>" ><?php echo esc_html( $student->display_name ) . ' - ' . esc_html( $student->user_login ); ?></option>
						<?php } ?>
					</select>
				</div>
				<div class="ld-dashboard-student-loader">
					<img src="<?php echo apply_filters( 'ld_dashboard_loader_img_url', $loader ); ?>">
					<p><?php echo apply_filters( 'ld_dashboard_waiting_text', __( 'Please wait while details are loading...', 'ld-dashboard' ) ); ?></p>
				</div>
				<div class="ld-dashboard-student-details"></div>
			</div>
		<?php } else { ?>
			<p class="ld-dashboard-warning">
				<?php
				if ( in_array( 'ld_instructor', (array) $curr_user->roles ) ) {
					echo apply_filters(
						'ld_dashboard_student_status_no_student_instructor_message',
						sprintf(
							esc_html__( 'Your %s currently has no enrolled students. Share it to attract learners!', 'ld-dashboard' ),
							esc_html( strtolower( LearnDash_Custom_Label::get_label( 'course' ) ) )
						)
					);
				} else {
					echo apply_filters( 'ld_dashboard_student_status_no_student_admin_message', esc_html__( 'No students are currently registered. Encourage enrollment to start tracking progress.', 'ld-dashboard' ) );
				}
				?>
			</p>
		<?php } ?>
		<?php do_action( 'ld_dashboard_student_status_after', $curr_user_id ); ?>
	</div>
</div>
