<?php
if ( isset( $_GET['filter'] ) ) {
    $group_ids = learndash_get_administrators_group_ids( get_current_user_id(), true );
    $page_num  = get_query_var( 'paged' ) ? get_query_var( 'paged' ) : 1;

    $args = array(
        'post_type'      => 'groups',
        'posts_per_page' => apply_filters( 'ld_dashboard_user_groups_per_page', 10 ),
        'post_status'    => array( 'publish', 'pending', 'draft' ),
        'paged'          => $page_num,
        'post__in'       => ! empty( $group_ids ) ? $group_ids : array( 0 ),
    );
    $groups = new WP_Query( $args );
} else {
    $groups = ld_dashboard_get_user_groups( false, get_current_user_id() );
}

$dashboard_page_url = Ld_Dashboard_Functions::instance()->ld_dashboard_get_url( 'dashboard' );

if ( learndash_is_admin_user( get_current_user_id() ) || ( learndash_is_group_leader_user( get_current_user_id() ) && in_array( learndash_get_group_leader_manage_groups(), array( 'basic', 'advanced' ), true ) ) ) : ?>
    <div class="ld-dashboard-section-head-title">
        <div class="ld-dashboard-header-button ld-dashboard-add-new-button-container">
            <a class="ld-dashboard-add-course ld-dashboard-add-new-button ld-dashboard-btn-bg" href="<?php echo esc_url( $dashboard_page_url ); ?>?tab=groups&action=add&_lddnonce=<?php echo esc_attr( wp_create_nonce( 'ld_dashboard_add_group' ) ); ?>">
                <span class="ldd-icons ldd-icon-add-line edit_square"></span>
                <?php
                printf( esc_html__( 'Add A New %s', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'group' ) ) );
                ?>
            </a>
        </div>
    </div>
<?php endif; ?>

<div class="ld-dashboard-inline-links">
    <ul class="ld-dashboard-inline-links-ul">
        <li>
            <a href="<?php echo esc_url( $dashboard_page_url ) . '?tab=groups'; ?>" class="ld-dashboard-groups-tab-switch <?php echo ! isset( $_GET['filter'] ) ? 'ld-dashboard-groups-active-tab' : ''; ?>" data-tab="builder">
                <?php esc_html_e( 'Groups', 'ld-dashboard' ); ?>
            </a>
        </li>
        <li>
            <a href="<?php echo esc_url( $dashboard_page_url ) . '?tab=groups&filter=1'; ?>" class="ld-dashboard-groups-tab-switch <?php echo isset( $_GET['filter'] ) ? 'ld-dashboard-groups-active-tab' : ''; ?>" data-tab="setting">
                <?php esc_html_e( 'Group Administrations', 'ld-dashboard' ); ?>
            </a>
        </li>
    </ul>
</div>

<?php if ( $groups->have_posts() ) : ?>
    <div class="ld-dashboard-groups-list-wrapper">
        <?php
        while ( $groups->have_posts() ) :
            $groups->the_post();
            $group          = get_the_ID();
            $featured_image = has_post_thumbnail() ? get_the_post_thumbnail_url() : LD_DASHBOARD_PLUGIN_URL . 'public/img/course-default.png';
            ?>
            <div class="ld-dashboard-single-group-wrapper">
                <div class="ld-dashboard-single-group-attachment">
                    <img src="<?php echo esc_url( $featured_image ); ?>" alt="<?php echo esc_attr( get_the_title() ); ?>">
                </div>
                <div class="ld-dashboard-single-group-content-wrap">
                    <div class="ld-dashboard-single-group-title"><h4><?php echo esc_html( get_the_title() ); ?></h4></div>
                    <div class="ld-dashboard-group-count">
                        <span><?php esc_html_e( 'Courses:', 'ld-dashboard' ); ?> <?php echo ! empty( learndash_get_groups_courses_ids( get_current_user_id(), array( $group ) ) ) ? count( learndash_get_groups_courses_ids( get_current_user_id(), array( $group ) ) ) : 0; ?></span>
                        <span><?php esc_html_e( 'Students:', 'ld-dashboard' ); ?> <?php echo ! empty( learndash_get_groups_user_ids( $group ) ) ? count( learndash_get_groups_user_ids( $group ) ) : 0; ?></span>
                    </div>
                    <div class="ld-dashboard-single-group-actions">
                        <?php if ( learndash_is_admin_user( get_current_user_id() ) || ( learndash_is_group_leader_user( get_current_user_id() ) && in_array( learndash_get_group_leader_manage_groups(), array( 'basic', 'advanced' ), true ) ) ) : ?>
                            <span class="ld-dashboard-single-group-edit">
                                <a href="<?php echo esc_url( $dashboard_page_url ) . '?tab=groups&action=edit&ld-group=' . esc_attr( $group ); ?>"><?php esc_html_e( 'Edit', 'ld-dashboard' ); ?></a>
                            </span>
                        <?php endif; ?>
                        <?php if ( learndash_is_admin_user( get_current_user_id() ) || ( learndash_is_group_leader_user( get_current_user_id() ) && in_array( learndash_get_group_leader_manage_courses(), array( 'basic', 'advanced' ), true ) ) ) : ?>
                            <span class="ld-dashboard-single-group-view">
                                <a href="<?php echo esc_url( $dashboard_page_url ) . '?tab=groups&action=view&ld-group=' . esc_attr( $group ); ?>">
                                    <?php
                                        printf( esc_html__( 'View %s', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'courses' ) ) );
                                    ?>
                                </a>
                            </span>
                        <?php endif; ?>
                    </div>
                </div>
            </div>
        <?php endwhile; ?>
        <?php wp_reset_postdata(); ?>
    </div>

    <?php if ( $groups->max_num_pages > 1 ) : ?>
        <nav class="custom-learndash-pagination-nav">
            <ul class="custom-learndash-pagination course-pagination-wrapper">
                <?php if ( $groups->query_vars['paged'] > 1 ) : ?>
                    <li class="custom-learndash-pagination-first">
                        <a href="<?php echo esc_url( $dashboard_page_url . '?tab=groups' . ( isset( $_GET['filter'] ) ? '&filter=1' : '' ) ); ?>" title="<?php esc_html_e( 'First', 'ld-dashboard' ); ?>">&#8606;</a>
                    </li>
                <?php endif; ?>
                <li class="custom-learndash-pagination-prev"><?php previous_posts_link( '&larr;', $groups->max_num_pages ); ?></li>
                <li class="custom-learndash-pagination-pagedisplay">
                    <span>
                        <?php esc_html_e( 'Page', 'ld-dashboard' ); ?>
                        <span class="pagedisplay">
                            <span class="current_page"><?php echo esc_html( max( 1, $groups->query_vars['paged'] ) ); ?></span> / 
                            <span class="total_pages"><?php echo esc_html( $groups->max_num_pages ); ?></span>
                            (<span class="total_items"><?php echo esc_html( $groups->found_posts ); ?></span>)
                        </span>
                    </span>
                </li>
                <li class="custom-learndash-pagination-next"><?php next_posts_link( '&rarr;', $groups->max_num_pages ); ?></li>
                <?php if ( $groups->query_vars['paged'] != $groups->max_num_pages ) : ?>
                    <li class="custom-learndash-pagination-last">
                        <a href="<?php echo esc_url( $dashboard_page_url . 'page/' . $groups->max_num_pages . '/?tab=groups' . ( isset( $_GET['filter'] ) ? '&filter=1' : '' ) ); ?>" title="<?php esc_html_e( 'Last', 'ld-dashboard' ); ?>">&#8608;</a>
                    </li>
                <?php endif; ?>
            </ul>
        </nav>
    <?php endif; ?>

<?php else : ?>
    <div class="ld-dashboard-groups-error-msg">
        <p class="ld-dashboard-warning">
        <?php
        printf( esc_html__( 'No %s available at the moment.', 'ld-dashboard' ), esc_html( LearnDash_Custom_Label::get_label( 'groups' ) ) );
        ?>
        </p>
    </div>
<?php endif; ?>
