<?php
$dashboard_url              = Ld_Dashboard_Functions::instance()->ld_dashboard_get_url( 'dashboard' );
$ld_dashboard_settings_data = Ld_Dashboard_Functions::instance()->ld_dashboard_settings_data();
$zoom_settings              = $ld_dashboard_settings_data['zoom_meeting_settings'];
?>

<div class="ld-dashboard-content-inner ld-dashboard-zoom-setting-fields">
<?php if ( ! is_user_logged_in() ) : ?>
<p class="warning ld-dashboard-warning">
	<?php esc_html_e( 'You must be logged in to update your profile settings.', 'ld-dashboard' ); ?>
</p><!-- .warning -->
<?php elseif ( isset( $zoom_settings['use-admin-account'] ) && 1 === $zoom_settings['use-admin-account'] ) : ?>
	<p class="warning ld-dashboard-warning">
		<?php esc_html_e( 'Meetings are being scheduled using the admin\'s Zoom account.', 'ld-dashboard' ); ?>
	</p><!-- .warning -->
<?php else : ?>
	<?php do_action( 'ld_dashboard_before_zoom_setting_form' ); ?>
	<form method="post" class="ld-dashboard-profile-form" id="adduser" action="<?php echo esc_url( $dashboard_url ) . '?tab=settings&action=zoom'; ?>">
		<?php
		do_action( 'ld_dashboard_before_zoom_setting_fields' );

		$status_icon = '';
		if ( class_exists( 'Zoom_Api' ) && '' !== get_user_meta( get_current_user_id(), 'zoom_api_key', true ) ) {
			$zoom_meeting = new Zoom_Api();
			$response     = $zoom_meeting->get_all_meetings( '?page_size=2&page_number=1' );

			if ( ! empty( $response ) && property_exists( $response, 'meetings' ) ) {
				$status_icon = '<span class="dashicons dashicons-yes-alt ld-dashboard-zoom-api-status zoom-api-active"></span>';
			} else {
				$status_icon = '<span class="dashicons dashicons-dismiss ld-dashboard-zoom-api-status zoom-api-inactive"></span>';
			}
		}

		$zoom_fields = array(
			'meeting'     => array(
				'zoom_account_id' => array(
					'title' => esc_html__( 'Account ID (Server-to-Server OAuth)', 'ld-dashboard' ),
					'tag'   => 'input',
					'type'  => 'password',
					'name'  => 'zoom_account_id',
					'value' => get_user_meta( get_current_user_id(), 'zoom_account_id', true ),
					'class' => 'form-url',
					'icon'  => ( '' !== $status_icon ) ? $status_icon : '',
				),
				'zoom_api_key'    => array(
					'title' => esc_html__( 'Client ID (Server-to-Server OAuth)', 'ld-dashboard' ),
					'tag'   => 'input',
					'type'  => 'password',
					'name'  => 'zoom_api_key',
					'value' => get_user_meta( get_current_user_id(), 'zoom_api_key', true ),
					'class' => 'form-url',
					'icon'  => ( '' !== $status_icon ) ? $status_icon : '',
				),
				'zoom_api_secret' => array(
					'title' => esc_html__( 'Client Secret (Server-to-Server OAuth)', 'ld-dashboard' ),
					'tag'   => 'input',
					'type'  => 'password',
					'name'  => 'zoom_api_secret',
					'value' => get_user_meta( get_current_user_id(), 'zoom_api_secret', true ),
					'class' => 'form-url',
					'icon'  => ( '' !== $status_icon ) ? $status_icon : '',
				),
				'zoom_email'      => array(
					'title' => esc_html__( 'Zoom Account Email', 'ld-dashboard' ),
					'tag'   => 'input',
					'type'  => 'email',
					'name'  => 'zoom_email',
					'value' => get_user_meta( get_current_user_id(), 'zoom_email', true ),
					'class' => 'form-url',
					'icon'  => ( '' !== $status_icon ) ? $status_icon : '',
				),
			),
			'meeting_sdk' => array(
				'client_id'     => array(
					'title' => esc_html__( 'SDK Key / Client ID', 'ld-dashboard' ),
					'tag'   => 'input',
					'type'  => 'password',
					'name'  => 'client_id',
					'value' => get_user_meta( get_current_user_id(), 'client_id', true ),
					'class' => 'instrutor-client-id',
					'icon'  => ( '' !== $status_icon ) ? $status_icon : '',
				),
				'client_secret' => array(
					'title' => esc_html__( 'SDK / Client Secret', 'ld-dashboard' ),
					'tag'   => 'input',
					'type'  => 'password',
					'name'  => 'client_secret',
					'value' => get_user_meta( get_current_user_id(), 'client_secret', true ),
					'class' => 'instrutor-client-secret',
					'icon'  => ( '' !== $status_icon ) ? $status_icon : '',
				),
			),
		);

		$zoom_fields = apply_filters( 'ld_dashboard_zoom_fields', $zoom_fields );
		?>
			<div class="ld-dashboard-profile-form-field-list">
				<?php
				do_action( 'ld_dashboard_before_zoom_setting_fields' );
				foreach ( $zoom_fields['meeting'] as $slug => $field ) {
					?>
					<div class="ld-dashboard-profile-form-field <?php echo esc_attr( $field['class'] ); ?>">
						<label for="first-name"><?php echo esc_html( $field['title'] ); ?> <?php echo ( isset( $field['icon'] ) && '' !== $field['icon'] ) ? wp_kses_post( $field['icon'] ) : ''; ?></label>
						<div class="ld-dashboard-zoom-setting-fields"><?php ld_dashboard_get_field_html( $field ); ?></div>
					</div>
					<?php
				}
				do_action( 'ld_dashboard_after_zoom_setting_fields' );
				?>
			</div>
			<div class="ld-dashboard-profile-form-field-list">
				<?php
				do_action( 'ld_dashboard_before_zoom_sdk_setting_fields' );
				foreach ( $zoom_fields['meeting_sdk'] as $slug => $field ) {
					?>
					<div class="ld-dashboard-profile-form-field <?php echo esc_attr( $field['class'] ); ?>">
						<label for="first-name"><?php echo esc_html( $field['title'] ); ?> <?php echo ( isset( $field['icon'] ) && '' !== $field['icon'] ) ? wp_kses_post( $field['icon'] ) : ''; ?></label>
						<div class="ld-dashboard-zoom-setting-fields"><?php ld_dashboard_get_field_html( $field ); ?></div>
					</div>
					<?php
				}
				do_action( 'ld_dashboard_after_zoom_sdk_setting_fields' );
				?>
			</div>
			<div class="ld-dashboard-profile-form-field field-full-width">
				<input name="updateuser" type="submit" id="updateuser" class="submit button ld-dashboard-btn-bg" value="<?php esc_html_e( 'Save', 'ld-dashboard' ); ?>" />
				<?php wp_nonce_field( 'update-zoom-settings' ); ?>
				<input name="action" type="hidden" id="action" value="update-zoom-settings" />
			</div>
	</form>
	<?php do_action( 'ld_dashboard_after_zoom_setting_form' ); ?>
	<?php endif; ?>
</div>
