<?php
/**
 * Plugin Name:       LearnDash LMS - Group Registration
 * Plugin URI:        https://learndash.com/group-registration-by-learndash
 * Description:       Allows Group leaders to purchase a course (or courses) on behalf of students, and then enroll members to the course.
 * Version:           4.3.14.1
 * Author:            LearnDash
 * Author URI:        https://learndash.com
 * Text Domain:       wdm_ld_group
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

require_once plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';

use LearnDash\Seats_Plus\Activation;
use LearnDash\Seats_Plus\Licensing\Migration as License_Migration;

/**
 * Set Plugin Version
 */
define( 'LD_GROUP_REGISTRATION_VERSION', '4.3.14.1' );

/**
 * Set Default Plugin File Path Constant
 */
if ( ! defined( 'WDM_LDGR_PLUGIN_FILE' ) ) {
	define( 'WDM_LDGR_PLUGIN_FILE', __FILE__ );
}

/**
 * Set the plugin slug as default text domain.
 */
if ( ! defined( 'WDM_LDGR_TXT_DOMAIN' ) ) {
	define( 'WDM_LDGR_TXT_DOMAIN', 'wdm_ld_group' );
}

/**
 * Set Default Plugin Directory Path Constant
 */
if ( ! defined( 'WDM_LDGR_PLUGIN_DIR' ) ) {
	define( 'WDM_LDGR_PLUGIN_DIR', __DIR__ );
}

/**
 * Define LearnDash Licensing URL.
 *
 * @since 4.3.14
 */
if ( ! defined( 'WDM_LDGR_LICENSING_SITE_URL' ) ) {
	define( 'WDM_LDGR_LICENSING_SITE_URL', 'https://checkout.learndash.com/wp-json/learndash/v2/site/auth_token' );
}

/**
 * Define LearnDash Check Licensing URL.
 *
 * @since 4.3.14
 */
if ( ! defined( 'WDM_LDGR_LICENSING_CHECK_LICENSE_URL' ) ) {
	define( 'WDM_LDGR_LICENSING_CHECK_LICENSE_URL', 'https://checkout.learndash.com/wp-json/learndash/v2/site/auth' );
}

register_activation_hook( WDM_LDGR_PLUGIN_FILE, [ Activation::class, 'run' ] );

add_action( 'admin_notices', [ License_Migration::class, 'run' ] );

require plugin_dir_path( __FILE__ ) . 'includes/class-ld-group-registration.php';

/**
 * Begins execution of the plugin.
 */
function run_ld_group_registration() {

	$plugin = new \LdGroupRegistration\Includes\Ld_Group_Registration();
	$plugin->run();

}
run_ld_group_registration();
