<?php
/**
 * PHPUnit bootstrap file.
 *
 * @package Learndash_Edd
 */

$_tests_dir = getenv( 'WP_TESTS_DIR' );

if ( ! $_tests_dir ) {
	$_tests_dir = rtrim( sys_get_temp_dir(), '/\\' ) . '/wordpress-tests-lib';
}

// Forward custom PHPUnit Polyfills configuration to PHPUnit bootstrap file.
$_phpunit_polyfills_path = getenv( 'WP_TESTS_PHPUNIT_POLYFILLS_PATH' );
if ( false !== $_phpunit_polyfills_path ) {
	define( 'WP_TESTS_PHPUNIT_POLYFILLS_PATH', $_phpunit_polyfills_path );
}

if ( ! file_exists( "{$_tests_dir}/includes/functions.php" ) ) {
	echo "Could not find {$_tests_dir}/includes/functions.php, have you run bin/install-wp-tests.sh ?" . PHP_EOL; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
	exit( 1 );
}

// Set globals
$active_plugins = [
	'sfwd-lms/sfwd_lms.php',
	'easy-digital-downloads/easy-digital-downloads.php',
	'edd-recurring/edd-recurring.php',
	'edd-auto-register/edd-auto-register.php',
];

$GLOBALS['wp_tests_options'] = [
	'active_plugins' => $active_plugins
];

// Give access to tests_add_filter() function.
require_once "{$_tests_dir}/includes/functions.php";

/**
 * Manually load the plugin being tested.
 */
function _manually_load_plugin() {
	require dirname( dirname( __FILE__ ) ) . '/learndash-edd.php';

	// Include LearnDash plugin
	define( 'LEARNDASH_LMS_PLUGIN_DIR', dirname( dirname( dirname( __FILE__ ) ) ) . '/sfwd-lms/' );
	define( 'LEARNDASH_FILE', dirname( dirname( dirname( __FILE__ ) ) ) . '/sfwd-lms/sfwd_lms.php' );
	require dirname( dirname( dirname( __FILE__ ) ) ) . '/sfwd-lms/sfwd_lms.php';

	// Include EDD and its addons
	require dirname( dirname( dirname( __FILE__ ) ) ) . '/easy-digital-downloads/easy-digital-downloads.php';

	// Required to init other addons
	require_once dirname( dirname( dirname( __FILE__ ) ) ) . '/easy-digital-downloads/includes/admin/upgrades/upgrade-functions.php';
	require dirname( dirname( dirname( __FILE__ ) ) ) . '/edd-recurring/edd-recurring.php';
	require dirname( dirname( dirname( __FILE__ ) ) ) . '/edd-auto-register/edd-auto-register.php';
}

tests_add_filter( 'muplugins_loaded', '_manually_load_plugin' );

// Start up the WP testing environment.
require "{$_tests_dir}/includes/bootstrap.php";
