/******/ (() => { // webpackBootstrap
var __webpack_exports__ = {};
/*!********************************************************!*\
  !*** ./src/assets/js/admin/common/copy-text-button.js ***!
  \********************************************************/
const $ = jQuery,
  copyTextButton = {
    /**
     * Initializes Copy Text Buttons by setting up event handlers.
     *
     * @since 1.1.0
     *
     * @return {void}
     */
    init() {
      $('.sfwd_option_label .sfwd_help_text').on('click touch', this.fixButtonClickInHelpText);
      $('.learndash-notes-copy-text').on('click touch', this.copyToClipBoard);
      $('.learndash-notes-copy-text').on('mouseover focus', this.resetTooltip);
      $('.learndash-notes-copy-text').on('mouseleave', this.closeTooltip);
    },
    /**
     * Fixes clicking a Copy Text Button within Help Text.
     * Help Text is a <label> and our Copy Text Button is a <button>, and because the <label>
     * doesn't have a `for` attribute, it defaults to clicking the <button> when it is clicked. This prevents that.
     *
     * @since 1.1.0
     *
     * @param {Event} event [event description]
     *
     * @return {void}
     */
    fixButtonClickInHelpText(event) {
      if ($(this).find('.learndash-notes-copy-text').length <= 0) {
        return;
      }
      event.preventDefault();
    },
    /**
     * When a Copy Text button is clicked, put its content in the clipboard.
     * Additionally, set the Tooltip text to the defined "Success" text.
     *
     * @since 1.1.0
     *
     * @param {Event} event JS Event.
     *
     * @return {void}
     */
    copyToClipBoard(event) {
      event.preventDefault();
      const text = $(this).data('text');
      if (text.length <= 0) {
        return;
      }
      window.navigator.clipboard.writeText(text);
      const tooltipSuccess = $(this).data('tooltip-success');
      if (tooltipSuccess.length <= 0) {
        return;
      }

      // Writing to the attribute so that CSS can see the text.
      $(this).attr('data-tooltip', tooltipSuccess);
    },
    /**
     * When mousing over the Copy Text button, ensure the Tooltip text is reset to the default.
     * This is important for after it has already been clicked.
     *
     * @since 1.1.0
     *
     * @return {void}
     */
    resetTooltip() {
      const tooltipDefault = $(this).data('tooltip-default');
      if (tooltipDefault.length <= 0) {
        return;
      }

      // Writing to the attribute so that CSS can see the text.
      $(this).attr('data-tooltip', tooltipDefault);
    },
    /**
     * Ensures that Focus is removed from the element once the mouse leaves the button to prevent confusion.
     *
     * @since 1.1.0
     *
     * @return {void}
     */
    closeTooltip() {
      $(this).trigger('blur');
    }
  };
$(document).on('ready', function () {
  copyTextButton.init();
});
/******/ })()
;