<?php
/**
 * Adds a Description to the bottom of each Field.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Fields\Common;

use LearnDash\Core\Utilities\Cast;

/**
 * Class to add a Description to the bottom of each Field.
 *
 * @since 1.1.0
 */
class Description_After {
	/**
	 * Injects a Description at the bottom of the Field.
	 *
	 * @since 1.1.0
	 *
	 * @param string       $html       Field HTML.
	 * @param array<mixed> $field_args Field Args.
	 *
	 * @return string
	 */
	public function add( $html, array $field_args ) {
		// @phpstan-ignore-next-line -- This is always false on older Core versions and always true on newer ones, which phpstan doesn't like.
		if ( version_compare( LEARNDASH_VERSION, '4.15.2', '>=' ) ) {
			return $html;
		}

		// @phpstan-ignore-next-line -- On newer Core versions, this is expected to be unreachable.
		if (
			empty( $field_args['description_after'] )
			|| empty( $field_args['type'] )
		) {
			return $html;
		}

		$type = Cast::to_string( $field_args['type'] );

		$styles = [];
		switch ( $type ) {
			case 'radio':
				$styles['padding-left'] = '0';
				// Intentionally not breaking so Radio also inherits styles from Checkbox.
			case 'checkbox':
				$styles['margin-top'] = '0.9375rem';
				break;
			case 'checkbox-switch':
				$styles['margin-top'] = '0.1875rem';
				break;
		}

		$html .= sprintf(
			'<p class="ld-%s-description" style="%s">%s</p>',
			esc_attr( $type ),
			esc_attr(
				implode(
					';',
					array_map(
						function ( $value, $key ) {
							return "{$value}: {$key}";
						},
						array_keys( $styles ),
						$styles
					)
				)
			),
			wp_kses_post( Cast::to_string( $field_args['description_after'] ) )
		);

		return $html;
	}
}
