<?php
/**
 * Notes by LearnDash How To Page.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Sections;

use LearnDash\Core\App;
use LearnDash\Core\Utilities\Cast;
use LearnDash\Notes\Settings\Traits\Supports_Getting_Active_Sub_Page;

/**
 * Class Notes by LearnDash How To Page.
 *
 * @since 1.1.0
 */
class How_To extends Sub_Page_With_Sections {
	use Supports_Getting_Active_Sub_Page;

	/**
	 * Public constructor for class
	 *
	 * @since 1.1.0
	 */
	public function __construct() {
		$this->settings_page_id = Cast::to_string(
			App::getVar( 'learndash_notes_settings_page_id' )
		);

		// This is the 'option_name' key used in the wp_options table.
		$this->setting_option_key = 'learndash_notes_settings';

		// This is the HTML form field prefix used.
		$this->setting_field_prefix = '';

		// Used within the Settings API to uniquely identify this section.
		$this->settings_section_key = Cast::to_string(
			App::getVar( 'learndash_notes_how_to_settings_section_id' )
		);

		// Section label/header.
		$this->settings_section_label = esc_html__( 'How To', 'learndash_student_notes' );

		parent::__construct();
	}

	/**
	 * Hide the "Save Options" meta box.
	 *
	 * @since 1.1.0
	 *
	 * @param bool   $show             Hide/Show.
	 * @param string $settings_page_id The current Settings Page ID.
	 *
	 * @return bool
	 */
	public function show_save_options_box( $show, $settings_page_id ) {
		if ( $settings_page_id !== $this->settings_page_id ) {
			return $show;
		}

		$current_sub_page = $this->get_current_sub_page_id();

		if ( $current_sub_page !== $this->settings_section_key ) {
			return $show;
		}

		return false;
	}

	/**
	 * Action hook to handle current settings page layout columns.
	 *
	 * @since 1.1.0
	 *
	 * @param array<string, int> $columns   Number of columns to show per Screen ID.
	 * @param string             $screen_id Current Screen ID.
	 *
	 * @return array<string, int>
	 */
	public function set_section_columns( $columns = [], $screen_id = '' ) {
		if ( $screen_id !== $this->settings_screen_id ) {
			return $columns;
		}

		$current_sub_page = $this->get_current_sub_page_id();

		if ( $current_sub_page !== $this->settings_section_key ) {
			return $columns;
		}

		/**
		 * Add this filter to override the get user option logic. This is to force
		 * the screen layout option for users who don't have this defined.
		 *
		 * @since 1.1.0
		 */
		add_filter(
			"get_user_option_screen_layout_{$screen_id}",
			function ( $option_value = '', $option_key = '' ) {
				if ( "screen_layout_{$this->settings_screen_id}" === $option_key ) {
					$option_value = 1;
				}
				return $option_value;
			},
			1,
			2
		);

		$columns[ $screen_id ] = 1;

		return $columns;
	}
}
