<?php
/**
 * Notes by LearnDash How To Page.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Sections\How_To;

use LearnDash\Core\App;
use LearnDash\Core\Utilities\Cast;
use LearnDash\Notes\Settings\Sections\Sub_Page_Section;
use LearnDash\Notes\Settings\Traits\Supports_Escaping_Shortcodes;
use LearnDash\Notes\Templates\Admin_Template;

/**
 * Class LearnDash Settings Section for Learner Notes Metabox.
 *
 * @since 1.1.0
 */
class Common_Shortcodes extends Sub_Page_Section {
	use Supports_Escaping_Shortcodes;

	/**
	 * Public constructor for class.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function __construct() {
		$this->settings_page_id = Cast::to_string(
			App::getVar( 'learndash_notes_settings_page_id' )
		);

		// Used by the Sub_Page_Section Parent Class.
		$parent_settings_section_id = Cast::to_string(
			App::getVar( 'learndash_notes_how_to_settings_section_id' )
		);

		// This is the 'option_name' key used in the wp_options table.
		$this->setting_option_key = 'learndash_notes_settings';

		// This is the HTML form field prefix used.
		$this->setting_field_prefix = '';

		/**
		 * Used within the Settings API to uniquely identify this section.
		 *
		 * The number works as a "priority" as metaboxes are normally displayed alphabetically
		 * within their set Metabox priority.
		 */
		$this->settings_section_key = 'learndash_notes_how_to_20_common_shortcodes';

		// Section label/header.
		$this->settings_section_label = esc_html__( 'Common Shortcodes', 'learndash_student_notes' );

		parent::__construct( $parent_settings_section_id );
	}

	/**
	 * Initialize the metabox settings fields.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function load_settings_fields() {
		$template                    = new Admin_Template();
		$this->setting_option_fields = [];

		$html = $this->escape_shortcodes(
			Cast::to_string(
				$template->template( 'how-to/common-shortcodes/new-note', [], false )
			)
		);

		$this->setting_option_fields['common_shortcodes_new_note'] = [
			'name'      => 'common_shortcodes_new_note',
			'name_wrap' => false,
			'type'      => 'html',
			'label'     => esc_html__( 'New Note', 'learndash_student_notes' ),
			'value'     => $html,
		];

		$html = $this->escape_shortcodes(
			Cast::to_string(
				$template->template( 'how-to/common-shortcodes/list-of-notes', [], false )
			)
		);

		$this->setting_option_fields['common_shortcodes_list_of_notes'] = [
			'name'      => 'common_shortcodes_list_of_notes',
			'name_wrap' => false,
			'type'      => 'html',
			'label'     => esc_html__( 'List of Notes', 'learndash_student_notes' ),
			'value'     => $html,
		];

		/** This filter is documented in sfwd-lms/includes/settings/settings-metaboxes/class-ld-settings-metabox-course-access-settings.php */
		$this->setting_option_fields = apply_filters( 'learndash_settings_fields', $this->setting_option_fields, $this->settings_section_key );

		parent::load_settings_fields();
	}
}
