<?php
/**
 * Getting Started Sub Page Section.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Sections\How_To;

use LearnDash\Core\App;
use LearnDash\Core\Utilities\Cast;
use LearnDash\Notes\Settings\Sections\Sub_Page_Section;
use LearnDash\Notes\Settings\Traits\Supports_Escaping_Shortcodes;

/**
 * Class Getting Started Sub Page Section.
 *
 * @since 1.1.0
 */
class Getting_Started extends Sub_Page_Section {
	use Supports_Escaping_Shortcodes;

	/**
	 * Public constructor for class.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function __construct() {
		$this->settings_page_id = Cast::to_string(
			App::getVar( 'learndash_notes_settings_page_id' )
		);

		// Used by the Sub_Page_Section Parent Class.
		$parent_settings_section_id = Cast::to_string(
			App::getVar( 'learndash_notes_how_to_settings_section_id' )
		);

		// This is the 'option_name' key used in the wp_options table.
		$this->setting_option_key = 'learndash_notes_settings';

		// This is the HTML form field prefix used.
		$this->setting_field_prefix = '';

		/**
		 * Used within the Settings API to uniquely identify this section.
		 *
		 * The number works as a "priority" as metaboxes are normally displayed alphabetically
		 * within their set Metabox priority.
		 */
		$this->settings_section_key = 'learndash_notes_how_to_10_getting_started';

		// Section label/header.
		$this->settings_section_label = esc_html__( 'Getting Started With Notes by LearnDash', 'learndash_student_notes' );

		parent::__construct( $parent_settings_section_id );
	}

	/**
	 * Initialize the metabox settings fields.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function load_settings_fields() {
		$this->setting_option_fields = [];

		ob_start();
		include LEARNDASH_NOTES_DIR . 'src/admin-views/how-to/getting-started/content.php';
		$html = $this->escape_shortcodes(
			Cast::to_string( ob_get_clean() )
		);

		$this->setting_option_fields['getting_started'] = [
			'name'       => 'getting_started',
			'name_wrap'  => false,
			'type'       => 'html',
			'label'      => '', // This has to be defined for LearnDash_Settings_Section::settings_page_init().
			'label_none' => true,
			'value'      => $html,
		];

		/** This filter is documented in sfwd-lms/includes/settings/settings-metaboxes/class-ld-settings-metabox-course-access-settings.php */
		$this->setting_option_fields = apply_filters( 'learndash_settings_fields', $this->setting_option_fields, $this->settings_section_key );

		parent::load_settings_fields();
	}
}
