<?php
/**
 * Notes by LearnDash How To Page.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Sections\How_To;

use LearnDash_Settings_Section;
use LearnDash\Core\App;
use LearnDash\Core\Utilities\Cast;
use LearnDash\Notes\Settings\Sections\Sub_Page_Section;
use LearnDash\Notes\Settings\Traits\Supports_Escaping_Shortcodes;

/**
 * Class LearnDash Settings Section for Learner Notes Metabox.
 *
 * @since 1.1.0
 */
class More_Resources extends Sub_Page_Section {
	use Supports_Escaping_Shortcodes;

	/**
	 * Public constructor for class.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function __construct() {
		$this->settings_page_id = Cast::to_string(
			App::getVar( 'learndash_notes_settings_page_id' )
		);

		// Used by the Sub_Page_Section Parent Class.
		$parent_settings_section_id = Cast::to_string(
			App::getVar( 'learndash_notes_how_to_settings_section_id' )
		);

		// This is the 'option_name' key used in the wp_options table.
		$this->setting_option_key = 'learndash_notes_settings';

		// This is the HTML form field prefix used.
		$this->setting_field_prefix = '';

		/**
		 * Used within the Settings API to uniquely identify this section.
		 *
		 * The number works as a "priority" as metaboxes are normally displayed alphabetically
		 * within their set Metabox priority.
		 */
		$this->settings_section_key = 'learndash_notes_how_to_40_more_resources';

		// Section label/header.
		$this->settings_section_label = esc_html__( 'More Resources', 'learndash_student_notes' );

		parent::__construct( $parent_settings_section_id );
	}

	/**
	 * Initialize the metabox settings fields.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function load_settings_fields() {
		$this->setting_option_fields = [];

		ob_start();
		include LEARNDASH_NOTES_DIR . 'src/admin-views/how-to/more-resources/video-playlist.php';
		$html = $this->escape_shortcodes(
			Cast::to_string( ob_get_clean() )
		);

		$this->setting_option_fields['more_resources_video_playlist'] = [
			'name'      => 'more_resources_video_playlist',
			'name_wrap' => false,
			'type'      => 'html',
			'label'     => esc_html__( 'How To: Video Playlist', 'learndash_student_notes' ),
			'value'     => $html,
		];

		ob_start();
		include LEARNDASH_NOTES_DIR . 'src/admin-views/how-to/more-resources/knowledge-base.php';
		$html = $this->escape_shortcodes(
			Cast::to_string( ob_get_clean() )
		);

		$this->setting_option_fields['more_resources_knowledge_base'] = [
			'name'      => 'more_resources_knowledge_base',
			'name_wrap' => false,
			'type'      => 'html',
			'label'     => esc_html__( 'Knowledge Base', 'learndash_student_notes' ),
			'value'     => $html,
		];

		/** This filter is documented in sfwd-lms/includes/settings/settings-metaboxes/class-ld-settings-metabox-course-access-settings.php */
		$this->setting_option_fields = apply_filters( 'learndash_settings_fields', $this->setting_option_fields, $this->settings_section_key );

		parent::load_settings_fields();
	}
}
