<?php
/**
 * How To Page provider class file.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Sections\How_To;

use StellarWP\Learndash\lucatume\DI52\ServiceProvider;

/**
 * How To Page service provider class.
 *
 * @since 1.1.0
 */
class Provider extends ServiceProvider {
	/**
	 * Register service providers.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function register(): void {
		/**
		 * Ensure that the Instance is only created once.
		 *
		 * We do not use add_section_instance() and instead simply create the object as registering the Instance
		 * within the object will cause these Sections to show as Sub Pages, when we instead only want their Metaboxes.
		 */
		$this->container->singleton( Getting_Started::class );
		$this->container->singleton( Common_Shortcodes::class );
		$this->container->singleton( Notification_Features::class );
		$this->container->singleton( More_Resources::class );

		$this->hooks();
	}

	/**
	 * Hooks wrapper.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function hooks(): void {
		add_action(
			'learndash_settings_sections_init',
			function () {
				/**
				 * This is to force the constructor to be ran early enough
				 * for all the hooks within it to be ran at the appropriate time.
				 *
				 * We specifically do not call add_section_instance()
				 * as we're avoiding the default Settings Section logic which would make each of these a Sub Page.
				 */
				$this->container->get( Getting_Started::class );
			}
		);

		add_action(
			'learndash_settings_sections_init',
			function () {
				/**
				 * This is to force the constructor to be ran early enough
				 * for all the hooks within it to be ran at the appropriate time.
				 *
				 * We specifically do not call add_section_instance()
				 * as we're avoiding the default Settings Section logic which would make each of these a Sub Page.
				 */
				$this->container->get( Common_Shortcodes::class );
			}
		);

		add_action(
			'learndash_settings_sections_init',
			function () {
				/**
				 * This is to force the constructor to be ran early enough
				 * for all the hooks within it to be ran at the appropriate time.
				 *
				 * We specifically do not call add_section_instance()
				 * as we're avoiding the default Settings Section logic which would make each of these a Sub Page.
				 */
				$this->container->get( Notification_Features::class );
			}
		);

		add_action(
			'learndash_settings_sections_init',
			function () {
				/**
				 * This is to force the constructor to be ran early enough
				 * for all the hooks within it to be ran at the appropriate time.
				 *
				 * We specifically do not call add_section_instance()
				 * as we're avoiding the default Settings Section logic which would make each of these a Sub Page.
				 */
				$this->container->get( More_Resources::class );
			}
		);
	}
}
