<?php
/**
 * Settings Sections provider class file.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Sections;

use StellarWP\Learndash\lucatume\DI52\ServiceProvider;
use LearnDash_Settings_Section;

/**
 * Settings service provider class.
 *
 * @since 1.1.0
 */
class Provider extends ServiceProvider {
	/**
	 * Register service providers.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function register(): void {
		$this->container->setVar( 'learndash_notes_how_to_settings_section_id', 'learndash_notes_general' );

		// Ensure that the Instance is only created once, using Core's methods to create and return the instance.
		$this->container->singleton(
			How_To::class,
			function () {
				How_To::add_section_instance();

				return LearnDash_Settings_Section::get_section_instance( How_To::class );
			}
		);

		$this->container->register( How_To\Provider::class );

		$this->hooks();
	}

	/**
	 * Hooks wrapper.
	 *
	 * @since 1.1.0
	 *
	 * @return void
	 */
	public function hooks(): void {
		add_action(
			'learndash_settings_sections_init',
			$this->container->callback( How_To::class, 'add_section_instance' ),
			50 // "50" to help denote that it will be our 5th tab.
		);

		add_filter(
			'learndash_settings_show_section_submit',
			$this->container->callback( How_To::class, 'show_save_options_box' ),
			10,
			2
		);

		add_filter(
			'screen_layout_columns',
			$this->container->callback( How_To::class, 'set_section_columns' ),
			20, // Run after LearnDash Core's Filter.
			2
		);
	}
}
