<?php
/**
 * Section Class to load a Section within another Section as a Metabox.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Sections;

use LearnDash_Settings_Section;
use LearnDash\Notes\Settings\Traits\Supports_Getting_Active_Sub_Page;

/**
 * Section Class to load a Section within another Section as a Metabox.
 *
 * @since 1.1.0
 */
class Sub_Page_Section extends LearnDash_Settings_Section {
	use Supports_Getting_Active_Sub_Page;

	/**
	 * Parent Settings Section ID for this "Child Section".
	 *
	 * @since 1.1.0
	 *
	 * @var string
	 */
	private string $parent_settings_section_id;

	/**
	 * Public constructor for class.
	 *
	 * @since 1.1.0
	 *
	 * @param string $parent_settings_section_id Used to set the Parent Settings Section ID.
	 *
	 * @return void
	 */
	public function __construct( string $parent_settings_section_id = '' ) {
		$this->parent_settings_section_id = $parent_settings_section_id;

		add_filter(
			'learndash_show_section',
			[ $this, 'should_show' ],
			20, // Run after LearnDash Core's Filter.
			3
		);

		parent::__construct();
	}

	/**
	 * Control when this "Child Section" should be displayed.
	 * Be sure to hook this on the `learndash_show_section` Filter for your Section.
	 *
	 * @since 1.1.0
	 *
	 * @param bool   $show               Whether to show the Section on the current Page or not.
	 * @param string $section_key        Section ID/Key.
	 * @param string $settings_screen_id Page ID (as defined by WordPress).
	 *
	 * @return bool
	 */
	public function should_show( $show, $section_key, $settings_screen_id ) {
		if ( $settings_screen_id !== $this->settings_screen_id ) {
			return $show;
		}

		if ( $section_key !== $this->settings_section_key ) {
			return $show;
		}

		$current_sub_page = $this->get_current_sub_page_id();

		// Check that we're viewing the "Parent Section".
		if ( $current_sub_page !== $this->parent_settings_section_id ) {
			return $show;
		}

		return true;
	}
}
