<?php
/**
 * Trait for providing a method to escape Shortcodes to display as text.
 *
 * @since 1.1.0
 *
 * @package LearnDash\Notes
 */

namespace LearnDash\Notes\Settings\Traits;

use LearnDash\Core\Utilities\Cast;

/**
 * Trait for providing a method to escape Shortcodes to display as text.
 *
 * @since 1.1.0
 */
trait Supports_Escaping_Shortcodes {
	/**
	 * Escapes any registered shortcodes within the content.
	 *
	 * @since 1.1.0
	 *
	 * @param string $content HTML Content.
	 *
	 * @return string
	 */
	protected function escape_shortcodes( string $content ): string {
		global $shortcode_tags;

		preg_match_all( '@\[([^<>&/\[\]\x00-\x20=]++)@', $content, $matches );
		$tag_names = array_intersect( array_keys( $shortcode_tags ), $matches[1] );

		$pattern = get_shortcode_regex( $tag_names );

		$content = Cast::to_string(
			preg_replace( "/{$pattern}/", '[$0]', $content )
		);

		return $content;
	}
}
