<?php
/**
 * LearnDash Quiz Import/Export Add-on Exporter
 */

defined('ABSPATH') || exit;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;

if( !class_exists('WN_LD_Quiz_Import_Export_Exporter') ) {
	/**
	 * Class WN_LD_Quiz_Import_Export_Importer
	 */
	class WN_LD_Quiz_Import_Export_Exporter {

		public function ld_qie_export_cb() {
			if ( isset( $_GET['ld_qie_action'] ) && isset( $_GET['ld_qie_quiz_post_id'] ) && isset( $_GET['ld_qie_export_type'] ) ) {
				$action       = sanitize_text_field( $_GET['ld_qie_action'] );
				$quiz_post_id = absint( $_GET['ld_qie_quiz_post_id'] );
				$export_type  = sanitize_text_field( $_GET['ld_qie_export_type'] );

				if ( $action === 'export' && ! empty( $quiz_post_id ) && in_array( $export_type, [ 'xls', 'xlsx', 'json' ] ) ) {
					$this->export_quiz( $export_type, array( $quiz_post_id ) );
				}
			}
		}

		/**
		 * Add Export column
		 *
		 * @param $columns
		 *
		 * @return array
		 */
		public function quiz_post_columns( $columns ) {
			$new_columns = array(
				'export_quiz'  => __( 'Export Options', 'wn-ld-quiz-import-export' ),
				// 'export_as_xlsx' => __( 'Export as XLSX', 'wn-ld-quiz-import-export' )
			);

			return array_merge( $columns, $new_columns );
		}

		/**
		 * Add export button to quizes
		 *
		 * @param $column
		 * @param $post_id
		 */
		public function quiz_post_columns_data( $column, $post_id ) {
			switch ( $column ) {
				case "export_quiz":
					$this->get_export_button( $post_id, 'xls' );
					$this->get_export_button( $post_id, 'xlsx' );
					$this->get_export_button( $post_id, 'json' );
					break;
				// case "export_as_xlsx":
					// $this->get_export_button( $post_id, 'xlsx' );
					// break;
			}
		}

		/**
		 * Render Export Button
		 *
		 * @param $post_id
		 * @param $export_type
		 */
		private function get_export_button( $post_id, $export_type ) {
			$download_url = add_query_arg( array(
				'ld_qie_action'       => 'export',
				'ld_qie_quiz_post_id' => $post_id,
				'ld_qie_export_type'  => $export_type
			), admin_url() );
			echo sprintf( '<a href="%s" class="button button-primary">%s</a>', $download_url, strtoupper( $export_type ) );

		}

		/**
		 * Add a new action in bulk export
		 *
		 * @param $bulk_actions
		 *
		 * @return array
		 */
		public function ldmqie_register_export_bulk_actions( $bulk_actions ) {
			$bulk_actions['ldmqie_bulk_export_xls']  = __( 'Export as XLS', 'wn-ld-quiz-import-export' );
			$bulk_actions['ldmqie_bulk_export_xlsx'] = __( 'Export as XLSX', 'wn-ld-quiz-import-export' );
			$bulk_actions['ldmqie_bulk_export_json'] = __( 'Export as JSON', 'wn-ld-quiz-import-export' );

			return $bulk_actions;
		}


		/**
		 * Add a new action in bulk export
		 *
		 * @param $bulk_actions
		 *
		 * @return array
		 */
		public function ldmqie_export_bulk_action_handler( $redirect_to, $doaction, $post_ids ) {

			if ( $doaction !== 'ldmqie_bulk_export_xls' && $doaction !== 'ldmqie_bulk_export_xlsx' 
				&& $doaction !== 'ldmqie_bulk_export_json' ) {
				return $redirect_to;
			}

			if ( $doaction == 'ldmqie_bulk_export_xls' ) {
				$this->export_quiz( 'xls', $post_ids );
			}
			if ( $doaction == 'ldmqie_bulk_export_xlsx' ) {
				$this->export_quiz( 'xlsx', $post_ids );
			}
			if ( $doaction == 'ldmqie_bulk_export_json' ) {
				$this->export_quiz( 'json', $post_ids );
			}

			$redirect_to = add_query_arg( 'bulk_quizzes_export', count( $post_ids ), $redirect_to );

			return $redirect_to;
		}

		/**
		 * Quiz ID
		 *
		 * @param $post_id
		 *
		 * @return mixed
		 */
		public function get_quiz_id( $post_id ) {
			$quiz_meta = get_post_meta( $post_id, '_sfwd-quiz', true );

			return ( isset( $quiz_meta['sfwd-quiz_quiz_pro'] ) ? $quiz_meta['sfwd-quiz_quiz_pro'] : 0 );
		}

		/**
		 * Question Category
		 *
		 * @param $category_id
		 *
		 * @return array|null|object
		 */
		public function get_category( $category_id ) {
			global $wpdb;
			$category_method = "SELECT category_name FROM " . LDLMS_DB::get_table_name( 'quiz_category' ) . " WHERE category_id = %d";
			$catgory         = $wpdb->get_results( $wpdb->prepare( $category_method, $category_id ), ARRAY_A );
			if ( is_array( $catgory ) && count( $catgory ) > 0 ) {
				if ( isset( $catgory[0]["category_name"] ) && ! empty( $catgory[0]["category_name"] ) ) {
					return $catgory[0]["category_name"];
				}
			}

			return '';
		}

		/**
		 * Multi Question Category
		 *
		 * @param $question_id
		 *
		 * @return array|null|object
		 */
		public function get_multi_question_category( $question_id, $question_title ) {

			global $wpdb;
			$term_taxonomy 		= $wpdb->prefix . 'term_taxonomy';
			$term_relationship 	= $wpdb->prefix . 'term_relationships';
			
			// Find all terms that are part related to question id.	
			$term_data = $wpdb->get_results( 
				$wpdb->prepare( 
					"SELECT {$term_relationship}.*, {$term_taxonomy}.*
						FROM {$term_relationship}
						RIGHT JOIN {$term_taxonomy}
						ON {$term_relationship}.term_taxonomy_id = {$term_taxonomy}.term_taxonomy_id 
						WHERE object_id = %d;", $question_id 
					), 
				ARRAY_A 
			);
				
			$term_ids = array();
			foreach ( $term_data as $key => $value ) {
				$term_ids[] = $value['term_id'];
	        }        
			$term_names = WN_LD_Quiz_Import_Export_Helper::multi_question_category_format_term_ids( $term_ids, 'multi_question_category' );        

			// Return all terms that are related to question.
			return $term_names;
		}

		/**
		 * Categories
		 *
		 * @param $category_id
		 * @return array|null|object
		 */
		public function get_post_categories( $post_id, $taxonomy ) {
			global $wpdb;

			$term_taxonomy 		= $wpdb->prefix . 'term_taxonomy';
			$term_relationship 	= $wpdb->prefix . 'term_relationships';
			
			// Find all terms that are part related to question id.	
			$term_data = $wpdb->get_results( 
				$wpdb->prepare( 
					"SELECT {$term_relationship}.*, {$term_taxonomy}.*
						FROM {$term_relationship}
						RIGHT JOIN {$term_taxonomy}
						ON {$term_relationship}.term_taxonomy_id = {$term_taxonomy}.term_taxonomy_id 
						WHERE object_id = %d", $post_id 
					), 
				ARRAY_A 
			);

			$term_ids = array();
			foreach ( $term_data as $key => $value ) {
				$term_ids[] = $value['term_id'];
	        }

	        if ( count( $term_ids ) < 1 ) {
	        	return '';
	        }

			$term_names = WN_LD_Quiz_Import_Export_Helper::multi_question_category_format_term_ids( $term_ids, $taxonomy );
			return $term_names;
		}
	
		/**
		 * Credits: Mayra M
		 * https://stackoverflow.com/questions/3797239/insert-new-item-in-array-on-any-position-in-php
		 **/
		private function insertArrayAtPosition( $array, $insert, $position ) {
			return array_slice($array, 0, $position, TRUE) + $insert + array_slice($array, $position, NULL, TRUE);
		}

		private function get_answers($question) {

			$controller_question = new WpProQuiz_Controller_Question();

			$answers_data = $controller_question->setAnswerObject( $question );

			foreach ( $answers_data as $answer_type => &$answers ) {

				if ( $question->getAnswerType() === 'single' || $question->getAnswerType() === 'multiple' ) {
					$answer_type_required = 'classic_answer';
				} else {
					$answer_type_required = $question->getAnswerType();
				}

				if ( $answer_type != $answer_type_required ) {
					unset($answers_data[$answer_type]);
				}
			}

			if ( $question->getAnswerType() === 'laq_jumbled_sentence' ) {
				$answers           = get_post_meta( $question->getQuestionPostId(), '_ld_advanced_jumbled_options', true );

				if ( empty( $answers ) ) {
					$answers = [];
				}

				$new_answers = [];
				foreach ( $answers as $answer ) {
					$answer['vals']        = json_encode( $answer['vals'] );
					$new_answer['options'] = implode( '|', array(
						$answer['vals'],
						$answer['pts_for_each_usage']
					) );
					$new_answer['correct'] = $answer['correct'];
					$new_answer['points']  = absint($answer['points']);
					$new_answers[]         = $new_answer;
				}

				$answers_data[$question->getAnswerType()] = $new_answers;
			} else if ( $question->getAnswerType() === 'laq_swipe_questions' ) {
				$answers = get_post_meta( $question->getQuestionPostId(), '_ld_advanced_swipe_questions', true );

				if ( empty( $answers ) ) {
					$answers = [];
				}
				$new_answers = [];
				foreach ( $answers as $index => $answer ) {

					$new_answer['options'] = $answer['question'];
					$new_answer['correct'] = absint($answer['correct']);
					$new_answer['points']  = absint($answer['points']);
					$new_answers[]         = $new_answer;
				}

				$answers_data[$question->getAnswerType()] = $new_answers;
			} else if ( $question->getAnswerType() === 'laq_calculated_formula' ) {
				$answers = get_post_meta( $question->getQuestionPostId(), '_ld_advanced_calculated_formula_questions', true );

				if ( empty( $answers ) ) {
					$answers = [];
				}

				$new_answers             = $ranges = [];
				foreach ( $answers as $index => $answer ) {

					foreach ( $answer['min'] as $index2 => $answer_min ) {
						$ranges[ $index ][] = "{$answer['min'][$index2]},{$answer['max'][$index2]}";
					}

					$options               = implode( '|', array(
						$answer['formula'],
						implode( '|', $ranges[ $index ] )
					) );

					$new_answer['question'] = $answer['question'];
					$new_answer['options'] = $options;
					$new_answer['correct'] = null;
					$new_answer['points']  = absint($answer['points']);
					$new_answers[]         = $new_answer;
				}

				$answers_data[$question->getAnswerType()] = $new_answers;
			} else if ( $question->getAnswerType() === 'laq_jumble_dnd_sentence' ) {
				$answers           = get_post_meta( $question->getQuestionPostId(), '_ld_advanced_dnd_jumbled_options', true );
				if ( empty( $answers ) ) {
					$answers = [];
				}

				$new_answers = [];
				foreach ( $answers as $answer ) {
					$answer['code']        = json_encode( $answer['code'] );
					$answer['vals']        = json_encode( $answer['vals'] );
					$new_answer['options'] = implode( '|', array(
						$answer['code'],
						$answer['vals'],
						$answer['pts_for_each_usage']
					) );
					// $new_answer['correct'] = $answer['correct'];
					$new_answer['points']  = absint($answer['points']);
					$new_answers[]         = $new_answer;
				}

				$answers_data[$question->getAnswerType()] = $new_answers;
			}

			//TODO: Implement LD Advanced Questions export using this hook
			$answers_data = apply_filters('ld_qie_export_answers_data', $answers_data, $question);

			return $answers_data;
		}

		public function export_quiz($exp_type, $ids) {

			if ( empty( $exp_type ) || empty( $ids ) ) {
				return;
			}


			if( $exp_type == 'json') {
				$export   = new WN_LD_Quiz_Import_Export_Json_Exporter();
				$filename = 'ldqie_export_' . time() . '.json';
				$a = $export->export( $ids );

				header( 'Content-Type: application/json' );
				header( 'Content-Disposition: attachment; filename="' . $filename . '"' );
				echo $a; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
				exit;
			}

			$spreadsheet = new Spreadsheet();

			foreach ($ids as $quiz_index => $quiz_post_id) {
				$sheet_number = absint($quiz_index) + 1;
				$quiz_pro_id      = $this->get_quiz_id( $quiz_post_id );
				$quizMapper = new WpProQuiz_Model_QuizMapper();
				$quizM = $quizMapper->fetch( $quiz_pro_id );
				if ( $quiz_post_id !== absint( $quizM->getPostId() ) ) {
					$quizM->setPostId( $quiz_post_id );
				}

				$questionMapper = new WpProQuiz_Model_QuestionMapper();
				$questionModels = $questionMapper->fetchAll( $quizM );
				$question_count = count( $questionModels );

				$quiz_meta = get_post_meta( $quiz_post_id, '_sfwd-quiz', true );

				if ( !is_array( $quiz_meta ) ) {
					$quiz_meta = [];
				}

				if( !empty($quiz_meta) ) {
					$quiz_data = [
						'quiz_title' 	=> $quizM->getName(),
						'quiz_content' 	=> get_the_content( null, false, $quiz_post_id ),
						'quiz_category' => $this->get_post_categories( $quiz_post_id, 'category' ),
						'quiz_tags' => $this->get_post_categories( $quiz_post_id, 'post_tag' )
					];

					$quiz_data_column_map = array(
						'sfwd-quiz_quiz_materials' => 'materials',
						'sfwd-quiz_threshold' => 'certificate_awarded_for',
						'sfwd-quiz_passingpercentage' => 'passing_percentage',
						'sfwd-quiz_course' => 'course',
						'sfwd-quiz_lesson' => 'lesson_or_topic',
						'sfwd-quiz_certificate' => 'certificate',
					);

					foreach ($quiz_data_column_map as $quiz_meta_key => $quiz_column_name) {
						if( (isset($quiz_meta[ $quiz_meta_key ]) && !empty($quiz_meta[ $quiz_meta_key ])) || (isset($quiz_meta[ $quiz_meta_key ]) && $quiz_meta[ $quiz_meta_key ] === 0) ) {
							$quiz_data[ $quiz_column_name ] = $quiz_meta[$quiz_meta_key];
						} else {
							$quiz_data[ $quiz_column_name ] = null;
						}
					}

					/**
					 * Calculater Certificate Award Percentage
					 */
					if ( (isset( $quiz_data['certificate_awarded_for'] ) && ( is_float( $quiz_data['certificate_awarded_for'] ) || is_numeric( $quiz_data['certificate_awarded_for'] ) )) || $quiz_data['certificate_awarded_for'] === 0 ) {
						$certificate_percent          = $quiz_data['certificate_awarded_for'];
						$quiz_data['certificate_awarded_for'] = ( $certificate_percent * 100 );
					}

					$quiz_data = $this->elementor_support_exporter($quiz_post_id, $quiz_data);
				}

				if($question_count > 0 ) {

					$answer_counts = $output_questions = [];

					foreach ( $questionModels as $question ) {
						if ( ( ! empty( $question ) ) && ( $question instanceof WpProQuiz_Model_Question ) ) {

							if ( !class_exists('Learndash_Advanced_Quiz')
							     && in_array($question->getAnswerType(), [
									'laq_jumbled_sentence',
									'laq_swipe_questions',
									'laq_calculated_formula',
									'laq_jumbled_sentence',
									'laq_audio_questions'
								]) ) {
								continue; //bail here if LD Advanced Quiz is not available
							}

							$answers_data = $this->get_answers($question);
							foreach ($answers_data as $answer_type => $answers) {
								$answer_counts[] = count( $answers );
							}
						}
					}

					$max_answers_count = 0;

					if(!empty($answer_counts)) {
						$max_answers_count = max( $answer_counts );
					}

					$question_index = -1;
					foreach ( $questionModels as $question ) {

						if ( !class_exists('Learndash_Advanced_Quiz')
						     && in_array($question->getAnswerType(), [
								'laq_jumbled_sentence',
								'laq_swipe_questions',
								'laq_calculated_formula',
								'laq_jumble_dnd_sentence',
								'laq_audio_questions'
							]) ) {
							continue; //bail here if LD Advanced Quiz is not available
						}

						$question_index++;

						if ( ( ! empty( $question ) ) && ( $question instanceof WpProQuiz_Model_Question ) ) {
							$question_id = $question->getQuestionPostId();
							$question_data = $question->get_object_as_array();
							$question_data['_category_id'] = $question->getCategoryId();


							$answers_data = $this->get_answers($question);

							if($question->getAnswerType() === 'laq_jumbled_sentence') {
								$sub_question_text_array = [];
							}

							$output_answer = [];
							foreach ($answers_data as $answer_type => $answers) {

								$correct_answers = [];
								$answer_number = 0;
								foreach ($answers as $answer_index => $answer) {

									$answer_number = absint( $answer_index ) + 1;

									if($question->getAnswerType() === 'laq_jumbled_sentence') {

										$output_answer["answer_{$answer_number}"] = $answer['options'];
										$output_answer["point_{$answer_number}"]  = $answer['points'];
										$correct_answers[] = $answer['correct'];
									} elseif($question->getAnswerType() === 'laq_jumble_dnd_sentence') {

										$output_answer["answer_{$answer_number}"] = $answer['options'];
										$output_answer["point_{$answer_number}"]  = $answer['points'];
										// $correct_answers[] = $answer['correct'];
									} elseif($question->getAnswerType() === 'laq_swipe_questions') {

										$output_answer["answer_{$answer_number}"] = $answer['options'];
										$output_answer["point_{$answer_number}"]  = $answer['points'];
										if($answer['correct'] === 1) {
											$correct_answers[] = $answer_number;
										}
									} elseif($question->getAnswerType() === 'laq_calculated_formula') {
										$sub_question_text_array[] = $answer['question']; //Collect Questions to Sub Questions array
										$output_answer["answer_{$answer_number}"] = $answer['options'];
										$output_answer["point_{$answer_number}"]  = $answer['points'];
										$correct_answers[] = $answer['correct'];
									} else {

										//Set answer columns
										if ( in_array( $answer_type, [
											'classic_answer',
											'sort_answer'
										] ) ) {
											$output_answer["answer_{$answer_number}"] = $answer->getAnswer();
											$output_answer["point_{$answer_number}"]  = $answer->getPoints();
										} elseif( $answer_type === 'matrix_sort_answer' ) {
											$output_answer["answer_{$answer_number}"] = "{{$answer->getAnswer()}}{{$answer->getSortString()}}";
											$output_answer["point_{$answer_number}"]  = $answer->getPoints();
										} else {
											$output_answer["answer_{$answer_number}"] = null;
											$output_answer["point_{$answer_number}"]  = null;
										}

										//Set correct answer column
										if ( in_array( $answer_type, [
											'classic_answer',
										] ) ) {
											if ( $answer->isCorrect() ) {
												$correct_answers[] = $answer_number;
											}
										} elseif ( $answer_type === 'essay' ) {
											$correct_answers[] = $answer->getGradedType();
											$correct_answers[] = $answer->getGradingProgression();
										} else {
											if ( !in_array( $answer_type, [
												'sort_answer',
												'matrix_sort_answer'
											] ) ) { //Correct answer column can be blank for sorting questions
												$correct_answers[] = $answer->getAnswer();
											}
										}
									}
								}

								//Set missing answer columns
								for($i=$answer_number; $i < $max_answers_count; $i++) {
									$answer_number = $i + 1;
									$output_answer["answer_{$answer_number}"] = null;
									$output_answer["point_{$answer_number}"] = null;
								}

								$output_answer['answer'] = implode('|', $correct_answers);
							}

							// Output question's data and answers.
							$output_question = [

								'question'   							=> $question_data['_answerType'],
								'category'   							=> $this->get_category($question_data['_category_id']),
								'multi_question_category'   		    => $this->get_multi_question_category( $question->getQuestionPostId(), $question_data['_title'] ),
								'title'      							=> $question_data['_title'],
								'total_point'          					=> $question_data['_points'],
								'show_points_in_box'   					=> $question->isShowPointsInBox() ? 'yes' : 'no',
								'different_points_for_each_answer' 		=> $question->isAnswerPointsActivated() ? 'yes' : 'no',
								'answer_points_diff_modus_activated' 	=> $question->isAnswerPointsDiffModusActivated() ? 'yes' : 'no',
								'question_text'    						=> $question_data['_question'],
								'message_with_correct_answer' 			=> $question_data['_correctMsg'],
								'message_with_incorrect_answer' 		=> $question_data['_incorrectMsg'],
								'message_same_text' 					=> $question_data['_correctSameText'],
								'hint'          						=> $question_data['_tipMsg']
								//'answers'         => $processed_answers,
								//'correctSameText' => $question_data['_correctSameText'],
								//'tipEnabled'      => $question_data['_tipEnabled'],
							];

							if ( class_exists( 'Learndash_Advanced_Quiz' ) ) {
								$sub_question_text = null;
								//Get LD Advanced Quizzes Sub Question Texts
								if( $question->getAnswerType() === 'laq_jumbled_sentence' ) {
									$sub_question_text = get_post_meta( $question->getQuestionPostId(), '_ld_advanced_jumbled_question', true );
								} elseif( $question->getAnswerType() === 'laq_calculated_formula' ) {
									$sub_question_text = implode( '|', $sub_question_text_array );
								} elseif( $question->getAnswerType() === 'laq_jumble_dnd_sentence' ) {
									$sub_question_text = get_post_meta( $question->getQuestionPostId(), '_ld_advanced_dnd_jumbled_question', true );
								}

								$ld_aq_settings = WN_LD_Quiz_Import_Export_Helper::get_advanced_quizzes_settings(); //LD Advance Quizzes Add-on settings

								$advanced_quiz_settings = [];

								if ( $ld_aq_settings['negative_points_enabled'] ) {
									$negative_points                           = absint( get_post_meta( $question->getQuestionPostId(), '_ld_retake_negative_points', true ) );
									$advanced_quiz_settings['negative_points'] = $negative_points;
								}

								if ( $ld_aq_settings['question_time_limit_enabled'] ) {
									$time_limit                           = absint( get_post_meta( $question->getQuestionPostId(), '_ld_advanced_time_limit', true ) );
									$advanced_quiz_settings['time_limit'] = $time_limit;
								}

								$ld_advanced_quiz_columns['sub_question_text'] = $sub_question_text;

								$blank_option = get_post_meta( $question->getQuestionPostId(), '_ld_advanced_question_blank_option_value', true);
								$randomize_option = get_post_meta( $question->getQuestionPostId(), '_ld_advanced_question_random_option_value', true);
								$ld_advanced_quiz_columns['question_blank_option'] = $blank_option;
								$ld_advanced_quiz_columns['question_randomize_option'] = $randomize_option;

								if(!empty($advanced_quiz_settings)) {
									$ld_advanced_quiz_columns['additional_question_settings'] = json_encode($advanced_quiz_settings);
								}

								$output_answer = $ld_advanced_quiz_columns + $output_answer;
							}

							//TODO: Implement LD Advanced Quizzes output_answer using this hook instead
							$output_answer = apply_filters('ld_qie_output_answer', $output_answer, $question);
							$output_question = $this->insertArrayAtPosition($output_question, $output_answer, 6);

							if( $question_index > 0 ) {
								$quiz_data = array_fill_keys(array_keys($quiz_data), null);
							}

							$output_question = array_merge($output_question,$quiz_data);

							//TODO: This should be removed in future version, it is available for the sake old export column order
							$this->resortColumns($output_question);

							$output_questions[] = $output_question;
						}
					}

					if( !empty($output_question) ) {

						$headers = array_keys( $output_question );
						foreach ( $headers as &$header_column ) {
							$header_column = ucfirst( str_replace( '_', ' ', $header_column ) );
						}

						$output_questions = array_merge( array( $headers ), $output_questions );

						$sheet_name_prefix = apply_filters( 'ld_qie_sheet_name_prefix', 'Sheet', $quiz_data );

						$sheet_name = "{$sheet_name_prefix}{$sheet_number}";

						if ( $quiz_index > 0 ) {
							$worksheet = new Worksheet( $spreadsheet, $sheet_name );
							$spreadsheet->addSheet( $worksheet );
							/** set the current active worksheet by name **/
							$spreadsheet->setActiveSheetIndexByName( $sheet_name );
						} else {
							$spreadsheet->getActiveSheet()->setTitle( $sheet_name );
						}

						$spreadsheet->getActiveSheet()->getDefaultColumnDimension()->setWidth( 25 );
						$spreadsheet->getActiveSheet()->getDefaultRowDimension()->setRowHeight( 25 );
						$spreadsheet->getActiveSheet()->fromArray( $output_questions );
					}
				}
			}

			$current_time = current_time('timestamp');
			$file_time_prefix = date('Y-m-d h-i-s', $current_time);
			$filename     = apply_filters( 'ld_qie_export_file_name', "quiz-{$file_time_prefix}", $current_time );
			$filename    .= ".{$exp_type}";

			if ( trim( $exp_type ) === 'xlsx' ) {
				$content_type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';
			} else {
				$content_type = 'application/vnd.ms-excel';
			}

			//force user to download the Excel file without writing it to server's HD
			header( "Content-Type: {$content_type}" );
			header( sprintf( 'Content-Disposition: attachment; filename="%s"', $filename ) );
			header( "Cache-Control: max-age=0" );

			//save it to Excel5 format (excel 2003 .XLS file), change this to 'Excel2007' (and adjust the filename extension, also the header mime type)
			//if you want to save it as .XLSX Excel 2007 format

			if ( trim( $exp_type ) == 'xlsx' ) {
				$objWriter = new \PhpOffice\PhpSpreadsheet\Writer\Xlsx( $spreadsheet );
				// $objWriter->setOffice2003Compatibility( true );
			} else {
				$objWriter = new \PhpOffice\PhpSpreadsheet\Writer\Xls( $spreadsheet );
			}

			$objWriter->save( 'php://output' );
			exit;

		}



		private function elementor_support_exporter($quiz_post_id, $quiz_data){
			if (!is_array($quiz_data)) {
				$quiz_data = []; // Initialize only if $quiz_data is not already an array
			}

			$edit_mode = get_post_meta($quiz_post_id, '_elementor_edit_mode', true);
			if( $edit_mode === 'builder' ){
				$quiz_elementor_data_column_map = array(
					'_elementor_edit_mode'         => 'elementor_edit_mode',
					'_elementor_template_type'     => 'elementor_template_type',
					'_elementor_version'           => 'elementor_version',
					'_elementor_pro_version'       => 'elementor_pro_version',
					'_elementor_page_assets'       => 'elementor_page_assets',
					'_elementor_data'              => 'elementor_data',
					'_elementor_css'               => 'elementor_css',
				);
				foreach ($quiz_elementor_data_column_map as $quiz_elementor_meta_key => $quiz_elementor_column_name) {
					// Check if the meta key exists
					$quiz_elementor_meta_value = get_post_meta($quiz_post_id, $quiz_elementor_meta_key, true);

					if (is_array($quiz_elementor_meta_value)) {
						$quiz_elementor_meta_value = json_encode($quiz_elementor_meta_value);
					}

					if( $quiz_elementor_meta_value !== '' ) {
						// If the meta key is _elementor_data, unserialize the value
						$quiz_data[ $quiz_elementor_column_name ] = $quiz_elementor_meta_value;
					} else {
						$quiz_data[ $quiz_elementor_column_name ] = null;
					}
				}
			}
			
			return $quiz_data;
		}



		private function resortColumns(&$output_question) {
			$quiz_title 	= $output_question['quiz_title'];
			$quiz_content 	= $output_question['quiz_content'];
			$quiz_category 	= $output_question['quiz_category'];
			$quiz_tags 		= $output_question['quiz_tags'];
			
			unset( 
				$output_question['quiz_title'], 
				$output_question['quiz_content'], 
				$output_question['quiz_category'],
				$output_question['quiz_tags']
			);
			
			$output_question = array_merge(
				[
					'quiz_title' 	=> $quiz_title, 
					'quiz_content' 	=> $quiz_content, 
					'quiz_category' => $quiz_category,
					'quiz_tags' 	=> $quiz_tags
				], 
				$output_question
			);
		}
	}
}