<?php

/**
 * The plugin bootstrap file
 *
 * @link              https://wooninjas.com/
 * @since             1.0.0
 * @package           WN_LD_Quiz_Import_Export
 *
 * @wordpress-plugin
 * Plugin Name:       LearnDash Quiz Import Export
 * Plugin URI:        https://wooninjas.com/downloads/learndash-quiz-import-export/
 * Description:       Easily Import/Export LearnDash quizzes (with questions and answers) in Microsoft Excel (XLS/XLSX) file formats. LearnDash Advanced Quizzes questions are also supported.
 * Version:           3.8.4
 * Requires at least: 5.0
 * Tested up to:      6.7.1
 * Requires PHP:      7.2
 * Author:            WooNinjas
 * Author URI:        http://wooninjas.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       wn-ld-quiz-import-export
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Set plugin FILE to access it globally
 */
define( 'WN_LD_QUIZ_IMPORT_EXPORT_FILE', __FILE__ );
define( 'WN_LD_QUIZ_IMPORT_EXPORT_URL', plugin_dir_url(  __FILE__  ) );
define( 'WN_LD_QUIZ_IMPORT_EXPORT_PLUGIN_NAME', 'LearnDash Quiz Import Export' );

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'WN_LD_QUIZ_IMPORT_EXPORT_VERSION', '3.8.4' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-wn-ld-quiz-import-export-activator.php
 */
function activate_wn_ld_quiz_import_export() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wn-ld-quiz-import-export-activator.php';
	WN_LD_Quiz_Import_Export_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-wn-ld-quiz-import-export-deactivator.php
 */
function deactivate_wn_ld_quiz_import_export() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-wn-ld-quiz-import-export-deactivator.php';
	WN_LD_Quiz_Import_Export_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_wn_ld_quiz_import_export' );
register_deactivation_hook( __FILE__, 'deactivate_wn_ld_quiz_import_export' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-wn-ld-quiz-import-export.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_wn_ld_quiz_import_export() {

	$plugin = new WN_LD_Quiz_Import_Export();
	$plugin->run();

}

//run_wn_ld_quiz_import_export(); //Run your wn_ld_quiz_import_export without checking any plugin dependency

/////////// EXAMPLE CODE FOR PLUGIN DEPENDENCY CHECKS ///////////

function wn_ld_quiz_import_export_missing_plugin_notice() {
	unset($_GET['activate']);  //unset this to hide default Plugin activated. notice

	$plugin_data = get_plugin_data( __FILE__ );

	$class = 'notice notice-error is-dismissible';
	$message = sprintf( __( '<strong>%s</strong> requires <a href="https://www.learndash.com">LearnDash LMS</a> plugin to be activated.', 'wn-ld-quiz-import-export' ), $plugin_data['Name'] );
	printf ( "<div id='message' class='%s'> <p>%s</p></div>", $class, $message );
}

function wn_ld_quiz_import_export_plugins_loaded_cb() {

	if ( current_user_can( 'activate_plugins' ) && !class_exists( 'SFWD_LMS' ) ) {

		if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
			include_once( ABSPATH . '/wp-admin/includes/plugin.php' );
		}

		deactivate_plugins ( plugin_basename ( __FILE__ ), true );

		add_action( "admin_notices", 'wn_ld_quiz_import_export_missing_plugin_notice' );

	} else {
		run_wn_ld_quiz_import_export(); //Run your wn_ld_quiz_import_export after checking any plugin dependency
	}

}
add_action('plugins_loaded', 'wn_ld_quiz_import_export_plugins_loaded_cb');