<?php

namespace wdmmrQuiz;

/**
* "DeleteStatistics" is used to add deletion tab into Quiz submenu. A user having "wpProQuiz_change_settings" cap can access this page
* and delete the statistics. A user just needs to select a range and delete the data. This will delete all the
* statistics prior to selected date.
*
* */
class DeleteStatistics
{

    // Index number in admin menu array of LearnDash.
    protected $deleteTab = 92;

    public function __construct()
    {
        add_filter("learndash_admin_tabs", array($this, "deleteStatisticTab"));
        add_filter("learndash_admin_tabs_on_page", array($this, "deleteStatisticTabOnPage"), 10, 2);

        add_action('admin_menu', array($this, 'deleteMenu'), 1000);
    }

    /**
     * Adds submenu in the LearnDash
     * @return void()
     */
    public function deleteMenu()
    {
        add_submenu_page(
            'learndash-lms-non-existant',
            'Delete Statistics',
            'Delete Statistics',
            'wpProQuiz_change_settings',
            'ldAdvQuiz-statisticDeletion',
            array($this, 'learndashCourseShortcodesPage')
        );
    }


    /**
     * Adds required details to add tab in the quiz sub menu of LearnDash.
     * @return array Array of LearnDash admin tabs
     */
    public function deleteStatisticTab($admin_tabs)
    {
        $tab = array(
            'link'      => 'admin.php?page=ldAdvQuiz-statisticDeletion',
            'name'      => 'Delete Statistics',
            'id'        => 'admin_page_ldAdvQuiz_statisticDeletion',
            'cap'       => 'wpProQuiz_change_settings',
            'menu_link' => 'edit.php?post_type=sfwd-quiz'
            );

        $admin_tabs[$this->deleteTab] = $tab;

        return $admin_tabs;
    }

    /**
     * Shows our custom tab while a user is on other tabs of the same sub menu.
     * @param  array $admin_tabs_on_page Current submenu admin tabs
     * @param  array $admin_tabs         All the admin tabs
     * @return array                     Current submenu admin tabs
     */
    public function deleteStatisticTabOnPage($admin_tabs_on_page, $admin_tabs)
    {

        $admin_tabs_on_page['edit-sfwd-essays'][]                       = $this->deleteTab;
        $admin_tabs_on_page['sfwd-essays'][]                            = $this->deleteTab;
        $admin_tabs_on_page['edit-sfwd-quiz'][]                         = $this->deleteTab;
        $admin_tabs_on_page['sfwd-quiz'][]                              = $this->deleteTab;
        $admin_tabs_on_page['sfwd-quiz_edit'][]                         = $this->deleteTab;
        $admin_tabs_on_page['admin_page_ldAdvQuiz'][]                   = $this->deleteTab;
        $admin_tabs_on_page['admin_page_ldAdvQuiz_globalSettings'][]    = $this->deleteTab;
        $admin_tabs_on_page['admin_page_ldAdvQuiz_statisticDeletion']   = $admin_tabs_on_page['sfwd-quiz'];

        return $admin_tabs_on_page;
        unset($admin_tabs);
    }

    /**
     * Callback function to our custom tab "Statistic deletion".
     * @return string HTML of the delete statistics page.
     */
    public function learndashCourseShortcodesPage()
    {
        wp_enqueue_script('jquery-ui-datepicker');
        wp_enqueue_style('wdmmr-jquery-ui', WDMMR_PLUGIN_URL . 'assets/css/jquery-ui.css');

        wp_enqueue_script('wdmmr-delete-statistic', WDMMR_PLUGIN_URL . 'assets/js/delete-statistics.js');

        $empty_date_error   = '<p>Please select the date to delete the statistics</p>';
        $confirm_del_msg    = '<div>
		<center><p>Do you want to delete the statistics?</p>
		<input type="button" class="button-primary wdmmr-confirm-dialog" value="Yes" id="wdmmr_del_yes" style="background:rgb(0, 128, 22); border-color:rgba(0, 128, 0, 0.82);">
		<input type="button" class="button-primary wdmmr-confirm-dialog" value="No" id="wdmmr_del_no" style="background:rgba(207,0,0,1); border-color:rgb(175, 11, 11);">
		</center></div>';

        $data = array(
            'empty_error_msg' => $empty_date_error,
            'confirm_del_msg' => $confirm_del_msg
            );

        wp_localize_script('wdmmr-delete-statistic', 'delete_statistic_obj', $data);

        // To add popup in the page
        add_thickbox();
?>
<style type="text/css">
#TB_window { width: 30% !important; margin-left: -15% !important; }
#TB_ajaxContent { width: 90% !important; }
</style>
<div class="wrap">
<h2>Delete Statistics</h2>
<p>
<label for="wdmmr_del_cal">Date</label>:
<input type="text" id="wdmmr_del_cal" name="wdmmr_del_cal">
<a class="button-primary thickbox" id="wdmmr_del_btn" href="#TB_inline?width=200&height=150&inlineId=modal-wdmmr_del_btn">Delete</a>
</p>
<div class="updated" id="problemInfo">
<h3>Please note</h3>
<p>
Deletion note to the admin goes here!!!
</p>
</div>
</div>
<?php
    }
}

new DeleteStatistics();
