<?php

namespace quizReport;

/*
 * Class QuizReport
 *
 */
class QuizReport {

	/**
	 * Instance of this class.
	 *
	 * @since 1.0
	 *
	 * @var object
	 */
	protected static $instance = null;

	/**
	 * Return an instance of this class.
	 *
	 * @since     1.0.0
	 *
	 * @return object A single instance of this class.
	 */
	public static function getInstance() {
		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function __construct() {
		add_action( 'init', array( $this, 'createQuizReportingZip' ), 5 );
		add_action( 'init', array( $this, 'wdmqeExportData' ) );
		add_action( 'wp_ajax_wdm_export_statistics', array( $this, 'wdmqeExportData' ) );
		add_action( 'admin_footer', array( $this, 'wdmLinkExport' ) );
	}

	/**
	 * To get quiz data
	 *
	 * @since     1.0.0
	 **/
	public function wdmLinkExport() {
        
		// Your Code goes here
		if ( isset( $_GET['page'] ) && isset( $_GET['module'] ) && $_GET['page'] == 'ldAdvQuiz' && $_GET['module'] == 'statistics' && isset( $_GET['id'] ) && $_GET['id'] != '' ) {
			// if page is 'module' and 'statistics' of quiz.
			$quiz_id = trim( $_GET['id'] );

			if ( is_numeric( $quiz_id ) ) {
				// to get all stat ids to export all responses - starts
				global $wpdb;

				if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
					$stat_ref_tbl = \LDLMS_DB::get_table_name( 'quiz_statistic_ref' );
				} else {
					$stat_ref_tbl = $wpdb->prefix . 'wp_pro_quiz_statistic_ref';
				}

				$query = "SELECT statistic_ref_id FROM {$stat_ref_tbl} WHERE quiz_id = {$quiz_id} AND is_old=0 ORDER BY statistic_ref_id DESC; ";

				$result = $wpdb->get_results( $query, ARRAY_A );
				$all_ids = '';

				if ( is_array( $result ) ) {
					$result = array_filter( $result );
				}

				if ( ! empty( $result ) ) {
					foreach ( $result as $stat_id ) {
						$all_ids .= $stat_id['statistic_ref_id'] . ',';
					}
				}
			}

			$all_ids = substr( $all_ids, 0, -1 ); // removing last char
			// to get all stat ids to export all responses - ends

			$wdm_nonce = wp_create_nonce( 'wdm' . get_current_user_id() );

			$plugin_dir = plugin_dir_url( __FILE__ );

			$loader_link = $plugin_dir . 'ajax_preloader.gif'; // loader image link after clicking export link

			wp_register_script( 'wdm_export', plugins_url( '/assets/js/wdm-export.js', __FILE__ ), array( 'jquery' ), '4.2.3' );
			wp_enqueue_script( 'wdm_export' );

			wp_register_script( 'page_blocker', plugins_url( '/assets/js/page_blocker.min.js', __FILE__ ), array( 'jquery' ), '4.2.3' );
			wp_enqueue_script( 'page_blocker' );

			$wdm_js_array = array(
				'ajax_url'                      => admin_url( 'admin-ajax.php' ),
				'loader_link'                   => $loader_link,
				'all_ids'                       => $all_ids,
				'wdm_nonce'                     => $wdm_nonce,
				'wdm_msg_export_all_in_csv'     => __( 'Export All in CSV ', WDM_LD_QRE_TXT_DOMAIN ),
				'wdm_msg_export_all_in_excel'   => __( 'Export All in Excel ', WDM_LD_QRE_TXT_DOMAIN ),
				'wdm_msg_export'                => __( 'Export', WDM_LD_QRE_TXT_DOMAIN ),
				'wdm_msg_export_response'       => __( 'Export Response ', WDM_LD_QRE_TXT_DOMAIN ),
				'wdm_msg_excel'                 => __( ' Excel ', WDM_LD_QRE_TXT_DOMAIN ),
				'notice_text'                   => __( '<p> Dear User, </p><p> This is to inform you that WisdmLabs does not collect any user data. The data that is being exported is your sole responsibility and we urge you to update the privacy policy of your website. </p> <p> Regards, </p> <p> WisdmLabs </p>', WDM_LD_QRE_TXT_DOMAIN ),
			);

			wp_localize_script( 'wdm_export', 'wdm_export_obj', $wdm_js_array );
			wp_enqueue_style( 'wdmNoticeStyle', plugins_url( '/assets/css/wdmNoticeStyle.css', __FILE__ ) );
		}
	}
	/**
	 * checkIsset
	 * Checks variable and if isset true returns variable else ''
	 *
	 * @since     1.0.0
	 *
	 * @return variable or null
	 */
	public function checkIsset( $variable, $parameter ) {
		if ( isset( $variable[ $parameter ] ) ) {
			return $variable[ $parameter ];
		}

		return '';
	}
	/**
	 * wdmsetAutoSize
	 *
	 * Set Column width to auto
	 *
	 * @since     1.0.0
	 */
	public function wdmsetAutoSize( $objPHPExcel ) {
		// Set Column width to auto
		for ( $col = 'A'; $col != 'J'; $col ++ ) {
			$objPHPExcel->getActiveSheet()->getColumnDimension( $col )->setAutoSize( true );
		}
	}

	/**
	 * createXLS
	 *
	 * Creates .xls file of report
	 *
	 * @since     1.0.0
	 */
	public function createXLS( $wdm_export_data, $filename ) {
		// Contains  PHPExcel_IOFactory::createWriter
		include 'Classes/PHPExcel/IOFactory.php';
		// Contains PHPExcel Class
		include 'Classes/PHPExcel.php';
		// Contains PHPExcel_Style_Alignment Class
		include 'Classes/PHPExcel/Style/Alignment.php';

		$objPHPExcel = new \PHPExcel();

		$excel_data = array();
		// $style_data = array();

		$data               = json_decode( $wdm_export_data, true );
		$wdm_export_data    = null;

		// Counter for $data array loop
		$dnt = 0;

		foreach ( $data as $row_key => $row_val ) {
			$ActiveSheet = $objPHPExcel->getActiveSheet();

			// Counter for $row_val array loop
			$rnt = 0;

			foreach ( $row_val as $cell_key => $cell_val ) {
				$excel_data[ $dnt ][ $rnt ] = $cell_val['value'];

				// Setting height if multi lines present
				if ( strpos( $cell_val['value'], "\n" ) !== false ) {
					$count_lines         = count( explode( "\n", $cell_val['value'] ) ) + 1;
					$current_row_height  = $ActiveSheet->getRowDimension( $dnt + 1 )->getRowHeight();
					if ( ( 20 * $count_lines ) > $current_row_height ) {
						$ActiveSheet->getRowDimension( $dnt + 1 )->setRowHeight( 15 * $count_lines );
					}
					$current_row_height = null;
					$count_lines        = null;
				}

				// Adding formatting if required
				if ( isset( $cell_val['font'] ) ) {
					// $style_data[chr(65+$rnt).''.($dnt+1)] = array('font' => $cell_val['font']);
					$ActiveSheet->getStyle( chr( 65 + $rnt ) . '' . ( $dnt + 1 ) )->applyFromArray( array( 'font' => $cell_val['font'] ) );
				}

				$data[ $row_key ][ $cell_key ] = null;

				$rnt++;
			}

			// Here 8 because we have 7 columns in the file
			for ( $snt = $rnt; $snt < 8; $snt++ ) {
				$excel_data[ $dnt ][ $snt ] = '';
			}

			$dnt++;

			$data[ $row_key ] = null;
		}

		// Adding whole data to object
		$objPHPExcel->getActiveSheet()->fromArray( $excel_data, null, 'A1' );

		// Setting auth widht to all the columns
		$objPHPExcel->getActiveSheet()->getColumnDimension( 'A' )->setAutoSize( true );
		$objPHPExcel->getActiveSheet()->getColumnDimension( 'B' )->setAutoSize( true );
		$objPHPExcel->getActiveSheet()->getColumnDimension( 'C' )->setAutoSize( true );
		$objPHPExcel->getActiveSheet()->getColumnDimension( 'D' )->setAutoSize( true );
		$objPHPExcel->getActiveSheet()->getColumnDimension( 'E' )->setAutoSize( true );
		$objPHPExcel->getActiveSheet()->getColumnDimension( 'F' )->setAutoSize( true );
		$objPHPExcel->getActiveSheet()->getColumnDimension( 'G' )->setAutoSize( true );
		$objPHPExcel->getActiveSheet()->getColumnDimension( 'H' )->setAutoSize( true );

		$ActiveSheet = $objPHPExcel->getActiveSheet();

		ob_end_clean();
		// Object to wrie into the file and save in Php output stream
		$upload_dir = wp_upload_dir();
		$objWriter = \PHPExcel_IOFactory::createWriter( $objPHPExcel, 'Excel2007' );
		$file = '';
		if ( isset( $_POST['page'] ) ) {
			if ( ! file_exists( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] ) ) {
				mkdir( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] );
			}
			$file      = $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] . '/' . $filename . '_' . $_POST['page'] . '.xls';
		} else {
			$file = 'php://output';
		}
		$objWriter->save( $file );

		$objPHPExcel->__destruct();
		$objPHPExcel = null;
	}

	/**
	 * createCSVFile
	 * creates csv file
	 *
	 * @param $wdm_export_data: data to put in file
	 **/
	public function createCSVFile( $wdm_export_data, $filename ) {
		// Include library which we used to convert Html to Csv data
		include 'simple_html_dom.php';
		// Library's function to get Html from the data
		$html    = str_get_html( htmlspecialchars_decode( $wdm_export_data ) );

		// Opens a file in write mode
		$upload_dir = wp_upload_dir();
		$file = null;
		if ( isset( $_POST['page'] ) ) {
			if ( ! file_exists( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] ) ) {
				mkdir( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] );
			}
			$file      = fopen( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] . '/' . $filename . '_' . $_POST['page'] . '.csv', 'w' );
		} else {
			$file      = fopen( 'php://output', 'w' );
		}
		// Checks if file opened on php output stream
		if ( $file ) {
			// For each row of the table in Data recieved
			foreach ( $html->find( 'tr' ) as $element ) {
				// For Headings
				$th = array();
				foreach ( $element->find( 'th' ) as $row ) {
					$th [] = $row->plaintext;
				}
				// Inserts Heading into the csv file
				if ( ! empty( $th ) ) {
					fputcsv( $file, $th );
				}
				// For cell's value
				$td = array();
				// For each cell
				foreach ( $element->find( 'td' ) as $row ) {
					$td [] = $row->plaintext;
				}
				// Inserts Each cell's value and points to next row
				if ( ! empty( $td ) ) {
					fputcsv( $file, $td );
				}
			}
			// Closes the Csv file
			fclose( $file );
		} else {
			_e( 'File Permission Issue!!!', WDM_LD_QRE_TXT_DOMAIN );
		}
	}


	public function createQuizReportingZip() {
		if ( isset( $_POST['export_zip'] ) ) {
			$upload_dir = wp_upload_dir();
			$rootPath = \realpath( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] );

			// Initialize archive object
			$zip = new \ZipArchive();
			$zip_opened = $zip->open( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] . '.zip', \ZipArchive::CREATE | \ZipArchive::OVERWRITE );
			if ( $zip_opened === true ) {
				// Create recursive directory iterator
				/** @var SplFileInfo[] $files */
				$files = new \RecursiveIteratorIterator(
					new \RecursiveDirectoryIterator( $rootPath ),
					\RecursiveIteratorIterator::LEAVES_ONLY
				);

				foreach ( $files as $name => $file ) {
					// Skip directories (they would be added automatically)
					if ( ! $file->isDir() ) {
						// Get real and relative path for current file
						$filePath = $file->getRealPath();
						$relativePath = substr( $filePath, strlen( $rootPath ) + 1 );

						// Add current file to archive
						$zip->addFile( $filePath, $relativePath );
					}
				}

				// . phpcs fix
				unset( $name );

				// Zip archive will be created only after closing object
				$zip->close();
				header( 'Content-disposition: attachment; filename=QuizReporting_' . $_POST['wdm_format'] . '.zip' );
				header( 'Content-type: application/zip' );
				readfile( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] . '.zip' );
				unlink( $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'] . '.zip' );
				$dir = $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'];
				$dir = $upload_dir['basedir'] . '/QuizReporting_' . $_POST['wdm_format'];
				$iterator = new \RecursiveDirectoryIterator( $dir, \RecursiveDirectoryIterator::SKIP_DOTS );
				$files = new \RecursiveIteratorIterator(
					$iterator,
					\RecursiveIteratorIterator::CHILD_FIRST
				);
				foreach ( $files as $file ) {
					if ( $file->isDir() ) {
						rmdir( $file->getRealPath() );
					} else {
						unlink( $file->getRealPath() );
					}
				}
				rmdir( $dir );
				die();
			} else {
				die( 'Could not create a ZIP file. Error: ' . $zip_opened );
			}
		}
	}

	/**
	 * wdmqeExportData
	 * creates csv file
	 *
	 * @param $wdm_export_data: data to put in file
	 **/
	public function wdmqeExportData() {

		if ( isset( $_POST['wdm_format'] ) && ! isset( $_POST['export_zip'] ) ) {
			// ini_set('memory_limit', '-1');
			// ini_set('max_execution_time', '-1');
			// Don't cache the result.
			wp_suspend_cache_addition( true );

			// Get all data of statistics

			$data = self::wdmExport();
			if ( empty( $data ) ) {
				die( __( 'Something went wrong OR data not found!!!', WDM_LD_QRE_TXT_DOMAIN ) );
			}

			// Stores all the required values from the form

			// Data to be inserted in Csv or Excel

			$wdm_export_data = self::checkIsset( $data, 'table' );

			// User name
			$wdm_uname       = self::checkIsset( $data, 'name' );

			// Quiz Title
			$wdm_quiz        = self::checkIsset( $data, 'quiz_title' );

			// Format of Export
			$wdm_format      = self::checkIsset( $_REQUEST, 'wdm_format' );

			// Checks for the data to be exported
			if ( $wdm_export_data !== '' && $wdm_format !== '' ) {
				// If username and Quiz title is not empty then set file name using Username and Quiz title
				if ( ! empty( $wdm_uname ) && ! empty( $wdm_quiz ) ) {
					$file_name   = $wdm_quiz . '-' . $wdm_uname;
					$file_name   = str_replace( ' ', '_', $file_name );
					$file_name   = preg_replace( '/[^A-Za-z0-9\-]/', '', $file_name );
				} // Else set it to sample
				else {
					$file_name = 'sample';
				}

				header( 'Content-type: application/ms-excel' );
				header( 'Content-Disposition: attachment; filename=' . $file_name . '.' . $wdm_format );

				// Checks if the format is Csv
				if ( $wdm_format === 'csv' ) {
					self::createCSVFile( $wdm_export_data, $file_name );
				} //Create Excel sheet if format is xls
				else {
					self::createXLS( $wdm_export_data, $file_name );
				}
			} elseif ( isset( $_POST['page'] ) ) {
				die();
			} else {
				_e( 'Something went wrong OR data not found!!!', WDM_LD_QRE_TXT_DOMAIN );
			}

			exit;
		}
	}


	/**
	 * wdmArrCustomData
	 *
	 * @param   $[arr_custom_data] [array] [custom data related to quiz]
	 **/
	public function wdmArrCustomData( $arr_custom_data ) {
		foreach ( $arr_custom_data as $cust_key => $cust_val ) {
			foreach ( $cust_val as $wdm_cust_key => $wdm_cust_value ) {
				if ( ! is_array( $wdm_cust_value ) ) {
					continue;
				}
				// echo "<pre>";
				// print_r($wdm_cust_value);
				// echo "</pre>";
				$cust_question_type = $wdm_cust_value['answer_type'];

				switch ( $cust_question_type ) {
					case '0': // text
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'text';
						break;

					case '1': // textarea
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'textarea';
						break;

					case '2': // number
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'number';
						break;

					case '3': // checkbox
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'checkbox';
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['qsanswer_data'] = ( $wdm_cust_value['qsanswer_data'] != '' ) ? ( ( $wdm_cust_value['qsanswer_data'] == '1' ) ? __( 'Yes', WDM_LD_QRE_TXT_DOMAIN ) : __( 'No', WDM_LD_QRE_TXT_DOMAIN ) ) : '';
						break;

					case '4': // email
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'email';
						break;

					case '5': // yes/no
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'yes/no';
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['qsanswer_data'] = ( $wdm_cust_value['qsanswer_data'] != '' ) ? ( ( $wdm_cust_value['qsanswer_data'] == '1' ) ? 'Yes' : 'No' ) : '';
						break;

					case '6': // date
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'date';
						break;

					case '7': // dropdown menu
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'dropdown';

						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_data'] = json_decode( $wdm_cust_value['answer_data'] );

						break;

					case '8': // radio buttons
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_type'] = 'radio';
						$arr_custom_data[ $cust_key ][ $wdm_cust_key ]['answer_data'] = json_decode( $wdm_cust_value['answer_data'] );
						break;

					default:
						break;
				} // switch ( $cust_question_type)
			} // foreach ( $cust_val as $wdm_cust_key => $wdm_cust_value )
		}

		return $arr_custom_data;
	}
	/**
	 * caseFreeAnswer
	 * sets user response and correct answers of ouestion type free answers
	 *
	 * @since     1.0.0
	 *
	 * @return array case_free_answer
	 */
	public function caseFreeAnswer( $wdm_answer_data, $wdm_user_response ) {
		$arr_answers = array();
		$arr_correct_answers = array();
		$ans_obj = self::checkIssetVar( $wdm_answer_data[0] );
		$ans_obj_answer = self::getProtectedValue( $ans_obj, '_answer' );

		$ans_obj_answer = str_replace( array( "\r", "\n" ), ',', $ans_obj_answer );
		$arr_correct_answers = explode( ',', $ans_obj_answer );
		$arr_correct_answers = array_filter( $arr_correct_answers );
		$arr_correct_answers = array_values( $arr_correct_answers );

		// Because "free answer" type questions have correct answers as same as options
		$arr_answers = $arr_correct_answers;

		if ( $ans_obj !== '' ) {
			$arr_user_response = $wdm_user_response;
		}

		$case_free_answer = array(
			'arr_correct_answers' => $arr_correct_answers,
			'arr_answers' => $arr_answers,
			'arr_user_response' => $arr_user_response,
		);

		return $case_free_answer;
	}
	/**
	 * caseMultiple
	 * sets user response and correct answers of ouestion type multiple
	 *
	 * @since     1.0.0
	 *
	 * @return array case_multiple
	 */
	public function caseMultiple( $wdm_answer_data, $wdm_user_response ) {
		$ans_cnt = 0;

		$arr_user_response = array();
		$arr_answers = array();
		$arr_correct_answers = array();
		foreach ( $wdm_answer_data as $ans_obj ) {
			$ans_obj_answer = self::getProtectedValue( $ans_obj, '_answer' );

			$arr_answers[ $ans_cnt ] = $ans_obj_answer;
			$ans_obj_correct = self::getProtectedValue( $ans_obj, '_correct' );
			$ans_obj_sortString = self::getProtectedValue( $ans_obj, '_sortString' );

			if ( $ans_obj_correct == 1 ) {
				// if correct answer, makes entry in $arr_correct_answers array
				array_push( $arr_correct_answers, $ans_obj_answer );
			}

			if ( isset( $wdm_user_response[ $ans_cnt ] ) && $wdm_user_response[ $ans_cnt ] == 1 ) {
				// if user has selected answer, '0' if not
				array_push( $arr_user_response, $ans_obj_answer );
			}

			$ans_cnt++;
		}
		$case_multiple = array(
			'arr_correct_answers' => $arr_correct_answers,
			'arr_user_response' => $arr_user_response,
			'ans_obj_sortString' => $ans_obj_sortString,
			'arr_answers' => $arr_answers,
		);

		return $case_multiple;
	}
	/**
	 * caseMatrix
	 * sets user response and correct answers of ouestion type Matrix sort
	 *
	 * @since     1.0.0
	 *
	 * @return array arr_correct_answers,array arr_user_response
	 */

	public function caseMatrix( $wdm_answer_data, $arr_answers, $wdm_question_id, $wdm_user_response, $cur_user_id ) {
		$ans_cnt = 0;
		$arr_correct_answers = array();

		foreach ( $wdm_answer_data as $ans_obj ) {
			$ans_obj_answer = self::getProtectedValue( $ans_obj, '_sortString' );
			$arr_answers[ $ans_cnt ] = $ans_obj_answer;

			array_push( $arr_correct_answers, $ans_obj_answer );

			$ans_cnt++;
		}

		$arr_user_response = array();

		foreach ( $arr_answers as $ans_key => $ans_val ) {
			// unset because not used further
			unset( $ans_val );
			$md5 = self::datapos( $cur_user_id, $wdm_question_id, $ans_key );
			if ( empty( $wdm_user_response ) ) {
				$res_key = false;
			} else {
				$res_key = array_search( $md5, $wdm_user_response );
			}

			if ( false !== $res_key ) {
				$arr_user_response[ $ans_key ] = $arr_answers[ $res_key ];
			}
		}
		$rArray = array(
			'arr_correct_answers' => $arr_correct_answers,
			'arr_user_response' => $arr_user_response,
			'arr_answers' => $arr_answers,
		);

		return $rArray;
	}
	/**
	 * caseSort
	 * sets user response and correct answers of ouestion type sort
	 *
	 * @since     1.0.0
	 *
	 * @return array arr_correct_answers,array arr_user_response
	 */
	public function sortCase( $wdm_answer_data, $arr_answers, $wdm_question_id, $wdm_user_response, $cur_user_id ) {
		$ans_cnt = 0;
		$arr_correct_answers = array();

		foreach ( $wdm_answer_data as $ans_obj ) {
			$ans_obj_answer = self::getProtectedValue( $ans_obj, '_answer' );
			$arr_answers[ $ans_cnt ] = $ans_obj_answer;

			array_push( $arr_correct_answers, $ans_obj_answer );

			$ans_cnt++;
		}

		$arr_user_response = array();

		foreach ( $arr_answers as $ans_key => $ans_val ) {
			$ans_val = $ans_val;
			$md5 = self::datapos( $cur_user_id, $wdm_question_id, $ans_key );
			if ( empty( $wdm_user_response ) ) {
				$res_key = false;
			} else {
				$res_key = array_search( $md5, $wdm_user_response );
			}

			if ( false !== $res_key && isset( $arr_answers[ $res_key ] ) ) {
				$arr_user_response[ $ans_key ] = $arr_answers[ $res_key ];
			}
		}

		$rArray = array(
			'arr_correct_answers' => $arr_correct_answers,
			'arr_user_response' => $arr_user_response,
			'arr_answers' => $arr_answers,
		);

		return $rArray;
	}
	/**
	 * getUserResponse
	 * checks user response for question
	 *
	 * return wdm_z_user_response
	 *
	 * @since     1.0.0
	 */

	public function getUserResponse( $wdm_user_response, $wdm_actual_ans ) {
		if ( ! empty( $wdm_user_response ) ) {
			$wdm_z_user_response = self::checkIssetVar( $wdm_user_response[0] );
		} else {
			$wdm_z_user_response = '';
		}
		if ( isset( $wdm_actual_ans[ $wdm_z_user_response ] ) && isset( $wdm_actual_ans ) && $wdm_z_user_response != 0 ) {
			$wdm_z_user_response = $wdm_actual_ans[ intval( $wdm_z_user_response ) - 1 ];
		}

		return $wdm_z_user_response;
	}
	/**
	 * setUserName
	 * Check whether user is exists and returns username
	 *
	 * @since     2.0.0
	 */
	public function setUserName( $wdm_userdata ) {
		if ( $wdm_userdata !== false ) {
			$username = $wdm_userdata->user_login;
		} else {
			$username = 'Deleted User';
		}

		return $username;
	}
	/**
	 * switchCase
	 * handles all result data of quiz
	 *
	 * @since     1.0.0
	 */
	public function switchCase( $res_value, $result, $arr_data, $arr_process_data ) {
		// global $wpdb;
		$main_cnt = 0; // index counter of main data array

		foreach ( $arr_process_data as $res_key => $res_value ) {
			$res_key = $res_key;
			$wdm_user_id = self::checkIsset( $res_value[0], 'user_id' );
			$wdm_quiz_id = self::checkIsset( $res_value[0], 'quiz_id' );
			$arr_data[ $main_cnt ]['user_id'] = $wdm_user_id;
			$arr_data[ $main_cnt ]['user_login_name'] = '';
			$arr_data[ $main_cnt ]['total_points'] = 0;
			$arr_data[ $main_cnt ]['tot_points_scored'] = 0;
			$arr_data[ $main_cnt ]['tot_time_taken'] = 0;

			if ( ! empty( $wdm_user_id ) ) {
				// to get user name
				$wdm_userdata = get_userdata( $wdm_user_id );

				$arr_data[ $main_cnt ]['user_login_name'] = self::setUserName( $wdm_userdata );
			}
			$arr_data['quiz_title'] = '';
			$arr_data['quiz_title'] = self::getQuizTitle( $wdm_quiz_id, $arr_data['quiz_title'] );

			$cnt = 0;

			if ( ! empty( $res_value ) ) {
				foreach ( $res_value as $key => $val ) {
					$result[ $key ]['answer_data'] = maybe_unserialize( $result[ $key ]['answer_data'] );

					$wdm_answer_data = maybe_unserialize( $result[ $key ]['answer_data'] );

					$question_type = $val['answer_type'];

					$cur_user_id = $val['user_id'];

					$wdm_user_response = json_decode( $val['qsanswer_data'], 1 );

					$wdm_question_id = $val['question_id'];

					$arr_user_response = array();
					$arr_answers = array();
					$arr_correct_answers = array();

					$switch_true = false; // if switch statement is true then make it TRUE.
					$is_attach_question = false; // if want to attache answer to question. For cloze questions.
					$wdm_answer = '';
					// var_dump($result[$key]);
					// die();
					switch ( $question_type ) {
						// if $question_type is single OR multiple
						case 'single':
						case 'multiple':
							$switch_true = true;
							$case_multiple = self::caseMultiple( $wdm_answer_data, $wdm_user_response );
							$arr_correct_answers = $case_multiple['arr_correct_answers'];
							$arr_user_response = $case_multiple['arr_user_response'];
							// $ans_obj_sortString =$case_multiple['ans_obj_sortString'];
							$arr_answers = $case_multiple['arr_answers'];

							break;

						case 'free_answer':
							$switch_true = true;

							$case_free_answer = self::caseFreeAnswer( $wdm_answer_data, $wdm_user_response );
							$arr_user_response = $case_free_answer['arr_user_response'];
							$arr_answers = $case_free_answer['arr_answers'];
							$arr_correct_answers = $case_free_answer['arr_correct_answers'];

							break;
						case 'sort_answer':
							$switch_true = true;
							$case_sort_answer = self::sortCase( $wdm_answer_data, $arr_answers, $wdm_question_id, $wdm_user_response, $cur_user_id );
							$arr_user_response = $case_sort_answer['arr_user_response'];
							$arr_answers = $case_sort_answer['arr_answers'];
							$arr_correct_answers = $case_sort_answer['arr_correct_answers'];

							break;
						case 'matrix_sort_answer':
							$switch_true = true;
							$case_msort_answer = self::caseMatrix( $wdm_answer_data, $arr_answers, $wdm_question_id, $wdm_user_response, $cur_user_id );
							$arr_user_response = $case_msort_answer['arr_user_response'];
							$arr_answers = $case_msort_answer['arr_answers'];
							$arr_correct_answers = $case_msort_answer['arr_correct_answers'];

							break;
						case 'cloze_answer':
							// var_dump($result[$key]);
							// die();
							$switch_true = true;
							$is_attach_question = true;
							$ans_obj = $wdm_answer_data[0];

							$arr_correct_answers = maybe_unserialize( $result[ $key ]['qsanswer_data'] );
							$wdm_answer_data = self::getProtectedValue( $ans_obj, '_answer' );

							// $ans_obj_answer =$ans_obj;
							$arr_wdm_answer = $ans_obj;

							// examples -
							// I {[play][love][hate]} soccer
							// I {play} soccer, with a {ball|3}

							$ans_obj = self::checkIssetVar( $wdm_answer_data );
							$wdm_answer = $ans_obj;

							$arr_wdm_answer = explode( '{', $wdm_answer );

							$arr_options = array();

							if ( ! empty( $arr_wdm_answer ) ) {
								$arr_options = self::getAnsString( $arr_wdm_answer );
							}

							$arr_answers = $arr_options;

							$arr_correct_answers = $arr_options;

							foreach ( $arr_answers as $ckey => $cval ) {
								$wdm_user_ckey_res = self::checkIsset( $wdm_user_response, $ckey );
								array_push( $arr_user_response, $wdm_user_ckey_res );
								$cval = $cval;
							}

							break;
						case 'assessment_answer':
							$switch_true = true;
							$ans_obj = self::checkIssetVar( $wdm_answer_data[0] );

							$wdm_answer = self::getProtectedValue( $ans_obj, '_answer' );

							$arr_wdm_answer = explode( '{', $wdm_answer );

							if ( ! empty( $arr_wdm_answer ) ) {
								$arr_wdm_answer2 = explode( '}', isset( $arr_wdm_answer[1] ) ? $arr_wdm_answer[1] : array() );
								$wdm_answer_str = self::checkIssetVar( $arr_wdm_answer2[0] );

								$wdm_answer_str = str_replace( array( '] [', ']', '[' ), array( ',', '', '' ), $wdm_answer_str );

								$arr_answers[0] = $wdm_answer_str;
								$arr_correct_answers[0] = $wdm_answer_str;
								$wdm_actual_ans = explode( ',', $wdm_answer_str );
							}
							$wdm_z_user_response = self::getUserResponse( $wdm_user_response, $wdm_actual_ans );

							array_push( $arr_user_response, $wdm_z_user_response );

							break;

						case 'essay':
							$switch_true = true;
							$arr_user_response = self::caseEassy( $wdm_user_response, $arr_user_response );
							break;
					}
					// var_dump($arr_user_response);
					// die();

					if ( $switch_true ) {
						$arr_data = self::setArrData( $is_attach_question, $wdm_answer, $arr_data, $main_cnt, $val, $cnt, $arr_answers, $arr_correct_answers, $arr_user_response );

						$cnt++;
					} //if( $switch_true )
				} // foreach ($result as $key => $val) {
			} // if( !empty( $res_value ) )

			   $main_cnt++;
		} //foreach ( $arr_process_data as $res_key => $res_value )

		return $arr_data;
	}

	public function caseEassy( $wdm_user_response, $arr_user_response ) {
		if ( ! empty( $wdm_user_response ) ) {
			$wdm_link = get_post_meta( $wdm_user_response['graded_id'], 'upload', true );
			if ( $wdm_link ) {
				array_push( $arr_user_response, $wdm_link );
			} else {
				$content_post = get_post( $wdm_user_response['graded_id'] );
				$content = $content_post->post_content;
				array_push( $arr_user_response, $content );
			}
		} else {
			$content = '';
			array_push( $arr_user_response, $content );
		}

		return $arr_user_response;
	}
	/**
	 * Returns a quiz title of quiz
	 *
	 * @since 2.0.0
	 *
	 * @param  int    $wdm_quiz_id
	 * @param  string $quiz_title
	 * @return string MD5 Checksum
	 */
	public function getQuizTitle( $wdm_quiz_id, $quiz_title ) {
		global $wpdb;
		if ( ! empty( $wdm_quiz_id ) && $quiz_title == '' ) {
			// to get quiz title
			if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
				$quiz_master_tbl = \LDLMS_DB::get_table_name( 'quiz_master' );
			} else {
				$quiz_master_tbl = $wpdb->prefix . 'wp_pro_quiz_master';
			}
			$quiz_query = "SELECT `name` FROM  {$quiz_master_tbl} WHERE id={$wdm_quiz_id}";
			$quiz_name = $wpdb->get_var( $quiz_query );
			$quiz_title = $quiz_name;

			return $quiz_title;
		}

		return '';
	}

	/**
	 * Returns a MD5 checksum on a concatenated string comprised of user id, question id, and pos
	 *
	 * @since 1.0.0
	 *
	 * @param  int $question_id
	 * @param  int $pos
	 * @return string MD5 Checksum
	 */
	public function datapos( $user_id, $question_id, $pos ) {
		$pos = intval( $pos );

		return md5( $user_id . $question_id . $pos );
	}

	public function getAnsString( $arr_wdm_answer ) {
		$arr_options = array();
		foreach ( $arr_wdm_answer as $cloze_key => $cloze_val ) {
			if ( $cloze_key == 0 ) {
				// first value never be ib
				continue;
			}

			$arr_wdm_answer2 = explode( '}', $cloze_val );

			$wdm_answer_str = self::checkIssetVar( $arr_wdm_answer2[0] );

			if ( $wdm_answer_str !== '' ) {
				$arr_wdm_answer_str = explode( '|', $wdm_answer_str );

				if ( isset( $arr_wdm_answer_str[1] ) ) {
					$wdm_answer_str = $arr_wdm_answer_str[0];
				}
			}

			$wdm_answer_str = str_replace( array( '][', ']', '[' ), array( ',', '', '' ), $wdm_answer_str );
			array_push( $arr_options, $wdm_answer_str );
		}

		return $arr_options;
	}
	 /**
	  * @function  setArrData
	  *
	  * @Description final data to put in files
	  */
	public function setArrData( $is_attach_question, $wdm_answer, $arr_data, $main_cnt, $val, $cnt, $arr_answers, $arr_correct_answers, $arr_user_response ) {
		// . adding quiz title for each element in arr_data
		$arr_data[ $main_cnt ]['quiz_title'] = $arr_data['quiz_title'];

		$arr_data[ $main_cnt ]['total_points'] = $arr_data[ $main_cnt ]['total_points'] + $val['points'];
		$arr_data[ $main_cnt ]['tot_points_scored'] = $arr_data[ $main_cnt ]['tot_points_scored'] + $val['qspoints'];
		$arr_data[ $main_cnt ]['tot_time_taken'] = $arr_data[ $main_cnt ]['tot_time_taken'] + $val['question_time'];

		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['question'] = $val['question'];
		if ( $is_attach_question && $wdm_answer != '' ) {
			// to attache answer in question
			$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['question'] .= '  ' . $wdm_answer;
		}

		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['points'] = $val['points'];

		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['points_scored'] = $val['qspoints'];

		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['time_taken'] = $val['question_time'];

		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['answers'] = $arr_answers;
		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['correct_answers'] = $arr_correct_answers;
		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['user_response'] = $arr_user_response;

		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['question_type'] = $val['answer_type'];

		$arr_data[ $main_cnt ]['ref_id'] = $val['statistic_ref_id']; // added for the use of custom fields

		$arr_data[ $main_cnt ]['question_meta'][ $cnt ]['question_id'] = $val['question_id'];

		return $arr_data;
	}


	// . check for pagination and get 30 entries
	public function wdmCheckCount( $ref_id ) {
		if ( isset( $_POST['page'] ) && ! empty( $ref_id ) ) {
			$refs = explode( ',', $ref_id );
			$total = count( $refs );
			$count = 0;
			$ref_array = array();
			for ( $i = $_POST['page'] * 30; $count < 30 && $i < $total; $i++ ) {
				if ( $count != 29 ) {
					array_push( $ref_array, $refs[ $i ] );
				}
				$count++;
			}
			$ref_id = implode( ',', $ref_array );
			echo $count;
		}
		return $ref_id;
	}

	/**
	 * wdmExport
	 *
	 * @param   $[format] [file format csv/xml]
	 */
	public function wdmExport() {
		global $wpdb;
		$wp_tbl_prefix = $wpdb->prefix;
		$wdm_quiz_id = 0;
		$arr_data = array();

		$ref_id = self::checkIsset( $_POST, 'ref_id' );
		$wdm_nonce = self::checkIsset( $_POST, 'wdm_nonce' );

		// If format of export is CSV
		$format = self::checkIsset( $_POST, 'wdm_format' );

		$table = '';

		$ref_id = self::wdmCheckCount( $ref_id );

		// if (isset($_POST['page']) && !empty($ref_id)) {
		// $refs=explode(',', $ref_id);
		// $total=count($refs);
		// $count=0;
		// $ref_array=array();
		// for ($i=$_POST['page']*30; $count < 30 && $i < $total; $i++) {
		// if ($count != 29) {
		// array_push($ref_array, $refs[$i]);
		// }
		// $count++;
		// }
		// $ref_id = implode(',', $ref_array);
		// echo $count;
		// }

		if ( ! empty( $ref_id ) && wp_verify_nonce( $wdm_nonce, 'wdm' . get_current_user_id() ) ) {

			if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
				$quiz_stat_ref_tbl = \LDLMS_DB::get_table_name( 'quiz_statistic_ref' );
				$quiz_question_tbl = \LDLMS_DB::get_table_name( 'quiz_question' );
				$quiz_statistic_tbl = \LDLMS_DB::get_table_name( 'quiz_statistic' );
			} else {
				$quiz_stat_ref_tbl = $wpdb->prefix . 'wp_pro_quiz_statistic_ref';
				$quiz_question_tbl = $wpdb->prefix . 'wp_pro_quiz_question';
				$quiz_statistic_tbl = $wpdb->prefix . 'wp_pro_quiz_statistic';
			}

			$query = "SELECT qsr.statistic_ref_id, qsr.quiz_id, qsr.user_id, qq.question, qq.points, qq.answer_type, qq.answer_data, qq.sort col_sort, qs.points qspoints, qs.answer_data qsanswer_data, qs.question_time, qs.question_id FROM {$quiz_stat_ref_tbl} qsr,  {$quiz_question_tbl} qq, {$quiz_statistic_tbl} qs WHERE qsr.statistic_ref_id = qs.statistic_ref_id AND qq.id=qs.question_id AND qsr.statistic_ref_id IN (" . $ref_id . ') ORDER BY qsr.statistic_ref_id DESC, col_sort ASC; ';
			if ( preg_match( '[update|delete|drop|alter]', strtolower( $query ) ) === true ) {
				// to prevent user from update,delete,drop,alter commands
				echo 'No cheating';
			} else {
				$result = self::getResultVar( $wpdb->get_results( $query, ARRAY_A ) );
				$wpdb->flush();

				if ( ! empty( $result ) ) {
					// not checked "$custom_result" because if there is only custom fields in quiz, then quiz doesn't show any question on the page.
					$arr_data = array(); // main array of final data
					$custom_form_data = array(); // array of custom form fields

					$arr_process_data = array(); // array to loop

					foreach ( $result as $res_key => $res_value ) {
						// to generate process array

						$wdm_ref_id = self::checkIsset( $res_value, 'statistic_ref_id' );
						// added in version 1.1.2
						$arr_process_data[ $wdm_ref_id ] = self::setArray( $arr_process_data, $wdm_ref_id );

						array_push( $arr_process_data[ $wdm_ref_id ], $res_value );

						$wdm_quiz_id = self::getQuizId( $res_value, $wdm_quiz_id );
					}

					$custom_form_data = self::getCustomFormData( $wdm_quiz_id );
					$arr_custom_data = array(); // final custom fields process data

					if ( ! empty( $custom_form_data ) ) {
						$arr_custom_inserted = array(); // to check if custom answers are already inserted in $arr_process_data
						foreach ( $result as $res_key => $res_value ) {
							$wdm_ref_id = self::checkIsset( $res_value, 'statistic_ref_id' );
							if ( ! in_array( $wdm_ref_id, $arr_custom_inserted ) ) {
								$custom_answers = self::customQuizAnswers( $wdm_ref_id, $wdm_quiz_id, $res_value['user_id'] ); // to get answers of a ref id

								if ( ! empty( $custom_answers ) ) {
									// added in version 1.1.2
									// if ((!is_array($arr_custom_data[$wdm_ref_id])) || (!isset($arr_custom_data[$wdm_ref_id]))) {
									if ( ( ! isset( $arr_custom_data[ $wdm_ref_id ] ) ) ) {
										$arr_custom_data[ $wdm_ref_id ] = array();
									}

									$wdm_custom_process = self::wdmArrCustomProcess( $wdm_ref_id, $res_value, $wdm_quiz_id, $custom_answers );
									// $cust_question = $cust_question;
									$arr_custom_data[ $wdm_ref_id ] = $wdm_custom_process;
									array_push( $arr_custom_inserted, $wdm_ref_id );
								} //     if ( ! empty( $custom_answers ) )
							} // if( !  in_array( $wdm_ref_id, $arr_custom_inserted ) )
						}//foreach
					}
					// for custom field answers - ends
					$arr_data = self::switchCase( $res_value, $result, $arr_data, $arr_process_data );
				} // if( !  empty( $result ) )
				// filter of question types for the custom fields

				if ( ! empty( $arr_custom_data ) ) {
					$arr_custom_data = self::wdmArrCustomData( $arr_custom_data );
				} // if ( !  empty( $arr_custom_data ) ) {

				   // $user_login_name = self::getLoginName($arr_data);

				// creating html data table for Csv format
				if ( $format === 'csv' ) {
					if ( ! empty( $arr_data ) ) {
						$table = self::CSVTable( $arr_data, $arr_custom_data );
					} // $arr_data
				} //Data for xls format
				else {
					if ( ! empty( $arr_data ) ) {
						$table = self::XLSTable( $arr_data, $arr_custom_data );
						// This array contains Data to be send to wdmExportCSV.php
						// It contains data for each row, each cell
						// Also contains style for each cell
						//
					} // if( !empty( $arr_data ) )

					$table = json_encode( $table );
					// return json_encode(array('table' => $table, 'quiz_title' => $arr_data['quiz_title'], 'name' => $user_login_name));
					return array(
						'table' => $table,
						'quiz_title' => self::getLoginName( $arr_data, 'quiz_title' ),
						'name' => self::getLoginName( $arr_data ),
					);
					// die();
				} //else part ends
			}
		}
		// return json_encode(array('table' => htmlentities($table), 'quiz_title' => $arr_data['quiz_title'], 'name' => $user_login_name));
		return array(
			'table' => htmlentities( $table ),
			'quiz_title' => self::getLoginName( $arr_data, 'quiz_title' ),
			'name' => self::getLoginName( $arr_data ),
		);
		// die();
	}

	/**
	 * to return custom field's answers
	 *
	 * @param  numeric $ref_id  statistics ref id
	 * @param  numeric $quiz_id quiz id
	 * @param  numeric $user_id user id of user
	 * @return array   answers of user
	 */
	public function customQuizAnswers( $ref_id, $quiz_id, $user_id ) {
		$custom_answers = array();
		if ( ! empty( $ref_id ) && ! empty( $quiz_id ) ) {
			// User ID check condition removed to get stat of anonymous users
			global $wpdb;

			if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
				$quiz_stat_ref_tbl = \LDLMS_DB::get_table_name( 'quiz_statistic_ref' );
			} else {
				$quiz_stat_ref_tbl = $wpdb->prefix . 'wp_pro_quiz_statistic_ref';
			}

			$custom_query = "SELECT form_data FROM {$quiz_stat_ref_tbl} WHERE statistic_ref_id={$ref_id} AND quiz_id={$quiz_id} AND user_id={$user_id};";

			$custom_answers = maybe_unserialize( $wpdb->get_var( $custom_query ) );
			$wpdb->flush();
			if ( $custom_answers != '' ) {
				$custom_answers = json_decode( $custom_answers, 1 );
			}
		}

		return $custom_answers;
	}

	/**
	 * @function  setArray
	 * @Description to declare array variable
	 * @param array $arr_process_data
	 * @param int   $wdm_ref_id
	 * @return array $arr_process_data[$wdm_ref_id]
	 */
	public function setArray( $arr_process_data, $wdm_ref_id ) {
		if ( ( ! isset( $arr_process_data[ $wdm_ref_id ] ) ) || ( ! is_array( $arr_process_data[ $wdm_ref_id ] ) ) ) {
			$arr_process_data[ $wdm_ref_id ] = array();
		}

		return $arr_process_data[ $wdm_ref_id ];
	}

	/**
	 * @function  getQuizId
	 * @Description to get quiz id
	 * @param array $res_value
	 * @param int   $wdm_quiz_id
	 * @return int $wdm_quiz_id
	 */
	public function getQuizId( $res_value, $wdm_quiz_id ) {
		if ( $wdm_quiz_id == 0 ) {
			// if quiz id not set
			$wdm_quiz_id = $res_value['quiz_id'];
		}

		return $wdm_quiz_id;
	}

	 /**
	  * @function  wdmArrCustomProcess
	  * @Description to get quiz id
	  * @param array $res_value
	  * @param int   $wdm_quiz_id
	  * @return array $arr_process_data[$wdm_ref_id]
	  */
	public function wdmArrCustomProcess( $wdm_ref_id, $res_value, $wdm_quiz_id, $custom_answers ) {
		$arr_custom_process = array();
		$arr_custom_data    = array();

		$arr_custom_data[ $wdm_ref_id ]    = array();

		$custom_form_data = $this->getCustomFormData( $wdm_quiz_id );

		// to get answers of a ref id
		// $custom_answers = self::wdm_custom_quiz_answers( $wdm_ref_id, $wdm_quiz_id, $res_value[ 'user_id' ] );

		// var_dump($custom_form_data);

		foreach ( $custom_form_data as $cust_question ) {
			$arr_custom_process['statistic_ref_id'] = $wdm_ref_id;
			$arr_custom_process['quiz_id'] = $wdm_quiz_id;
			$arr_custom_process['user_id'] = $res_value['user_id'];
			$arr_custom_process['question'] = $cust_question['fieldname'];
			$arr_custom_process['points'] = '';
			$arr_custom_process['answer_type'] = $cust_question['type'];
			$arr_custom_process['answer_data'] = $cust_question['data'];
			$arr_custom_process['col_sort'] = '';
			$arr_custom_process['qspoints'] = '';

			$form_id = $cust_question['form_id']; // id of custom question

			// $arr_custom_process['qsanswer_data'] = isset($custom_answers[$form_id]) ? $custom_answers[$form_id] : '';
			$arr_custom_process['qsanswer_data'] = self::checkIsset( $custom_answers, $form_id );
			$arr_custom_process['question_time'] = '';

			array_push( $arr_custom_data[ $wdm_ref_id ], $arr_custom_process );
			// array_push($arr_custom_data[$wdm_ref_id], $arr_custom_process, $wdm_quiz_id, $custom_answers, $form_id);
		}

		return $arr_custom_data[ $wdm_ref_id ];
	}

	/**
	 * getLoginName
	 * verifies request for export all or for specific user or quiz and returns login name or quiz title
	 *
	 * @param   $[arr_data] [array]
	 * returns  $return_name [login name of user or quiz title depending on $key]
	 **/
	public function getLoginName( $arr_data, $key = 'user_login_name' ) {
		if ( ! empty( $arr_data ) ) {
			$all_values = array_column( $arr_data, $key );
			if ( count( array_unique( $all_values ) ) > 1 ) {
				// if all records are for same user
				$return_name = 'all';
			} else {
				$return_name = $arr_data[0][ $key ];
			}

			return $return_name;
		}
	}

	/**
	 * checkIssetVar
	 * verifies whether variable is set or empty
	 *
	 * @param   $[variable] [int]
	 * returns  $variable or ''
	 **/
	public function checkIssetVar( $variable ) {
		if ( isset( $variable ) ) {
			return $variable;
		}

		return '';
	}
	 /**
	  * getCustomFormData
	  * To get custom form data
	  *
	  * @param   $[wdm_quiz_id] [int] [id of quiz]
	  * returns  $custom_form_data [array]
	  **/
	public function getCustomFormData( $wdm_quiz_id ) {
		if ( ! empty( $wdm_quiz_id ) ) {
			$custom_form_data = self::wdmGetQuizFormData( $wdm_quiz_id ); // gets form data of a quiz
		} else {
			$custom_form_data = array();
		}

		return $custom_form_data;
	}
	 /**
	  * XLSTable
	  * manages information to put in xls file
	  *
	  * @since     1.0.0
	  */
	public function XLSTable( $arr_data, $arr_custom_data ) {
		$table = array(
			0 => array(
				0 => array(
					'value' => '',
					'font' => array(),
				),
			),
		);

		// For row number
		$index = 0;

		foreach ( $arr_data as $val ) {
			if ( isset( $val['question_meta'] ) && ! is_array( $val['question_meta'] ) ) {
				continue;
			}
			if ( isset( $val['question_meta'] ) ) {
				$table[ $index ][0]['value'] = sprintf( __( 'QUIZ TITLE: %s', WDM_LD_QRE_TXT_DOMAIN ), str_replace( '&#39;', "'", html_entity_decode( $val['quiz_title'] ) ) );
				$table[ $index ][1]['value'] = sprintf( __( 'USER LOGIN: %s', WDM_LD_QRE_TXT_DOMAIN ), $val['user_login_name'] );
				$table[ $index ][2]['value'] = sprintf( __( 'USER ID: %s', WDM_LD_QRE_TXT_DOMAIN ), $val['user_id'] );

				$table[ $index ][0]['font'] = array(
					'bold' => 1,
					'italic' => 1,
				);
				$table[ $index ][1]['font'] = array(
					'bold' => 1,
					'italic' => 1,
				);
				$table[ $index ][2]['font'] = array(
					'bold' => 1,
					'italic' => 1,
				);

				$index++;

				$table[ $index ][0]['value'] = __( 'QUESTION', WDM_LD_QRE_TXT_DOMAIN );
				$table[ $index ][1]['value'] = __( 'OPTIONS', WDM_LD_QRE_TXT_DOMAIN );
				$table[ $index ][2]['value'] = __( 'CORRECT ANSWERS', WDM_LD_QRE_TXT_DOMAIN );
				$table[ $index ][3]['value'] = __( 'USER RESPONSE', WDM_LD_QRE_TXT_DOMAIN );
				$table[ $index ][4]['value'] = __( 'POINTS', WDM_LD_QRE_TXT_DOMAIN );
				$table[ $index ][5]['value'] = __( 'POINTS SCORED', WDM_LD_QRE_TXT_DOMAIN );
				$table[ $index ][6]['value'] = __( 'TIME TAKEN', WDM_LD_QRE_TXT_DOMAIN );
				$table[ $index ][7]['value'] = __( 'QUESTION TYPE', WDM_LD_QRE_TXT_DOMAIN );

				$table[ $index ][0]['font'] = array( 'bold' => 1 );
				$table[ $index ][1]['font'] = array( 'bold' => 1 );
				$table[ $index ][2]['font'] = array( 'bold' => 1 );
				$table[ $index ][3]['font'] = array( 'bold' => 1 );
				$table[ $index ][4]['font'] = array( 'bold' => 1 );
				$table[ $index ][5]['font'] = array( 'bold' => 1 );
				$table[ $index ][6]['font'] = array( 'bold' => 1 );
				$table[ $index ][7]['font'] = array( 'bold' => 1 );

				// For next row
				$index++;

				foreach ( $val['question_meta'] as $qkey => $qval ) {
					$qkey = $qkey;
					$table[ $index ][0]['value'] = str_replace( '&#39;', "'", html_entity_decode( $qval['question'] ) ); // Question Column

					// To number the options

					// echo html_entity_decode($qval['question']);

					// To append array values
					$question_str  = '';
					$question_str .= str_replace( '&#39;', "'", html_entity_decode( self::appendQstnStr( $qval['answers'] ) ) );

					$table[ $index ][1]['value'] = $question_str;
					$question_str = '';
					 // $cnt = 1;
					$question_str .= str_replace( '&#39;', "'", html_entity_decode( self::appendQstnStr( $qval['correct_answers'] ) ) );

					$table[ $index ][2]['value'] = $question_str;
					$question_str = '';

					 // $cnt = 1;
					$question_str .= str_replace( '&#39;', "'", html_entity_decode( self::appendQstnStr( $qval['user_response'] ) ) );

					$table[ $index ][3]['value'] = $question_str;
					$question_str = '';

					$table[ $index ][4]['value'] = $qval['points'];

					$table[ $index ][5]['value'] = $qval['points_scored'];

					$table[ $index ][6]['value'] = gmdate( 'H:i:s', $qval['time_taken'] );

					$table[ $index ][7]['value'] = $qval['question_type'];

					// Sets font color for different situations
					if ( $qval['points'] === $qval['points_scored'] ) {
						$table[ $index ][3]['font'] = array( 'color' => array( 'rgb' => '#008000' ) );
					} elseif ( $qval['points_scored'] <= 0 ) {
						$table[ $index ][3]['font'] = array( 'color' => array( 'rgb' => '#FF0000' ) );
					} else {
						$table[ $index ][3]['font'] = array( 'color' => array( 'rgb' => '#0000FF' ) );
					}
					// Next row
					$index++;
				}

				// for custom fields - starts
				if ( isset( $arr_custom_data[ $val['ref_id'] ] ) ) {
					// To append array values

					$table[ $index ][0]['value'] = __( 'CUSTOM FIELDS', WDM_LD_QRE_TXT_DOMAIN );
					$table[ $index ][0]['font'] = array( 'bold' => 1 );

					$options_types = array( 'dropdown', 'radio' );

					foreach ( $arr_custom_data[ $val['ref_id'] ] as $cust_val ) {
						// echo "<pre>";
						// print_r($cust_val);
						// echo "</pre>";
						// exit;
						$question_str_custom = '';
						$index++;
						$table[ $index ][0]['value'] = str_replace( '&#39;', "'", html_entity_decode( $cust_val['question'] ) );

						if ( in_array( $cust_val['answer_type'], $options_types ) ) {
							$cnt = 1;
							if ( is_array( $cust_val['answer_data'] ) ) {
								foreach ( $cust_val['answer_data'] as $answer ) {
									// options
									$question_str_custom .= $cnt . ') ' . str_replace( '&#39;', "'", html_entity_decode( $answer ) ) . "\n";
									$cnt++;
								}
							} else {
								$question_str_custom = str_replace( '&#39;', "'", html_entity_decode( $cust_val['qsanswer_data'] ) );
							}
						}

						$table[ $index ][1]['value'] = $question_str_custom;

						$table[ $index ][2]['value'] = '';

						$table[ $index ][3]['value'] = str_replace( '&#39;', "'", html_entity_decode( $cust_val['qsanswer_data'] ) );

						$table[ $index ][4]['value'] = '';

						$table[ $index ][5]['value'] = '';

						$table[ $index ][6]['value'] = '';

						$table[ $index ][7]['value'] = $cust_val['answer_type'];
					}
					// Next row
					$index++;
				} // for custom fields - ends
				// For total

				$table[ $index ][0]['value'] = __( 'TOTAL', WDM_LD_QRE_TXT_DOMAIN );
				$table[ $index ][0]['font'] = array( 'bold' => 1 );

				$table[ $index ][1]['value'] = '';

				$table[ $index ][2]['value'] = '';

				$table[ $index ][3]['value'] = '';

				$table[ $index ][4]['value'] = $val['total_points'];
				$table[ $index ][4]['font'] = array( 'bold' => 1 );

				$table[ $index ][5]['value'] = $val['tot_points_scored'] . ' ( ' . round( ( ( $val['tot_points_scored'] / $val['total_points'] ) * 100 ), 2 ) . '% )';
				$table[ $index ][5]['font'] = array( 'bold' => 1 );

				$table[ $index ][6]['value'] = gmdate( 'H:i:s', $val['tot_time_taken'] );
				$table[ $index ][6]['font'] = array( 'bold' => 1 );

				$table[ $index ][7]['value'] = '';
				$index++;

				// For a blank row
				// $m as a loop variable
				for ( $m = 0; $m < 8; $m++ ) {
					$table[ $index ][ $m ]['value'] = '';
				}
				$index++;
			} // if( isset( $val[ 'question_meta' ] ) )
		}

		return $table;
	}

	/**
	 * getResultVar
	 * To get result
	 *
	 * @param   $[prevresult] [array]
	 * returns  $prevresult [array]
	 **/

	public function getResultVar( $prevresult ) {
		if ( is_array( $prevresult ) ) {
			$result = array_filter( $prevresult );

			return $result;
		}

		return $prevresult;
	}

	/**
	 * appendQstnStr
	 * To string of result.
	 *
	 * @param   $[qval] [array]
	 * returns  $question_str [string]
	 **/

	public function appendQstnStr( $qval ) {
		$cnt = 1;
		$question_str = '';
		foreach ( $qval as $answer ) {
			if ( $answer != '' ) {
				// user response
				$question_str .= $cnt . ') ' . $answer . "\n";
			}
			$cnt++;
		}

		return $question_str;
	}
	/**
	 * CSVTable
	 * manages information to put in csv file
	 *
	 * @since     1.0.0
	 */
	public function CSVTable( $arr_data, $arr_custom_data ) {
		$table = '';
		$table .= '<table id="wdm_export">';

		foreach ( $arr_data as $key => $val ) {
			if ( isset( $val['question_meta'] ) && ! is_array( $val['question_meta'] ) ) {
				continue;
			}
			$key = $key;
			if ( isset( $val['question_meta'] ) ) {
				$table .= '<tr>
                                            <td>' . sprintf( __( 'QUIZ TITLE: %s', WDM_LD_QRE_TXT_DOMAIN ), str_replace( '&#39;', "'", html_entity_decode( $val['quiz_title'] ) ) ) . '</td>
                                            <td>' . sprintf( __( 'USER LOGIN: %s', WDM_LD_QRE_TXT_DOMAIN ), $val['user_login_name'] ) . '</td>
                                            <td>' . sprintf( __( 'USER ID: %s', WDM_LD_QRE_TXT_DOMAIN ), $val['user_id'] ) . '</td>
                                        </tr><tr>
                                            <td>' . __( 'QUESTION', WDM_LD_QRE_TXT_DOMAIN ) . '</td>
                                            <td>' . __( 'OPTIONS', WDM_LD_QRE_TXT_DOMAIN ) . '</td>
                                            <td>' . __( 'CORRECT ANSWERS', WDM_LD_QRE_TXT_DOMAIN ) . '</td>
                                            <td>' . __( 'USER RESPONSE', WDM_LD_QRE_TXT_DOMAIN ) . '</td>
                                            <td>' . __( 'POINTS', WDM_LD_QRE_TXT_DOMAIN ) . '</td>
                                            <td>' . __( 'POINTS SCORED', WDM_LD_QRE_TXT_DOMAIN ) . '</td>
                                            <td>' . __( 'TIME TAKEN', WDM_LD_QRE_TXT_DOMAIN ) . '</td>
                                            <td>' . __( 'QUESTION TYPE', WDM_LD_QRE_TXT_DOMAIN ) . '</td>
                                        </tr>';

				$question_str = '';
				foreach ( $val['question_meta'] as $qkey => $qval ) {
					$qkey = $qkey;
					$question_str .= '<tr>
                                        <td>' . str_replace( '&#39;', "'", html_entity_decode( $qval['question'] ) ) . '</td>';
					$question_str .= '<td>';

					$question_str .= self::getQuestionStr( $qval['answers'] );

					// $cnt = 1;
					// foreach ($qval['answers'] as $answer) {
					// options column
					// $question_str .= $cnt . ') ' . str_replace("&#39;", "'", html_entity_decode($answer)) . '<br />';
					// $cnt++;
					// }
					$question_str .= '</td>';

					$question_str .= '<td>';

					$question_str .= self::getQuestionStr( $qval['correct_answers'] );

					// $cnt = 1;
					// foreach ($qval['correct_answers'] as $answer) {
					// correct answers
					// $question_str .= $cnt . ') ' . str_replace("&#39;", "'", html_entity_decode($answer)) . '<br />';
					// $cnt++;
					// }
					$question_str .= '</td>';

					$question_str .= '<td>';
					$cnt = 1;
					foreach ( $qval['user_response'] as $answer ) {
						// user response
						if ( $answer != '' ) {
							$question_str .= $cnt . ') ' . str_replace( '&#39;', "'", html_entity_decode( $answer ) ) . '<br />';
						}
						$cnt++;
					}
					$question_str .= '</td>';

					$question_str .= '<td>' . $qval['points'] . '</td>
                                                    <td>' . $qval['points_scored'] . '</td>
                                                    <td>' . gmdate( 'H:i:s', $qval['time_taken'] ) . '</td>
                                                    <td>' . $qval['question_type'] . '</td>
                                                </tr>';
				}

				$table .= $question_str;

				// for custom fields - starts
				if ( isset( $arr_custom_data[ $val['ref_id'] ] ) ) {
					$options_types = array( 'dropdown', 'radio' );
					$question_str_custom = '';
					$question_str_custom .= '<tr><td>' . __( 'CUSTOM FIELDS', WDM_LD_QRE_TXT_DOMAIN ) . '</td></tr>';
					foreach ( $arr_custom_data[ $val['ref_id'] ] as $cust_val ) {
						// if (!is_array($cust_val)) {
						// continue;
						// }
						$question_str_custom .= '<tr>
                                                        <td>' . str_replace( '&#39;', "'", html_entity_decode( $cust_val['question'] ) ) . '</td>';

						$question_str_custom .= '<td>';

						if ( in_array( $cust_val['answer_type'], $options_types ) ) {
							if ( is_array( $cust_val['answer_data'] ) ) {
								$cnt = 1;
								foreach ( $cust_val['answer_data'] as $answer ) {
									// options
									$question_str_custom .= $cnt . ') ' . str_replace( '&#39;', "'", html_entity_decode( $answer ) ) . '<br />';
									$cnt++;
								}
							} else {
								$question_str_custom .= str_replace( '&#39;', "'", html_entity_decode( $cust_val['qsanswer_data'] ) );
							}
						}

						$question_str_custom .= '</td>';

						$question_str_custom .= '<td> </td>
                                                        <td>' . str_replace( '&#39;', "'", html_entity_decode( $cust_val['qsanswer_data'] ) ) . '</td>
                                                        <td> </td>
                                                        <td> </td>
                                                        <td> </td>
                                                        <td>' . $cust_val['answer_type'] . '</td>
                                                    </tr>';
					}
					$table .= $question_str_custom;
				}

				// for custom fields - ends

				$table .= '<tr>
                                <td> ' . __( 'TOTAL', WDM_LD_QRE_TXT_DOMAIN ) . ' </td>
                                <td> </td>
                                <td> </td>
                                <td> </td>
                                <td>' . $val['total_points'] . '</td>
                                <td>' . $val['tot_points_scored'] . ' ( ' . round( ( ( $val['tot_points_scored'] / $val['total_points'] ) * 100 ), 2 ) . '% )' . '</td>
                                <td>' . gmdate( 'H:i:s', $val['tot_time_taken'] ) . '</td>
                                <td> </td>
                            </tr>';
				$table .= '<tr>
                                                <td> </td>
                                            </tr>
                                            <tr>
                                                <td> </td>
                                            </tr>';
			} // if( isset( $arr_data[ 'question_meta' ] ) )
		} // foreach ($arr_data as $key => $val) {

				$table .= '</table>';

		return $table;
	}

	/**
	 * [getQuestionStr Loops through array and generates string]
	 *
	 * @param  array $data [Array of a data]
	 * @return [string] [String of data]
	 */
	public function getQuestionStr( $data = array() ) {
		$question_str = '';
		$cnt = 1;
		foreach ( $data as $value ) {
			// correct answers
			$question_str .= $cnt . ') ' . str_replace( '&#39;', "'", html_entity_decode( $value ) ) . '<br />';
			$cnt++;
		}

		return $question_str;
	}
	 /**
	  * [getProtectedValue To access protected elements from the object as an array].
	  *
	  * @param [obj]    $obj  [object to get protected fields]
	  * @param [string] $name [field name from the object]
	  *
	  * @return [array] [associative array of an protected field]
	  */

	public function getProtectedValue( $obj, $name ) {
		$array = (array) $obj;
		$prefix = chr( 0 ) . '*' . chr( 0 );

		return $array[ $prefix . $name ];
	}
	/**
	 * To get form questions data of custom field by quiz id
	 *
	 * @param number $quiz_id quiz id
	 * @return array $custom_form_data custom form field questions
	 */
	public function wdmGetQuizFormData( $quiz_id ) {
		$custom_form_data = array();
		if ( ! empty( $quiz_id ) ) {
			global $wpdb;
			$wp_tbl_prefix = $wpdb->prefix;

			if ( ! empty( $quiz_id ) ) {
				if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
					$quiz_form_tbl = \LDLMS_DB::get_table_name( 'quiz_form' );
				} else {
					$quiz_form_tbl = $wpdb->prefix . 'wp_pro_quiz_form';
				}
				$custom_form_query = "SELECT form_id, fieldname, type, data FROM {$quiz_form_tbl} WHERE quiz_id={$quiz_id} ORDER BY sort ASC;";
				$custom_form_data = $wpdb->get_results( $custom_form_query, ARRAY_A );
				$wpdb->flush();
			}
		}

		return $custom_form_data;
	}
}

// add_action('plugins_loaded', array('quizReport\QuizReport', 'getInstance'));


// "htmlspecialchars_decode" function is supported after PHP "5.1.0" version. For older PHP versions, following functions is used.
if ( ! function_exists( 'htmlspecialchars_decode' ) ) {
	function htmlspecialchars_decode( $text ) {
		return strtr( $text, array_flip( get_html_translation_table( HTML_SPECIALCHARS ) ) );
	}
}
