<?php

namespace wdmQuizStatistics;

/**
* This class is used to show the quiz attempts and their details made by the user.
*/
class QuizStatistics
{
    public function __construct()
    {
        add_shortcode('wdm_quiz_statistics_details', array($this, 'quizStatisticsShortcode'));
        add_action('wp_ajax_display_attempted_questions', array($this, 'displayAttemptedQuestions'));
        add_action('wp_ajax_wdm_user_data_display', array($this, 'userDataDisplay'));
    }
    public function userDataDisplay()
    {
        $is_ajax=true;
        if (!current_user_can('manage_options') && !current_user_can('group_leader')) {
            $is_ajax=false;
        }
        $user_id=get_current_user_id();
        if (isset($_POST['wdm_user_id'])) {
            $user_id = $_POST['wdm_user_id'];
        }
        if (!empty(filter_input(INPUT_POST, 'user_id'))) {
            $user_id = filter_input(INPUT_POST, 'user_id');
        }
        $wdm_courses = $this->getGroupCourses();
        $quizzes_data = get_user_meta($user_id, '_sfwd-quizzes', true);

        //. shamali - added function to check if statistics red ids are not deleted
        $quizzes_data = $this->wdmFilterQuizzesData($quizzes_data);
        $wdmResult ='';
        if (empty($quizzes_data) || ($quizzes_data === false) || $user_id==0) {
            $wdmResult .='<div class="wdm_quiz_data" data-counter="">
                            <div><center>No Data to display</center></div>
                        </div>';
            echo $wdmResult;

            if ($is_ajax) {
                die();
            }
        }

        $this->displayQuizTable($wdm_courses, $quizzes_data, $is_ajax, $wdmResult, $user_id);
    }

    //. shamali - filter quizzes data by checking if their corresponding statistics data is deleted or not.
    public function wdmFilterQuizzesData($quizzes_data)
    {
        if (empty($quizzes_data)) {
            return $quizzes_data;
        }

        global $wpdb;

        //. check for permanently deleted quizzes
        $quiz_ids = implode(',', array_unique(array_column($quizzes_data, 'quiz')));

        $quiz_qry = "SELECT ID FROM  {$wpdb->prefix}posts where ID IN ({$quiz_ids})";
        $quiz_ids_present = $wpdb->get_col($quiz_qry);

        //. check for deleted/ reset quiz statistics
        $statistics_ids = implode(',', array_filter(array_column($quizzes_data, 'statistic_ref_id')));

        if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
            $stat_ref_tbl = \LDLMS_DB::get_table_name( 'quiz_statistic_ref' );
        } else {
            $stat_ref_tbl = $wpdb->prefix . 'wp_pro_quiz_statistic_ref';
        }

        $check_qry = "SELECT statistic_ref_id FROM  {$stat_ref_tbl} where statistic_ref_id IN ({$statistics_ids})";
        $ids_present = $wpdb->get_col($check_qry);

        if (empty($quiz_ids_present) || empty($ids_present)) {
            return array();
        }

        $quizzes_data = array_filter(
            $quizzes_data,
            function ($data) use ($quiz_ids_present, $ids_present) {
                if (in_array($data['quiz'], $quiz_ids_present) && in_array($data['statistic_ref_id'], $ids_present)) {
                    return true;
                }
                return false;
            }
        );

        return $quizzes_data;
    }

    public function displayQuizTable($wdm_courses, $quizzes_data, $is_ajax, $wdmResult, $user_id = null)
    {
        if (is_array($quizzes_data) && !empty($quizzes_data)) {
            $quiz=array_reverse($quizzes_data);
            echo $this->displayQuizDataInTable($wdmResult, $quiz, $wdm_courses, $user_id);
        }
        if ($is_ajax) {
            wp_die();
        } elseif (!empty(filter_input(INPUT_POST, 'is_ajax')) && filter_input(INPUT_POST, 'is_ajax') == 'is_ajax') {
            wp_die();
        }
    }
    public function getGroupCourses()
    {
        $wdm_courses=array();
        if (current_user_can('group_leader')) {
            $associatedgroups = learndash_get_administrators_group_ids(get_current_user_id());

            if (!empty($associatedgroups)) {
                foreach ($associatedgroups as $associatedgroup) {
                    if (empty($wdm_courses)) {
                        $wdm_courses = learndash_group_enrolled_courses($associatedgroup);
                    } else {
                        array_merge($wdm_courses, learndash_group_enrolled_courses($associatedgroup));
                    }
                }
            }
        }
        return $wdm_courses;
    }
    public function sortQuiz($quiz, $wdm_courses, $user_id)
    {
        $sorted_quiz = array();
        if (current_user_can('group_leader') && get_current_user_id() != $user_id) {
            foreach ($quiz as $quiz_value) {
                $wdmC = learndash_get_course_id($quiz_value["quiz"]);
                if ($wdmC=='') {
                    $wdmC=$quiz_value["course"];
                }
                if (!empty($wdmC) && in_array($wdmC, $wdm_courses) && !empty($wdm_courses)) {
                    $sorted_quiz[] = $quiz_value;
                }
            }
        } else {
            return $quiz;
        }
        return $sorted_quiz;
    }

    public function displayQuizDataInTable($wdmResult, $quiz, $wdm_courses, $user_id = null)
    {
        /* aman code start */
        /* Pagination for quiz result start*/
        $quiz = $this->sortQuiz($quiz, $wdm_courses, $user_id);

        $perPage = new \paginationclass\PerPage();
        $paginationlink = admin_url('admin-ajax.php')."?page=";
        $rowcount = filter_input(INPUT_POST, 'rowcount');
        $page = 1;
        if (!empty($_GET["page"])) {
            $page = $_GET["page"];
        }
        $start = ($page-1)*$perPage->perpage;
        if ($start < 0) {
            $start = 0;
        }
        if (empty($rowcount)) {
            $rowcount = sizeof($quiz);
        }
        $perpageresult = $perPage->getAllPageLinks($rowcount, $paginationlink);
        ?>
         <div id="pagination" style="float:right">
            <?php echo $perpageresult;
        ?>
        </div>
        <?php
        echo '<input type="hidden" id="rowcount" name="rowcount" value="' . $rowcount . '" />';
        $start = ($page - 1) * $perPage->perpage;
        $end = $start + $perPage->perpage;
        /* Pagination for quiz result end*/
        /* aman code end */
        $end = (sizeof($quiz) < $end) ? sizeof($quiz) : $end;
        $all_ids = array();
        for ($count=$start; $count < $end; ++$count) {
            $wdmC = learndash_get_course_id($quiz[$count]["quiz"]);
            $download_report_link = "<a href='#' data-ref_id='". $quiz[$count]["statistic_ref_id"]. "' class=\"wdm-export  wdm-exp-csv\"> CSV </a>/<a href='#' data-ref_id='" . $quiz[$count]["statistic_ref_id"] . "' class=\"wdm-export  wdm-exp-xls\">Excel</a>";
            if (current_user_can('group_leader') && get_current_user_id() != $user_id) {
                if ($wdmC=='') {
                    $wdmC=$quiz[$count]["course"];
                }
                if (!empty($wdmC) && in_array($wdmC, $wdm_courses) && !empty($wdm_courses)) {
                    $wdmResult .='<div class="wdm_quiz_data" data-counter="'.$count.'">
                        <div class="wdmcolumn_1 columns">
                            <input type="hidden" value="'.$quiz[$count]["time"].'" class="details_id">
                            <input type="hidden" value="'.$quiz[$count]["quiz"].'" class="quiz_ids">';
                    $wdmResult .= '<input type="hidden" value="'.$quiz[$count]["statistic_ref_id"].'" class="stat_ref_id">';

                    $quiz_title=get_the_title($quiz[$count]["quiz"]);

                    $wdmResult .='  <a href="javascript:void(0)" style="text-decoration:none;" class="quiz_load"> '.$quiz_title.'</a>
                        </div>
                        <div class="wdmcolumn_2 columns">';

                    $quiz_time=$quiz[$count]["time"];
                    $quiz_date=date("d-M-Y", $quiz_time);
                    $wdmResult .=$quiz_date.'

                            </div>
                        <div class="wdmcolumn_3 columns">';

                    $quiz_percentage=$quiz[$count]["percentage"];
                    $wdmResult .=$quiz_percentage.' %

                        </div>
                        <div class="wdmcolumn_4 columns">'
                        .$download_report_link.
                        '</div>
                    </div>
                    <div class="wdm_prev_div" id="wdm_quiz_details_'.$count.'"></div>';
                        // var_dump($wdmResult);die();
                }
            } elseif (current_user_can('manage_options')) {
                $wdmResult .='<div class="wdm_quiz_data" data-counter="'.$count.'">
                        <div class="wdmcolumn_1 columns">
                            <input type="hidden" value="'.$quiz[$count]["time"].'" class="details_id">
                            <input type="hidden" value="'.$quiz[$count]["quiz"].'" class="quiz_ids">';
                $wdmResult .= '<input type="hidden" value="'.$quiz[$count]["statistic_ref_id"].'" class="stat_ref_id">';

                $quiz_title=get_the_title($quiz[$count]["quiz"]);

                $wdmResult .='  <a href="javascript:void(0)" style="text-decoration:none;" class="quiz_load"> '.$quiz_title.'</a>
                        </div>
                        <div class="wdmcolumn_2 columns">';

                $quiz_time=$quiz[$count]["time"];
                $quiz_date=date("d-M-Y", $quiz_time);
                $wdmResult .=$quiz_date.'

                            </div>
                        <div class="wdmcolumn_3 columns">';

                $quiz_percentage=$quiz[$count]["percentage"];
                $wdmResult .=$quiz_percentage.' %

                        </div>
                        <div class="wdmcolumn_4 columns">'
                        .$download_report_link.
                        '</div>
                    </div>
                    <div class="wdm_prev_div" id="wdm_quiz_details_'.$count.'"></div>';
            } else {
                $wdmResult .='<div class="wdm_quiz_data" data-counter="'.$count.'">
                        <div class="wdmcolumn_1 columns">
                            <input type="hidden" value="'.$quiz[$count]["time"].'" class="details_id">
                            <input type="hidden" value="'.$quiz[$count]["quiz"].'" class="quiz_ids">';

                $wdmResult .= '<input type="hidden" value="'.$quiz[$count]["statistic_ref_id"].'" class="stat_ref_id">';

                $quiz_title=get_the_title($quiz[$count]["quiz"]);

                $wdmResult .='  <a href="javascript:void(0)" style="text-decoration:none;" class="quiz_load"> '.$quiz_title.'</a>
                        </div>
                        <div class="wdmcolumn_2 columns">';

                $quiz_time=$quiz[$count]["time"];
                $quiz_date=date("d-M-Y", $quiz_time);
                $wdmResult .=$quiz_date.'

                            </div>
                        <div class="wdmcolumn_3 columns">';

                $quiz_percentage=$quiz[$count]["percentage"];
                $wdmResult .=$quiz_percentage.' %
                        </div>
                        <div class="wdmcolumn_4 columns">'
                        .$download_report_link.
                        '</div>
                    </div>
                    <div class="wdm_prev_div" id="wdm_quiz_details_'.$count.'"></div>';
            }
            # code...
            // $all_ids[] = $quiz[$count]["statistic_ref_id"];
        }
        $all_ids = array_column($quiz, 'statistic_ref_id');
        $wdmResult = $this->getExportAllLink($wdmResult, $all_ids);
        return $wdmResult;
    }

    public function getExportAllLink($wdmResult, $all_ids)
    {
        if (count($all_ids) > 1) {
            $wdmResult .= '<a id="wdm_export_all" href="#" data-ref_id="' . implode(',', $all_ids) . '" class="wdm-export wdm-export-all button-secondary wdm-exp-csv" style="margin-right:15px;">Export All in CSV</a><a id="wdm_export_all" href="#" data-ref_id="' . implode(',', $all_ids) . '" class="wdm-export wdm-export-all button-secondary wdm-exp-xls" style="margin-right:3px;">Export All in Excel</a>';
        }

        return $wdmResult;
    }
    // }

    // }
    /**
     * This class is used to create the shortcode for showing the quiz attempts made by the user
     */
    public function quizStatisticsShortcode()
    {
        if (!is_user_logged_in() || isset($_GET['_locale']) || is_admin()) {
            return false;
        }
        wp_enqueue_script('QuizData', plugins_url('../assets/js/LoadQuizData.js', __FILE__), array('jquery'));

        wp_enqueue_script('wdm_export_frontend', plugins_url('../assets/js/wdm-export-frontend.js', __FILE__), array('jquery'), '4.2.3');
        wp_localize_script('wdm_export_frontend', 'wdm_export_obj', array(
            'wdm_nonce'     => wp_create_nonce('wdm' . get_current_user_id()),
            'ajax_url'      => admin_url('admin-ajax.php'),
        ));

        wp_register_script('page_blocker', plugins_url('../assets/js/page_blocker.min.js', __FILE__), array('jquery'), '4.2.3');
            wp_enqueue_script('page_blocker');

        $wp_localize_data=array();
        $wp_localize_data['ajax_url']=admin_url('admin-ajax.php');
        wp_localize_script('QuizData', 'ajax_new_object', $wp_localize_data);
        wp_enqueue_style('wdmCustomStyle', plugins_url('../assets/css/wdmCustomStyle.css', __FILE__));
        ?>
        <img id="wdm_loader" src="<?php echo plugins_url('../assets/images/loader.gif', __FILE__);
        ?>" />
        <a id="wdm_arrow_sec" title="Collapse Result" href="javascript:void(0)"><img id="wdm_arrow-up" src="<?php echo plugins_url('../assets/images/arrow-up.png', __FILE__);
        ?>" style="display:none;" /></a>

        <?php
        //New Code
        $nonce = wp_create_nonce('Quiz-Attempt-nonce');
        $wdm_courses = array();
        $users = array();
        if (current_user_can('manage_options')) {
            $students = get_users();
            foreach ($students as $student) {
                array_push($users, $student->ID);
                # code...
            }
        } elseif (current_user_can('group_leader')) {
            $curr_user_id = get_current_user_id();
            $associatedgroups = learndash_get_administrators_group_ids($curr_user_id);
            array_push($users, $curr_user_id);

            if (!empty($associatedgroups)) {
                foreach ($associatedgroups as $associatedgroup) {
                    $students = learndash_get_groups_user_ids($associatedgroup);
                    if (!empty($students)) {
                        foreach ($students as $student) {
                            array_push($users, $student);
                        }
                    }
                }
            }
        }
        if (!empty($users)) {
            $users = array_unique($users);
            ?>
        <input type="hidden" value="<?php echo $nonce;
            ?>" class="wdm_nonce"/>
        <strong>Select The User:</strong>
        <select class="UserID" style="margin-left:5px;">
        <option value="0"><?php echo '--Select the User--';
            ?></option>
        <?php
        foreach ($users as $userId) {
            $user = get_user_by('id', $userId);
            ?>
            <option value="<?php echo $user->ID;
            ?>">
            <?php
            echo $user->data->user_login;
            ?>
            </option>
        <?php
        }
            ?>
        </select>
        <?php
        }
        ?>
        <div class="wdmQuizData" style="margin-top:5px;"></div>

    <div id="wdm_exams_table">
        <div class="wdmcolumn_1 columns">
            <strong>Exam</strong>
        </div>
        <div class="wdmcolumn_2 columns">
            <strong>Date Of Attempt</strong>
        </div>
        <div class="wdmcolumn_3 columns">
            <strong>Score</strong>
        </div>
        <div class="wdmcolumn_4 columns">
            <strong>Download Report</strong>
        </div>
        </div>
        <?php
        if (!empty($users)) {
            ?>
        <div class="wdm_quiz_data" data-counter="<?php //echo $count;?>">
            <div><center>No Data to display</center></div>
        </div>
        <?php
        } else {
            echo "<div class= wdm_quiz_data>";
            $this->userDataDisplay();
            echo "</div>";
        }
        unset($wdm_courses);
    }
    /**
     * To get the statistic_ref_id we used creation_time of the quiz, because in the LearnDash version 2.0.6.3, auto generation of statistic_ref_id facility is not provided.
     */
    public function displayAttemptedQuestions()
    {
        global $wpdb;
        $user_id=get_current_user_id();
        if (isset($_POST['user_id'])) {
            $user_id = $_POST['user_id'];
        }
        $quiz_id = $_POST['quiz_id'];
        $statistics_id = 0;
        $pro_quiz_id  = learndash_get_setting(intval($quiz_id), 'quiz_pro', true);
        $time = $_POST['time_id'];
        if (isset($_POST['stat_ref_id']) && !empty($_POST['stat_ref_id'])) {
            $statistics_id = $_POST['stat_ref_id'];
        } else {

            if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
                $stat_ref_tbl = \LDLMS_DB::get_table_name( 'quiz_statistic_ref' );
            } else {
                $stat_ref_tbl = $wpdb->prefix . 'wp_pro_quiz_statistic_ref';
            }

            $qry = "SELECT statistic_ref_id FROM  {$stat_ref_tbl} WHERE user_id = {$user_id} AND quiz_id = {$pro_quiz_id} AND create_time = {$time} ORDER BY quiz_id DESC";
            $myrows = $wpdb->get_results($qry);
            if (!empty($myrows) || $myrows != "") {
                foreach ($myrows[0] as $key => $value) {
                    $statistics_id = $value;
                }
            }
        }

        if (!empty($statistics_id)) {
            $questions = $this->getStatisticsData($pro_quiz_id, $user_id, $statistics_id, false);
            $questions_key = array();
            if (!empty($questions)) {
                $questions_key = array_keys($questions);
            }
            if (!empty($questions_key)) {
                $counter = 0;
                echo $this->showResultBox($que_count, $pro_quiz_id, $statistics_id);
                foreach ($questions_key as $key) {
                    foreach ($questions[$key]['questions'] as $value) {
                        // echo "<pre>";
                        // print_r($value['statistcAnswerData']);
                        // echo"</pre>";

                        $question_id = $value['question_id'];
                        $question_text = !empty($value['questionName']) ? $value['questionName'] : $this->getQuestionText($pro_quiz_id, $question_id);
                        // $question_text = $this->getQuestionText($pro_quiz_id, $question_id);
                        $cmsg=$this->getQuestionDetails($question_id, $pro_quiz_id);
                        echo $this->showUserAnswer($question_text, $value['questionAnswerData'], $value['statistcAnswerData'], $value['answerType'], ++$counter, $value['correct'], $cmsg);
                    }
                }
                $que_count=count($questions[$key]['questions']);
            }
        } else {
            echo "<center>Statistics not found</center>";
        }
        die();
    }
    /**
     * This function is used to get the question details like answer options,description etc.
     * @param  [int] $queID       [Question ID]
     * @param  [int] $pro_quiz_id [Pro Quiz ID]
     * @return [string]      $cmg        [correct message]
     */
    public function getQuestionDetails($queID, $pro_quiz_id)
    {
        $view = new \WpProQuiz_View_FrontQuiz();
        $quizMapper = new \WpProQuiz_Model_QuizMapper();
        $questionMapper = new \WpProQuiz_Model_QuestionMapper();
        $question=$questionMapper->fetchAll($pro_quiz_id);
        $categoryMapper = new \WpProQuiz_Model_CategoryMapper();
        $formMapper = new \WpProQuiz_Model_FormMapper();
        $quiz = $quizMapper->fetch($pro_quiz_id);
        if ($quiz->isShowMaxQuestion() && $quiz->getShowMaxQuestionValue() > 0) {
            $value = $quiz->getShowMaxQuestionValue();
            if ($quiz->isShowMaxQuestionPercent()) {
                $count = $questionMapper->count($pro_quiz_id);
                $value = ceil($count * $value / 100);
            }
            $question = $questionMapper->fetchAll($pro_quiz_id, true, $value);
        } else {
            $question = $questionMapper->fetchAll($pro_quiz_id);
        }
        $view->quiz = $quiz;
        $view->question = $question;
        $view->category = $categoryMapper->fetchByQuiz($quiz->getId());
        $view->forms = $formMapper->fetch($quiz->getId());
        foreach ($view->question as $que) {
            $qid=$this->getProtectedValue($que, '_id');
            if ($qid==$queID) {
                $cmsg=$this->getProtectedValue($que, '_correctMsg');
                return $cmsg;
            }
        }
    }
    /**
     * This function is used to get the statistics of the quiz for particular user
     * @param  [int] $quizId [Quiz ID]
     * @param  [int] $userId [User ID]
     * @param  [int] $refId  [Statistics reference ID]
     * @param  [int] $avg    [Average of Quiz]
     * @return [string]  $output       [output of questions]
     */
    public function getStatisticsData($quizId, $userId, $refId, $avg)
    {
        $refIdUserId = $avg ? $userId : $refId;
        $statisticUserMapper = new \WpProQuiz_Model_StatisticUserMapper();
        $statisticUsers = $statisticUserMapper->fetchUserStatistic($refIdUserId, $quizId, $avg);
        $output = $this->getStatistics($statisticUsers, $userId);
        return $output;
    }

    public function getStatistics($statisticUsers, $userId)
    {
        $output = array();
        foreach ($statisticUsers as $statistic) {
            if (!isset($output[$statistic->getCategoryId()])) {
                $output[$statistic->getCategoryId()] = array(
                    'questions' => array(),
                    'categoryId' => $statistic->getCategoryId(),
                    'categoryName' => $statistic->getCategoryId() ? $statistic->getCategoryName() : __('No category', 'wp-pro-quiz')
                );
            }
            $output_str = &$output[$statistic->getCategoryId()];
            $question_item = array(
                'question_id' => $statistic->getQuestionId(),
                'correct' => $statistic->getCorrectCount(),
                'incorrect' => $statistic->getIncorrectCount(),
                'hintCount' => $statistic->getIncorrectCount(),
                'time' => $statistic->getQuestionTime(),
                'points' => $statistic->getPoints(),
                'gPoints' => $statistic->getGPoints(),
                'statistcAnswerData' => $statistic->getStatisticAnswerData(),
                'questionName' => $statistic->getQuestionName(),
                'questionAnswerData' => $statistic->getQuestionAnswerData(),
                'answerType' => $statistic->getAnswerType()
            );
            // For the sort_answer items. This worked correctly with LD 2.0.6.8. But in 2.1.x there was a change where
            // the stored value for 'statistcAnswerData' was not simply keys to match 'questionAnswerData' but md5 value.
            // This causes a mis-match when viewing statistics data. To complicate things we will have a mix of LD 2.0.6.8
            // quiz values and 2.1.x quiz values.
            if (($question_item['answerType'] == 'sort_answer') || ($question_item['answerType'] == 'matrix_sort_answer')) {
                if ((isset($question_item['questionAnswerData'])) && (!empty($question_item['questionAnswerData']))
                 && (isset($question_item['statistcAnswerData'])) && (!empty($question_item['statistcAnswerData']))) {
                    // So first we check the value of the first item from 'statistcAnswerData'. If the value
                    // is a simple int then we can move on. If not, then we have some work to do.
                    $statistcAnswerData_item = $question_item['statistcAnswerData'][0];
                    if (($statistcAnswerData_item == -1)
                     || (strcmp($statistcAnswerData_item, intval($statistcAnswerData_item)) !== 0)) {
                        $questionId = $statistic->getQuestionId();
                        // Next we loop over the 'questionAnswerData' items.
                        foreach ($question_item['questionAnswerData'] as $q_k => $q_v) {
                            // Take the item key and encode it.
                            $datapos=md5($userId . $questionId . intval($q_k));
                            $s_pos = array_search($datapos, $question_item['statistcAnswerData'], true);

                            if ($s_pos !== false) {
                                $question_item['statistcAnswerData'][$s_pos] = intval($q_k);
                            }
                        }
                    }
                }
            }
            $output_str['questions'][] = $question_item;
        }
        return $output;
        unset($q_v);
    }
    /**
     * This function is used to fetch the quiz assessment
     * @param  [string] $answerText [Answer Text of Question]
     * @param  [int] $quizId     [Quiz ID]
     * @param  [int] $questionId [QUestion ID]
     * @return [string]             [description]
     */
    public function fetchAssessments($answerText, $quizId, $questionId)
    {
        //This will math the answer text having braces around it.
        preg_match_all('#\{(.*?)\}#im', $answerText, $matches);
        $data = array();
        for ($count = 0, $count_i = count($matches[1]); $count < $count_i; $count++) {
            $match = $matches[1][$count];
            preg_match_all('#\[([^\|\]]+)(?:\|(\d+))?\]#im', $match, $wdm_ms);
            $ans_text = '';
            for ($loop = 0, $count_ = count($wdm_ms[1]); $loop < $count_; $loop++) {
                $que_v = $wdm_ms[1][$loop];
                $ans_text .= '<label>
                    <input type="radio" value="'.($loop+1).'" name="question_'.$quizId.'_'.$questionId.'_'.$count.'" class="wpProQuiz_questionInput" data-index="'.$count.'">
                    '.$que_v.'
                </label>';
            }
        }
        $data['replace'] = preg_replace('#\{(.*?)\}#im', '@@wpProQuizAssessment@@', $answerText);
        return $data;
    }
    /**
     * This function is an assessment callback.
     * @param  [int] $term [AnswerTerm]
     * @return [string]  ans  [Assessment Answer]
     */
    public function assessmentCallback($term)
    {
        $ans = array_shift($this->_assessmetTemp);
        return $ans === null ? '' : $ans;
        unset($term);
    }
    /**
     * This function is used to show the anser to the user in a HTML format
     * @param  [string] $question_text [Question Text]
     * @param  [string] $qAnswerData   [Question Answer Data]
     * @param  [string] $sAnswerData   [Answer Data]
     * @param  [string] $anserType     [Answer Type]
     * @param  [int] $question_no   [Question ID]
     * @param  [bool] $is_correct    [Answer is correct or not]
     * @param  [string] $cmg           [description]
     */
    public function showUserAnswer($question_text, $qAnswerData, $sAnswerData, $anserType, $question_no, $is_correct, $cmg = null)
    {
        $matrix = array();
        if ($anserType == 'matrix_sort_answer') {
            foreach ($qAnswerData as $ans_key => $ans_value) {
                $matrix[$ans_key][] = $ans_key;
                foreach ($qAnswerData as $ans_key2 => $ans_value2) {
                    if ($ans_key != $ans_key2) {
                        if ($ans_value->getAnswer() == $ans_value2->getAnswer()) {
                            $matrix[$ans_key][] = $ans_key2;
                        } elseif ($ans_value->getSortString() == $ans_value2->getSortString()) {
                            $matrix[$ans_key][] = $ans_key2;
                        }
                    }
                }
            }
        }
        ?>
        <h5 class="wpProQuiz_header">
        <span><?php echo $question_no;
        ?></span>. Question  </h5>
        <div class="wpProQuiz_question_text">
            <p>
            <?php echo $question_text;
        ?>
            </p>
        </div>
        <?php
        if ($anserType == 'matrix_sort_answer') {
            $this->getMatrixSortAnswer($qAnswerData);
        }
        $this->getQuestionList($question_text, $qAnswerData, $sAnswerData, $anserType, $question_no, $is_correct, $matrix, $cmg);
        if ($anserType != 'essay') {
        ?>
        <div class="wpProQuiz_response">
                <?php
                if ($is_correct) {
                    ?>
                    <div style="" class="wpProQuiz_correct">
                        <span>
                            Correct
                        </span>
                        <p class="wpProQuiz_AnswerMessage">
                        <?php echo do_shortcode(apply_filters('comment_text', $cmg));
                    ?>
                        </p>
                    </div>
                    <?php
                } else {
                    ?>
                    <div style="" class="wpProQuiz_incorrect">
                    <span>
                            Incorrect
                    </span>
                    <p class="wpProQuiz_AnswerMessage">
                        <?php echo do_shortcode(apply_filters('comment_text', $cmg));
                    ?>
                        </p>
                        </div>
                    <?php
                }
        ?>
        </div>
        <?php
        }
    }
    public function getQuestionList($question_text, $qAnswerData, $sAnswerData, $anserType, $question_no, $is_correct, $matrix, $cmg = null)
    {
        ?>
        <ul class="wpProQuiz_questionList">
        <?php
        for ($cnt = 0; $cnt < count($qAnswerData); $cnt++) {
            $answerText = $qAnswerData[$cnt]->isHtml() ? $qAnswerData[$cnt]->getAnswer() : esc_html($qAnswerData[$cnt]->getAnswer());
            $correct = '';
            ?>
            <?php
            if ($anserType === 'single' || $anserType === 'multiple') {
                $this->displaySingleMultiple($qAnswerData, $sAnswerData, $answerText, $anserType, $cnt);
            } elseif ($anserType === 'free_answer') {
                $this->displayFreeAnswer($sAnswerData, $qAnswerData, $cnt);
            } elseif ($anserType === 'sort_answer') {
                $correct = 'wpProQuiz_answerIncorrect';
                $sortText = '';

                if (isset($sAnswerData[$cnt]) && isset($qAnswerData[$sAnswerData[$cnt]])) {
                    if ($sAnswerData[$cnt] == $cnt) {
                        $correct = 'wpProQuiz_answerCorrect';
                    }
                    $ans_val = $qAnswerData[$sAnswerData[$cnt]];
                    $sortText = $ans_val->isHtml() ? $ans_val->getAnswer() : esc_html($ans_val->getAnswer());
                }
                ?>
                            <li class="<?php echo $correct;
                ?>" style="margin-left:0px;padding: 3px;">
                                <div class="wpProQuiz_sortable wdmProQuiz_sortable">
                                    <?php echo $sortText;
                ?>
                                </div>
                            </li>
                            <?php
            } elseif ($anserType == 'matrix_sort_answer') {
                $correct = 'wpProQuiz_answerIncorrect';
                $sortText = '';
                if (isset($sAnswerData[$cnt]) && isset($qAnswerData[$sAnswerData[$cnt]])) {
                    if (in_array($sAnswerData[$cnt], $matrix[$cnt])) {
                        $correct = 'wpProQuiz_answerCorrect';
                    }
                    $ans_val = $qAnswerData[$sAnswerData[$cnt]];
                    $sortText = $ans_val->isSortStringHtml() ? $ans_val->getSortString() : esc_html($ans_val->getSortString());
                }
                ?>
                            <li>
                                <table>
                                    <tbody>
                                        <tr class="wpProQuiz_mextrixTr">
                                            <td style = "width:20%">
                                                <div class="wpProQuiz_maxtrixSortText"><?php echo $answerText;
                ?></div>
                                            </td>
                                            <td style = "width:80%">
                                                <ul class="wpProQuiz_maxtrixSortCriterion <?php echo $correct;
                ?>">
                                                    <li class="wpProQuiz_sortStringItem" data-pos="0" style="box-shadow: 0px 0px; cursor: auto;">
                                                        <?php echo $sortText;
                ?>
                                                    </li>
                                                </ul>
                                            </td>
                                        </tr>
                                    </tbody>
                                </table>
                            </li>
                            <?php
            } elseif ($anserType == 'cloze_answer') {
                $clozeData = $this->fetchCloze($qAnswerData[$cnt]->getAnswer(), $sAnswerData);
                echo $clozeData['replace'];
            } elseif ($anserType == 'assessment_answer') {
                $assessmentData = $this->fetchAssessment($qAnswerData[$cnt]->getAnswer(), $sAnswerData);
                echo $assessmentData['replace'];
            } elseif ($anserType == 'essay') {
                $this->displayEssayAnswer($sAnswerData);
            }
        }
        ?>
        </ul>
        <?php
        unset($question_text);
        unset($question_no);
        unset($is_correct);
        unset($cmg);
        unset($matrix);
    }
    public function displayEssayAnswer($sAnswerData)
    {
        if ((isset($sAnswerData['graded_id'])) && (!empty($sAnswerData['graded_id']))) {
            $essay_post_status = get_post_status($sAnswerData['graded_id']);
            $essay_post_status_str = '';
            if ($essay_post_status == 'graded') {
                $essay_post_status_str = __('Graded', 'wp-pro-quiz');
            } else {
                $essay_post_status_str = __('Not Graded', 'wp-pro-quiz');
            }
            ?><li>
                <div class="wpProQuiz_sortable">
                    <?php
                    echo __('Status', 'wp_pro_quiz') .' : '. $essay_post_status_str;
                    if (current_user_can('edit_post', $sAnswerData['graded_id'])) {
                        ?> (<a target="_blank" href="<?php echo get_post_permalink($sAnswerData['graded_id']); ?>"><?php _e('view', 'wp-pro-quiz') ?></a>)<?php
                    }
                    ?>
                </div>
            </li>
            <?php
        }
    }
    public function displaySingleMultiple($qAnswerData, $sAnswerData, $answerText, $anserType, $cnt)
    {
        if ($qAnswerData[$cnt]->isCorrect()) {
            $correct = 'wpProQuiz_answerCorrect';
        } elseif (isset($sAnswerData[$cnt]) && $sAnswerData[$cnt]) {
            $correct = 'wpProQuiz_answerIncorrect';
        }
        ?>
                <li class="<?php echo $correct;
        ?>" style="margin-left:0px;padding: 3px;">
                    <label>
                        <input disabled="disabled" type="<?php echo $anserType === 'single' ? 'radio' : 'checkbox';
        ?>"
                            <?php echo $sAnswerData[$cnt] ? 'checked="checked"' : '' ?>>
                        <?php echo $answerText;
        ?>
                    </label>
                </li>
                <?php
    }

    public function displayFreeAnswer($sAnswerData, $qAnswerData, $cnt)
    {
        $term_ans = str_replace("\r\n", "\n", strtolower($qAnswerData[$cnt]->getAnswer()));
        $term_ans = str_replace("\r", "\n", $term_ans);
        $term_ans = explode("\n", $term_ans);
        $term_ans = array_values(array_filter(array_map('trim', $term_ans)));
        if (isset($sAnswerData[0]) && in_array(strtolower(trim($sAnswerData[0])), $term_ans)) {
            $correct = 'wpProQuiz_answerCorrect';
        } else {
            $correct = 'wpProQuiz_answerIncorrect';
        }
        ?>
<li class="<?php echo $correct?>" style="margin-left:0px;padding: 3px;">
<label>
    <input type="text" disabled="disabled" style="width: 300px; padding: 3px;margin-bottom: 5px;"
        value="<?php echo esc_attr($sAnswerData[0]);
        ?>">
</label>
<br>
                </li>
                <?php
    }
    public function getMatrixSortAnswer($qAnswerData)
    {
        ?>
        <div class="wpProQuiz_matrixSortString">
            <h5 class="wpProQuiz_header"><?php _e('Sort elements', 'wp-pro-quiz');
        ?></h5>
            <ul class="wpProQuiz_sortStringList">
                <?php
                foreach ($qAnswerData as $key => $value) {
                    ?>
                    <li class="wpProQuiz_sortStringItem" data-pos="<?php echo $key;
                    ?>">
                        <?php echo $value->isSortStringHtml() ? $value->getSortString() : esc_html($value->getSortString());
                    ?>
                    </li>
                <?php
                }
        ?>
            </ul>
            <div style="clear: both;"></div>
        </div>
        <?php
    }
    /**
     * This function is used to fetch the data for cloze answer type
     * @param  [string] $answer_text [Answer Text]
     * @param  [string] $answerData  [Answer Data]
     * @return [string] $data        [Answer Data]
     */
    public function fetchCloze($answer_text, $answerData)
    {
        preg_match_all('#\{(.*?)(?:\|(\d+))?(?:[\s]+)?\}#im', $answer_text, $matches, PREG_SET_ORDER);
        $data = array();
        $index = 0;
        foreach ($matches as $key => $value) {
            $text = $value[1];
            $points = !empty($value[2]) ? (int)$value[2] : 1;
            $rowText = $multiTextData = array();
            $len = array();
            if (preg_match_all('#\[(.*?)\]#im', $text, $multiTextMatches)) {
                foreach ($multiTextMatches[1] as $multiText) {
                    $str_x = mb_strtolower(trim(html_entity_decode($multiText, ENT_QUOTES)));
                    $len[] = strlen($str_x);
                    $multiTextData[] = $str_x;
                    $rowText[] = $multiText;
                }
            } else {
                $str_x = mb_strtolower(trim(html_entity_decode($text, ENT_QUOTES)));
                $len[] = strlen($str_x);
                $multiTextData[] = $str_x;
                $rowText[] = $text;
            }
            $correct=$this->getCorrectClass($answerData, $index, $rowText);
            $data['data'][]=$this->getClozeAnswer($correct, $answerData, $index, $rowText);
            $data['correct'][] = $multiTextData;
            $data['points'][] = $points;
            $index++;
        }
        foreach ($data['data'] as $key => $value) {
            $answer_text = preg_replace("/\{[^}]+\}/", $value, $answer_text, 1);
        }
        $data['replace'] = $answer_text;
        return $data;
    }

    public function getClozeAnswer($correct, $answerData, $index, $rowText)
    {
        $ans_tag = '<span class="wpProQuiz_cloze '.$correct.'">'.esc_html(isset($answerData[$index]) ? empty($answerData[$index]) ? '---' : $answerData[$index]
                    : '---').'</span> ';
        $ans_tag .= '<span>('.implode(', ', $rowText).')</span>';
        return $ans_tag;
    }
    public function getCorrectClass($answerData, $index, $rowText)
    {
        $correct = 'wpProQuiz_answerIncorrect';
        if (isset($answerData[$index]) && in_array($answerData[$index], $rowText)) {
            $correct = 'wpProQuiz_answerCorrect';
        }
        return $correct;
    }
    /**
     * This is the callback function for cloze type of question.
     * @param  [int] $term [Answer Term]
     * @return [string]    [Cloze answer data]
     */
    public function clozeCallback($term)
    {
        return '<span class="wpProQuiz_cloze wpProQuiz_answerIncorrect">sdfdsf</span> <span>(play, love, hate)</span>';
        unset($term);
    }
    /**
     * This function is used to fetch the assessment type question data
     * @param  [string] $answerText [Answer Text]
     * @param  [string] $answerData [Answer Data]
     * @return [string]             [Answer Data]
     */
    public function fetchAssessment($answerText, $answerData)
    {
        preg_match_all('#\{(.*?)\}#im', $answerText, $matches);
        $data = array();
        for ($count = 0, $count_1 = count($matches[1]); $count < $count_1; $count++) {
            $match = $matches[1][$count];
            preg_match_all('#\[([^\|\]]+)(?:\|(\d+))?\]#im', $match, $wdm_ms);
            $ans_html = '';
            $checked = isset($answerData[$count]) ? $answerData[$count]-1 : -1;
            for ($count_j = 0, $counterj = count($wdm_ms[1]); $count_j < $counterj; $count_j++) {
                $ans_val = $wdm_ms[1][$count_j];
                $ans_html .= '<label><input type="radio" disabled="disabled" '.($checked == $count_j ? 'checked="checked"' : '').'>
                    '.$ans_val.'
                </label>';
            }
        }
        $data['replace'] = $ans_html;
        return $data;
    }
    /**
     * This function is used to get the question text
     * @param  [int] $pro_quiz_id [Pro Quiz ID]
     * @param  [int] $question_id [Question ID]
     * @return [string] [Returns protected value]
     */
    public function getQuestionText($pro_quiz_id, $question_id)
    {
        $questionMapper = new \WpProQuiz_Model_QuestionMapper();
        $questions = $questionMapper->fetchAll($pro_quiz_id);
        foreach ($questions as $ques) {
            if ($question_id == $this->getProtectedValue($ques, '_id')) {
                return $this->getProtectedValue($ques, '_question');
            }
        }
    }
    /**
     * This function is used to get the protected value
     * @param  [object] $obj  [Relative object]
     * @param  [string] $name [value name]
     * @return                [Protected value]
     */
    public function getProtectedValue($obj, $name)
    {
        $array = (array) $obj;
        $prefix = chr(0).'*'.chr(0);
        return $array[$prefix.$name];
    }
    /**
     * This function is used to show the the Result section.
     * @param  [int] $que_count     [Question count]
     * @param  [int] $pqid          [Pro quiz ID]
     * @param  [int] $statistics_id [Statistics Reference ID]
     */
    public function showResultBox($que_count, $pqid, $statistics_id)
    {
        // $user_id=get_current_user_id();
        // $quizzes_data = get_user_meta($user_id, '_sfwd-quizzes', true);
        global $wpdb;
        // $LD_QuizPro = new \LD_QuizPro();
        // $res = $LD_QuizPro->wp_pro_quiz_completed($statistics_id);
        // foreach ($quizzes_data as $quiz) {
        
        $tables = \LDLMS_DB::get_tables();

        if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
            $quiz_statistic_tbl = \LDLMS_DB::get_table_name( 'quiz_statistic' );
        } else {
            $quiz_statistic_tbl = $wpdb->prefix . 'wp_pro_quiz_statistic';
        } 
        
        $qcount     = $wpdb->get_results('SELECT count(*) as count FROM ' . $quiz_statistic_tbl . ' WHERE statistic_ref_id ='.$statistics_id.'', OBJECT);
        $qscore     = $wpdb->get_results('SELECT count(correct_count) as score FROM ' . $quiz_statistic_tbl . ' WHERE correct_count = 1 AND statistic_ref_id = '.$statistics_id.'', OBJECT);
        $qpoints    = $wpdb->get_results('SELECT SUM(points) as points FROM ' . $quiz_statistic_tbl . ' WHERE statistic_ref_id = '.$statistics_id.'', OBJECT);
        $qids   = $wpdb->get_results('SELECT question_id FROM ' . $quiz_statistic_tbl . ' WHERE statistic_ref_id = '.$statistics_id.'', OBJECT);
        $qtime      = $wpdb->get_results('SELECT SUM(question_time) as quet FROM ' . $quiz_statistic_tbl . ' WHERE statistic_ref_id ='.$statistics_id.'', OBJECT);
        // }
        $total_points = $this->getTotalPoints($qids);
        $count  = $qcount[0]->count;
        $score  = $qscore[0]->score;
        $points = $qpoints[0]->points;
        $per    = round(($points/$total_points)*100, 2);
        $seconds= $qtime[0]->quet;
        $time   = round($seconds);
        $statisticController = new \WpProQuiz_Controller_Statistics();
        $avg_res=$statisticController->getAverageResult($pqid);
        $avg_per=round($avg_res, 2);
        ?>
        <h4 class="wpProQuiz_header"><?php _e('Results', 'wp-pro-quiz');
        ?></h4>
        <?php  ?>
        <p style="padding:1%;">
            <?php printf(__('%s of %s questions answered correctly', 'wp-pro-quiz'), '<span class="wpProQuiz_correct_answer">'.$score.'</span>', '<span>'.$count.'</span>');
        ?>
        </p>
        <p class="wpProQuiz_quiz_time">
            <?php _e('Your time: <span></span>', 'wp-pro-quiz');
            printf('%02d:%02d:%02d', ($time/3600), ($time/60%60), ($time%60));
        ?>
            </p>
            <p class="wpProQuiz_points">
                <?php printf(__('You have reached %s of %s points, (%s)', 'wp-pro-quiz'), '<span>'.$points.'</span>', '<span>'.$total_points.'</span>', '<span>'.$per.'%'.'</span>');
        ?>
            </p>
            <div class="wpProQuiz_resultTable">
                <table>
                    <tbody>
                        <tr>
                            <td class="wpProQuiz_resultName"><?php _e('Average score', 'wp-pro-quiz');
        ?></td>
                            <td class="wpProQuiz_resultValue">
                                <div style="background-color: #6CA54C;width:<?php echo "$avg_per"."%";
        ?>;">&nbsp;</div>
                                <span><?php echo $avg_per."%";
        ?></span>
                            </td>
                        </tr>
                        <tr>
                            <td class="wpProQuiz_resultName"><?php _e('Your score', 'wp-pro-quiz');
        ?></td>
                            <td class="wpProQuiz_resultValue">
                                <div style="width:<?php echo "$per"."%";
        ?>;">&nbsp;</div>
                                <span><?php echo $per."%";
        ?></span>
                            </td>
                        </tr>
                    </tbody>
                </table>
            </div>
            <?php
        unset($que_count);
    }
    
    public function getTotalPoints($qids)
    {
        global $wpdb;
        $total_points = 0;

        if ( version_compare( LEARNDASH_VERSION, '3.1' ) >= 0 ) {
            $quiz_question_tbl = \LDLMS_DB::get_table_name( 'quiz_question' );
        } else {
            $quiz_question_tbl = $wpdb->prefix . 'wp_pro_quiz_question';
        }

        foreach ($qids as $qid) {
            $qpoint     = $wpdb->get_results('SELECT points FROM ' . $quiz_question_tbl . ' WHERE id = '.$qid->question_id.'', OBJECT);
            $total_points = $total_points + $qpoint[0]->points;
        }
        return $total_points;
    }
}
new QuizStatistics();
