<?php
/*
 * Plugin Name: Quiz Reporting Extension for LearnDash
 * Plugin URI: https://wisdmlabs.com/
 * Description: To export quiz data. The plugin exports single/all quiz data in .xls and .csv files.
 * Version: 2.1.1
 * Author: WisdmLabs
 * Author URI: https://wisdmlabs.com/
 * Text Domain: quiz_reporting_learndash
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}
/*
 * Set the plugin slug as default text domain.
 */
define( 'WDM_LD_QRE_TXT_DOMAIN', 'quiz_reporting_learndash' );
if ( ! defined( 'QRE_PLUGIN_VERSION' ) ) {
	define( 'QRE_PLUGIN_VERSION', '2.1.1' );
}
if ( ! defined( 'WDMMR_PLUGIN_DIR' ) ) {
	define( 'WDMMR_PLUGIN_DIR', dirname( __FILE__ ) );
}
if ( ! defined( 'WDMMR_PLUGIN_URL' ) ) {
	define( 'WDMMR_PLUGIN_URL', plugins_url( '/', __FILE__ ) );
}
if ( ! defined( 'MPDF_ALL_FONTS_URL' ) ) {
	define( 'MPDF_ALL_FONTS_URL', 'https://wisdmlabs.com/all-fonts/' );
}

global $qrePluginData;

add_action( 'plugins_loaded', 'qreloadLicense', 9 );

function qreloadLicense() {
	 // If LearnDash is active.
	if ( defined( 'LEARNDASH_VERSION' ) ) {
		global $qrePluginData;
		$qrePluginData = include_once( 'license.config.php' );
		require_once 'licensing/class-wdm-license.php';
		new Licensing\WdmLicense( $qrePluginData );
		wdmldqreLoadTextDomain();
	} else {
		add_action( 'admin_notices', 'wdmqr_admin_notices' );
	}
}

define( 'REQUIRE_VERSION', '2.0.2' );// Required version of LD

// Include files

include_once 'generate-quiz-report.php';

/**
 *Pagination class file
 */
require_once WDMMR_PLUGIN_DIR . '/vendor/pagination.class.php';

/**
 * Function wdmldqreLoadTextDomain() to load plugins text domain.
 */
function wdmldqreLoadTextDomain() {
	 load_plugin_textdomain( WDM_LD_QRE_TXT_DOMAIN, false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );

	$getDataFromDb = Licensing\WdmLicense::checkLicenseAvailiblity( 'quiz-reporting-extension' );
	if ( $getDataFromDb == 'available' ) {
		quizReport\QuizReport::getInstance();
		/**
		 * Load front-end files and classes.
		 */
		require_once dirname( __FILE__ ) . '/public/public.php';
	}
}


/**
 * To show admin notice if LearnDash LMS is deactivated.
 *
 * @since 1.2.0
 */
function wdmqr_admin_notices() {
	$err_inst_msg = sprintf(
		__( '%1$s plugin is not active. In order to make %2$s plugin work, you need to install and activate %3$s first.', WDM_LD_QRE_TXT_DOMAIN ),
		'<b>' . __( 'LearnDash LMS', WDM_LD_QRE_TXT_DOMAIN ) . '</b>',
		'<b>' . __( 'Quiz Reporting Plugin', WDM_LD_QRE_TXT_DOMAIN ) . '</b>',
		'<b>' . __( 'LearnDash LMS', WDM_LD_QRE_TXT_DOMAIN ) . '</b>'
	);
	echo '<div class="error"><p>' . $err_inst_msg . '</p></div>';
}
