<?php

namespace uncanny_learndash_codes;

/**
 * Class GenerateCodes
 *
 * @package uncanny_learndash_codes
 */
class GenerateCodes extends Config {

	/**
	 * @var array|null|object
	 */
	public static $courses;
	/**
	 * @var array|null|object
	 */
	public static $coupons;
	/**
	 * @var array|null|object
	 */
	public static $rejected_coupons;
	/**
	 * @var array|null|object
	 */
	public static $groups;

	/**
	 * @var array|null|object
	 */
	public static $num_coupons_added;
	/**
	 * @var array|null|object
	 */
	public static $num_coupons_requested;
	/**
	 * @var
	 */
	public static $generation_type;
	/**
	 * @var mixed|void
	 */
	public static $chars;
	/**
	 * @var
	 */
	protected $table;
	/**
	 * @var
	 */
	public static $manual_code_length;

	/**
	 * GenerateCodes constructor.
	 */
	function __construct() {
		self::$chars              = apply_filters( 'ulc_code_characters', 'abcdefghjkmnpqrstuvwxyz123456789ABCDEFGHJKLMNPQRSTUVWXYZ' );
		self::$manual_code_length = apply_filters( 'ulc_manual_code_length', 30 );

		if ( is_admin() ) {
			parent::__construct();
			self::$courses = LearnDash::get_courses();
			self::$groups  = LearnDash::get_groups();
		}
		add_action( 'admin_init', [ __CLASS__, 'process_submit' ], 9 );

		if ( isset( $_GET['msg'] ) && 'success' === sanitize_text_field( $_GET['msg'] ) ) {
			add_action( 'admin_notices', [ __CLASS__, 'success_notice' ] );
		}

		if ( isset( $_GET['msg'] ) && 'failed' === $_GET['msg'] ) {
			add_action( 'admin_notices', [ __CLASS__, 'failed_notice' ] );
		}
	}

	/**
	 *
	 */
	public static function process_submit() {
		if ( ( ! empty( $_POST ) && isset( $_POST['_custom_wpnonce'] ) ) && wp_verify_nonce( $_POST['_custom_wpnonce'], Config::get_project_name() ) ) {
			if ( isset( $_POST['edit_group'] ) && 'yes' === sanitize_text_field( $_POST['edit_group'] ) ) {
				$group_id           = absint( $_POST['group_id'] );
				$coupon_for         = sanitize_text_field( $_POST['coupon-for'] );
				$coupon_courses     = isset( $_POST['coupon-courses'] ) ? array_map( 'esc_attr', $_POST['coupon-courses'] ) : [];
				$coupon_group       = isset( $_POST['coupon-group'] ) ? array_map( 'esc_attr', $_POST['coupon-group'] ) : [];
				$coupon_paid_unpaid = sanitize_text_field( $_POST['coupon-paid-unpaid'] );
				$coupon_max_usage   = sanitize_text_field( $_POST['coupon-max-usage'] );
				$expiry_date        = sanitize_text_field( $_POST['expiry-date'] );
				$expiry_time        = sanitize_text_field( $_POST['expiry-time'] );
				$expiry             = '0000-00-00 00:00:00';
				if ( ! empty( $expiry_date ) ) {
					if ( ! empty( $expiry_time ) ) {
						$expiry = date( 'Y-m-d H:i:s', strtotime( $expiry_date . ' ' . $expiry_time ) );
					} else {
						$expiry = date( 'Y-m-d 23:59:59', strtotime( $expiry_date ) );
					}
				}

				if ( 'course' === $coupon_for ) {
					$linked_to = $coupon_courses;
				} else {
					$linked_to = $coupon_group;
				}

				global $wpdb;

				$wpdb->update( $wpdb->prefix . Config::$tbl_groups,
					[
						'code_for'        => $coupon_for,
						'paid_unpaid'     => $coupon_paid_unpaid,
						'expire_date'     => $expiry,
						'linked_to'       => maybe_serialize( $linked_to ),
						'issue_max_count' => $coupon_max_usage,
					],
					[ 'ID' => $group_id ],
					[
						'%s',
						'%s',
						'%s',
						'%s',
						'%d',
					],
					[ '%d' ] );
				wp_safe_redirect( admin_url( 'admin.php?page=uncanny-learndash-codes&message=Codes+Modified' ) );
				exit();
			} else {
				$coupons               = [];
				$prefix                = sanitize_text_field( $_POST['coupon-prefix'] );
				$suffix                = sanitize_text_field( $_POST['coupon-suffix'] );
				$dashes                = explode( '-', sanitize_text_field( $_POST['coupon-dash'] ) );
				$code_length           = sanitize_text_field( $_POST['coupon-length'] ) - strlen( $prefix ) - strlen( $suffix );
				$coupon_amount         = sanitize_text_field( $_POST['coupon-amount'] );
				$type                  = sanitize_text_field( $_POST['generation-type'] );
				$codes                 = Database::get_coupon_codes( sanitize_text_field( $_POST['coupon-length'] ), $type );
				$custom_codes          = sanitize_textarea_field( $_POST['manual-codes'] );
				$character_type        = isset( $_POST['coupon-character-type'] ) ? array_map( 'esc_attr', $_POST['coupon-character-type'] ) : [ 'uppercase-letters', 'numbers' ];
				$threshold             = 5;
				self::$generation_type = $type;

				if ( 'manual' === $type ) {
					$passed_coupons = [];
					$custom_coupons = explode( PHP_EOL, $custom_codes );
					if ( $custom_coupons ) {
						foreach ( $custom_coupons as $k => $code ) {
							if ( strlen( trim( $code ) ) >= 4 && strlen( trim( $code ) ) <= self::$manual_code_length ) {
								if ( empty( trim( $code ) ) ) {
									unset( $custom_coupons[ $k ] );
								} else {
									preg_match( '/[^A-Za-z0-9\-]/', trim( $code ), $matches );
									if ( empty( $matches ) && ! in_array( trim( $code ), $passed_coupons ) ) {
										$passed_coupons[] = sanitize_text_field( trim( $code ) );
									} else {
										self::$rejected_coupons[] = $code;
									}
								}
							} else {
								self::$rejected_coupons[] = $code;
							}
						}
					}
					$coupon_amount = count( $passed_coupons );
				}
				if ( $coupon_amount > 10000 ) {
					if ( $threshold > strlen( $coupon_amount ) ) {
						$threshold = absint( $threshold - strlen( $coupon_amount ) );
					} else {
						$threshold = 0;
					}
				}

				$looping                     = 0;
				self::$num_coupons_requested = $coupon_amount;

				if ( ! empty( $character_type ) ) {
					self::$chars = '';
					if ( in_array( 'numbers', $character_type ) ) {
						self::$chars .= '123456789';
					}
					if ( in_array( 'uppercase-letters', $character_type ) ) {
						self::$chars .= 'ABCDEFGHJKLMNPQRSTUVWXYZ';
					}
					if ( in_array( 'lowercase-letters', $character_type ) ) {
						self::$chars .= 'abcdefghjkmnpqrstuvwxyz';
					}
				}
				for ( $i = 0; $i < $coupon_amount; $i ++ ) {
					if ( 'auto' === $type ) {
						$coupon = $prefix . self::generate_random_string( $code_length ) . $suffix;
					} elseif ( 'manual' === $type ) {
						$coupon = isset( $passed_coupons[ $i ] ) ? $passed_coupons[ $i ] : '';
					}
					$coupon_new = [];
					$pointer    = 0;
					// check for unique code.
					$is_unique  = false;
					$force_exit = 0;
					while ( false === $is_unique ) {
						$looping ++;
						if ( ! in_array( $coupon, $codes ) ) {
							$is_unique = true;
						} elseif ( 'auto' === $type ) {
							$coupon    = $prefix . self::generate_random_string( $code_length ) . $suffix;
							$is_unique = false;
							$force_exit ++;
						} elseif ( 'manual' === $type && in_array( $coupon, $codes ) ) {
							$is_unique = false;
							$coupon    = '';
							$force_exit ++;
						}

						if ( $force_exit == $threshold ) {
							$is_unique = true;
							$coupon    = '';
						}
					}
					if ( $force_exit < $threshold && ! empty( $coupon ) ) {
						$codes[] = $coupon;
						if ( 'auto' === $type ) {
							foreach ( $dashes as $dash ) {
								$dash = (int) $dash;
								if ( $dash ) {
									if ( strlen( $coupon ) < $pointer + $dash ) {
										$dash = strlen( $coupon ) - $pointer;
									}
									if ( $pointer < strlen( $coupon ) ) {
										$coupon_new[] = substr( $coupon, $pointer, $dash );
										$pointer      = $pointer + $dash;
									}
								}
							}

							if ( $pointer < strlen( $coupon ) ) {
								$coupon_new[] = substr( $coupon, $pointer );
							}
							$coupons[] = implode( '-', $coupon_new );
						} elseif ( 'manual' === $type ) {
							//reverify
							preg_match( '/[^A-Za-z0-9\-]/', trim( $coupon ), $matches );
							if ( empty( $matches ) && ! in_array( $coupon, $coupons ) ) {
								$coupons[] = $coupon;
							} elseif ( ! in_array( $coupon, self::$rejected_coupons ) ) {
								self::$rejected_coupons[] = $coupon;
							}
						}
					}

				}
				//}
				self::$num_coupons_added = count( $coupons );
				//Sanitize values
				$data = [
					'coupon-amount'         => self::$num_coupons_added,
					'coupon-prefix'         => $prefix,
					'coupon-suffix'         => $suffix,
					'coupon-dash'           => sanitize_text_field( $_POST['coupon-dash'] ),
					'coupon-length'         => sanitize_text_field( $_POST['coupon-length'] ),
					'generation-type'       => $type,
					'manual-codes'          => $custom_codes,
					'coupon-for'            => sanitize_text_field( $_POST['coupon-for'] ),
					'coupon-courses'        => isset( $_POST['coupon-courses'] ) ? array_map( 'esc_attr', $_POST['coupon-courses'] ) : [],
					'coupon-group'          => isset( $_POST['coupon-group'] ) ? array_map( 'esc_attr', $_POST['coupon-group'] ) : [],
					'expiry-date'           => sanitize_text_field( $_POST['expiry-date'] ),
					'coupon-paid-unpaid'    => sanitize_text_field( $_POST['coupon-paid-unpaid'] ),
					'coupon-max-usage'      => sanitize_text_field( $_POST['coupon-max-usage'] ),
					'coupon-character-type' => isset( $_POST['coupon-character-type'] ) ? array_map( 'esc_attr', $_POST['coupon-character-type'] ) : [],
				];

				if ( ! empty( $coupons ) ) {
					$group_id = Database::add_coupons( $data, $coupons );
					if ( (int) self::$num_coupons_added === (int) self::$num_coupons_requested && empty( self::$rejected_coupons ) ) {
						wp_safe_redirect( admin_url( 'admin.php' ) . '?page=uncanny-learndash-codes&msg=success&codes_generated=' . self::$num_coupons_added . '&group_id=' . $group_id );
						exit();
					} else {
						add_action( 'admin_notices', [ __CLASS__, 'success_notice' ] );
					}
				} elseif ( empty( $coupons ) || 0 === self::$num_coupons_added ) {
					add_action( 'admin_notices', [ __CLASS__, 'failed_notice' ] );
				}
			}
		}
	}

	/**
	 * @param $length
	 *
	 * @return string
	 */
	private static function generate_random_string( $length ) {
		$string = "";
		for ( $i = 0; $i < $length; $i ++ ) {
			$string .= self::$chars[ mt_rand( 0, strlen( self::$chars ) - 1 ) ];
		}

		return $string;
	}

	/**
	 *
	 */
	public static function success_notice() {
		$codes_generated = '';
		if ( isset( $_GET['codes_generated'] ) ) {
			$codes_generated = absint( $_GET['codes_generated'] );
		}

		$message = sprintf( __( '<h4>%s Codes created listed below.</h4>', 'uncanny-learndash-codes' ), $codes_generated );
		if ( self::$num_coupons_added < self::$num_coupons_requested && 'auto' === self::$generation_type ) {
			$message = sprintf( __( '<h4>Only %s unique codes were generated. Try increasing the number of characters to generate more codes.</h4>', 'uncanny-learndash-codes' ), self::$num_coupons_added );
		}

		if ( 'manual' === self::$generation_type && ! empty( self::$rejected_coupons ) ) {
			$message = sprintf( __( '<h4>Only %s unique codes were added out of %s codes. The following codes were rejected:<br />%s.<br /><a href="%s">Manage Codes</a></h4>', 'uncanny-learndash-codes' ), self::$num_coupons_added, (int) self::$num_coupons_added + (int) count( self::$rejected_coupons ), join( '<br />', self::$rejected_coupons ), esc_attr( add_query_arg( 'page', 'uncanny-learndash-codes' ) ) );
		}

		if ( self::$num_coupons_added < self::$num_coupons_requested && 'manual' === self::$generation_type && empty( self::$rejected_coupons ) ) {
			$message = sprintf( __( '<h4>Only %s unique codes were added out of %s codes. <a href="%s">Manage Codes</a></h4>', 'uncanny-learndash-codes' ), self::$num_coupons_added, self::$num_coupons_requested, esc_attr( add_query_arg( 'page', 'uncanny-learndash-codes' ) ) );
		}
		?>
        <div class="updated notice">
            <p><?php echo $message; ?></p>
        </div>
		<?php
	}

	/**
	 *
	 */
	public static function failed_notice() {
		$message = sprintf( __( '<h4>0 Codes Created! None of the generated codes were unique.</h4>', 'uncanny-learndash-codes' ) );
		if ( 'manual' === self::$generation_type && ! empty( self::$rejected_coupons ) ) {
			$message = sprintf( __( '<h4>The following code(s) were rejected:<br />%s.<br /></h4>', 'uncanny-learndash-codes' ), join( '<br />', self::$rejected_coupons ) );
		}
		?>
        <div class="notice notice-error is-dismissible">
            <p><?php echo $message; ?></p>
        </div>
		<?php
	}
}