<?php
/**
 * Handles Plugin Activation logic
 *
 * @since 2.2.2
 *
 * @package LearnDash\Reviews_Plus
 */

namespace LearnDash\Reviews_Plus;

/**
 * Plugin Activation class.
 *
 * @since 2.2.2
 */
class Activation {
	/**
	 * Runs an authentication check. If the check fails, the plugin is deactivated and an error message is shown.
	 *
	 * @since 2.2.2
	 *
	 * @return void
	 */
	public static function run(): void {
		$result = Licensing\Authentication::verify_token();

		if ( ! is_wp_error( $result ) ) {
			return;
		}

		deactivate_plugins( plugin_basename( RRF_PLUGIN_FILE ) );

		die( esc_html( $result->get_error_message() ) );
	}
}
