<?php
/**
 * Plugin Name: LearnDash LMS - Ratings, Reviews and Feedback
 * Plugin URI: https://go.learndash.com/rrf
 * Description: This plugin provides a feature to rate, review and feedback on course.
 * Version: 2.2.4
 * Requires PHP: 7.4
 * Requires at least: 6.1
 * Tested up to: 6.7.2
 * Author: LearnDash
 * Author URI: https://learndash.com
 * Text Domain: wdm_ld_course_review
 * Domain Path: /languages
 *
 * @package LearnDash\Reviews_Plus
 *
 * cSpell:ignore capabilites
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

require_once plugin_dir_path( __FILE__ ) . 'vendor/autoload.php';
require_once plugin_dir_path( __FILE__ ) . 'vendor-prefixed/autoload.php';

use LearnDash\Reviews_Plus\Activation;
use LearnDash\Reviews_Plus\Licensing\Migration as License_Migration;
use LearnDash\Reviews_Plus\Dependency_Checker;
use LearnDash\Reviews_Plus\Plugin;

if ( ! defined( 'RRF_PLUGIN_FILE' ) ) {
	define( 'RRF_PLUGIN_FILE', __FILE__ );
}

if ( ! defined( 'RRF_PLUGIN_PATH' ) ) {
	define( 'RRF_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );
}

if ( ! defined( 'RRF_PLUGIN_URL' ) ) {
	define( 'RRF_PLUGIN_URL', plugins_url( '', __FILE__ ) );
}

/**
 * Define LearnDash Licensing URL.
 *
 * @since 2.2.2
 */
if ( ! defined( 'RRF_LICENSING_SITE_URL' ) ) {
	define( 'RRF_LICENSING_SITE_URL', 'https://checkout.learndash.com/wp-json/learndash/v2/site/auth_token' );
}

/**
 * Define LearnDash Check Licensing URL.
 *
 * @since 2.2.2
 */
if ( ! defined( 'RRF_LICENSING_CHECK_LICENSE_URL' ) ) {
	define( 'RRF_LICENSING_CHECK_LICENSE_URL', 'https://checkout.learndash.com/wp-json/learndash/v2/site/auth' );
}

add_action( 'after_setup_theme', 'rrf_load_textdomain' );

/**
 * Load plugin textdomain.
 */
function rrf_load_textdomain() {
	load_plugin_textdomain( 'wdm_ld_course_review', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
}

register_activation_hook( RRF_PLUGIN_FILE, [ Activation::class, 'run' ] );

add_action( 'admin_notices', [ License_Migration::class, 'run' ] );

$learndash_reviews_plus_dependency_checker = new Dependency_Checker();

$learndash_reviews_plus_dependency_checker->set_dependencies(
	[
		'sfwd-lms/sfwd_lms.php' => [
			'label'            => '<a href="https://www.learndash.com" target="_blank">' . __( 'LearnDash LMS', 'wdm_ld_course_review' ) . '</a>',
			'class'            => 'SFWD_LMS',
			'version_constant' => 'LEARNDASH_VERSION',
			'min_version'      => '4.7.0',
		],
	]
);

$learndash_reviews_plus_dependency_checker->set_message(
	esc_html__( 'LearnDash LMS - Ratings, Reviews and Feedback requires the following plugin(s) to be active:', 'wdm_ld_course_review' )
);

// Loading all constants.
require_once 'includes/constants.php';
// Loading all the helper(pluggable) functions.
require_once 'includes/functions.php';
// Loading all deprecated functions.
require_once 'includes/deprecated-functions.php';

global $rrf_plugin_data;

if ( ! function_exists( 'rrf_load_licensing_module' ) ) {
	/**
	 * Change file paths if not accessed from main file
	 */
	function rrf_load_licensing_module() {
		_deprecated_function( __FUNCTION__, '2.2.2' );
	}
}

if ( ! function_exists( 'rrf_load_plugin_files' ) ) {
	/**
	 * Load all the plugin files.
	 */
	function rrf_load_plugin_files() {
		include_once RRF_PLUGIN_PATH . 'includes/class-rrf-custom-db-table.php';
		include_once RRF_PLUGIN_PATH . 'admin/admin.php';        // loading all admin e.g(dashboard) related functionality.
		include_once RRF_PLUGIN_PATH . 'public/public.php';      // loading all front-end related functionality.
	}
}
if ( ! function_exists( 'wdm_crr_check_ld_activation' ) ) {
	/**
	 * Check whether LearnDash is activated or not. If not then notify the admin.
	 *
	 * @since 1.0.0
	 * @deprecated 2.2.3 Use the Dependency_Checker class instead.
	 *
	 * @return void
	 */
	function wdm_crr_check_ld_activation() {
		_deprecated_function( __FUNCTION__, '2.2.3', 'Use the Dependency_Checker class instead' );

		// if multisite.
		if ( is_multisite() ) {
			if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
				include_once ABSPATH . '/wp-admin/includes/plugin.php';
			}

			if ( in_array( 'sfwd-lms/sfwd_lms.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ), true ) ) {
				return;
			}

			if ( ! is_plugin_active_for_network( 'sfwd-lms/sfwd_lms.php' ) ) {
				echo WDM_LD_COURSE_ACTIVATION_MSG;// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			}
		} elseif ( ! in_array( 'sfwd-lms/sfwd_lms.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ), true ) ) {
			echo WDM_LD_COURSE_ACTIVATION_MSG;// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		}
	}
}

if ( ! function_exists( 'wdm_course_review_plugin_activate' ) ) {
	/**
	 * Adding capabilites to admin and instructor role.
	 *
	 * @param bool $network_wide [is network wide].
	 */
	function wdm_course_review_plugin_activate( $network_wide ) {
		include_once RRF_PLUGIN_PATH . 'admin/admin.php';
		global $wp_roles;
		$roles        = $wp_roles->roles;
		$all_role_ids = array();

		foreach ( $roles as $role_id => $role ) {
			$all_role_ids[] = $role_id;
			unset( $role );
		}

		$admin_capabilities  = array(
			// for rating and review.
			'read_private_wdm_course_reviews',
			'publish_wdm_course_reviews',
			'edit_wdm_course_reviews',
			'edit_published_wdm_course_reviews',
			'edit_private_wdm_course_reviews',
			'edit_others_wdm_course_reviews',
			'delete_wdm_course_reviews',
			'delete_published_wdm_course_reviews',
			'delete_private_wdm_course_reviews',
			'delete_others_wdm_course_reviews',
			// for feedback.
			'read_private_wdm_course_feedbacks',
			'publish_wdm_course_feedbacks',
			'edit_wdm_course_feedbacks',
			'edit_published_wdm_course_feedbacks',
			'edit_private_wdm_course_feedbacks',
			'edit_others_wdm_course_feedbacks',
			'delete_wdm_course_feedbacks',
			'delete_published_wdm_course_feedbacks',
			'delete_private_wdm_course_feedbacks',
			'delete_others_wdm_course_feedbacks',
		);
		$limited_capabilites = array(
			'read_private_wdm_course_reviews',
			'edit_wdm_course_reviews',
			// 'edit_published_wdm_course_reviews' ,
			'edit_private_wdm_course_reviews',
		);
		$full_access_users   = array(
			'administrator',
			// 'wdm_instructor',
		);

		$limited_access_users = array_diff( $all_role_ids, $full_access_users );

		if ( is_multisite() && $network_wide ) {
			global $wpdb;
			foreach ( $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" ) as $blog_id ) {
				switch_to_blog( $blog_id );
				rrf_add_capabilities( $full_access_users, $admin_capabilities );
				/*rrf_add_capabilities($limited_access_users, $limited_capabilites);*/
				restore_current_blog();
			}
		} else {
			rrf_add_capabilities( $full_access_users, $admin_capabilities );
			/*rrf_add_capabilities($limited_access_users, $limited_capabilites);*/
		}
		unset( $limited_capabilites );
		unset( $limited_access_users );

		// Register post types and flush Rewrite rules
		if ( class_exists( '\ns_wdm_ld_course_review\Course_Review_CPT' ) && class_exists( '\ns_wdm_ld_course_review\Course_Feedback_CPT' ) ) {
			$review_instance   = \ns_wdm_ld_course_review\Course_Review_CPT::get_instance();
			$feedback_instance = \ns_wdm_ld_course_review\Course_Feedback_CPT::get_instance();
			$review_instance->create_post_type();
			$feedback_instance->create_post_type();

			flush_rewrite_rules();
		} else {
			throw new Exception( 'Class does not exist.' );
		}
	}
}

add_action(
	'plugins_loaded',
	function () use ( $learndash_reviews_plus_dependency_checker ) {
		// If plugin requirements aren't met, don't run anything else to prevent possible fatal errors.
		if ( ! $learndash_reviews_plus_dependency_checker->check_dependency_results() || php_sapi_name() === 'cli' ) {
			return;
		}

		rrf_load_plugin_files();
		learndash_register_provider( Plugin::class );
	},
	50
);
